//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bb401KDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.7 com/java/src/com/ibm/beans/demos/a401k/b401kDap.java, bb401KDemo, bb97c, bb97c9747e 10/31/97 17:31:15 [11/26/97 04:21:45]

package com.ibm.beans.demos.a401k;

import com.ibm.beans.demos.a401k.acctid.AccountIDChangedEvent;
import com.ibm.beans.demos.a401k.acctid.AccountIDChangedListener;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 * <p>
 * This class is part of the 401k demo and implements the b401kDap or data
 * access part (DAP) bean.  The DAP bean serves as the event sink for
 * the AccountIDInformationChanged event and the PercentChanged event.
 * The AccountIDInformationChanged event is sent by the AccountID bean when the user types in a
 * new account id and hits the submit button on the AccountID bean.  The
 * PercentChanged event is sent by the b401kViewer bean when the user changes
 * the percentage of one of the investment choices.
 * <p>
 * This DAP bean serves as the event source for the b401kInformationChanged
 * event.  The event is sent the the b401kViewer bean when the investment
 * choices and percentages need to be updated on the viewer.
 *
 * @see com.ibm.beans.demos.a401k.acctid.AccountIDChangedEvent
 * @see com.ibm.beans.demos.a401k.acctid.AccountIDChangedListener
 * @see com.ibm.beans.demos.a401k.PercentEvent
 * @see com.ibm.beans.demos.a401k.b401kEvent
 * @see com.ibm.beans.demos.a401k.PercentListener
 * @see com.ibm.beans.demos.a401k.b401kListener
 */
public class b401kDap implements Serializable, AccountIDChangedListener, PercentListener
{
  private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
  private final static long serialVersionUID = 2000L; // serial version unique ID

  private final static int NUMROWS    = 8;

  private Vector listeners = new Vector();  // list of listeners

  //hard code values for demo purposes
  private String[] investChoices             = new String[NUMROWS];
  private int[]    investChoicesPercentages1 = new int[NUMROWS];
  private int[]    investChoicesPercentages2 = new int[NUMROWS];
  private transient b401kResourceBundle msgs = null;

  private static boolean bUseFirst = true;

   /**
   * Constructs an instance of the b401kDap bean.  The constructor initializes
   * the investment choices and the percentage of each investment choice that
   * make up the total portfolio.
   */
  public b401kDap()
  {

    investChoices[0]  = getMsg("MoneyMarketFund");
    investChoices[1]  = getMsg("LargeCompanyIndex");
    investChoices[2]  = getMsg("SmallCompanyStock");
    investChoices[3]  = getMsg("IBMStockFund");
    investChoices[4]  = getMsg("FixedIncomeFund");
    investChoices[5]  = getMsg("TotalBondMarket");
    investChoices[6]  = getMsg("InternationalStock");
    investChoices[7]  = getMsg("ModerateLifeStrategy");

    investChoicesPercentages1[0] = 5;
    investChoicesPercentages1[1] = 15;
    investChoicesPercentages1[2] = 5;
    investChoicesPercentages1[3] = 25;
    investChoicesPercentages1[4] = 5;
    investChoicesPercentages1[5] = 5;
    investChoicesPercentages1[6] = 35;
    investChoicesPercentages1[7] = 5;

    investChoicesPercentages2[0] = 0;
    investChoicesPercentages2[1] = 0;
    investChoicesPercentages2[2] = 20;
    investChoicesPercentages2[3] = 0;
    investChoicesPercentages2[4] = 30;
    investChoicesPercentages2[5] = 10;
    investChoicesPercentages2[6] = 0;
    investChoicesPercentages2[7] = 40;
  }

   /**
    * Responds to calls on objects of this class when the objects are deserialized.
    * This method is called whenever the b401kDap bean is instantiated from
    * serialized data (such as a .ser file).  It is overridden so that transient
    * data (that is, data not serialized) can be initialized (then it just calls
    * defaultReadObject on the ObjectInputStream to let it take care of initializing
    * all of the non-transient data).
    */
  private void readObject(ObjectInputStream in) throws ClassNotFoundException, IOException
  {
    msgs = null;
    in.defaultReadObject();
  }

   /**
   * Accessor (set) method for the percentage value of a particular
   * investment choice.
   * @param index The index into the investment choices array or list.
   * @param value The new percentage value for that particular investment
   * choice.
   */
  public void setInvestChoicesPercentage(int index, int value)
  {
    if (bUseFirst)
      investChoicesPercentages1[index] = value;
    else
      investChoicesPercentages2[index] = value;
  }

   /**
    * Responds to an AccountIDChangedEvent. This method is invoked when
    * the AccountID bean has been wired to this DAP bean and
    * the user enters an account number and presses the submit button
    * on the AccountID.
    * @param  e The event signifying that a new account id has been submitted.
    */
  public void AccountIDInformationChanged(AccountIDChangedEvent e)
  {
    if (bUseFirst)
    {
      fireB401k(investChoices,investChoicesPercentages1);
      bUseFirst = false;
    }
    else
    {
      fireB401k(investChoices, investChoicesPercentages2);
      bUseFirst = true;
    }
  }

   /**
    * Responds to a PercentEvent. This method is invoked when
    * the b401kViewer bean has been wired to this DAP bean and
    * the user changes a percentage of one of the investment
    * choices.
    * @param  pE The event signifying that a new percentage has been chosen
   */
  public void PercentChanged(PercentEvent pE)
  {
    int indx    = pE.getIndex();
    int percent = pE.getNewPercent();

    if (!bUseFirst)  // inverse of flag since changed after notify
    {
      investChoicesPercentages1[indx] = percent;
      fireB401k(investChoices,investChoicesPercentages1);
    }
    else
    {
      investChoicesPercentages2[indx] = percent;
      fireB401k(investChoices, investChoicesPercentages2);
    }
  }

   /**
    * Allows other beans to listen to the b401kInformationChanged events this bean generates.
    * This is the addListener method for all listeners that want to be notified
    * when a b401kInformationChanged event occurs.  In the demo, the b401kViewer
    * bean is wired to this DAP bean and is notified when the b401kInformationChanged event happens.
    * @param  b401kL The registering listener.
    * @see #removeb401kListener
    */
  public synchronized void addb401kListener(b401kListener b401kL)
  {
    listeners.addElement(b401kL);
  }

   /**
    * Allows beans to remove themselves from the listener list for this bean's
    * b401kInformationChanged event.
    * This is the removeListener method for all listeners that want to be notified
    * when a b401kInformationChanged event occurs.  In the demo, the b401kViewer
    * bean is wired to this DAP bean and is notified when the b401kInformationChanged event
    * happens.
    * @param  b401kL The unregistering listener.
    * @see #addb401kListener
    */
  public synchronized void removeb401kListener(b401kListener b401kL)
  {
    listeners.removeElement(b401kL);
  }

   /**
   * Notifies all registered listeners that a b401kInformationChanged event has
   * occurred.  In the demo, this event fires in response to a users
   * investment portfolio being selected or changed.
   * @param  ic  The string array of the names of the investment choices that make up the portfolio.
   * @param  icp The integer array of the corresponding percentages of the investment choices that make up the portfolio.
   *
   */
  public void fireB401k(String[] ic, int[] icp)
  {
    Vector l;

    int totp = icp[0] + icp[1] + icp[2] + icp[3] + icp[4] + icp[5] + icp[6] +icp[7];

    b401kEvent b401kEvt = new b401kEvent(this, ic, icp, totp);

    // must copy the Vector here in order to freeze the state of the
    // set of EventListeners the event should be delivered to prior
    // to delivery.  This ensures that any changes made to the Vector
    // from a target listener's method, during the delivery of this
    // event will not take effect until after the event is delivered.
    synchronized(this) { l = (Vector)listeners.clone(); }

    for (int i = 0; i < l.size(); i++)
    {
      ((b401kListener)l.elementAt(i)).b401kInformationChanged(b401kEvt);
    }
  }

  String getMsg(String key)
  {
    if (msgs == null)
      msgs = (b401kResourceBundle)ResourceBundle.getBundle(
        "com.ibm.beans.demos.a401k.b401kResourceBundle");

    return (String)msgs.handleGetObject(key);
  }

}

