//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bb401KDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.6 com/java/src/com/ibm/beans/demos/a401k/b401kViewer.java, bb401KDemo, bb97c, bb97c9747e 10/31/97 17:31:30 [11/26/97 04:21:39]

package com.ibm.beans.demos.a401k;

import java.awt.BorderLayout;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.ResourceBundle;
import java.util.Vector;

/**
 * <p>
 * This bean is a simple viewer bean for 401K Investment Portfolio Percentage Information.
 * It is wired to the b401kDap bean which itself is wired to the accountID bean.  When
 * a user enters a new account id and hits the submit button, a b401kInformationChanged event
 * is sent from the b401kDap to this viewer with information detailing the names and percentages
 * of that users 401K investment account.
 * @see com.ibm.beans.demos.a401k.b401kDap
 * @see com.ibm.beans.demos.a401k.b401kListener
 * @see com.ibm.beans.demos.a401k.PercentEvent
 * @see com.ibm.beans.demos.a401k.b401kEvent
 */
public class b401kViewer extends Panel implements b401kListener, ItemListener, Serializable
{
  private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
  private final static long serialVersionUID = 2000L; // serial version unique ID

  private final static int    GAP     = 4;         // used for insets
  private final static int    NUMROWS = 8;

  private Vector listeners = new Vector();  // list of listeners
  private Panel innerPanel;
  private Label mainTitle;
  private transient b401kResourceBundle msgs = null;
  private String my401KViewer;

  transient private Label[]  choiceLabels;      // Must be transient to not be saved as null
  transient private Choice[] choicePercentages;
  transient private Label   totLabel;

  static boolean bPaint = true;
  static boolean bFirstTime = false;

  // -- constructor --
  /**
   * Constructs an instance of the b401kViewer bean.  The constructor
   * initializes instance data.
   */
  public b401kViewer ( )
  {
    setBackground( new Color(255,255,204) ); //Creme
    setSize( new Dimension(300,210) );
    my401KViewer = getMsg("401KViewer");
  }

  /**
   * Draws the title string when this bean is displayed at design time.
   *
   */
  public void paint(Graphics g)
  {
     if (bPaint)
     {
        g.drawString(my401KViewer,10,30);
     }
     if (bFirstTime)
     {
        validate();
        invalidate();
        bFirstTime = false;
     }
  }

  /**
   * Return the minimum size of the b401kViewer bean.
   * @return The dimensions of the minimum size of the b401kViewer bean.
   *
   */
  public Dimension getMinimumSize ( )
  {
    return new Dimension( 200, 200 );
  }

  /**
   * Initialize and build the graphical user interface for the b401kViewer bean.
   *
   */
  public void initialize()
  {
    bPaint = false;

    setLayout( new BorderLayout() );

    setBackground( new Color(255,255,204) );

    mainTitle  = new Label( getMsg("401KChoices"), Label.CENTER );

    Font _font = new Font( "Helvetica", Font.BOLD, 10 );
    mainTitle.setFont( _font );
    mainTitle.setBackground( new Color(204,0,0) ); // Red background
    mainTitle.setForeground( Color.white );        // White letters
    add( "North", mainTitle );

    innerPanel = new Panel();
    innerPanel.setLayout( new GridLayout( 9, 2, GAP, GAP ) );

    choiceLabels      = new Label[NUMROWS];
    choicePercentages = new Choice[NUMROWS];

    Color color1 = new Color( 102, 102, 51 );         // @X1, Green
    Color color2 = new Color( 206, 206, 156 );        // @X1, Green Light

    for(int i=0; i<NUMROWS; ++i)
    {
      choiceLabels[i]      = new Label("");
      choicePercentages[i] = new Choice();
      choicePercentages[i].addItemListener(this);

      for(int p=0; p<=100; p=p+5)                   // Pre-fill with 0%, 5%, 10%, ... 100%
        choicePercentages[i].addItem(""+p);

      choicePercentages[i].select("0");             // Default to 0%

      if (i % 2 == 0) { /* jwa: alternate colors */
         choiceLabels[i].setBackground(color1);
         choiceLabels[i].setForeground(Color.white);

         choicePercentages[i].setBackground(color1);
         choicePercentages[i].setForeground(Color.white);
      } else {
         choiceLabels[i].setBackground(color2);
         choiceLabels[i].setForeground(Color.black);

         choicePercentages[i].setBackground(color2);
         choicePercentages[i].setForeground(Color.black);
      } /* endif */

      innerPanel.add(choiceLabels[i]     );
      innerPanel.add(choicePercentages[i]);
    }


    innerPanel.add(new Label( getMsg("TotalPercentage") ));
    totLabel = new Label(" 0%");
    innerPanel.add(totLabel);

    add( "Center", innerPanel );

    invalidate();
    validate();

    bFirstTime = true;
  }

   /**
    * Allows other beans to listen to the PercentChanged events this bean generates.
    * This is the addListener method for all listeners that want to be notified
    * when a PercentChanged event occurs.  In the demo, the b401kViewer
    * bean is wired to the b401kDap bean and is notified when the PercentChanged event happens.
    * @param  pL The registering listener.
    * @see #removePercentListener
    */
  public synchronized void addPercentListener(PercentListener pL)
  {
    listeners.addElement(pL);
  }

   /**
    * Allows beans to remove themselves from the listener list for this bean's
    * PercentChanged event.
    * This is the removeListener method for all listeners that want to be notified
    * when a PercentChanged event occurs.  In the demo, the b401kViewer
    * bean is wired to the b401kDap bean and is notified when the PercentChanged event
    * happens.
    * @param  pL The unregistering listener.
    * @see #addPercentListener
    */
  public synchronized void removePercentListener(PercentListener pL)
  {
    listeners.removeElement(pL);
  }


   /**
    * Responds to an ItemEvent. This method is invoked when
    * a user changes one of the percentages associated with
    * an investment choice.  A PercentChanged event is then fired
    * to the b401kDap providing this information.
    * @param  e The event signifying that a new percentage has been chosen
   */
  public void itemStateChanged(ItemEvent e)
  {
    String newPercentStr = (String) e.getItem();
    Object obj           = e.getSource();

    try
    {
      int newPercent = Integer.decode(newPercentStr).intValue();

      for(int i=0; i<NUMROWS; ++i)
      {
        if (choicePercentages[i] == obj)
        {
          PercentEvent pEvt = new PercentEvent(this, i, "name", newPercent);
          firePercent(pEvt);

          break;
        }
      }
    }
    catch (NumberFormatException nfex)
    {
    }
  }

   /**
   * Notifies all registered listeners that a PercentChanged event has
   * occurred.  In the demo, this event fires in response to a users
   * changing the percentage of one of the investment portfolio choices.
   * @param  pEvt  The PercentEvent that is the parameter of the PercentChanged event method.
   *
   */
  public void firePercent(PercentEvent pEvt)
  {
    Vector vl;
    synchronized(this) { vl = (Vector)listeners.clone(); }

    for (int il = 0; il < vl.size(); il++)
    {
      ((PercentListener)vl.elementAt(il)).PercentChanged(pEvt);
    }
  }

   /**
    * Responds to a b401kInformationChanged event. This method is invoked by
    * the b401kDap bean when it is wired to the b401kViewer bean and
    * the b401kDap bean receives notification that the 401K Investment
    * Choice Percentages have been updated.
    * @param  b401kE The event signifying that a the 401K Investment Choice Percentages have been updated
   */
  public void b401kInformationChanged(b401kEvent b401kE)
  {
    String[] ic = b401kE.getInvestmentChoices();
    for(int i=0; i<NUMROWS; ++i)
      choiceLabels[i].setText(ic[i]);      // Set each investment choice name

    int[] icp = b401kE.getInvestmentChoicePercentages();
    for(int i=0; i<NUMROWS; ++i)
      choicePercentages[i].select(""+icp[i]);  // Select item on string type

    int total = b401kE.getTotalPercentage();
    if (total != 100)                                // Warn user if total is not 100%
    {
      totLabel.setBackground( Color.red );   // Red background
      totLabel.setForeground( Color.white ); // White text
    }
    else
    {
      totLabel.setBackground( new Color(255,255,204) ); // Creme background
      totLabel.setForeground( Color.black );            // Black text
    }

    totLabel.setText(" "+total+"%");
  }

  /**
   * This method is called whenever the b401kViewer bean is instantiated from
   * serialized data (such as a .ser file).  It is overridden so that
   * data can be initialized (then it just calls
   * defaultReadObject on the ObjectInputStream to let it take care of initializing
   * all of the non-transient data).
   *
   */
  private void readObject(java.io.ObjectInputStream in) throws ClassNotFoundException, IOException
  {
    msgs = null;
    initialize();
    in.defaultReadObject();
  }

   /**
   * Set the insets for the b401kViewer.
   *
   * @return The insets of the b401kViewer bean.
   */
  public Insets getInsets()
  {
    return new Insets(GAP,GAP,GAP,GAP);   // top,bottom,left,right edges
  }

  String getMsg(String key)
  {
    if (msgs == null)
      msgs = (b401kResourceBundle)ResourceBundle.getBundle(
        "com.ibm.beans.demos.a401k.b401kResourceBundle");

    return (String)msgs.handleGetObject(key);
  }

} // class b401KViewer, april29 test

