//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbTravelDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
//   @(#) 1.28 com/java/src/com/ibm/beans/demos/travel/Itinerary.java, bbTravelDemo, bb97c, bb97c9750e 11/20/97 18:12:05 [12/11/97 17:03:02]
package com.ibm.beans.demos.travel;

import java.awt.Panel;
import java.awt.Dimension;
import java.awt.Button;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Component;
import java.awt.Insets;
import java.awt.Container;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;

import java.util.Vector;
import java.util.ResourceBundle;

import java.beans.Customizer;
import java.beans.PropertyChangeListener;

import com.ibm.beans.demos.travel.flight.FlightReservedListener;
import com.ibm.beans.demos.travel.flight.FlightRequestedListener;
import com.ibm.beans.demos.travel.flight.FlightInformation;
import com.ibm.beans.demos.travel.flight.FlightRequestedEvent;
import com.ibm.beans.demos.travel.flight.FlightReservedEvent;
import com.ibm.beans.samples.media.ImageViewer;


/**
 * The Itinerary class displays the Travel planner UI.
 * <p>
 * The Itinerary contains:
 * <ol>
 * <li>Header GIF
 * <li>0 or more "legs of trip"
 * <li>0 a row of buttons
 * <li>Footer GIF
 * </ol>
 */
public class Itinerary extends Panel
	implements Serializable, FlightRequestedListener, FlightReservedListener
{
  /**
   * Constructs an Itinerary
   *
   */
  public Itinerary()
  {
    super();

    listeners = new Vector();
    frlisteners = new Vector();
    allItineraryFlightSubInfo = new Vector();

    //    --------------------------------
    //             Header GIF                 headerPanel
    //    --------------------------------
    //           SubInfo[0]   (0 or more)     infoPanel
    //           SubInfo[1]
    //           SubInfo[2]
    //             ...
    //    --------------------------------
    //    Add Flight                Ticket    buttonPanel
    //    --------------------------------
    //            Footer GIF                  footerPanel
    //    --------------------------------


    // HeaderPanel ===================
    headerPanel = new Panel();
    headerPanel.setBackground(Color.white);

    // infoPanel   ===================
    infoPanel = new Panel();
    infoPanel.setBackground(Color.white);
    GridLayout gridLayout = new GridLayout(0,1); // any # of rows, one column
    gridLayout.setVgap(2);                       // set vertical gap
    infoPanel.setLayout(gridLayout);

    // ButtonPanel ===================
    buttonPanel = new Panel();
    buttonPanel.setBackground(Color.white);

    // FooterPanel
    footerPanel = new Panel();
    footerPanel.setBackground(Color.white);

    GridBagLayout gridbag = new GridBagLayout();

    this.setLayout(gridbag);

    constrain(this, headerPanel,
              0, 0, 1, 1,
              GridBagConstraints.BOTH,  // Grow in both directions
              GridBagConstraints.NORTHWEST,
              0, 0,
              0, 0, 0, 0);    // above, left, bottom, right

    constrain(this, infoPanel,
              0, 1, 1, 1,
              GridBagConstraints.BOTH,
              GridBagConstraints.NORTHWEST,
              1.0, 1.0,
              5, 5, 5, 5);

    constrain(this, buttonPanel,
              0, 2, 1, 1,
              GridBagConstraints.HORIZONTAL,
              GridBagConstraints.NORTHWEST,
              0, 0,
              5, 5, 5, 5);

    constrain(this, footerPanel,
              0, 3, 1, 1,
              GridBagConstraints.BOTH,
              GridBagConstraints.NORTHWEST,
              0, 0,
              0, 0, 0, 0);

    // Add Buttons to the ButtonPanel
    addFlightButton    = new Button(getMsg("addFlight_button"));
    rentAutoButton     = new Button(getMsg("rentAuto_button"));
    reserveHotelButton = new Button(getMsg("reserveHotel_button"));
    ticketButton       = new Button(getMsg("ticket_button"));

    buttonPanel.add(addFlightButton);
    buttonPanel.add(rentAutoButton);
    buttonPanel.add(reserveHotelButton);
    buttonPanel.add(ticketButton);

    // Set up all the actions to flow to the listener
    itineraryListener = new ItineraryListener(this, infoPanel, buttonPanel);

    addFlightButton.addActionListener(itineraryListener);
    ticketButton.addActionListener(itineraryListener);

    // Set up default Header and Footer images
    setHeader(new ImageViewer("/com/ibm/beans/demos/travel/travlog1.gif"));
    setFooter(new ImageViewer("/com/ibm/beans/demos/travel/befooter.gif"));
  }

  /**
   * Returns the preferred size of the Itinerary UI.
   *
   * @return A Dimension object indicating the component's preferred size
   * is returned.
   */
  public Dimension getPreferredSize()
  {
    Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
    return new Dimension((7*screen.width)/8,5*(screen.height)/6);
    // 7/8 width, 5/6 height
  }

  /**
   * Sets the header panel of the Itinerary.
   *
   * @param c The component (typically a GIF image)
   * to be put in the header.
   */
  public void setHeader(Component c)
  {
    headerPanel.removeAll();
    headerPanel.add(c);
    headerPanel.setSize(c.getPreferredSize());
  }

  /**
   * Sets the footer panel of the Itinerary.
   *
   * @param c The component (typically a GIF image) to be put in the footer.
   */
  public void setFooter(Component c)
  {
    footerPanel.removeAll();
    footerPanel.add(c);
    footerPanel.setSize(c.getPreferredSize());
  }


  /**
   *  Initiates a TravelInfoChangedEvent to all
   *  registered listeners.  It is called by an ItineraryFlightSubInfo
   *  object when the user modifies its travel information.
   *
   *  @param ifsi The ItineraryFlightSubInfo panel
   *  whose travel information has been changed by the user.
   */
  public void fireTravelInfoChanged(ItineraryFlightSubInfo ifsi)
  {
    java.util.Date fd = ifsi.getFlightDate();
    String originCity = ifsi.getOriginCity();
    String destinationCity = ifsi.getDestinationCity();
    TravelInfoChangedEvent tice = new TravelInfoChangedEvent(this,
		originCity, destinationCity, fd);

    // Copy the listeners vector:
    Vector vector_copy;
    synchronized(this)
    {
      vector_copy = (Vector)listeners.clone();
    }

    // Notify all the listeners of the event:
    for (int i = 0; i < vector_copy.size(); i++)
    {
      ((TravelInfoChangedListener)vector_copy.elementAt(i)).TravelInfoChanged(tice);
    }
  }

  /**
   *  Initiates a FlightRequestedEvent to all
   *  registered listeners.  It is called by the Adapter when the
   *  user presses the "TICKET" button.
   *
   */
  public void fireFlightRequested()
  {
    Vector flightInfoVector = new Vector();
    // Collect the flight information from all the FlightSubInfo objects:
    for (int j = 0; j < allItineraryFlightSubInfo.size(); j++)
    {
      FlightInformation fi =
	  ((ItineraryFlightSubInfo)allItineraryFlightSubInfo.elementAt(j)).
		getFlightInformation();
      if (fi != null)
         flightInfoVector.addElement(fi);
    }
    FlightRequestedEvent fre = new FlightRequestedEvent(this, flightInfoVector);

    // Copy the frlisteners vector:
    Vector vector_copy;
    synchronized(this)
    {
      vector_copy = (Vector)frlisteners.clone();
    }

    // Notify all the frlisteners of the event:
    for (int i = 0; i < vector_copy.size(); i++)
    {
      ((FlightRequestedListener)vector_copy.elementAt(i)).FlightRequested(fre);
    }
  }

    /**
    * Responds to the FlightReserved event. It is Invoked by
    * the flight information data access part bean to signify
    * that a flight has been successfully reserved.
    *
    * @param fRE The FlightReservedEvent, which contains flight
    *            reservation information.
    */
    public void FlightReserved ( FlightReservedEvent fRE )
    {
       // Tell all the ItineraryFlightSubInfo objects to mark
       // themselves as "reserved".
       for (int j = 0; j < allItineraryFlightSubInfo.size(); j++)
       {
	 ((ItineraryFlightSubInfo)allItineraryFlightSubInfo.elementAt(j)).
		   markFlightReserved();
       }
       validate();
    }


  /**
   * Adds listeners for TravelInfoChangedEvents.
   *
   * @param l Indicates the
   * object implementing the TravelInfoChangedListener
   * interface that wants to listen for TravelInfoChangedEvents.
   */
  public synchronized void addTravelInfoChangedListener(TravelInfoChangedListener l)
  {
       listeners.addElement(l);
  }

  /**
   * Removes listeners for TravelInfoChangedEvents.
   *
   * @param l Indicates the
   * object implementing the TravelInfoChangedListener
   * interface that no longer wants to listen for TravelInfoChangedEvents.
   */
  public synchronized void removeTravelInfoChangedListener(TravelInfoChangedListener l)
  {
    listeners.removeElement(l);
  }

  /**
   * Adds listeners for FlightRequestedEvents.
   *
   * @param l Indicates the
   * object implementing the FlightRequestedListener
   * interface that wants to listen for FlightRequestedEvents.
   */
  public synchronized void addFlightRequestedListener(FlightRequestedListener l)
  {
       frlisteners.addElement(l);
  }

  /**
   * Removes listeners for FlightRequestedEvents.
   *
   * @param l Indicates the
   * object implementing the FlightRequestedListener
   * interface that no longer wants to listen for FlightRequestedEvents.
   */
  public synchronized void removeFlightRequestedListener(FlightRequestedListener l)
  {
    frlisteners.removeElement(l);
  }

  /**
   *  Responds to the Adapter (Listener) when
   *  the user has pressed the "Add Flight" button.
   *
   *  @param p Specifies a newly-created ItineraryFlightSubInfo
   *  object to be added.
   */
  void addSubInfoPanel(Panel p)
  {
    infoPanel.add(p);
    infoPanel.validate();    // Force a repaint!
    allItineraryFlightSubInfo.addElement(p);
  }

  /**
    * Invoked by the flight information
    * viewer bean when a flight has been selected.
    *
    * @param fre The FlightRequestedEvent, which contains flight
    *            reservation information.
    */
  public void FlightRequested(FlightRequestedEvent fre)
  {
    Vector vFlightInfo = fre.getFlightInfo();
    FlightInformation flightInformation =
			(FlightInformation)vFlightInfo.elementAt(0);
    currentItineraryFlightSubInfo.setFlightInformation(flightInformation);
    validate();                      // Force a repaint!
  }

  /**
   *  Sets the given ItineraryFlightSubInfo object to be
   *  the one currently "in focus".
   *
   *  @param ifsi The ItineraryFlightSubInfo object making the call.
   */
  public void setCurrentItineraryFlightSubInfo(ItineraryFlightSubInfo ifsi)
  {
    currentItineraryFlightSubInfo = ifsi;
  }

  /**
   *  Returns the  ItineraryFlightSubInfo object currently
   *  in focus (the last one to call setCurrentItineraryFlightSubInfo).
   *
   *  @return The ItineraryFlightSubInfo object currently in focus is
   *  returned.
   */
  public ItineraryFlightSubInfo getCurrentItineraryFlightSubInfo()
  {
    return currentItineraryFlightSubInfo;
  }

  void constrain(Container container, Component component,
                 int grid_x, int grid_y, int grid_width, int grid_height,
                 int fill, int anchor, double weight_x, double weight_y,
                 int top, int left, int bottom, int right)
  {
    GridBagConstraints c = new GridBagConstraints();
    c.gridx      = grid_x;
    c.gridy      = grid_y;
    c.gridwidth  = grid_width;
    c.gridheight = grid_height;
    c.fill       = fill;
    c.anchor     = anchor;
    c.weightx    = weight_x;
    c.weighty    = weight_y;

    if (top+bottom+left+right>0)
      c.insets = new Insets(top,left,bottom,right);

    ((GridBagLayout)container.getLayout()).setConstraints(component, c);
    container.add(component);
  }

  /**
   * This method is called whenever the Itinerary bean is serialized out
   * (such as a .ser file).  It is overridden so that transient
   * data (that is, data not serialized) can be removed (then it just calls
   * defaultWriteObject on the ObjectInputStream to let it
   * do the default processing.
   */
  private void writeObject(ObjectOutputStream out) throws IOException
  {
    headerPanel.removeAll();
    footerPanel.removeAll();
    // Don't save any SubInfo panels created during design phase:
    infoPanel.removeAll();
    allItineraryFlightSubInfo = null;
    out.defaultWriteObject();
  }

  /**
   * This method is called whenever the Itinerary bean is instantiated from
   * serialized data (such as a .ser file).  It is overridden so that transient
   * data (that is, data not serialized) can be initialized (then it just calls
   * defaultReadObject on the ObjectInputStream to let it take care of
   * initializing all of the non-transient data).
   */
  private void readObject(ObjectInputStream in)
				throws IOException, ClassNotFoundException
  {
    // Initialize transient data

    currentItineraryFlightSubInfo = null;
    msgs                          = null;
    listeners                     = new Vector();
    frlisteners                   = new Vector();
    allItineraryFlightSubInfo     = new Vector();

    in.defaultReadObject();

    itineraryListener = new ItineraryListener(this, infoPanel, buttonPanel);
    addFlightButton.addActionListener(itineraryListener);
    ticketButton.addActionListener(itineraryListener);

    // Set up default Header and Footer images
    setHeader(new ImageViewer("/com/ibm/beans/demos/travel/travlog1.gif"));
    setFooter(new ImageViewer("/com/ibm/beans/demos/travel/befooter.gif"));
  }

  String getMsg(String key)
  {
    if (msgs == null)
      msgs = (TravelResourceBundle)ResourceBundle.getBundle(
        "com.ibm.beans.demos.travel.TravelResourceBundle");

    return (String)msgs.handleGetObject(key);
  }

  private Button addFlightButton;
  private Button rentAutoButton;
  private Button reserveHotelButton;
  private Button ticketButton;
  private Panel  headerPanel;
  private Panel  infoPanel;
  private Panel  buttonPanel;
  private Panel  footerPanel;

  private transient ItineraryFlightSubInfo currentItineraryFlightSubInfo = null;
  private transient Vector                 listeners;
  private transient Vector                 frlisteners;
  private transient ItineraryListener      itineraryListener;
  private transient TravelResourceBundle   msgs = null;
  private transient Vector                 allItineraryFlightSubInfo;
  private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
  private static final long serialVersionUID = 2000L;  // serial version unique ID
}

class ItineraryListener implements ActionListener
{
  ItineraryListener(Itinerary itin, Panel infop,  Panel buttonp)
  {
    itinerary          = itin;
    infoPanel          = infop;
    buttonPanel        = buttonp;
    ticketButtonStr    = getMsg("ticket_button");
    addFlightButtonStr = getMsg("addFlight_button");
  }

  /**
   * This method handles action events on the Itinerary.
   * <p>
   * @param e This parameter indicates the action event that occurred.
   */
  public void actionPerformed(ActionEvent e)
  {
    String aCmd = e.getActionCommand();

    if(aCmd.equals(addFlightButtonStr))
    {
      itinerary.addSubInfoPanel(new ItineraryFlightSubInfo(itinerary));
    }
    else if(aCmd.equals(ticketButtonStr))
    {
      itinerary.fireFlightRequested();
    }
  }

  String getMsg(String key)
  {
    if (msgs == null)
      msgs = (TravelResourceBundle)ResourceBundle.getBundle(
        "com.ibm.beans.demos.travel.TravelResourceBundle");

    return (String)msgs.handleGetObject(key);
  }

  private Itinerary            itinerary;
  private Panel                infoPanel          = null;
  private Panel                buttonPanel        = null;
  private TravelResourceBundle msgs               = null;
  private String               addFlightButtonStr = null;
  private String               ticketButtonStr    = null;
  private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
  private static final long serialVersionUID = 2000L;  // serial version unique ID
}
