//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbTravelDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================

package com.ibm.beans.demos.travel.flight;

import com.ibm.beans.samples.daps.sql.Connection;
import com.ibm.beans.samples.daps.sql.DDL;
import com.ibm.beans.samples.daps.sql.InsertUpdateDelete;
import com.ibm.beans.samples.daps.sql.Select;
import com.ibm.beans.samples.daps.sql.Viewer;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Color;
import java.awt.Frame;

import java.util.ResourceBundle;

import java.beans.Beans;


/**
 * <p>
 * This class builds an example database that can be used in the remote travel demo.
 * The database is built with the help of beans from the "com.ibm.beans.samples.daps.sql"
 * package.  These beans utilize the JDBC Java capabilities. The database is built from
 * values defined in the "com.ibm.beans.demos.travel.flight.FlightResourceBundle".
 */
public class CreateDB extends Frame implements ActionListener {

   /**
   * Constructs an instance of the CreateDB class.
   */
   public CreateDB() {

     super();
     super.setTitle(CreateDBTitle);

     try {
       viewer     = (Viewer)Beans.instantiate(null,"com.ibm.beans.samples.daps.sql.Viewer");
       viewer2    = (Viewer)Beans.instantiate(null,"com.ibm.beans.samples.daps.sql.Viewer");
       connection = (Connection)Beans.instantiate(null,"com.ibm.beans.samples.daps.sql.Connection");
       insert     = (InsertUpdateDelete)Beans.instantiate(null,"com.ibm.beans.samples.daps.sql.InsertUpdateDelete");
       select     = (Select)Beans.instantiate(null,"com.ibm.beans.samples.daps.sql.Select");
       ddl        = (DDL)Beans.instantiate(null,"com.ibm.beans.samples.daps.sql.DDL");
     } catch (Exception e) {
       e.printStackTrace();
       System.exit(0);
     }

     setBackground(Color.lightGray);
     setLayout(new BorderLayout());
     add("Center",viewer);
     add("North",viewer2);
     addWindowListener(new ActionWindowListener()); // allows us to receive the windowClosing event (we ignore the others)
     OKbutton = new Button(OKButton);
     OKbutton.setEnabled(false);
     add("South", OKbutton);
     OKbutton.addActionListener(this);

     viewer2.setText(PleaseWaitMsg);

   }

   // Methods from ActionListener interface
   /**
   * Dismiss the frame when the OK button is pressed.
   * @param evt An action event.
   */
   public void actionPerformed(ActionEvent evt)
   {
      System.exit(0);
   }

   // Method from WindowListener interface
   /**
   * This class extends WindowAdapter and implements
   * the windowClosing method.
   */
   class ActionWindowListener extends WindowAdapter {

      /**
      * Dismiss the frame when the user selects close.
      * @param evt A window event.
      */
      public void windowClosing(WindowEvent evt)
      {
         System.exit(0);
      }
   }

   /**
   * Add the database records defined in the resource file.
   */
   private static void addDBRecords() {
     // Clear the viewer
     viewer.clear();

     // Set Connection properties
     connection.setJDBCURL(JDBCURL);

     // Perform the Database Connection
     connection.connect();

     // If successful connection
     if (connection.getSuccessful())
     {
        // Get the JDBCConnection
        java.sql.Connection connect = connection.getJDBCConnection();

        // Set DDL properties
        ddl.setJDBCConnection(connect);

        // delete the old table
        ddl.setSQLText("DROP TABLE " + dbTableName);
        ddl.execute();

        // create the new table
        String         mySQLString;
        mySQLString  = "CREATE TABLE ";
        mySQLString += dbTableName;
        mySQLString += " (ORIGIN CHAR(32), DESTINATION CHAR(32), AIRLINE CHAR(32), FLTNUM CHAR(32),";
        mySQLString += " STRTCTY CHAR(32), ACSCITY CHAR(32),ENDCITY CHAR(32), ACECITY CHAR(32), WHCLASS CHAR(32), DTIME CHAR(32),";
        mySQLString += "ATIME CHAR(32),PRICE CHAR(32), SEATNUM CHAR(32), BHASCON CHAR(1), AIRLINE2 CHAR(32), FLTNUM2 CHAR(32),";
        mySQLString += "ENDCITY2 CHAR(32), ACECITY2 CHAR(32), WHCLASS2 CHAR(32), DTIME2 CHAR(32), ATIME2 CHAR(32),";
        mySQLString += "PRICE2 CHAR(32), SEATNUM2 CHAR(32), FIRSTN CHAR(32), LASTN CHAR(32), STREET CHAR(32),CITY CHAR(32),";
        mySQLString += "STATE CHAR(32), ZIP CHAR(32), PHONE CHAR(32), FFNUM CHAR(32), IMAGELOC CHAR(64),IMAGECAP CHAR(64))";
        ddl.setSQLText(mySQLString);

        // Create the table
        ddl.execute();

        // If successful creation
        if (ddl.getSuccessful())
        {
            // Relinquish DDL resources - ignore errors
            ddl.close();

            // Set Insert properties.
            insert.setJDBCConnection(connect);

            // insert first record
            mySQLString  = "INSERT INTO ";
            mySQLString += dbTableName;
            mySQLString += " ";
            mySQLString += CreateDB_Rec1_1;
            mySQLString += CreateDB_Rec1_2;
            mySQLString += CreateDB_Rec1_3;
            mySQLString += CreateDB_Rec1_4;
            insert.setSQLText(mySQLString);
            insert.execute();

            // insert second record
            mySQLString  = "INSERT INTO ";
            mySQLString += dbTableName;
            mySQLString += " ";
            mySQLString += CreateDB_Rec2_1;
            mySQLString += CreateDB_Rec2_2;
            mySQLString += CreateDB_Rec2_3;
            mySQLString += CreateDB_Rec2_4;
            insert.setSQLText (mySQLString);
            insert.execute();


            // insert third record
            mySQLString  = "INSERT INTO ";
            mySQLString += dbTableName;
            mySQLString += " ";
            mySQLString += CreateDB_Rec3_1;
            mySQLString += CreateDB_Rec3_2;
            mySQLString += CreateDB_Rec3_3;
            mySQLString += CreateDB_Rec3_4;
            insert.setSQLText(mySQLString);
            insert.execute();

            // insert fourth record
            mySQLString  = "INSERT INTO ";
            mySQLString += dbTableName;
            mySQLString += " ";
            mySQLString += CreateDB_Rec4_1;
            mySQLString += CreateDB_Rec4_2;
            mySQLString += CreateDB_Rec4_3;
            mySQLString += CreateDB_Rec4_4;
            insert.setSQLText(mySQLString);
            insert.execute();

            // insert fifth record
            mySQLString  = "INSERT INTO ";
            mySQLString += dbTableName;
            mySQLString += " ";
            mySQLString += CreateDB_Rec5_1;
            mySQLString += CreateDB_Rec5_2;
            mySQLString += CreateDB_Rec5_3;
            mySQLString += CreateDB_Rec5_4;
            insert.setSQLText(mySQLString);
            insert.execute();

            // insert sixth record
            mySQLString  = "INSERT INTO ";
            mySQLString += dbTableName;
            mySQLString += " ";
            mySQLString += CreateDB_Rec6_1;
            mySQLString += CreateDB_Rec6_2;
            mySQLString += CreateDB_Rec6_3;
            mySQLString += CreateDB_Rec6_4;
            insert.setSQLText(mySQLString);
            insert.execute();

            // If successful population
            if (insert.getSuccessful())
            {
                // Relinquish Insert resources - ignore errors
                insert.close();

                // Set Select Properties
                select.setJDBCConnection(connect);

                select.setSQLText("SELECT * FROM " + dbTableName);

                // Execute the Select
                select.execute();

                // If successful
                if (select.getSuccessful())
                {
                    // Do forever
                    while (true)
                    {
                        // Get the next row
                        select.next();

                        // If successful and row found
                        if ((select.getSuccessful()) &&
                            (select.getRowFound()))
                        {
                            // Add the result row to the Viewer
                            viewer.setAdditionalRow(select.getResultRow());
                        }
                        else
                        {
                            // Break out of the loop
                            break;
                        }
                    }

                    // If the loop was terminated because no longer successful
                    if (!select.getSuccessful())
                    {
                        // Display exception text
                        viewer.setText(select.getException().toString());
                    }

                    // all records entered successfully
                    viewer2.setText(SuccessfulMsg);

                    OKbutton.setEnabled(true);

                    // Relinquish resources for the Select - ignore errors
                    select.close();
                }
                else
                {
                    // Display exception text
                    viewer.setText(select.getException().toString());
                    OKbutton.setEnabled(true);
                }
            }
            else
            {
                // Display exception text
                viewer.setText(insert.getException().toString());
                OKbutton.setEnabled(true);
            }
        }
        else
        {
            // Display exception text
            viewer.setText(ddl.getException().toString());
            OKbutton.setEnabled(true);
        }

        // Rollback changes - ignore errors
//        connection.rollback();

        connection.commit();  //commit changes

        // Disconnect - ignore errors
        connection.disconnect();
     }
     else
     {
        // Display exception text
        viewer.setText(connection.getException().toString());
        OKbutton.setEnabled(true);
     }
   }


   /**
   * Given a key, get a message from the FlightResourceBundle
   * @param key The key for the message
   * @return The message retrieved from the resource bundle
   */
   private static String getMsg(String key)
   {
        if (msgs == null)
          msgs = (FlightResourceBundle)ResourceBundle.getBundle(
            "com.ibm.beans.demos.travel.flight.FlightResourceBundle");

        return (String)msgs.handleGetObject(key);
   }


   /**
   * Retrieve the needed resource strings from the FlightResourceBundle.
   */
   private static void getResourceStrings() {

        // the strings for the first database record
        CreateDB_Rec1_1                                   = getMsg("CreateDB_Rec1_1");
        CreateDB_Rec1_2                                   = getMsg("CreateDB_Rec1_2");
        CreateDB_Rec1_3                                   = getMsg("CreateDB_Rec1_3");
        CreateDB_Rec1_4                                   = getMsg("CreateDB_Rec1_4");

        // the strings for the second database record
        CreateDB_Rec2_1                                   = getMsg("CreateDB_Rec2_1");
        CreateDB_Rec2_2                                   = getMsg("CreateDB_Rec2_2");
        CreateDB_Rec2_3                                   = getMsg("CreateDB_Rec2_3");
        CreateDB_Rec2_4                                   = getMsg("CreateDB_Rec2_4");

        // the strings for the third database record
        CreateDB_Rec3_1                                   = getMsg("CreateDB_Rec3_1");
        CreateDB_Rec3_2                                   = getMsg("CreateDB_Rec3_2");
        CreateDB_Rec3_3                                   = getMsg("CreateDB_Rec3_3");
        CreateDB_Rec3_4                                   = getMsg("CreateDB_Rec3_4");

        // the strings for the fourth database record
        CreateDB_Rec4_1                                   = getMsg("CreateDB_Rec4_1");
        CreateDB_Rec4_2                                   = getMsg("CreateDB_Rec4_2");
        CreateDB_Rec4_3                                   = getMsg("CreateDB_Rec4_3");
        CreateDB_Rec4_4                                   = getMsg("CreateDB_Rec4_4");

        // the strings for the fifth database record
        CreateDB_Rec5_1                                   = getMsg("CreateDB_Rec5_1");
        CreateDB_Rec5_2                                   = getMsg("CreateDB_Rec5_2");
        CreateDB_Rec5_3                                   = getMsg("CreateDB_Rec5_3");
        CreateDB_Rec5_4                                   = getMsg("CreateDB_Rec5_4");

        // the strings for the sixth database record
        CreateDB_Rec6_1                                   = getMsg("CreateDB_Rec6_1");
        CreateDB_Rec6_2                                   = getMsg("CreateDB_Rec6_2");
        CreateDB_Rec6_3                                   = getMsg("CreateDB_Rec6_3");
        CreateDB_Rec6_4                                   = getMsg("CreateDB_Rec6_4");

        dbTableName                                       = getMsg("dbTableName");
        JDBCURL                                           = getMsg("JDBCURL");

        CreateDBTitle                                     = getMsg("CreateDB_Title");
        OKButton                                          = getMsg("OKbutton");

        PleaseWaitMsg                                     = getMsg("CreateDB_msg1");
        SuccessfulMsg                                     = getMsg("CreateDB_msg2");
   }



   public static void main(String argv[]) {
     getResourceStrings();
     if (argv.length == 2) {
        JDBCURL     = argv[0];
        dbTableName = argv[1];
     }
     System.out.println("JDBCURL     = " + JDBCURL);
     System.out.println("dbTableName = " + dbTableName);
     CreateDB app = new CreateDB();
     app.setSize(400,1000);
     app.pack();
     app.show();
     addDBRecords();
   }



   private static Viewer viewer;
   private static Viewer viewer2;
   private static Connection connection;
   private static InsertUpdateDelete insert;
   private static Select select;
   private static DDL ddl;
   private static Button OKbutton;
   private static String PleaseWaitMsg;
   private static String SuccessfulMsg;
   private static transient FlightResourceBundle msgs = null;
   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";


   // strings from the FlightResourceBundle

   private static String CreateDB_Rec1_1;
   private static String CreateDB_Rec1_2;
   private static String CreateDB_Rec1_3;
   private static String CreateDB_Rec1_4;

   private static String CreateDB_Rec2_1;
   private static String CreateDB_Rec2_2;
   private static String CreateDB_Rec2_3;
   private static String CreateDB_Rec2_4;

   private static String CreateDB_Rec3_1;
   private static String CreateDB_Rec3_2;
   private static String CreateDB_Rec3_3;
   private static String CreateDB_Rec3_4;

   private static String CreateDB_Rec4_1;
   private static String CreateDB_Rec4_2;
   private static String CreateDB_Rec4_3;
   private static String CreateDB_Rec4_4;

   private static String CreateDB_Rec5_1;
   private static String CreateDB_Rec5_2;
   private static String CreateDB_Rec5_3;
   private static String CreateDB_Rec5_4;

   private static String CreateDB_Rec6_1;
   private static String CreateDB_Rec6_2;
   private static String CreateDB_Rec6_3;
   private static String CreateDB_Rec6_4;

   private static String dbTableName;
   private static String JDBCURL;

   private static String CreateDBTitle;
   private static String OKButton;
}
