//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbTravelDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.25 com/java/src/com/ibm/beans/demos/travel/flight/FlightInfoDap.java, bbTravelDemo, bb97c, bb97c9750e 11/17/97 15:46:48 [12/11/97 17:02:56]

package com.ibm.beans.demos.travel.flight;

import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;
import java.beans.VetoableChangeSupport;

import java.io.IOException;
import java.io.ObjectInputStream;

import java.util.ResourceBundle;
import java.util.Vector;
import java.util.StringTokenizer;

/**
 * <p>
 * This class is part of the travel demo and implements the flight
 * information data access part (DAP) bean.  The DAP bean serves as the
 * event sink for the TravelInfoChanged event and the FlightRequested
 * event.  The TravelInfoChanged event is sent by the itinerary
 * bean when the user selects an origin city, a destination city, and a departure date.
 * The FlightRequested event is sent by the itinerary bean when the user requests
 * confirmation for a selected flight itinerary.
 * <p>
 * This DAP bean serves as the event source for the FlightInformationChanged
 * and the FlightReserved events.  The FlightInformationChanged event is
 * sent to the flight selector bean when the DAP bean receives the
 * TravelInfoChanged event notification from the itinerary
 * bean.  The FlightReserved event is sent to the itinerary bean
 * when the DAP bean receives the FlightRequested event notification from
 * the itinerary bean.
 * <p>
 *
 * @author K. David Johnson (kdjohn@us.ibm.com)
 * @see com.ibm.beans.demos.travel.flight.FlightInfoEvent
 * @see com.ibm.beans.demos.travel.flight.FlightInfoListener
 * @see com.ibm.beans.demos.travel.flight.FlightReservedEvent
 * @see com.ibm.beans.demos.travel.flight.FlightReservedListener
 * @see com.ibm.beans.demos.travel.flight.FlightInfoDapBeanInfo
 * @see com.ibm.beans.demos.travel.flight.FlightInformation
 * @see com.ibm.beans.demos.travel.flight.FlightRequestedEvent
 * @see com.ibm.beans.demos.travel.flight.FlightRequestedListener
 */
public class FlightInfoDap implements java.io.Serializable,
                           com.ibm.beans.demos.travel.flight.FlightInfoDapProperties,
                           com.ibm.beans.demos.travel.TravelInfoChangedListener,
                           com.ibm.beans.demos.travel.flight.FlightRequestedListener
{


   /**
   * Constructs an instance of the FlightInfoDap bean.  The constructor initializes
   * instance data.
   */
   public FlightInfoDap() {
        FlightInformationChangedListeners = new Vector();
        FlightReservedListeners = new Vector();
        flightInfo = new Vector();
        vetos = new VetoableChangeSupport(this);
        changes = new PropertyChangeSupport(this);
        SQLString = "";
        dbTableName = "FlightInfoTbl";

        // get the strings from the resource file.
        getResourceStrings();
   }

   /**
    * Responds to calls on objects of this class when the objects are deserialized.
    * This method is called whenever the FlightInfoDap bean is instantiated from
    * serialized data (such as a .ser file).  It is overridden so that transient
    * data (that is, data not serialized) can be initialized (then it just calls
    * defaultReadObject on the ObjectInputStream to let it take care of initializing
    * all of the non-transient data).
    */
   private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
        // Initialize transient data
        FlightInformationChangedListeners = new Vector();
        FlightReservedListeners = new Vector();
        flightInfo = new Vector();
        vetos = new VetoableChangeSupport(this);
        changes = new PropertyChangeSupport(this);

        // get the strings from the resource file.
        dbTableName = "FlightInfoTbl";
        getResourceStrings();

        in.defaultReadObject();
   }


   //------------------------------------------------------------------
   // Event Sinks start
   //------------------------------------------------------------------

   /**
    * Responds to a TravelInfoChanged event. This method is invoked when
    * the Itinerary bean has been wired to this DAP bean and
    * the user submits preliminary information
    * needed to query the database for flights that meet the users preliminary
    * requirements.
    * @param  travelInfoChange The event containing the preliminary travel requirements
    */
   public void TravelInfoChanged(
        com.ibm.beans.demos.travel.TravelInfoChangedEvent travelInfoChange) {
        FlightInfoEvent fIE = queryDataBase(travelInfoChange);
        fireFlightInfo(fIE); // jwa 7/24: Renamed to avoid warnings when morphing
   }


   /**
    * Responds to a FlightRequested event. This method is invoked when the
    * itinerary bean has been wired to this DAP bean and the user has selected
    * a preferred itinerary.  If the itinerary is approved, notification is sent
    * back to the itinerary bean via a FlightReserveredEvent.
    * @param  flightRequestedEvent The FlightRequestedEvent containing the preferred flight itinerary.
    */
   public void FlightRequested(com.ibm.beans.demos.travel.flight.FlightRequestedEvent
                               flightRequestedEvent) {

        // for now we will just accept the itinerary and send a FlightReservedEvent to
        // the itinerary bean.

        FlightReservedEvent flightReservedEvent = reserveFlight(flightRequestedEvent);
        fireFlightReserved(flightReservedEvent);
   }

   //------------------------------------------------------------------
   // Event Syncs end
   //------------------------------------------------------------------


   //------------------------------------------------------------------
   // Event Sources start
   //------------------------------------------------------------------

   /**
    * Allows other beans to listen to the FlightInfo events this bean generates.
    * This is the addListener method for all listeners that want to be notified
    * when a FlightInformationChanged event occurs.  In the demo, the flight
    * selector bean is wired to this DAP bean and is notified when the
    * FlightInformationChanged event happens.
    * @param  fIL The registering listener.
    * @see #removeFlightInfoListener
    */
   public synchronized void addFlightInfoListener(FlightInfoListener fIL) {
        FlightInformationChangedListeners.addElement(fIL);
   }

   /**
    * Allows beans to remove themselves from the listener list for this bean's
    * FlightInfo event.
    * This is the removeListener method for all listeners that want to be notified
    * when a FlightInformationChanged event occurs.  In the demo, the flight
    * selector bean is wired to this DAP bean and is notified when the
    * FlightInformationChanged event happens.
    * @param  fIL The unregistering listener.
    * @see #addFlightInfoListener
    */
   public synchronized void removeFlightInfoListener(FlightInfoListener fIL) {
        FlightInformationChangedListeners.removeElement(fIL);
   }

   /**
   * Allows other beans to listen to the FlightReserved events this bean generates.
   * This is the addListener method for all listeners that want to be notified
   * when a FlightReserved event occurs.  In the demo, this DAP bean
   * is wired to the itinerary viewer bean and is notified by this DAP bean
   * when the FlightReserved event happens.
   * @param  fRL The registering listener.
   * @see #removeFlightReservedListener
   */
   public synchronized void addFlightReservedListener(FlightReservedListener fRL) {
        FlightReservedListeners.addElement(fRL);
   }

   /**
    * Allows beans to remove themselves from the listener list for this bean's
    * FlightReserved event.
    * This is the removeListener method for all listeners that want to be notified
    * when a FlightReserved event occurs.  In the demo, this DAP bean
    * is wired to the itinerary viewer bean and is notified by this DAP bean
    * when the FlightReserved event happens.
    * @param  fRL The registering FlightReservedListener object.
    * @see #addFlightReservedListener
    */
   public synchronized void removeFlightReservedListener(FlightReservedListener fRL) {
        FlightReservedListeners.removeElement(fRL);
   }

   /**
   * Adds a listener to be notified when the <i>SQLString</i> property or the
   * <i>flightInfo</i> property is about to be changed.
   *
   * @param listener   The object to be notified.
   */
   public void addVetoableChangeListener(VetoableChangeListener listener) {
        vetos.addVetoableChangeListener(listener);
   }

   /**
   * Removes a listener from the notification list.
   *
   * @param listener   The object to be removed from the notification list.
   */
   public void removeVetoableChangeListener(VetoableChangeListener listener) {
        vetos.removeVetoableChangeListener(listener);
   }

   /**
   * Adds a listener to be notified when the <i>SQLString</i> property or the <i>flightInfo</i> property has been changed.
   *
   * @param listener   The object to be notified.
   */
   public void addPropertyChangeListener(PropertyChangeListener listener) {
        changes.addPropertyChangeListener(listener);
   }

   /**
   * Removes a listener from the notification list.
   *
   * @param listener   The object to be removed from the notification list.
   */
   public void removePropertyChangeListener(PropertyChangeListener listener) {
        changes.removePropertyChangeListener(listener);
   }

   //------------------------------------------------------------------
   // Event Sources end
   //------------------------------------------------------------------


   /**
   * Queries the database to find all of the possible flight
   * choices that match the user request of origin city, destination
   * city, departure date, and return date.
   * @param travelInfoChange The TravelInfoChanged event containing the travel info
   * @return The FlightInfoEvent containing the flight info
   */
   public com.ibm.beans.demos.travel.flight.FlightInfoEvent queryDataBase(
   com.ibm.beans.demos.travel.TravelInfoChangedEvent travelInfoChange) {

      String originCity         = travelInfoChange.getOriginCity();
      String destinationCity    = travelInfoChange.getDestinationCity();
      java.util.Date departDate = travelInfoChange.getDepartDate();
      // String returnDate   = travelInfoChange.getReturnDate();

      Vector l;
      FlightInfoEvent flightInfoEvent;

      synchronized (this) {                     // keep from having problems with multithreaded access

        if ( !(flightInfo.isEmpty()) ) {        // clear out the vector
            flightInfo.removeAllElements();
        }

        Vector flightInfoTemp = new Vector();

        // build the SQL statement that can be modified by a business rules dip if desired. We will only use the originCity
        // and destinationCity and ignore the dates for now.
        String mySQLString;
        mySQLString =  "SELECT * FROM " ;
        mySQLString += dbTableName;
        mySQLString += " WHERE ORIGIN = " + originCity;
        mySQLString += " AND DESTINATION = " + destinationCity;

        // set the SQL string as a property.  This will allow the business rules DIP to
        // hook the property and modify it if desired - i.e. to restrict access to a certain
        // airline - American, Delta, or TWA.  The DIP will add the following to the end of the
        // SQL string:  "AND AIRLINE <> AMERICAN", and/or, "AND AIRLINE <> DELTA",
        // and/or, "AND AIRLINE <> TWA".
        try {
          setSQLString(mySQLString);
        } catch(PropertyVetoException e) {
        }

        // retrieve the SQL string which may have been altered by the DIP
        String myFinalSQLString = getSQLString();

        //parse the string to see if it contains any substrings of the type - "AND AIRLINE <> AMERICAN", etc.
        //for the demo we will use the StringTokenizer with delemiter of "<>" and assume
        //that the "AND AIRLINE" was the prefix.

        boolean bDontIncludeAirline1      = false;                     // should we include American flights?
        boolean bDontIncludeAirline2      = false;                     // should we include Delta flights?
        boolean bDontIncludeAirline3      = false;                     // should we include TWA flights?

        String airline1 = " ";
        airline1       += FIDS_Airline1;
        String airline2 = " ";
        airline2       += FIDS_Airline2;
        String airline3 = " ";
        airline3       += FIDS_Airline3;

        StringTokenizer t = new StringTokenizer(myFinalSQLString, "<>");
        if (t.countTokens() > 0) {
           String myToken;
           while (t.hasMoreTokens()) {
             myToken = t.nextToken().toUpperCase();
             if (myToken.startsWith(airline1.toUpperCase())) {
                bDontIncludeAirline1 = true;
             }
             if (myToken.startsWith(airline2.toUpperCase())) {
                bDontIncludeAirline2 = true;
             }
             if (myToken.startsWith(airline3.toUpperCase())) {
                bDontIncludeAirline3 = true;
             }
           }
        }

        // variables used to convert the resource file price "String" values to double values
        Double tempDouble;
        double price1;
        double price2;

        if ( (originCity.equals(FIDS_FirstOriginCity)) && (destinationCity.equals(FIDS_FirstDestinationCity)) ) {


           // eliminate flights from the list if the business rules DIP has changed the SQL statement

           if (!bDontIncludeAirline1) {
              tempDouble = new Double(FIDS_FirstRec_Price);
              price1     = tempDouble.doubleValue();
              flightInfoTemp.addElement(new FlightInformation(FIDS_Airline1,  FIDS_FirstRec_FlightNum1,
                                        FIDS_FirstOriginCity,FIDS_FirstOriginCity_AirportCode,FIDS_FirstDestinationCity,
                                        FIDS_FirstDestinationCity_AirportCode,
                                        FIDS_FirstRec_WhichClass,FIDS_FirstRec_DepartTime,FIDS_FirstRec_ArriveTime,
                                        price1,FIDS_FirstRec_SeatNum,
                                        false,"","","","","","","",
                                        0,"","","","","","","","","",FIDS_FirstRec_ImageLocation,
                                        FIDS_FirstRec_ImageCaption)  );
           }

           if (!bDontIncludeAirline2) {
              tempDouble = new Double(FIDS_SecondRec_Price);
              price1     = tempDouble.doubleValue();
              flightInfoTemp.addElement(new FlightInformation(FIDS_Airline2, FIDS_SecondRec_FlightNum1,
                                        FIDS_FirstOriginCity,FIDS_FirstOriginCity_AirportCode,FIDS_FirstDestinationCity,
                                        FIDS_FirstDestinationCity_AirportCode,
                                        FIDS_SecondRec_WhichClass,FIDS_SecondRec_DepartTime,FIDS_SecondRec_ArriveTime,
                                        price1,FIDS_SecondRec_SeatNum,
                                        false,"","","","","","","",
                                        0,"","","","","","","","","",FIDS_SecondRec_ImageLocation,
                                        FIDS_SecondRec_ImageCaption)  );
           }

           if (!bDontIncludeAirline3) {
              tempDouble = new Double(FIDS_ThirdRec_Price);
              price1     = tempDouble.doubleValue();
              flightInfoTemp.addElement(new FlightInformation(FIDS_Airline3, FIDS_ThirdRec_FlightNum1,
                                        FIDS_FirstOriginCity,FIDS_FirstOriginCity_AirportCode,FIDS_FirstDestinationCity,
                                        FIDS_FirstDestinationCity_AirportCode,
                                        FIDS_ThirdRec_WhichClass,FIDS_ThirdRec_DepartTime,FIDS_ThirdRec_ArriveTime,
                                        price1,FIDS_ThirdRec_SeatNum,
                                        false,"","","","","","","",
                                        0,"","","","","","","","","",FIDS_ThirdRec_ImageLocation,
                                        FIDS_ThirdRec_ImageCaption)  );
           }
        }

        if ( (originCity.equals(FIDS_SecondOriginCity)) && (destinationCity.equals(FIDS_SecondDestinationCity)) ) {

           if (!bDontIncludeAirline1) {
               // a connecting flight
               tempDouble = new Double(FIDS_FourthRec_Price1);
               price1     = tempDouble.doubleValue();
               tempDouble = new Double(FIDS_FourthRec_Price2);
               price2     = tempDouble.doubleValue();
               flightInfoTemp.addElement(new FlightInformation(FIDS_Airline1, FIDS_FourthRec_FlightNum1,
                                         FIDS_SecondOriginCity,FIDS_SecondOriginCity_AirportCode,
                                         FIDS_FourthRec_IntermediateDestination,
                                         FIDS_FourthRec_IntermediateDestinationAirportCode,FIDS_FourthRec_WhichClass,
                                         FIDS_FourthRec_DepartTime1,FIDS_FourthRec_ArriveTime1,
                                         price1,FIDS_FourthRec_SeatNum1,
                                         true,FIDS_Airline1, FIDS_FourthRec_FlightNum2 ,FIDS_SecondDestinationCity,
                                         FIDS_SecondDestinationCity_AirportCode,
                                         FIDS_FourthRec_WhichClass,FIDS_FourthRec_DepartTime2 ,FIDS_FourthRec_ArriveTime2 ,
                                         price2,FIDS_FourthRec_SeatNum2 ,"","","","","","","","",
                                         FIDS_FourthRec_ImageLocation,FIDS_FourthRec_ImageCaption)  );
           }
           if (!bDontIncludeAirline2) {
               // a connecting flight
               tempDouble = new Double(FIDS_FifthRec_Price1);
               price1     = tempDouble.doubleValue();
               tempDouble = new Double(FIDS_FifthRec_Price2);
               price2     = tempDouble.doubleValue();
               flightInfoTemp.addElement(new FlightInformation(FIDS_Airline2, FIDS_FifthRec_FlightNum1,
                                         FIDS_SecondOriginCity,FIDS_SecondOriginCity_AirportCode,
                                         FIDS_FifthRec_IntermediateDestination,
                                         FIDS_FifthRec_IntermediateDestinationAirportCode,FIDS_FifthRec_WhichClass,
                                         FIDS_FifthRec_DepartTime1,FIDS_FifthRec_ArriveTime1,
                                         price1,FIDS_FifthRec_SeatNum1,
                                         true,FIDS_Airline2,FIDS_FifthRec_FlightNum2,FIDS_SecondDestinationCity,
                                         FIDS_SecondDestinationCity_AirportCode,
                                         FIDS_FifthRec_WhichClass,FIDS_FifthRec_DepartTime2,FIDS_FifthRec_ArriveTime2,
                                         price2,FIDS_FifthRec_SeatNum2,"","","","","","","","",
                                         FIDS_FifthRec_ImageLocation, FIDS_FifthRec_ImageCaption)  );
           }

           // note that this is a direct flight but costs $110.00 more that the alternative connecting
           // flights.  Therefore this is a candidate for the business rules dip to eliminate this
           // flight from the list.
           if (!bDontIncludeAirline3) {
               // a direct flight
               tempDouble = new Double(FIDS_SixthRec_Price);
               price1     = tempDouble.doubleValue();
               flightInfoTemp.addElement(new FlightInformation(FIDS_Airline3,  FIDS_SixthRec_FlightNum1,
                                         FIDS_SecondOriginCity,FIDS_SecondOriginCity_AirportCode,FIDS_SecondDestinationCity,
                                         FIDS_SecondDestinationCity_AirportCode,FIDS_SixthRec_WhichClass,
                                         FIDS_SixthRec_DepartTime,FIDS_SixthRec_ArriveTime,
                                         price1,FIDS_SixthRec_SeatNum,
                                         false,"","","","","","","",
                                         0,"None","","","","","","","","", FIDS_SixthRec_ImageLocation,
                                         FIDS_SixthRec_ImageCaption)  );
           }
        }

        // setting this property gives the business rules DIP an opportunity to hook all flights and
        // modify the list if necessary - i.e. see the above scenario where the direct Austin to
        // Chicago flight is $110.00 more than the alternate connecting flights and could be
        // removed from the list by the DIP.
        try {
          setFlightInfo(flightInfoTemp);
        } catch(PropertyVetoException e) {
        }

        flightInfoEvent = new FlightInfoEvent(this, flightInfo);
      } /* end synchronized */


      return(flightInfoEvent);
   }

   /**
   * Attempts to reserve the specified flight(s).
   * @param  flightRequestedEvent The FlightRequestedEvent containing the preferred flight itinerary.
   * @return The FlightReservedEvent containing the actually reserved flight itinerary.
   */
   public com.ibm.beans.demos.travel.flight.FlightReservedEvent reserveFlight(
   com.ibm.beans.demos.travel.flight.FlightRequestedEvent flightRequestedEvent) {

      FlightReservedEvent flightReservedEvent;

      synchronized (this) { //keep from having problems with multithreaded access
        // for now we will just accept the itinerary and prepare a FlightReservedEvent
        // to send to the itinerary bean.

        flightReservedEvent = new FlightReservedEvent(this, flightRequestedEvent.flightInfo);
      } /* end synchronized */

      return(flightReservedEvent);
   }


   /**
   * Notifies all registered listeners that a flight has been reserved.
   * In the demo, this event fires in response to a
   * FlightRequested event notification from the itinerary
   * bean when the requested flight itinerary is confirmed.
   * @param  fRE The FlightReservedEvent containing the preferred flight itinerary.
   */
   public void fireFlightReserved(com.ibm.beans.demos.travel.flight.FlightReservedEvent fRE) {
        Vector l;

        // notify registered listeners

        // must copy the Vector here in order to freeze the state of the
        // set of EventListeners the event should be delivered to prior
        // to delivery.  This ensures that any changes made to the Vector
        // from a target listener's method, during the delivery of this
        // event will not take effect until after the event is delivered.
        synchronized(this) { l = (Vector)FlightReservedListeners.clone(); }

        for (int i = 0; i < l.size(); i++) {
           ((FlightReservedListener)l.elementAt(i)).FlightReserved(fRE);
        }
   }



   /**
   * Notifies all registered listeners. A list of
   * flights are sent as a Vector.  In the demo,
   * this event fires in response to a TavelInfoChangedEvent
   * which is fired by the itinerary bean after
   * a user has entered the origin and destination cities
   * and dates and has pressed the submit button.  This DAP
   * then queries the database for matching flights, builds
   * this information into a vector,  and fires the
   * FlightInformationChanged notification to the flight
   * selector bean.
   * @param  fIE The FlightInfoEvent containing the possible flight selections.
   */
   public void fireFlightInfo(FlightInfoEvent fIE) {
        Vector l;

        // notify registered listeners

        // must copy the Vector here in order to freeze the state of the
        // set of EventListeners the event should be delivered to prior
        // to delivery.  This ensures that any changes made to the Vector
        // from a target listener's method, during the delivery of this
        // event will not take effect until after the event is delivered.

        synchronized(this) { l = (Vector)FlightInformationChangedListeners.clone(); }

        for (int i = 0; i < l.size(); i++) {
           ((FlightInfoListener)l.elementAt(i)).FlightInformationChanged(fIE);
        }
   }


   /**
   * Accessor (set) method for the <i>FlightInfo</i> property.
   * @param newValue A Vector representing the  FlightInformation.
   * @exception PropertyVetoException Indicates that a constrained property
   * listener has vetoed the change to the flightInfo property.
   */
   public void setFlightInfo(Vector newValue) throws PropertyVetoException{
        Vector oldFlightInfo;

        synchronized(this) { oldFlightInfo = (Vector)flightInfo.clone(); }

        // first tell the vetoers about the change.  If anyone objects, we
        // let the PropertyVetoException propagate back to our caller.
        vetos.fireVetoableChange("flightInfo",oldFlightInfo,newValue);

        // No one vetoed, so go ahead and make the change
        synchronized(this) { flightInfo    = (Vector)newValue.clone(); }
        changes.firePropertyChange("flightInfo",oldFlightInfo,flightInfo);
   }

   /**
   * Accessor (get) method for the <i>FlightInfo</i> property.
   *
   * @return The Vector containing FlightInformation objects.
   */
   public Vector getFlightInfo() {
        return flightInfo;
   }

   /**
   * Accessor (set) method for the <i>SQLString</i> property.
   * @param SQLString The String representing the SQL property of the bean.
   * @exception PropertyVetoException Indicates that a constrained property
   * listener has vetoed the change to the SQLString property.
   */
   public void setSQLString(String SQLString) throws PropertyVetoException{
        String oldString = this.SQLString;

        // first tell the vetoers about the change.  If anyone objects, we
        // let the PropertyVetoException propagate back to our caller.
        vetos.fireVetoableChange("SQLString",oldString,SQLString);

        // No one vetoed, so go ahead and make the change
        this.SQLString   = SQLString;
        changes.firePropertyChange("SQLString",oldString,SQLString);

   }

   /**
   * Accessor (get) method for the SQLString property.
   * @return The String containing the SQL database query
   */
   public String getSQLString() {
        return SQLString;
   }


   /**
   * Retrieve the needed resource strings from the FlightResourceBundle.
   */
   private void getResourceStrings() {
        FIDS_FirstOriginCity                              = getMsg("FIDS_FirstOriginCity");
        FIDS_FirstOriginCity_AirportCode                  = getMsg("FIDS_FirstOriginCity_AirportCode");

        FIDS_FirstDestinationCity                         = getMsg("FIDS_FirstDestinationCity");
        FIDS_FirstDestinationCity_AirportCode             = getMsg("FIDS_FirstDestinationCity_AirportCode");

        FIDS_SecondOriginCity                             = getMsg("FIDS_SecondOriginCity");
        FIDS_SecondOriginCity_AirportCode                 = getMsg("FIDS_SecondOriginCity_AirportCode");

        FIDS_SecondDestinationCity                        = getMsg("FIDS_SecondDestinationCity");
        FIDS_SecondDestinationCity_AirportCode            = getMsg("FIDS_SecondDestinationCity_AirportCode");

        FIDS_Airline1                                     = getMsg("FIDS_Airline1");
        FIDS_Airline2                                     = getMsg("FIDS_Airline2");
        FIDS_Airline3                                     = getMsg("FIDS_Airline3");

        FIDS_FirstRec_FlightNum1                          = getMsg("FIDS_FirstRec_FlightNum1");
        FIDS_FirstRec_WhichClass                          = getMsg("FIDS_FirstRec_WhichClass");
        FIDS_FirstRec_DepartTime                          = getMsg("FIDS_FirstRec_DepartTime");
        FIDS_FirstRec_ArriveTime                          = getMsg("FIDS_FirstRec_ArriveTime");
        FIDS_FirstRec_Price                               = getMsg("FIDS_FirstRec_Price");
        FIDS_FirstRec_SeatNum                             = getMsg("FIDS_FirstRec_SeatNum");
        FIDS_FirstRec_ImageLocation                       = getMsg("FIDS_FirstRec_ImageLocation");
        FIDS_FirstRec_ImageCaption                        = getMsg("FIDS_FirstRec_ImageCaption");

        FIDS_SecondRec_FlightNum1                         = getMsg("FIDS_SecondRec_FlightNum1");
        FIDS_SecondRec_WhichClass                         = getMsg("FIDS_SecondRec_WhichClass");
        FIDS_SecondRec_DepartTime                         = getMsg("FIDS_SecondRec_DepartTime");
        FIDS_SecondRec_ArriveTime                         = getMsg("FIDS_SecondRec_ArriveTime");
        FIDS_SecondRec_Price                              = getMsg("FIDS_SecondRec_Price");
        FIDS_SecondRec_SeatNum                            = getMsg("FIDS_SecondRec_SeatNum");
        FIDS_SecondRec_ImageLocation                      = getMsg("FIDS_SecondRec_ImageLocation");
        FIDS_SecondRec_ImageCaption                       = getMsg("FIDS_SecondRec_ImageCaption");

        FIDS_ThirdRec_FlightNum1                          = getMsg("FIDS_ThirdRec_FlightNum1");
        FIDS_ThirdRec_WhichClass                          = getMsg("FIDS_ThirdRec_WhichClass");
        FIDS_ThirdRec_DepartTime                          = getMsg("FIDS_ThirdRec_DepartTime");
        FIDS_ThirdRec_ArriveTime                          = getMsg("FIDS_ThirdRec_ArriveTime");
        FIDS_ThirdRec_Price                               = getMsg("FIDS_ThirdRec_Price");
        FIDS_ThirdRec_SeatNum                             = getMsg("FIDS_ThirdRec_SeatNum");
        FIDS_ThirdRec_ImageLocation                       = getMsg("FIDS_ThirdRec_ImageLocation");
        FIDS_ThirdRec_ImageCaption                        = getMsg("FIDS_ThirdRec_ImageCaption");

        FIDS_FourthRec_FlightNum1                         = getMsg("FIDS_FourthRec_FlightNum1");
        FIDS_FourthRec_IntermediateDestination            = getMsg("FIDS_FourthRec_IntermediateDestination");
        FIDS_FourthRec_IntermediateDestinationAirportCode = getMsg("FIDS_FourthRec_IntermediateDestinationAirportCode");
        FIDS_FourthRec_WhichClass                         = getMsg("FIDS_FourthRec_WhichClass");
        FIDS_FourthRec_DepartTime1                        = getMsg("FIDS_FourthRec_DepartTime1");
        FIDS_FourthRec_ArriveTime1                        = getMsg("FIDS_FourthRec_ArriveTime1");
        FIDS_FourthRec_Price1                             = getMsg("FIDS_FourthRec_Price1");
        FIDS_FourthRec_SeatNum1                           = getMsg("FIDS_FourthRec_SeatNum1");
        FIDS_FourthRec_FlightNum2                         = getMsg("FIDS_FourthRec_FlightNum2");
        FIDS_FourthRec_DepartTime2                        = getMsg("FIDS_FourthRec_DepartTime2");
        FIDS_FourthRec_ArriveTime2                        = getMsg("FIDS_FourthRec_ArriveTime2");
        FIDS_FourthRec_Price2                             = getMsg("FIDS_FourthRec_Price2");
        FIDS_FourthRec_SeatNum2                           = getMsg("FIDS_FourthRec_SeatNum2");
        FIDS_FourthRec_ImageLocation                      = getMsg("FIDS_FourthRec_ImageLocation");
        FIDS_FourthRec_ImageCaption                       = getMsg("FIDS_FourthRec_ImageCaption");

        FIDS_FifthRec_FlightNum1                          = getMsg("FIDS_FifthRec_FlightNum1");
        FIDS_FifthRec_IntermediateDestination             = getMsg("FIDS_FifthRec_IntermediateDestination");
        FIDS_FifthRec_IntermediateDestinationAirportCode  = getMsg("FIDS_FifthRec_IntermediateDestinationAirportCode");
        FIDS_FifthRec_WhichClass                          = getMsg("FIDS_FifthRec_WhichClass");
        FIDS_FifthRec_DepartTime1                         = getMsg("FIDS_FifthRec_DepartTime1");
        FIDS_FifthRec_ArriveTime1                         = getMsg("FIDS_FifthRec_ArriveTime1");
        FIDS_FifthRec_Price1                              = getMsg("FIDS_FifthRec_Price1");
        FIDS_FifthRec_SeatNum1                            = getMsg("FIDS_FifthRec_SeatNum1");
        FIDS_FifthRec_FlightNum2                          = getMsg("FIDS_FifthRec_FlightNum2");
        FIDS_FifthRec_DepartTime2                         = getMsg("FIDS_FifthRec_DepartTime2");
        FIDS_FifthRec_ArriveTime2                         = getMsg("FIDS_FifthRec_ArriveTime2");
        FIDS_FifthRec_Price2                              = getMsg("FIDS_FifthRec_Price2");
        FIDS_FifthRec_SeatNum2                            = getMsg("FIDS_FifthRec_SeatNum2");
        FIDS_FifthRec_ImageLocation                       = getMsg("FIDS_FifthRec_ImageLocation");
        FIDS_FifthRec_ImageCaption                        = getMsg("FIDS_FifthRec_ImageCaption");

        FIDS_SixthRec_FlightNum1                          = getMsg("FIDS_SixthRec_FlightNum1");
        FIDS_SixthRec_WhichClass                          = getMsg("FIDS_SixthRec_WhichClass");
        FIDS_SixthRec_DepartTime                          = getMsg("FIDS_SixthRec_DepartTime");
        FIDS_SixthRec_ArriveTime                          = getMsg("FIDS_SixthRec_ArriveTime");
        FIDS_SixthRec_Price                               = getMsg("FIDS_SixthRec_Price");
        FIDS_SixthRec_SeatNum                             = getMsg("FIDS_SixthRec_SeatNum");
        FIDS_SixthRec_ImageLocation                       = getMsg("FIDS_SixthRec_ImageLocation");
        FIDS_SixthRec_ImageCaption                        = getMsg("FIDS_SixthRec_ImageCaption");
   }

   /**
   * Given a key, get a message from the FlightResourceBundle
   * @param key The key for the message
   * @return The message retrieved from the resource bundle
   */
   private String getMsg(String key)
   {
        if (msgs == null)
          msgs = (FlightResourceBundle)ResourceBundle.getBundle(
            "com.ibm.beans.demos.travel.flight.FlightResourceBundle");

        return (String)msgs.handleGetObject(key);
   }


   // strings from the FlightResourceBundle
   private String FIDS_FirstOriginCity;
   private String FIDS_FirstOriginCity_AirportCode;
   private String FIDS_FirstDestinationCity;
   private String FIDS_FirstDestinationCity_AirportCode;
   private String FIDS_SecondOriginCity;
   private String FIDS_SecondOriginCity_AirportCode;
   private String FIDS_SecondDestinationCity;
   private String FIDS_SecondDestinationCity_AirportCode;
   private String FIDS_Airline1;
   private String FIDS_Airline2;
   private String FIDS_Airline3;
   private String FIDS_FirstRec_FlightNum1;
   private String FIDS_FirstRec_WhichClass;
   private String FIDS_FirstRec_DepartTime;
   private String FIDS_FirstRec_ArriveTime;
   private String FIDS_FirstRec_Price;
   private String FIDS_FirstRec_SeatNum;
   private String FIDS_FirstRec_ImageLocation;
   private String FIDS_FirstRec_ImageCaption;
   private String FIDS_SecondRec_FlightNum1;
   private String FIDS_SecondRec_WhichClass;
   private String FIDS_SecondRec_DepartTime;
   private String FIDS_SecondRec_ArriveTime;
   private String FIDS_SecondRec_Price;
   private String FIDS_SecondRec_SeatNum;
   private String FIDS_SecondRec_ImageLocation;
   private String FIDS_SecondRec_ImageCaption;
   private String FIDS_ThirdRec_FlightNum1;
   private String FIDS_ThirdRec_WhichClass;
   private String FIDS_ThirdRec_DepartTime;
   private String FIDS_ThirdRec_ArriveTime;
   private String FIDS_ThirdRec_Price;
   private String FIDS_ThirdRec_SeatNum;
   private String FIDS_ThirdRec_ImageLocation;
   private String FIDS_ThirdRec_ImageCaption;
   private String FIDS_FourthRec_FlightNum1;
   private String FIDS_FourthRec_IntermediateDestination;
   private String FIDS_FourthRec_IntermediateDestinationAirportCode;
   private String FIDS_FourthRec_WhichClass;
   private String FIDS_FourthRec_DepartTime1;
   private String FIDS_FourthRec_ArriveTime1;
   private String FIDS_FourthRec_Price1;
   private String FIDS_FourthRec_SeatNum1;
   private String FIDS_FourthRec_FlightNum2;
   private String FIDS_FourthRec_DepartTime2;
   private String FIDS_FourthRec_ArriveTime2;
   private String FIDS_FourthRec_Price2;
   private String FIDS_FourthRec_SeatNum2;
   private String FIDS_FourthRec_ImageLocation;
   private String FIDS_FourthRec_ImageCaption;
   private String FIDS_FifthRec_FlightNum1;
   private String FIDS_FifthRec_IntermediateDestination;
   private String FIDS_FifthRec_IntermediateDestinationAirportCode;
   private String FIDS_FifthRec_WhichClass;
   private String FIDS_FifthRec_DepartTime1;
   private String FIDS_FifthRec_ArriveTime1;
   private String FIDS_FifthRec_Price1;
   private String FIDS_FifthRec_SeatNum1;
   private String FIDS_FifthRec_FlightNum2;
   private String FIDS_FifthRec_DepartTime2;
   private String FIDS_FifthRec_ArriveTime2;
   private String FIDS_FifthRec_Price2;
   private String FIDS_FifthRec_SeatNum2;
   private String FIDS_FifthRec_ImageLocation;
   private String FIDS_FifthRec_ImageCaption;
   private String FIDS_SixthRec_FlightNum1;
   private String FIDS_SixthRec_WhichClass;
   private String FIDS_SixthRec_DepartTime;
   private String FIDS_SixthRec_ArriveTime;
   private String FIDS_SixthRec_Price;
   private String FIDS_SixthRec_SeatNum;
   private String FIDS_SixthRec_ImageLocation;
   private String FIDS_SixthRec_ImageCaption;

   private String dbTableName;

   private transient FlightResourceBundle   msgs = null;



   private transient Vector FlightInformationChangedListeners;
                                                                     // list of listeners for the
                                                                     // FlightInformationChanged event

   private transient Vector FlightReservedListeners;
                                                                     // list of listeners for the
                                                                     // FlightReserved event

   private transient Vector flightInfo;
                                                                     // contains the information queried
                                                                     // from the database (and possibly
                                                                     // changed by the business rules DIP).
                                                                     // This is the information sent to
                                                                     // the flight information viewer.

   private String SQLString;                                         // the SQL string used to access the
                                                                     // database

   private VetoableChangeSupport vetos = null;                       // helper class for handling property updates
   private PropertyChangeSupport changes = null;                     // helper class for handling property updates

   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
   private static final long serialVersionUID = 2000L;  // serial version unique ID
}

