//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbTravelDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.13 com/java/src/com/ibm/beans/demos/travel/flight/FlightInfoDapClient.java, bbTravelDemo, bb97c, bb97c9750e 10/28/97 15:02:11 [12/11/97 17:02:52]

package com.ibm.beans.demos.travel.flight;

import com.ibm.beans.util.BeanSystem;
import com.ibm.beans.util.LogStream;
import com.ibm.beans.util.MessageFormatter;

import java.io.IOException;
import java.io.ObjectInputStream;

import java.rmi.Naming;

import java.util.ResourceBundle;
import java.util.Vector;

/**
 * <p>
 * This class is part of the remote travel demo and implements the flight
 * information data access part (DAP) client bean.  For the remote travel
 * demo, the flight information data access part (DAP) bean has been divided
 * into a FlightInfoDapClient bean and a FlightInfoDapServer bean.
 * <p>
 * The FlightInfoDapClient bean serves as the event sink for the TravelInfoChanged
 * event and the FlightRequested event. The TravelInfoChanged event is sent by the itinerary
 * bean when the user selects an origin city, a destination city, and a departure date.
 * The FlightRequested event is sent by the itinerary bean when the user requests
 * confirmation for a selected flight itinerary.
 * <p>
 * The FlightInfoDapClient bean serves as the event source for the FlightInformationChanged
 * and the FlightReserved events.  The FlightInformationChanged event is
 * sent to the flight selector bean when the Client DAP bean receives the
 * TravelInfoChanged event notification from the itinerary
 * bean.  The FlightReserved event is sent to the itinerary bean
 * when the Client DAP bean receives the FlightRequested event notification from
 * the itinerary bean.
 * <p>
 * A customizer is provided for this DAP client so that a user can configure the
 * <i>serverAddress</i> property value of the FlightInfoDapClient instance.
 *
 * @author K. David Johnson (kdjohn@us.ibm.com)
 * @see com.ibm.beans.demos.travel.flight.FlightInfoDapClientBeanInfo
 * @see com.ibm.beans.demos.travel.flight.FlightInfoDapClientCustomizer
 * @see com.ibm.beans.demos.travel.flight.FlightInfoEvent
 * @see com.ibm.beans.demos.travel.flight.FlightInfoListener
 * @see com.ibm.beans.demos.travel.flight.FlightRequestedEvent
 * @see com.ibm.beans.demos.travel.flight.FlightRequestedListener
 * @see com.ibm.beans.demos.travel.flight.FlightReservedEvent
 * @see com.ibm.beans.demos.travel.flight.FlightReservedListener
 * @see com.ibm.beans.demos.travel.TravelInfoChangedEvent
 * @see com.ibm.beans.demos.travel.TravelInfoChangedListener
 */
public class FlightInfoDapClient implements java.io.Serializable,
                                 com.ibm.beans.demos.travel.TravelInfoChangedListener,
                                 com.ibm.beans.demos.travel.flight.FlightRequestedListener
{


   /**
   * Constructs an instance of the FlightInfoDapClient bean.  The constructor initializes
   * instance data.
   */
   public FlightInfoDapClient() {
        FlightInformationChangedListeners = new Vector();
        FlightReservedListeners           = new Vector();
        errorLog                          = BeanSystem.getErrorLog();
   }

   /**
    * Responds to calls on objects of this class when the objects are deserialized.
    * This method is called whenever the FlightInfoDap bean is instantiated from
    * serialized data (such as a .ser file).  It is overridden so that transient
    * data (that is, data not serialized) can be initialized (then it just calls
    * defaultReadObject on the ObjectInputStream to let it take care of initializing
    * all of the non-transient data).
    */
   private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
        // Initialize transient data
        FlightInformationChangedListeners = new Vector();
        FlightReservedListeners           = new Vector();
        errorLog                          = BeanSystem.getErrorLog();

        in.defaultReadObject();
   }


   //------------------------------------------------------------------
   // Event Sinks start
   //------------------------------------------------------------------

   /**
    * Responds to a TravelInfoChanged event. This method is invoked when
    * the Itinerary bean has been wired to this DAP bean and
    * the user submits preliminary information
    * needed to query the database for flights that meet the users preliminary
    * requirements.
    * @param  travelInfoChange The event containing the preliminary travel requirements
    */
   public void TravelInfoChanged(
           com.ibm.beans.demos.travel.TravelInfoChangedEvent travelInfoChange) {
           // jwa: Note that we purposely re-lookup the server each time, so that
           // we can handle the server going down and back up, or someone changing
           // our serverAddress property, etc.
           try {
              String serverName = "FlightInfoDapServer";
              if (!serverAddress.equals("")) {
                 serverName = "//" + serverAddress + "/" + serverName;
              } /* endif */
              FlightInfoDapServerExports serverReference = (FlightInfoDapServerExports)
                 Naming.lookup(serverName);

              if (serverReference!=null) {
                 FlightInfoEvent flightInfoEvent = serverReference.queryDataBase(travelInfoChange);

                 /* Following call necessary so 'source' of event can be changed from the
                    FlightInfoDapServer to the FlightInfoDapClient */
                 FlightInfoEvent fIE = new FlightInfoEvent(this, flightInfoEvent.flightInfo);

                 fireFlightInfo(fIE); // jwa 7/24: Renamed to avoid warnings when morphing
              } else {
                 mf = new MessageFormatter("com.ibm.beans.demos.travel.flight.FlightResourceBundle");
                 errorLog.println( mf, "NamingLookupFailed_msg");
              } /* endif */
           } catch(Exception ex) { ex.printStackTrace(); }
   }


   /**
    * Responds to a FlightRequested event. This method is invoked when the
    * itinerary bean has been wired to this DAP bean and the user has selected
    * a preferred itinerary.  If the itinerary is approved, notification is sent
    * back to the itinerary bean via a FlightReservedEvent.
    * @param  flightRequestedEvent The FlightRequestedEvent containing the preferred flight itinerary.
    */
   public void FlightRequested(com.ibm.beans.demos.travel.flight.FlightRequestedEvent
                               flightRequestedEvent) {
           // jwa: Note that we purposely re-lookup the server each time, so that
           // we can handle the server going down and back up, etc.
           try {
              String serverName = "FlightInfoDapServer";
              if (!serverAddress.equals("")) {
                 serverName = "//" + serverAddress + "/" + serverName;
              } /* endif */
              FlightInfoDapServerExports serverReference = (FlightInfoDapServerExports)
                 Naming.lookup(serverName);

              if (serverReference!=null) {
                 FlightReservedEvent fRE = serverReference.reserveFlight(flightRequestedEvent);

                 /* Following call necessary so 'source' of event can be changed from the
                    FlightInfoDapServer to the FlightInfoDapClient */
                 FlightReservedEvent flightReservedEvent = new FlightReservedEvent(this, fRE.flightInfo);

                 fireFlightReserved(flightReservedEvent);
              } else {
                 mf = new MessageFormatter("com.ibm.beans.demos.travel.flight.FlightResourceBundle");
                 errorLog.println( mf, "NamingLookupFailed_msg");
              } /* endif */
           } catch(Exception ex) { ex.printStackTrace(); }
   }

   //------------------------------------------------------------------
   // Event Syncs end
   //------------------------------------------------------------------


   //------------------------------------------------------------------
   // Event Sources start
   //------------------------------------------------------------------

   /**
    * Allows other beans to listen to the FlightInfo events this bean generates.
    * This is the addListener method for all listeners that want to be notified
    * when a FlightInformationChanged event occurs.  In the demo, the flight
    * selector bean is wired to this DAP bean and is notified when the
    * FlightInformationChanged event happens.
    * @param  fIL The registering listener.
    * @see #removeFlightInfoListener
    */
   public synchronized void addFlightInfoListener(FlightInfoListener fIL) {
        FlightInformationChangedListeners.addElement(fIL);
   }

   /**
    * Allows beans to remove themselves from the listener list for this bean's
    * FlightInfo event. This is the removeListener method for all listeners that
    * want to be notified when a FlightInformationChanged event occurs.  In the demo,
    * the flight selector bean is wired to this DAP bean and is notified when the
    * FlightInformationChanged event happens.
    * @param  fIL The unregistering listener.
    * @see #addFlightInfoListener
    */
   public synchronized void removeFlightInfoListener(FlightInfoListener fIL) {
        FlightInformationChangedListeners.removeElement(fIL);
   }

   /**
   * Allows other beans to listen to the FlightReserved events this bean generates.
   * This is the addListener method for all listeners that want to be notified
   * when a FlightReserved event occurs.  In the demo, this DAP bean
   * is wired to the itinerary viewer bean and is notified by this DAP bean
   * when the FlightReserved event happens.
   * @param  fRL The registering listener.
   * @see #removeFlightReservedListener
   */
   public synchronized void addFlightReservedListener(FlightReservedListener fRL) {
        FlightReservedListeners.addElement(fRL);
   }

   /**
    * Allows beans to remove themselves from the listener list for this bean's
    * FlightReserved event.
    * This is the removeListener method for all listeners that want to be notified
    * when a FlightReserved event occurs.  In the demo, this DAP bean
    * is wired to the itinerary viewer bean and is notified by this DAP bean
    * when the FlightReserved event happens.
    * @param  fRL The registering FlightReservedListener object.
    * @see #addFlightReservedListener
    */
   public synchronized void removeFlightReservedListener(FlightReservedListener fRL) {
        FlightReservedListeners.removeElement(fRL);
   }


   //------------------------------------------------------------------
   // Event Sources end
   //------------------------------------------------------------------


   /**
   * Notifies all registered listeners that a flight has been reserved.
   * In the demo, this event fires in response to a
   * FlightRequested event notification from the itinerary
   * bean when the requested flight itinerary is confirmed.
   * @param  fRE The FlightRequestedEvent containing the preferred flight itinerary.
   */
   public void fireFlightReserved(com.ibm.beans.demos.travel.flight.FlightReservedEvent fRE) {
        Vector l;

        // notify registered listeners

        // must copy the Vector here in order to freeze the state of the
        // set of EventListeners the event should be delivered to prior
        // to delivery.  This ensures that any changes made to the Vector
        // from a target listener's method, during the delivery of this
        // event will not take effect until after the event is delivered.
        synchronized(this) { l = (Vector)FlightReservedListeners.clone(); }

        for (int i = 0; i < l.size(); i++) {
           ((FlightReservedListener)l.elementAt(i)).FlightReserved(fRE);
        }
   }



   /**
   * Notifies all registered listeners. A list of
   * flights are sent as a Vector.  In the demo,
   * this event fires in response to a TavelInfoChangedEvent
   * which is fired by the itinerary bean after
   * a user has entered the origin and destination cities
   * and dates and has pressed the submit button.  This DAP
   * then queries the database for matching flights, builds
   * this information into a vector,  and fires the
   * FlightInformationChanged notification to the flight
   * selector bean.
   * @param  fIE The FlightInfoEvent containing the possible flight selections.
   */
   public void fireFlightInfo(FlightInfoEvent fIE) {
        Vector l;

        // notify registered listeners

        // must copy the Vector here in order to freeze the state of the
        // set of EventListeners the event should be delivered to prior
        // to delivery.  This ensures that any changes made to the Vector
        // from a target listener's method, during the delivery of this
        // event will not take effect until after the event is delivered.

        synchronized(this) { l = (Vector)FlightInformationChangedListeners.clone(); }

        for (int i = 0; i < l.size(); i++) {
           ((FlightInfoListener)l.elementAt(i)).FlightInformationChanged(fIE);
        }
   }


   /**
   * Retrieves the hostname or IP address of the FlightInfoDapServer.
   *
   * @return The TCP/IP designation of the machine on which the FlightInfoDapServer
   * is running.  An empty string (NOT <tt>null</tt>) indicates that the
   * FlightInfoDapServer is running on the same machine as the FlightInfoDapClient.
   */
   public String getServerAddress() { return serverAddress; }

   /**
   * Specifies the hostname or IP address of the FlightInfoDapServer.
   *
   * @param newAddress The TCP/IP designation of the machine on which the
   *  FlightInfoDapServer is running.  Specify the empty string (NOT
   *  <tt>null</tt>) to indicate the FlightInfoDapServer is running on
   *  the same machine as the FlightInfoDapClient.
   */
   public void setServerAddress(String newAddress) {
      serverAddress = newAddress;
   }

   /**
   * Given a key, get a message from the FlightResourceBundle
   * @param key The key for the message
   * @return The message retrieved from the resource bundle
   */
   private String getMsg(String key)
   {
        if (msgs == null)
          msgs = (FlightResourceBundle)ResourceBundle.getBundle(
            "com.ibm.beans.demos.travel.flight.FlightResourceBundle");

        return (String)msgs.handleGetObject(key);
   }



   private transient Vector FlightInformationChangedListeners;       // list of listeners for the
                                                                     // FlightInformationChanged event

   private transient Vector FlightReservedListeners;                 // list of listeners for the
                                                                     // FlightReserved event

   private           String serverAddress = "";                      // server's hostname or IP address
                                                                     // initialized to please PropertySheet


   private transient FlightResourceBundle   msgs         = null;
   private MessageFormatter mf;
   private transient LogStream errorLog;

   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
   private static final long serialVersionUID = 2000L;  // serial version unique ID
}

