//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbTravelDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================

package com.ibm.beans.demos.travel.flight;

import java.beans.Beans;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;
import java.beans.VetoableChangeSupport;

import java.io.IOException;
import java.io.ObjectInputStream;

import java.rmi.Naming;
import java.rmi.RMISecurityManager;

import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;

import com.ibm.beans.demos.travel.flight.FlightResourceBundle;
import com.ibm.beans.samples.daps.sql.Connection;
import com.ibm.beans.samples.daps.sql.Select;
import com.ibm.beans.util.BeanSystem;
import com.ibm.beans.util.LogStream;
import com.ibm.beans.util.MessageFormatter;

/**
 * <p>
 * This class is part of the remote travel demo and implements the flight
 * information data access part (DAP) Server bean.  For the remote travel demo,
 * the flight information data access part (DAP) bean has been divided into
 * a FlightInfoDapClient bean and a FlightInfoDapServer bean.  The FlightInfoDapServer
 * bean implements the FlightInfoDapProperties interface which contains methods that
 * are used by the dips from the FlightRules.jar (from the com.ibm.beans.demos.travel.flight.rules
 * package) to alter the flight data returned from the database query.  The FlightInfoDapServerExports
 * interface is also implemented.  This interface contains the methods that are called
 * on the FlightInfoDapServer from the FlightInfoDapClient to query that database for matching
 * flights and to reserve a particular flight schedule.
 * <p>
 * A customizer is provided for this server so that a user can configure whether or not
 * to access the database via JDBC or to access database values from the FlightResourceBundle.
 *
 * @author K. David Johnson (kdjohn@us.ibm.com)
 * @see com.ibm.beans.demos.travel.flight.FlightInfoDapServerCustomizer
 * @see com.ibm.beans.demos.travel.flight.FlightInfoDapServerBeanInfo
 * @see com.ibm.beans.demos.travel.flight.FlightInfoDapServerExports
 * @see com.ibm.beans.demos.travel.flight.FlightInfoDapClient
 * @see com.ibm.beans.demos.travel.flight.FlightInfoDapClientCustomizer
 * @see com.ibm.beans.demos.travel.flight.FlightInfoDapClientBeanInfo
 * @see com.ibm.beans.demos.travel.flight.FlightInfoDapProperties
 * @see com.ibm.beans.demos.travel.flight.FlightInformation
 * @see com.ibm.beans.demos.travel.TravelInfoChangedEvent
 * @see com.ibm.beans.demos.travel.flight.FlightInfoEvent
 * @see com.ibm.beans.demos.travel.flight.FlightRequestedEvent
 * @see com.ibm.beans.demos.travel.flight.FlightReservedEvent
 */
public class FlightInfoDapServer extends java.rmi.server.UnicastRemoteObject implements
                                 java.io.Serializable,
                                 com.ibm.beans.demos.travel.flight.FlightInfoDapProperties,
                                 com.ibm.beans.demos.travel.flight.FlightInfoDapServerExports
{

   /**
   * Runs the FlightInfoDapServer as an application.  By default, Beans.instantiate()
   * is used, passing "com.ibm.beans.demos.travel.flight.FlightInfoDapServer" as
   * the name of the object to be instantiated; however, if "-dippable" is specified,
   * it will use "com.ibm.beans.demos.travel.flight.FlightInfoDapServerDippable" as
   * the name of the object to be instantiated.  Note that by using Beans.instantiate,
   * this allows for a customized version of the FlightInfoDapServer to be used, so
   * long as it is named FlightInfoDapServer.ser (or FlightInfoDapServerDippable.ser,
   * if the "-dippable" option is specified).
   */
   public static void main(String argv[]) {
      String name = "com.ibm.beans.demos.travel.flight.FlightInfoDapServer";

      // Instantiate FlightInfoDapServerDippable if "-dippable" is specified
      if ((argv.length==1) && argv[0].equalsIgnoreCase("-dippable")) {
         name = "com.ibm.beans.demos.travel.flight.FlightInfoDapServerDippable";
      } /* endif */

      try {
         /* By using Beans.instantiate instead of new, we can cause a serialized
            version, which might have dips applied, to be instantiated. */
         FlightInfoDapServer server = (FlightInfoDapServer) Beans.instantiate(null, name);
      } catch(Exception ex) { ex.printStackTrace(); }
   }

   /**
   * Constructs an instance of the FlightInfoDapServer bean.  The constructor initializes
   * instance data.
   * @exception java.rmi.RemoteException
   *            Thrown if registry could not be contacted
   */
   public FlightInfoDapServer() throws java.rmi.RemoteException {

        flightInfo  = new Vector();
        vetos       = new VetoableChangeSupport(this);
        changes     = new PropertyChangeSupport(this);
        SQLString   = "";
        errorLog    = BeanSystem.getErrorLog();

        // initialize JDBC instance data properties used by the FlightInfoDapServerCustomizer
        bUseJDBC    = new Boolean(false);

        // get the strings from the resource file that will be used if JDBC is not chosen.
        getResourceStrings();

        // set up initial values
        dbTableName  = getMsg("dbTableName");
        JDBCURL      = getMsg("JDBCURL");

        bindWithRegistry();
   }

   /**
    * Responds to calls on objects of this class when the objects are deserialized.
    * This method is called whenever the FlightInfoDapServer bean is instantiated from
    * serialized data (such as a .ser file).  It is overridden so that transient
    * data (that is, data not serialized) can be initialized (then it just calls
    * defaultReadObject on the ObjectInputStream to let it take care of initializing
    * all of the non-transient data).
    */
   private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {

        // Initialize transient data
        flightInfo = new Vector();
        vetos      = new VetoableChangeSupport(this);
        changes    = new PropertyChangeSupport(this);

        errorLog   = BeanSystem.getErrorLog();

        in.defaultReadObject();

        // get the strings from the resource file.  These will be used if JDBC is not chosen.
        getResourceStrings();

        bindWithRegistry();
   }

   /**
    * Create and install the security manager and rebind the FlightInfoDapServer name
    * to this object instance.
    */
   public void bindWithRegistry() {
        try {
           SecurityManager currentSecurityManager = System.getSecurityManager();
           // Create and install the security manager
           if (currentSecurityManager==null) {
              System.setSecurityManager(new RMISecurityManager());
           } else {
              if (!(currentSecurityManager instanceof RMISecurityManager)) {
                 mf = new MessageFormatter("com.ibm.beans.demos.travel.flight.FlightResourceBundle");
                 errorLog.println( mf, "securityAlreadyInstalled_msg");
              } /* endif */
           } /* endif */

           Naming.rebind("FlightInfoDapServer", this);
        } catch (Exception ex) {
           ex.printStackTrace();
        } /* end try */
   }

   /**
    * Unbind the FlightInfoDapServer
    */
   protected void finalize() {
      try {
         Naming.unbind("FlightInfoDapServer");
      } catch(Exception ex) {
         ex.printStackTrace();
      } /* end try */
   }


   //------------------------------------------------------------------
   // Event Sources start
   //------------------------------------------------------------------

   /**
   * Adds a listener to be notified when the <i>SQLString</i> property or the
   * <i>flightInfo</i> property is about to be changed.
   *
   * @param listener   The object to be notified.
   */
   public void addVetoableChangeListener(VetoableChangeListener listener) {
        vetos.addVetoableChangeListener(listener);
   }

   /**
   * Removes a listener from the notification list.
   *
   * @param listener   The object to be removed from the notification list.
   */
   public void removeVetoableChangeListener(VetoableChangeListener listener) {
        vetos.removeVetoableChangeListener(listener);
   }

   /**
   * Adds a listener to be notified when the <i>SQLString</i> property or the <i>flightInfo</i> property has been changed.
   *
   * @param listener   The object to be notified.
   */
   public void addPropertyChangeListener(PropertyChangeListener listener) {
        changes.addPropertyChangeListener(listener);
   }

   /**
   * Removes a listener from the notification list.
   *
   * @param listener   The object to be removed from the notification list.
   */
   public void removePropertyChangeListener(PropertyChangeListener listener) {
        changes.removePropertyChangeListener(listener);
   }

   //------------------------------------------------------------------
   // Event Sources end
   //------------------------------------------------------------------


   /**
   * Queries the database to find all of the possible flight
   * choices that match the user request of origin city, destination
   * city, departure date, and return date.
   * @param travelInfoChange The TravelInfoChanged event containing the travel info
   * @return The FlightInfoEvent containing the flight info
   * @exception java.rmi.RemoteException
   *            Thrown if registry could not be contacted
   */
   public com.ibm.beans.demos.travel.flight.FlightInfoEvent queryDataBase(
   com.ibm.beans.demos.travel.TravelInfoChangedEvent travelInfoChange)
   throws java.rmi.RemoteException {

      String originCity         = travelInfoChange.getOriginCity();
      String destinationCity    = travelInfoChange.getDestinationCity();
      java.util.Date departDate = travelInfoChange.getDepartDate();

      Vector l;
      FlightInfoEvent flightInfoEvent;

      synchronized (this) {                     // keep from having problems with multithreaded access

        if (bUseJDBC.booleanValue()) {
           // if this instance has been configured to use JDBC
           // to access the database, invoke the appropriate method
           return queryJDBCDataBase(travelInfoChange);
        }

        if ( !(flightInfo.isEmpty()) ) {        // clear out the vector
            flightInfo.removeAllElements();
        }

        // JDBC will not be used to access the database, so we will use the hard coded values.

        Vector flightInfoTemp = new Vector();

        // build the SQL statement that can be modified by a business rules dip if desired. We will only use the originCity
        // and destinationCity and ignore the dates for now.
        String mySQLString;
        mySQLString =  "SELECT * FROM " ;
        mySQLString += dbTableName;
        mySQLString += " WHERE ORIGIN = " + originCity;
        mySQLString += " AND DESTINATION = " + destinationCity;

        // set the SQL string as a property.  This will allow the business rules DIP to
        // hook the property and modify it if desired - i.e. to restrict access to a certain
        // airline - American, Delta, or TWA.  The DIP will add the following to the end of the
        // SQL string:  "AND AIRLINE <> AMERICAN", and/or, "AND AIRLINE <> DELTA",
        // and/or, "AND AIRLINE <> TWA".
        try {
          setSQLString(mySQLString);
        } catch(PropertyVetoException e) {
        }

        // retrieve the SQL string which may have been altered by the DIP
        String myFinalSQLString = getSQLString();

        //parse the string to see if it contains any substrings of the type - "AND AIRLINE <> AMERICAN", etc.
        //for the demo we will use the StringTokenizer with delemiter of "<>" and assume
        //that the "AND AIRLINE" was the prefix.

        boolean bDontIncludeAirline1      = false;                     // should we include airline1 flights?
        boolean bDontIncludeAirline2      = false;                     // should we include airline2 flights?
        boolean bDontIncludeAirline3      = false;                     // should we include airline3 flights?

        String airline1 = " ";
        airline1       += FIDS_Airline1;
        String airline2 = " ";
        airline2       += FIDS_Airline2;
        String airline3 = " ";
        airline3       += FIDS_Airline3;

        StringTokenizer t = new StringTokenizer(myFinalSQLString, "<>");
        if (t.countTokens() > 0) {
           String myToken;
           while (t.hasMoreTokens()) {
             myToken = t.nextToken().toUpperCase();
             if (myToken.startsWith(airline1.toUpperCase())) {
                bDontIncludeAirline1 = true;
             }
             if (myToken.startsWith(airline2.toUpperCase())) {
                bDontIncludeAirline2 = true;
             }
             if (myToken.startsWith(airline3.toUpperCase())) {
                bDontIncludeAirline3 = true;
             }
           }
        }

        // variables used to convert the resource file price "String" values to double values
        Double tempDouble;
        double price1;
        double price2;

        if ( (originCity.equals(FIDS_FirstOriginCity)) && (destinationCity.equals(FIDS_FirstDestinationCity)) ) {


           // eliminate flights from the list if the business rules DIP has changed the SQL statement

           if (!bDontIncludeAirline1) {
              tempDouble = new Double(FIDS_FirstRec_Price);
              price1     = tempDouble.doubleValue();
              flightInfoTemp.addElement(new FlightInformation(FIDS_Airline1,  FIDS_FirstRec_FlightNum1,
                                        FIDS_FirstOriginCity,FIDS_FirstOriginCity_AirportCode,FIDS_FirstDestinationCity,
                                        FIDS_FirstDestinationCity_AirportCode,
                                        FIDS_FirstRec_WhichClass,FIDS_FirstRec_DepartTime,FIDS_FirstRec_ArriveTime,
                                        price1,FIDS_FirstRec_SeatNum,
                                        false,"","","","","","","",
                                        0,"","","","","","","","","",FIDS_FirstRec_ImageLocation,
                                        FIDS_FirstRec_ImageCaption)  );
           }

           if (!bDontIncludeAirline2) {
              tempDouble = new Double(FIDS_SecondRec_Price);
              price1     = tempDouble.doubleValue();
              flightInfoTemp.addElement(new FlightInformation(FIDS_Airline2, FIDS_SecondRec_FlightNum1,
                                        FIDS_FirstOriginCity,FIDS_FirstOriginCity_AirportCode,FIDS_FirstDestinationCity,
                                        FIDS_FirstDestinationCity_AirportCode,
                                        FIDS_SecondRec_WhichClass,FIDS_SecondRec_DepartTime,FIDS_SecondRec_ArriveTime,
                                        price1,FIDS_SecondRec_SeatNum,
                                        false,"","","","","","","",
                                        0,"","","","","","","","","",FIDS_SecondRec_ImageLocation,
                                        FIDS_SecondRec_ImageCaption)  );
           }

           if (!bDontIncludeAirline3) {
              tempDouble = new Double(FIDS_ThirdRec_Price);
              price1     = tempDouble.doubleValue();
              flightInfoTemp.addElement(new FlightInformation(FIDS_Airline3, FIDS_ThirdRec_FlightNum1,
                                        FIDS_FirstOriginCity,FIDS_FirstOriginCity_AirportCode,FIDS_FirstDestinationCity,
                                        FIDS_FirstDestinationCity_AirportCode,
                                        FIDS_ThirdRec_WhichClass,FIDS_ThirdRec_DepartTime,FIDS_ThirdRec_ArriveTime,
                                        price1,FIDS_ThirdRec_SeatNum,
                                        false,"","","","","","","",
                                        0,"","","","","","","","","",FIDS_ThirdRec_ImageLocation,
                                        FIDS_ThirdRec_ImageCaption)  );
           }
        }

        if ( (originCity.equals(FIDS_SecondOriginCity)) && (destinationCity.equals(FIDS_SecondDestinationCity)) ) {


           if (!bDontIncludeAirline1) {
               // a connecting flight
               tempDouble = new Double(FIDS_FourthRec_Price1);
               price1     = tempDouble.doubleValue();
               tempDouble = new Double(FIDS_FourthRec_Price2);
               price2     = tempDouble.doubleValue();
               flightInfoTemp.addElement(new FlightInformation(FIDS_Airline1, FIDS_FourthRec_FlightNum1,
                                         FIDS_SecondOriginCity,FIDS_SecondOriginCity_AirportCode,
                                         FIDS_FourthRec_IntermediateDestination,
                                         FIDS_FourthRec_IntermediateDestinationAirportCode,FIDS_FourthRec_WhichClass,
                                         FIDS_FourthRec_DepartTime1,FIDS_FourthRec_ArriveTime1,
                                         price1,FIDS_FourthRec_SeatNum1,
                                         true,FIDS_Airline1, FIDS_FourthRec_FlightNum2 ,FIDS_SecondDestinationCity,
                                         FIDS_SecondDestinationCity_AirportCode,
                                         FIDS_FourthRec_WhichClass,FIDS_FourthRec_DepartTime2 ,FIDS_FourthRec_ArriveTime2 ,
                                         price2,FIDS_FourthRec_SeatNum2 ,"","","","","","","","",
                                         FIDS_FourthRec_ImageLocation,FIDS_FourthRec_ImageCaption)  );
           }
           if (!bDontIncludeAirline2) {
               // a connecting flight
               tempDouble = new Double(FIDS_FifthRec_Price1);
               price1     = tempDouble.doubleValue();
               tempDouble = new Double(FIDS_FifthRec_Price2);
               price2     = tempDouble.doubleValue();
               flightInfoTemp.addElement(new FlightInformation(FIDS_Airline2, FIDS_FifthRec_FlightNum1,
                                         FIDS_SecondOriginCity,FIDS_SecondOriginCity_AirportCode,
                                         FIDS_FifthRec_IntermediateDestination,
                                         FIDS_FifthRec_IntermediateDestinationAirportCode,FIDS_FifthRec_WhichClass,
                                         FIDS_FifthRec_DepartTime1,FIDS_FifthRec_ArriveTime1,
                                         price1,FIDS_FifthRec_SeatNum1,
                                         true,FIDS_Airline2,FIDS_FifthRec_FlightNum2,FIDS_SecondDestinationCity,
                                         FIDS_SecondDestinationCity_AirportCode,
                                         FIDS_FifthRec_WhichClass,FIDS_FifthRec_DepartTime2,FIDS_FifthRec_ArriveTime2,
                                         price2,FIDS_FifthRec_SeatNum2,"","","","","","","","",
                                         FIDS_FifthRec_ImageLocation, FIDS_FifthRec_ImageCaption)  );
           }

           // note that this is a direct flight but costs $110.00 more that the alternative connecting
           // flights.  Therefore this is a candidate for the business rules dip to eliminate this
           // flight from the list.
           if (!bDontIncludeAirline3) {
               // a direct flight
               tempDouble = new Double(FIDS_SixthRec_Price);
               price1     = tempDouble.doubleValue();
               flightInfoTemp.addElement(new FlightInformation(FIDS_Airline3,  FIDS_SixthRec_FlightNum1,
                                         FIDS_SecondOriginCity,FIDS_SecondOriginCity_AirportCode,FIDS_SecondDestinationCity,
                                         FIDS_SecondDestinationCity_AirportCode,FIDS_SixthRec_WhichClass,
                                         FIDS_SixthRec_DepartTime,FIDS_SixthRec_ArriveTime,
                                         price1,FIDS_SixthRec_SeatNum,
                                         false,"","","","","","","",
                                         0,"None","","","","","","","","", FIDS_SixthRec_ImageLocation,
                                         FIDS_SixthRec_ImageCaption)  );
           }
        }

        // setting this property gives the business rules DIP an opportunity to hook all flights and
        // modify the list if necessary - i.e. see the above scenario where the direct Austin to
        // Chicago flight is $110.00 more than the alternate connecting flights and could be
        // removed from the list by the DIP.
        try {
          setFlightInfo(flightInfoTemp);
        } catch(PropertyVetoException e) {
        }

        flightInfoEvent = new FlightInfoEvent(this, flightInfo);
        } /* end synchronized */

      return(flightInfoEvent);
   }

   /**
   * Attempts to reserve the specified flight(s).  This method is invoked by the FlightInfoDapClient
   * when the Itinerary bean has been wired to the FlightInfoDapClient bean and the user has selected
   * a preferred itinerary.
   * @param  flightRequestedEvent The FlightRequestedEvent containing the preferred flight itinerary.
   * @return The FlightReservedEvent containing the reserved flight itinerary.
   */
   public com.ibm.beans.demos.travel.flight.FlightReservedEvent reserveFlight(
   com.ibm.beans.demos.travel.flight.FlightRequestedEvent flightRequestedEvent)
   throws java.rmi.RemoteException {

      FlightReservedEvent flightReservedEvent;

      synchronized (this) { //keep from having problems with multithreaded access
        // for now we will just accept the itinerary and prepare a FlightReservedEvent
        // to send to the itinerary bean.

        flightReservedEvent = new FlightReservedEvent(this, flightRequestedEvent.flightInfo);
      } /* end synchronized */

      return(flightReservedEvent);
   }


   /**
   * Accessor (set) method for the <i>flightInfo</i> property.
   * @param newValue A Vector representing the  FlightInformation.
   * @exception PropertyVetoException Indicates that a constrained property
   * listener has vetoed the change to the <i>flightInfo</i> property.
   */
   public void setFlightInfo(Vector newValue) throws PropertyVetoException {
        Vector oldFlightInfo;

        synchronized(this) { oldFlightInfo = (Vector)flightInfo.clone(); }

        // first tell the vetoers about the change.  If anyone objects, we
        // let the PropertyVetoException propagate back to our caller.
        vetos.fireVetoableChange("flightInfo",oldFlightInfo,newValue);

        // No one vetoed, so go ahead and make the change
        synchronized(this) { flightInfo    = (Vector)newValue.clone(); }
        changes.firePropertyChange("flightInfo",oldFlightInfo,flightInfo);
   }

   /**
   * Accessor (get) method for the <i>flightInfo</i> property.
   *
   * @return The Vector containing FlightInformation objects.
   */
   public Vector getFlightInfo() {
        return flightInfo;
   }

   /**
   * Accessor (set) method for the <i>SQLString</i> property.
   * @param SQLString The String representing the SQL property of the bean.
   * @exception PropertyVetoException Indicates that a constrained property
   * listener has vetoed the change to the <i>SQLString</i> property.
   */
   public void setSQLString(String SQLString) throws PropertyVetoException {
        String oldString = this.SQLString;

        // first tell the vetoers about the change.  If anyone objects, we
        // let the PropertyVetoException propagate back to our caller.
        vetos.fireVetoableChange("SQLString",oldString,SQLString);

        // No one vetoed, so go ahead and make the change
        this.SQLString   = SQLString;
        changes.firePropertyChange("SQLString",oldString,SQLString);

   }

   /**
   * Accessor (get) method for the <i>SQLString</i> property.
   * @return The String containing the SQL database query
   */
   public String getSQLString() {
        return SQLString;
   }

   /**
   * Queries the JDBC database to find all of the possible flight
   * choices that match the user request of origin city, destination
   * city, departure date, and return date.
   * @param travelInfoChange The TravelInfoChanged event containing the travel info
   * @return The FlightInfoEvent containing the flight info
   */
   private com.ibm.beans.demos.travel.flight.FlightInfoEvent queryJDBCDataBase(
   com.ibm.beans.demos.travel.TravelInfoChangedEvent travelInfoChange)
   throws java.rmi.RemoteException {

      String originCity         = travelInfoChange.getOriginCity();
      String destinationCity    = travelInfoChange.getDestinationCity();
      java.util.Date departDate = travelInfoChange.getDepartDate();
      // String returnDate   = travelInfoChange.getReturnDate();

      Vector l;
      FlightInfoEvent flightInfoEvent;

      synchronized (this) { //keep from having problems with multithreaded access


        if ( !(flightInfo.isEmpty()) ) {         // clear out the vector
           flightInfo.removeAllElements();
        }

        Vector flightInfoTemp = new Vector();

        // build the SQL statement that can be changed by a business rules dip.  We will only use the originCity and
        // destinationCity and ignore the dates for now.
        String mySQLString;

        mySQLString  = "SELECT * FROM ";
        mySQLString += dbTableName;
        mySQLString += " WHERE ORIGIN = "    + originCity;
        mySQLString += " AND DESTINATION = " + destinationCity;

        // set the SQL string as a property.  This will allow the business rules DIP to
        // hook the property and modify it if desired - i.e. to restrict access to a certain
        // airline - American, Delta, or TWA.  The DIP will add the following to the end of the
        // SQL string:  "AND AIRLINE <> AMERICAN", and/or, "AND AIRLINE <> DELTA",
        // and/or, "AND AIRLINE <> TWA".
        try {
           setSQLString(mySQLString);
        } catch(PropertyVetoException e) {
        }

        // retrieve the SQL string which may have been altered by the DIP
        String myFinalSQLString = getSQLString();

        // parse the string to see if it contains any substrings of the type - "AND AIRLINE <> AMERICAN", etc.
        // for the demo we will use the StringTokenizer with delemiter of "<>" and assume
        // that the "AND AIRLINE" was the prefix.

        boolean bDontIncludeAirline1      = false;                     // should we include airline1 flights?
        boolean bDontIncludeAirline2      = false;                     // should we include airline2 flights?
        boolean bDontIncludeAirline3      = false;                     // should we include airline3 flights?

        String airline1   = " ";
        airline1         += FIDS_Airline1;
        String myairline2 = " ";
        myairline2       += FIDS_Airline2;
        String airline3   = " ";
        airline3         += FIDS_Airline3;

        StringTokenizer t = new StringTokenizer(myFinalSQLString, "<>");
        if (t.countTokens() > 0) {
           String myToken;
           while (t.hasMoreTokens()) {
             myToken = t.nextToken().toUpperCase();
             if (myToken.startsWith(airline1.toUpperCase())) {
                bDontIncludeAirline1 = true;
             }
             if (myToken.startsWith(myairline2.toUpperCase())) {
                bDontIncludeAirline2 = true;
             }
             if (myToken.startsWith(airline3.toUpperCase())) {
                bDontIncludeAirline3 = true;
             }
           }
        }


        // instantiate the connection and select JDBC beans
        try {
           connection = (Connection)Beans.instantiate(null,"com.ibm.beans.samples.daps.sql.Connection");
           select     = (Select)Beans.instantiate(null,"com.ibm.beans.samples.daps.sql.Select");
        } catch (Exception e) {
           e.printStackTrace();
           mf = new MessageFormatter("com.ibm.beans.demos.travel.flight.FlightResourceBundle");
           errorLog.println( mf, "JDBCBeansNotInstalled_msg");
           System.exit(0);
        }

        // Set Connection properties
        connection.setJDBCURL(JDBCURL);

        // Perform the Database Connection
        connection.connect();

        // If successful connection
        if (connection.getSuccessful())
        {
           // Get the JDBCConnection
           java.sql.Connection connect = connection.getJDBCConnection();

           // Set Select Properties
           select.setJDBCConnection(connect);
        } else {
           mf = new MessageFormatter("com.ibm.beans.demos.travel.flight.FlightResourceBundle");
           errorLog.println( mf, "JDBCDatabase_cant_connect_msg");
           System.exit(0);
        }

        // build the SQL string to access the database
        // eliminate flights from the list if the business rules DIP has changed the SQL statement
        String myJDBCString;

        if (bDontIncludeAirline1) {

           myJDBCString  = "SELECT * FROM ";
           myJDBCString += dbTableName;
           myJDBCString += " WHERE ORIGIN = " + "'" + originCity + "'";
           myJDBCString += " AND DESTINATION = " + "'" + destinationCity + "'";
           myJDBCString += " AND AIRLINE <> 'American' AND AIRLINE2 <> 'American'";
           select.setSQLText(myJDBCString);

        } else if (bDontIncludeAirline2) {

           myJDBCString  = "SELECT * FROM ";
           myJDBCString += dbTableName;
           myJDBCString += " WHERE ORIGIN = " + "'" + originCity + "'";
           myJDBCString += " AND DESTINATION = " + "'" + destinationCity + "'";
           myJDBCString += " AND AIRLINE <> 'Delta' AND AIRLINE2 <> 'Delta'";
           select.setSQLText(myJDBCString);

        } else if (bDontIncludeAirline3) {

           myJDBCString  = "SELECT * FROM ";
           myJDBCString += dbTableName;
           myJDBCString += " WHERE ORIGIN = " + "'" + originCity + "'";
           myJDBCString += " AND DESTINATION = " + "'" + destinationCity + "'";
           myJDBCString += " AND AIRLINE <> 'TWA' AND AIRLINE2 <> 'TWA'";
           select.setSQLText(myJDBCString);

        } else {

           myJDBCString  = "SELECT * FROM ";
           myJDBCString += dbTableName;
           myJDBCString += " WHERE ORIGIN = " + "'" + originCity + "'";
           myJDBCString += " AND DESTINATION = " + "'" + destinationCity + "'";
           select.setSQLText(myJDBCString);
        }

        // Execute the Select
        select.execute();

        // If the execute was successful, build the flightInfoTemp with all pertinent records from the database
        if (select.getSuccessful())
        {
              String  airline;
              String  flightNum;
              String  startCity;
              String  airportCodeStartCity;
              String  endCity;
              String  airportCodeEndCity;
              String  whichClass;
              String  departTime;
              String  arriveTime;
              double  price;
              String  seatNum;
              boolean bHasConnection;
              String  airline2;
              String  flightNum2;
              String  endCity2;
              String  airportCodeEndCity2;
              String  whichClass2;
              String  departTime2;
              String  arriveTime2;
              double  price2;
              String  seatNum2;
              String  firstName;
              String  lastName;
              String  street;
              String  city;
              String  state;
              String  zip;
              String  phone;
              String  frequentFlyerNum;
              String  imageLocation;
              String  imageCaption;
              String  strbHasConnection;
              String  temp;
              Double  tempDouble;

              // Do forever
              while (true)
              {
                  // Get the next row
                  select.next();

                  // If successful and row found
                  if ((select.getSuccessful()) &&
                     (select.getRowFound()))
                  {

                     // skip the origin and destination, the first two fields -
                     // put the other elements of the row into flightInfoTemp

                     Vector myVector = select.getResultRow();

                     airline              = (String) myVector.elementAt(2);
                     flightNum            = (String) myVector.elementAt(3);
                     startCity            = (String) myVector.elementAt(4);
                     airportCodeStartCity = (String) myVector.elementAt(5);
                     endCity              = (String) myVector.elementAt(6);
                     airportCodeEndCity   = (String) myVector.elementAt(7);
                     whichClass           = (String) myVector.elementAt(8);
                     departTime           = (String) myVector.elementAt(9);
                     arriveTime           = (String) myVector.elementAt(10);
                     temp                 = (String)myVector.elementAt(11);
                     tempDouble           = new Double(temp);
                     price                = tempDouble.doubleValue();
                     seatNum              = (String) myVector.elementAt(12);

                     strbHasConnection    = (String) myVector.elementAt(13);
                     bHasConnection       = false;

                     if (strbHasConnection.equals("1")) {
                        bHasConnection    = true;
                     }

                     airline2             = (String) myVector.elementAt(14);
                     flightNum2           = (String) myVector.elementAt(15);
                     endCity2             = (String) myVector.elementAt(16);
                     airportCodeEndCity2  = (String) myVector.elementAt(17);
                     whichClass2          = (String) myVector.elementAt(18);
                     departTime2          = (String) myVector.elementAt(19);
                     arriveTime2          = (String) myVector.elementAt(20);
                     seatNum2             = (String) myVector.elementAt(22);
                     price2               = 0;

                     if (bHasConnection) {
                        temp = (String)myVector.elementAt(21);
                        tempDouble = new Double(temp);
                        price2 = tempDouble.doubleValue();
                     }

                     firstName            = (String) myVector.elementAt(23);
                     lastName             = (String) myVector.elementAt(24);
                     street               = (String) myVector.elementAt(25);
                     city                 = (String) myVector.elementAt(26);
                     state                = (String) myVector.elementAt(27);
                     zip                  = (String) myVector.elementAt(28);
                     phone                = (String) myVector.elementAt(29);
                     frequentFlyerNum     = (String) myVector.elementAt(30);
                     imageLocation        = (String) myVector.elementAt(31);
                     imageCaption         = (String) myVector.elementAt(32);

                     flightInfoTemp.addElement(new FlightInformation(airline, flightNum, startCity,airportCodeStartCity,endCity,airportCodeEndCity,whichClass,departTime,arriveTime,price,seatNum,bHasConnection,airline2,flightNum2,endCity2,airportCodeEndCity2,whichClass2,departTime2,arriveTime2,price2,seatNum2,firstName,lastName,street,city,state,zip,phone,frequentFlyerNum,imageLocation,imageCaption));

                  }
                  else
                  {
                     // Break out of the loop
                     break;
                  }
              }

              // If the loop was terminated because no longer successful
              if (!select.getSuccessful())
              {
                  mf = new MessageFormatter("com.ibm.beans.demos.travel.flight.FlightResourceBundle");
                  errorLog.println( mf, "JDBCDatabase_select_not_successful_msg");
                  System.out.println(select.getException().toString());
              }
              // Relinquish resources for the Select - ignore errors
              select.close();
        }
        else
        {
              // Display exception text - select.setSQLText was not successful
              mf = new MessageFormatter("com.ibm.beans.demos.travel.flight.FlightResourceBundle");
              errorLog.println( mf, "JDBCDatabase_select_not_successful_msg");
              System.out.println(select.getException().toString());
        }


        // setting this property gives the business rules DIP an opportunity to hook all flights and
        // modify the list if necessary - i.e. see the above scenario where the direct Austin to
        // Chicago flight is $110.00 more than the alternate connecting flights and could be
        // removed from the list by the DIP.
        try {
          setFlightInfo(flightInfoTemp);
        } catch(PropertyVetoException e) {
        }

        flightInfoEvent = new FlightInfoEvent(this, flightInfo);
      } /* end synchronized */

      return(flightInfoEvent);
   }

   /**
   * Accessor (set) method for the <i>JDBCURL</i> property.
   * @param JDBCURL The JDBC URL for the JDBC connection
   * @exception PropertyVetoException Indicates that a constrained property
   * listener has vetoed the change to the <i>JDBCURL</i> property.
   */
   public void setJDBCURL(String JDBCURL) throws PropertyVetoException {
        String oldJDBCURL = this.JDBCURL;

        // first tell the vetoers about the change.  If anyone objects, we
        // let the PropertyVetoException propagate back to our caller.
        vetos.fireVetoableChange("JDBCURL",oldJDBCURL,JDBCURL);

        // No one vetoed, so go ahead and make the change
        this.JDBCURL   = JDBCURL;
        changes.firePropertyChange("JDBCURL",oldJDBCURL,JDBCURL);
   }

   /**
   * Accessor (get) method for the <i>JDBCURL</i> property.
   *
   * @return The string containing the JDBCURL value.
   */
   public String getJDBCURL() {
        return JDBCURL;
   }


   /**
   * Accessor (set) method for the <i>bUseJDBC</i> property.
   * @param bUseJDBC A boolean that signifies whether a JDBC database or
   * the hard coded database will be used.
   * @exception PropertyVetoException Indicates that a constrained property
   * listener has vetoed the change to the <i>bUseJDBC</i> property.
   */
   public void setbUseJDBC(Boolean bUseJDBC) throws PropertyVetoException {
        Boolean oldboolean = this.bUseJDBC;

        // first tell the vetoers about the change.  If anyone objects, we
        // let the PropertyVetoException propagate back to our caller.
        vetos.fireVetoableChange("bUseJDBC",oldboolean,bUseJDBC);

        // No one vetoed, so go ahead and make the change
        this.bUseJDBC   = bUseJDBC;
        changes.firePropertyChange("bUseJDBC",oldboolean,bUseJDBC);
   }

   /**
   * Accessor (get) method for the <i>bUseJDBC</i> property.
   *
   * @return The Boolean object containing the bUseJDBC value.
   */
   public Boolean getbUseJDBC() {
        return bUseJDBC;
   }


   /**
   * Accessor (set) method for the <i>dbTableName</i> property.
   * @param dbTableName The database table name
   * @exception PropertyVetoException Indicates that a constrained property
   * listener has vetoed the change to the <i>dbTableName</i> property.
   */
   public void setdbTableName(String dbTableName) throws PropertyVetoException {
        String olddbTableName = this.dbTableName;

        // first tell the vetoers about the change.  If anyone objects, we
        // let the PropertyVetoException propagate back to our caller.
        vetos.fireVetoableChange("dbTableName",olddbTableName,dbTableName);

        // No one vetoed, so go ahead and make the change
        this.dbTableName   = dbTableName;
        changes.firePropertyChange("dbTableName",olddbTableName,dbTableName);
   }


   /**
   * Accessor (get) method for the <i>dbTableName</i> property.
   *
   * @return The string containing the dbTableName value.
   */
   public String getdbTableName() {
        return dbTableName;
   }


   /**
   * Given a key, get a message from the FlightResourceBundle
   * @param key The key for the message
   * @return The message retrieved from the resource bundle
   */
   private String getMsg(String key)
   {
        if (msgs == null)
          msgs = (FlightResourceBundle)ResourceBundle.getBundle(
            "com.ibm.beans.demos.travel.flight.FlightResourceBundle");

        return (String)msgs.handleGetObject(key);
   }


   /**
   * Retrieve the needed resource strings from the FlightResourceBundle.
   */
   private void getResourceStrings() {


        FIDS_FirstOriginCity                              = getMsg("FIDS_FirstOriginCity");
        FIDS_FirstOriginCity_AirportCode                  = getMsg("FIDS_FirstOriginCity_AirportCode");

        FIDS_FirstDestinationCity                         = getMsg("FIDS_FirstDestinationCity");
        FIDS_FirstDestinationCity_AirportCode             = getMsg("FIDS_FirstDestinationCity_AirportCode");

        FIDS_SecondOriginCity                             = getMsg("FIDS_SecondOriginCity");
        FIDS_SecondOriginCity_AirportCode                 = getMsg("FIDS_SecondOriginCity_AirportCode");

        FIDS_SecondDestinationCity                        = getMsg("FIDS_SecondDestinationCity");
        FIDS_SecondDestinationCity_AirportCode            = getMsg("FIDS_SecondDestinationCity_AirportCode");

        FIDS_Airline1                                     = getMsg("FIDS_Airline1");
        FIDS_Airline2                                     = getMsg("FIDS_Airline2");
        FIDS_Airline3                                     = getMsg("FIDS_Airline3");

        FIDS_FirstRec_FlightNum1                          = getMsg("FIDS_FirstRec_FlightNum1");
        FIDS_FirstRec_WhichClass                          = getMsg("FIDS_FirstRec_WhichClass");
        FIDS_FirstRec_DepartTime                          = getMsg("FIDS_FirstRec_DepartTime");
        FIDS_FirstRec_ArriveTime                          = getMsg("FIDS_FirstRec_ArriveTime");
        FIDS_FirstRec_Price                               = getMsg("FIDS_FirstRec_Price");
        FIDS_FirstRec_SeatNum                             = getMsg("FIDS_FirstRec_SeatNum");
        FIDS_FirstRec_ImageLocation                       = getMsg("FIDS_FirstRec_ImageLocation");
        FIDS_FirstRec_ImageCaption                        = getMsg("FIDS_FirstRec_ImageCaption");

        FIDS_SecondRec_FlightNum1                         = getMsg("FIDS_SecondRec_FlightNum1");
        FIDS_SecondRec_WhichClass                         = getMsg("FIDS_SecondRec_WhichClass");
        FIDS_SecondRec_DepartTime                         = getMsg("FIDS_SecondRec_DepartTime");
        FIDS_SecondRec_ArriveTime                         = getMsg("FIDS_SecondRec_ArriveTime");
        FIDS_SecondRec_Price                              = getMsg("FIDS_SecondRec_Price");
        FIDS_SecondRec_SeatNum                            = getMsg("FIDS_SecondRec_SeatNum");
        FIDS_SecondRec_ImageLocation                      = getMsg("FIDS_SecondRec_ImageLocation");
        FIDS_SecondRec_ImageCaption                       = getMsg("FIDS_SecondRec_ImageCaption");

        FIDS_ThirdRec_FlightNum1                          = getMsg("FIDS_ThirdRec_FlightNum1");
        FIDS_ThirdRec_WhichClass                          = getMsg("FIDS_ThirdRec_WhichClass");
        FIDS_ThirdRec_DepartTime                          = getMsg("FIDS_ThirdRec_DepartTime");
        FIDS_ThirdRec_ArriveTime                          = getMsg("FIDS_ThirdRec_ArriveTime");
        FIDS_ThirdRec_Price                               = getMsg("FIDS_ThirdRec_Price");
        FIDS_ThirdRec_SeatNum                             = getMsg("FIDS_ThirdRec_SeatNum");
        FIDS_ThirdRec_ImageLocation                       = getMsg("FIDS_ThirdRec_ImageLocation");
        FIDS_ThirdRec_ImageCaption                        = getMsg("FIDS_ThirdRec_ImageCaption");

        FIDS_FourthRec_FlightNum1                         = getMsg("FIDS_FourthRec_FlightNum1");
        FIDS_FourthRec_IntermediateDestination            = getMsg("FIDS_FourthRec_IntermediateDestination");
        FIDS_FourthRec_IntermediateDestinationAirportCode = getMsg("FIDS_FourthRec_IntermediateDestinationAirportCode");
        FIDS_FourthRec_WhichClass                         = getMsg("FIDS_FourthRec_WhichClass");
        FIDS_FourthRec_DepartTime1                        = getMsg("FIDS_FourthRec_DepartTime1");
        FIDS_FourthRec_ArriveTime1                        = getMsg("FIDS_FourthRec_ArriveTime1");
        FIDS_FourthRec_Price1                             = getMsg("FIDS_FourthRec_Price1");
        FIDS_FourthRec_SeatNum1                           = getMsg("FIDS_FourthRec_SeatNum1");
        FIDS_FourthRec_FlightNum2                         = getMsg("FIDS_FourthRec_FlightNum2");
        FIDS_FourthRec_DepartTime2                        = getMsg("FIDS_FourthRec_DepartTime2");
        FIDS_FourthRec_ArriveTime2                        = getMsg("FIDS_FourthRec_ArriveTime2");
        FIDS_FourthRec_Price2                             = getMsg("FIDS_FourthRec_Price2");
        FIDS_FourthRec_SeatNum2                           = getMsg("FIDS_FourthRec_SeatNum2");
        FIDS_FourthRec_ImageLocation                      = getMsg("FIDS_FourthRec_ImageLocation");
        FIDS_FourthRec_ImageCaption                       = getMsg("FIDS_FourthRec_ImageCaption");

        FIDS_FifthRec_FlightNum1                          = getMsg("FIDS_FifthRec_FlightNum1");
        FIDS_FifthRec_IntermediateDestination             = getMsg("FIDS_FifthRec_IntermediateDestination");
        FIDS_FifthRec_IntermediateDestinationAirportCode  = getMsg("FIDS_FifthRec_IntermediateDestinationAirportCode");
        FIDS_FifthRec_WhichClass                          = getMsg("FIDS_FifthRec_WhichClass");
        FIDS_FifthRec_DepartTime1                         = getMsg("FIDS_FifthRec_DepartTime1");
        FIDS_FifthRec_ArriveTime1                         = getMsg("FIDS_FifthRec_ArriveTime1");
        FIDS_FifthRec_Price1                              = getMsg("FIDS_FifthRec_Price1");
        FIDS_FifthRec_SeatNum1                            = getMsg("FIDS_FifthRec_SeatNum1");
        FIDS_FifthRec_FlightNum2                          = getMsg("FIDS_FifthRec_FlightNum2");
        FIDS_FifthRec_DepartTime2                         = getMsg("FIDS_FifthRec_DepartTime2");
        FIDS_FifthRec_ArriveTime2                         = getMsg("FIDS_FifthRec_ArriveTime2");
        FIDS_FifthRec_Price2                              = getMsg("FIDS_FifthRec_Price2");
        FIDS_FifthRec_SeatNum2                            = getMsg("FIDS_FifthRec_SeatNum2");
        FIDS_FifthRec_ImageLocation                       = getMsg("FIDS_FifthRec_ImageLocation");
        FIDS_FifthRec_ImageCaption                        = getMsg("FIDS_FifthRec_ImageCaption");

        FIDS_SixthRec_FlightNum1                          = getMsg("FIDS_SixthRec_FlightNum1");
        FIDS_SixthRec_WhichClass                          = getMsg("FIDS_SixthRec_WhichClass");
        FIDS_SixthRec_DepartTime                          = getMsg("FIDS_SixthRec_DepartTime");
        FIDS_SixthRec_ArriveTime                          = getMsg("FIDS_SixthRec_ArriveTime");
        FIDS_SixthRec_Price                               = getMsg("FIDS_SixthRec_Price");
        FIDS_SixthRec_SeatNum                             = getMsg("FIDS_SixthRec_SeatNum");
        FIDS_SixthRec_ImageLocation                       = getMsg("FIDS_SixthRec_ImageLocation");
        FIDS_SixthRec_ImageCaption                        = getMsg("FIDS_SixthRec_ImageCaption");
   }


   // set up JDBC beans
   private static Connection connection;
   private static Select select;
   private Boolean bUseJDBC;
   private String JDBCURL;
   private String dbTableName;

   // strings from the FlightResourceBundle
   private String FIDS_FirstOriginCity;
   private String FIDS_FirstOriginCity_AirportCode;
   private String FIDS_FirstDestinationCity;
   private String FIDS_FirstDestinationCity_AirportCode;
   private String FIDS_SecondOriginCity;
   private String FIDS_SecondOriginCity_AirportCode;
   private String FIDS_SecondDestinationCity;
   private String FIDS_SecondDestinationCity_AirportCode;
   private String FIDS_Airline1;
   private String FIDS_Airline2;
   private String FIDS_Airline3;
   private String FIDS_FirstRec_FlightNum1;
   private String FIDS_FirstRec_WhichClass;
   private String FIDS_FirstRec_DepartTime;
   private String FIDS_FirstRec_ArriveTime;
   private String FIDS_FirstRec_Price;
   private String FIDS_FirstRec_SeatNum;
   private String FIDS_FirstRec_ImageLocation;
   private String FIDS_FirstRec_ImageCaption;
   private String FIDS_SecondRec_FlightNum1;
   private String FIDS_SecondRec_WhichClass;
   private String FIDS_SecondRec_DepartTime;
   private String FIDS_SecondRec_ArriveTime;
   private String FIDS_SecondRec_Price;
   private String FIDS_SecondRec_SeatNum;
   private String FIDS_SecondRec_ImageLocation;
   private String FIDS_SecondRec_ImageCaption;
   private String FIDS_ThirdRec_FlightNum1;
   private String FIDS_ThirdRec_WhichClass;
   private String FIDS_ThirdRec_DepartTime;
   private String FIDS_ThirdRec_ArriveTime;
   private String FIDS_ThirdRec_Price;
   private String FIDS_ThirdRec_SeatNum;
   private String FIDS_ThirdRec_ImageLocation;
   private String FIDS_ThirdRec_ImageCaption;
   private String FIDS_FourthRec_FlightNum1;
   private String FIDS_FourthRec_IntermediateDestination;
   private String FIDS_FourthRec_IntermediateDestinationAirportCode;
   private String FIDS_FourthRec_WhichClass;
   private String FIDS_FourthRec_DepartTime1;
   private String FIDS_FourthRec_ArriveTime1;
   private String FIDS_FourthRec_Price1;
   private String FIDS_FourthRec_SeatNum1;
   private String FIDS_FourthRec_FlightNum2;
   private String FIDS_FourthRec_DepartTime2;
   private String FIDS_FourthRec_ArriveTime2;
   private String FIDS_FourthRec_Price2;
   private String FIDS_FourthRec_SeatNum2;
   private String FIDS_FourthRec_ImageLocation;
   private String FIDS_FourthRec_ImageCaption;
   private String FIDS_FifthRec_FlightNum1;
   private String FIDS_FifthRec_IntermediateDestination;
   private String FIDS_FifthRec_IntermediateDestinationAirportCode;
   private String FIDS_FifthRec_WhichClass;
   private String FIDS_FifthRec_DepartTime1;
   private String FIDS_FifthRec_ArriveTime1;
   private String FIDS_FifthRec_Price1;
   private String FIDS_FifthRec_SeatNum1;
   private String FIDS_FifthRec_FlightNum2;
   private String FIDS_FifthRec_DepartTime2;
   private String FIDS_FifthRec_ArriveTime2;
   private String FIDS_FifthRec_Price2;
   private String FIDS_FifthRec_SeatNum2;
   private String FIDS_FifthRec_ImageLocation;
   private String FIDS_FifthRec_ImageCaption;
   private String FIDS_SixthRec_FlightNum1;
   private String FIDS_SixthRec_WhichClass;
   private String FIDS_SixthRec_DepartTime;
   private String FIDS_SixthRec_ArriveTime;
   private String FIDS_SixthRec_Price;
   private String FIDS_SixthRec_SeatNum;
   private String FIDS_SixthRec_ImageLocation;
   private String FIDS_SixthRec_ImageCaption;


   private transient Vector flightInfo;
                                                                     // contains the information queried
                                                                     // from the database (and possibly
                                                                     // changed by the business rules DIP).
                                                                     // This is the information sent to
                                                                     // the flight information viewer.

   private String SQLString;                                         // the SQL string used to access the
                                                                     // database

   private VetoableChangeSupport vetos = null;                       // helper class for handling property updates
   private PropertyChangeSupport changes = null;                     // helper class for handling property updates
   private transient LogStream errorLog;
   private MessageFormatter mf;

   private transient FlightResourceBundle   msgs                      = null;

   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
   private static final long serialVersionUID = 2000L;  // serial version unique ID
}

