//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbTravelDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
package com.ibm.beans.demos.travel.flight;

import java.awt.Checkbox;
import java.awt.Dimension;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.TextListener;
import java.awt.event.TextEvent;

import java.beans.Customizer;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ResourceBundle;

import com.ibm.beans.demos.travel.flight.FlightResourceBundle;
import com.ibm.beans.widgets.VerticalHorizontalLayout;

/**
 * This class defines a FlightInfoDapServerCustomizer instance that is the
 * customizer for an instance of the FlightInfoDapServer class.  The
 * customizer is used to decide whether or not the database should be accessed
 * using JDBC.  The following properties can be modified: the <i>JDBCURL</i> property,
 * the <i>dbTableName</i> property, and the <i>bUseJDBC</i> property.  If these
 * properties are modified their new values will be reflected in the
 * FlightInfoDapServer instance.
 */
public class FlightInfoDapServerCustomizer extends Panel implements Customizer, ItemListener, TextListener {

   /**
    * Constructs a FlightInfoDapServerCustomizer instance and creates
    * controls used in customizing the properties of the FlightInfoDapServer.
    */
   public FlightInfoDapServerCustomizer() {

      support = new PropertyChangeSupport(this);

      setLayout(new VerticalHorizontalLayout(VerticalHorizontalLayout.VERTICAL,10,
                                             VerticalHorizontalLayout.UPPERLEFT));

      useJDBCStr           = getMsg("useJDBC_checkbox");           // string for checkbox label

      JDBCURLStr           = getMsg("JDBCURL_label");              // string for text field label
      dataBaseTableNameStr = getMsg("DatabaseTableName_label");    // string for text field label

      useJDBC              = new Checkbox(useJDBCStr);             // create and add checkbox and label
      add(useJDBC);

      Label label1         = new Label(JDBCURLStr);                // create and add "JDBC URL:" label
      add(label1);

      JDBCURL              = new TextField(30);                    // create and add JDBC URL text field
      add(JDBCURL);

      Label label2         = new Label(dataBaseTableNameStr);      // create and add "Database table name:" label
      add(label2);

      dbTableName          = new TextField(30);                    // create and add JDBC Database table name text field
      add(dbTableName);

      useJDBC.addItemListener(this);                               // set up listeners for controls
      JDBCURL.addTextListener(this);
      dbTableName.addTextListener(this);

   }


   // Implementation of ItemListener interface
   /**
    * Handles item events on the FlightInfoDapServer Customizer.
    * The <i>bUseJDBC</i> property of the FlightInfoDapServer
    * may be updated.
    * <p>
    * @param e This parameter indicates the item event that occurred.
    */
   public void itemStateChanged(ItemEvent e) {
     String aCmd = e.getItem().toString();
     if (aCmd.equals(useJDBCStr)) {
      try {
        if (useJDBC.getState()) {

          // User has selected to use JDBC to access the database.
          // Notify the FlightInfoDapServer and enable the
          // customizer text fields.
          target.setbUseJDBC(new Boolean(true));
          JDBCURL.setEnabled(true);
          dbTableName.setEnabled(true);

        } else {

          // User has selected to not use JDBC to access the database.
          // Notify the FlightInfoDapServer and disable the
          // customizer text fields.
          target.setbUseJDBC(new Boolean(false));
          JDBCURL.setEnabled(false);
          dbTableName.setEnabled(false);

        }
      } catch (Exception ex) {
        ex.printStackTrace();
      }
     }
   }


   // Implementation of TextListener interface
   /**
    * Receives keystrokes from the text fields.
    * The <i>JDBCURL</i> property and/or the <i>dbTableName</i>
    * property of the FlightInfoDapServer may be updated.
    * <p>
    * @param evt Describes the event that occurred.
    */
   public void textValueChanged(TextEvent evt) {
     try {
       if (evt.getSource()==JDBCURL) {

         // JDBCURL value changed
         String newURL = JDBCURL.getText();
         String oldURL = target.getJDBCURL();
         target.setJDBCURL(newURL);
         support.firePropertyChange("JDBCURL", oldURL, newURL);

       } else if (evt.getSource()==dbTableName) {

         // dbTableName value changed
         String newTableName = dbTableName.getText();
         String oldTableName = target.getdbTableName();
         target.setdbTableName(newTableName);
         support.firePropertyChange("dbTableName",oldTableName,newTableName);

       }
     } catch (Exception ex) {
       ex.printStackTrace();
     }
   }

   /**
    * Sets the object instance with which this customizer interacts.
    * Controls are initialized to contain the current value of
    * properties being customized.
    *
    * @param obj A FlightInfoDapServer instance to be customized.
    */
   public void setObject(Object obj) {
      if ((obj!=null) && (obj instanceof FlightInfoDapServer)) {
         target = (FlightInfoDapServer) obj;
      } else {
         return; // nothing we can do - caller isn't expecting any exceptions
      } /* endif */

      // get current values of properties being customized
      JDBCURL.setText(target.getJDBCURL());
      dbTableName.setText(target.getdbTableName());
      useJDBC.setState( (target.getbUseJDBC()).booleanValue() );

      // if checkbox is not checked, disable the entry fields
      if ( !(useJDBC.getState()) ) {
         JDBCURL.setEnabled(false);
         dbTableName.setEnabled(false);
      }
   }


   /**
    * Returns the preferred size for this customizer panel.
    * @return The dimensions of the customizer.
    */
   public Dimension getPreferredSize() {
      return new Dimension(400, 100);
   }


   /**
    * Adds a listener to the property-change notification list.
    * @param l The listener to add.
    */
   public void addPropertyChangeListener(PropertyChangeListener l) {
      support.addPropertyChangeListener(l);
   }


   /**
    * Removes a listener from the property-change notification list.
    * @param l The listener to remove.
    */
   public void removePropertyChangeListener(PropertyChangeListener l) {
      support.removePropertyChangeListener(l);
   }

   // Given a key, get a message from the TravelResourceBundle
   String getMsg(String key)
   {
     if (msgs == null)
       msgs = (FlightResourceBundle)ResourceBundle.getBundle(
         "com.ibm.beans.demos.travel.flight.FlightResourceBundle");

     return (String)msgs.handleGetObject(key);
   }


   // awt controls
   private Checkbox               useJDBC;
   private TextField              dbTableName;
   private TextField              JDBCURL;

   // the FlightInfoDapServer object instance with which this customizer interacts
   private FlightInfoDapServer    target;

   // helper class for handling property updates via the customizer
   private PropertyChangeSupport  support                   = null;

   // strings for the controls
   private String                 dataBaseTableNameStr      = null;
   private String                 JDBCURLStr                = null;
   private String                 useJDBCStr                = null;

   // the resource bundle
   private FlightResourceBundle   msgs                      = null;

   // copyright string
   private final static String    kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
}

