//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbTravelDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.22 com/java/src/com/ibm/beans/demos/travel/flight/FlightInfoViewer.java, bbTravelDemo, bb97c, bb97c9750e 11/19/97 14:09:54 [12/11/97 17:03:10]

package com.ibm.beans.demos.travel.flight;

import java.awt.Panel;
import java.awt.ItemSelectable;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Rectangle;

import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.ActionEvent;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.io.IOException;

import java.lang.ClassNotFoundException;

import java.util.ResourceBundle;
import java.util.Vector;

import COM.taligent.widget.MultiColumnListbox;
import COM.taligent.widget.ListboxColumn;

import COM.taligent.widget.event.ListboxListener;
import COM.taligent.widget.event.ListboxEvent;

/**
 * <p>
 * This bean is a simple viewer for Airline Flight Information.  It is
 * one of the beans that make up the complex composite flight selector
 * bean.  Through republishing, the flight selector bean receives
 * FlightInfoEvent's sent from the FlightInfoDap that are then forwarded
 * to the flight info viewer which displays information representing
 * available flights from a Vector of FlightInformation objects stored
 * in the FlightInfoEvent object.  A script ties the flight info viewer bean
 * to other beans that make up the composite flight selector bean so that each
 * time the user clicks on a flight, information about the flight is sent to the
 * image viewer bean and the text area bean.  When the user has decided on a particular
 * flight, he or she can then click on the <b>Ok</b> button, which causes the viewer
 * to fire a FlightRequestedEvent to the itinerary bean.
 *
 *
 * @author John W. Alcorn (jalcorn@us.ibm.com)
 * @see com.ibm.beans.demos.travel.flight.FlightInfoDap
 * @see com.ibm.beans.demos.travel.flight.FlightInfoEvent
 * @see com.ibm.beans.demos.travel.flight.FlightInformation
 * @see com.ibm.beans.demos.travel.flight.FlightRequestedEvent
 */
public class FlightInfoViewer extends Panel implements FlightInfoListener, Serializable,
                                               ActionListener, ListboxListener, ItemSelectable {

        private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
        private static final long serialVersionUID = 2000L; // serial version unique ID

        private transient FlightResourceBundle   msgs = null;

        private transient MultiColumnListbox listBox;       //displays the available flights
        private transient FlightInformation selectedFlight; //the currently selected flight
        private transient Vector availableFlights;          //Vector of FlightInformation objects
        private transient Vector flightRequestedListeners;  //recipients of FlightRequestedEvents
        private transient Vector itemListeners;             //recipients of ItemEvents

        /**
         * Constructs an instance of the FlightInfoViewer bean.  The constructor
         * initializes instance data and creates and embeds the MultiColumnListbox
         * control, which is used to convey the Airline Flight Information to the
         * user.  The MultiColumnListbox is initially empty; it will be filled in
         * when the FlightInformationChanged() method is invoked.
         *
         * @see #createListBox
         */
        public FlightInfoViewer() {
                super(null);
                setLayout(null);

                selectedFlight = null;
                availableFlights = new Vector();
                flightRequestedListeners = new Vector();
                itemListeners = new Vector();

                createListBox();
        }

        /**
         * This method is called whenever the FlightInfoViewer bean is instantiated from
         * serialized data (such as a .ser file).  It is overridden so that transient
         * data (that is, data not serialized) can be initialized (then it just calls
         * defaultReadObject on the ObjectInputStream to let it take care of initializing
         * all of the non-transient data).
         *
         * @see #createListBox
         */
        private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
                // Initialize transient data
                selectedFlight = null;
                availableFlights = new Vector();
                flightRequestedListeners = new Vector();
                itemListeners = new Vector();

                in.defaultReadObject();
                removeAll(); //jwa: Workaround for serialization bug with MultiColumnListbox

                createListBox();
        }

        /**
         * Creates, initializes, and embeds the MultiColumnListbox.
         */
        public void createListBox() {

                listBox = new MultiColumnListbox();

                listBox.setName(getMsg("FIV_Name"));
                listBox.addColumn(getMsg("FIV_Airline"));

                ListboxColumn column = listBox.getColumnInfo(0);
                column.setWidth(70); //see comment below re: autoSizeColumn()

                listBox.addColumn(getMsg("FIV_Flight#"));
                listBox.addColumn(getMsg("FIV_Depart"));
                listBox.addColumn(getMsg("FIV_Arrive"));
                listBox.addColumn(getMsg("FIV_Class"));

                column = listBox.getColumnInfo(4);
                column.setWidth(60);

                listBox.addColumn(getMsg("FIV_TakeOff"));

                column = listBox.getColumnInfo(5);
                column.setWidth(55);

                listBox.addColumn(getMsg("FIV_Land"));
                listBox.addColumn(getMsg("FIV_Price"));

//              Following line causes ClassNotFoundException???
//              for (int index=0; index<8; index++) listBox.autoSizeColumn(index);

                listBox.setHorizontalSeparatorVisible(true);
                listBox.setVerticalSeparatorVisible(true);

//    Horizontal scrollbar thinks listbox is wider than it is???
//              listBox.setHorizontalScrollbarVisible(true);

                listBox.setBackground(new Color(204, 255, 255)); //light cyan
                listBox.setCaptionBarBackground(new Color(255, 255, 204)); //cream

                listBox.setMultipleSelections(true); //so multi-line entries can have all lines highlighted

                listBox.addListboxListener(this);
                add(listBox);
        }

        /**
         * Receives Flight Information data from the FlightInfoDap.  The Vector of
         * FlightInformation objects is retrieved from the FlightInfoEvent, and
         * passed to the setAvailableFlights() method.
         *
         * @param flightInfoEvent A FlightInfoEvent object describing the event that occurred.
         * @see #setAvailableFlights
         * @see com.ibm.beans.demos.travel.flight.FlightInfoEvent
         */
        public void FlightInformationChanged ( FlightInfoEvent flightInfoEvent ) {
                setAvailableFlights(flightInfoEvent.getFlightInfo());
        }

        /**
         * Responds to the user selecting a flight from the list of available
         * flights, by firing a new ItemEvent to each of the registered listeners.
         *
         * @param listboxEvent An ListboxEvent object describing the event that occurred.
         * @see #fireItem
         */
        public void rowSelected(ListboxEvent listboxEvent) {
                listBox.deselectAllRows(); //have to clear old selection, since we turned on multiple selection

                selectedFlight = (FlightInformation) listBox.getRowKey(listboxEvent.getRow());

                if (selectedFlight!=null) { //select the row(s) corresponding to selectedFlight
                        int firstRowForKey = listBox.getIndexByKey(selectedFlight);
                        listBox.selectRow(firstRowForKey);
                        if (selectedFlight.getbHasConnection()) { //select other leg of connecting flight
                                listBox.selectRow(firstRowForKey+1);
                        } /* endif */
                        fireItem();
                } /* endif */

        }

        /**
         * Responds to the user deselecting a flight from the list of available
         * flights, by firing a new ItemEvent to each of the registered listeners.
         *
         * @param listboxEvent An ListboxEvent object describing the event that occurred.
         * @see #fireItem
         */
        public void rowDeselected(ListboxEvent listboxEvent) {
                listBox.deselectAllRows(); //have to clear old selection, since we turned on multiple selection
                selectedFlight = null;

                fireItem();
        }

        /**
         * Notifies all registered listeners that a flight has been selected.  In
         * the demo scenario, this event fires in response to the user clicking on
         * one of the items in the MultiColumnListbox, and the ImageViewer bean and
        * TextArea bean are updated using data from the FlightInformation object contained in the
         * ItemEvent by means of a script attached to this method.
         *
         * @see #rowSelected
         * @see #rowDeselected
         */
        public void fireItem() {
                ItemEvent item = new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED, selectedFlight,
                        selectedFlight==null ? ItemEvent.DESELECTED : ItemEvent.SELECTED);
                for (int index = 0; index<itemListeners.size(); index++)
                        ((ItemListener) itemListeners.elementAt(index)).itemStateChanged(item);
        }

        /**
         * Registers a listener for an ItemEvent.
         *
         * @param itemListener The registering ItemListener.
         * @see #removeItemListener
         */
        public synchronized void addItemListener(ItemListener itemListener) {
                itemListeners.addElement(itemListener);
        }

        /**
         * Deregisters a listener for an ItemEvent.
         *
         * @param itemListener The deregistering ItemListener.
         * @see #addItemListener
         */
        public synchronized void removeItemListener(ItemListener itemListener) {
                itemListeners.removeElement(itemListener);
        }

        /**
         * Returns an array of FlightInformation objects corresponding to the
         * currently selected flight.  In the demo, since the MultiColumnListbox
         * containing the flight information is set to single-select mode, this
         * will always be either <tt>null</tt> or an array with one element.  This
         * method is required by the <tt>ItemSelectable</tt> interface; in general,
         * it is probably easier to use the getSelectedFlight() method, which
         * avoids the need to have to index into an array).
         *
         * @return An array of FlightInformation objects describing the selected flight.
         * @see #getSelectedFlight
         */
        public Object[] getSelectedObjects() {
                Object[] selectedObjects = null;
                selectedObjects[0] = selectedFlight;
                return selectedObjects;
        }

        /**
         * Returns the FlightInformation object corresponding to the
         * currently selected flight.  If multiple selections were enabled
         * for the MultiColumnListbox, the getSelectedObjects() method
         * would have to be used instead of this method.
         *
         * @return A FlightInformation object describing the selected flight.
         * @see #getSelectedObjects
         */
        public FlightInformation getSelectedFlight() {
                return selectedFlight;
        }

        /**
         * Returns the Vector of FlightInformation objects corresponding to the
         * list of available flights.
         *
         * @return A Vector of FlightInformation objects corresponding to the available flights.
         * @see #setAvailableFlights
         * @see com.ibm.beans.demos.travel.flight.FlightInformation
         */
        public Vector getAvailableFlights() {
                return availableFlights;
        }

        /**
         * Updates Flight Information data.  In the demo scenario, this information
         * comes from a FlightInfoEvent fired by the FlightInfoDap.  Any currently
         * displayed flight information is cleared, and a new list of flight
         * information is presented.
         *
         * @param newAvailableFlights A Vector of FlightInformation objects.
         * @see #FlightInformationChanged
         * @see #getAvailableFlights
         * @see com.ibm.beans.demos.travel.flight.FlightInformation
         */
        synchronized public void setAvailableFlights(Vector newAvailableFlights) {
                availableFlights = newAvailableFlights;

                listBox.removeAllRows(); //clear any old flight info from the list box

                String blankRow[] = {""};
                for (int index=0; index<availableFlights.size(); index++) {
                        if (index!=0) listBox.addRow(blankRow, null);

                        FlightInformation info = (FlightInformation) availableFlights.elementAt(index);

                        String row[] = {info.getAirline(), info.getFlightNum(), info.getAirportCodeStartCity(),
                                info.getAirportCodeEndCity(), info.getWhichClass(), info.getDepartTime(),
                                info.getArriveTime(),
                                java.text.NumberFormat.getCurrencyInstance().
                                        format(info.getPrice())}; // lea, 7/16

                        listBox.addRow(row, info);

                        if (info.getbHasConnection()) {
                                String row2[] = {info.getAirline2(), info.getFlightNum2(), info.getAirportCodeEndCity(),
                                        info.getAirportCodeEndCity2(), info.getWhichClass2(), info.getDepartTime2(),
                                        info.getArriveTime2(),
                                        java.text.NumberFormat. // lea, 7/16
                                                getCurrencyInstance().
                                                   format(info.getPrice2())};

                                listBox.addRow(row2, info);
                        } /* endif */
                } /* endfor */

//              listBox.selectRow(0); //cause it to come it with first item preselected
//              Above line doesn't cause rowSelected to be called
                ListboxEvent evt = new ListboxEvent(listBox, ListboxEvent.SELECTED, 0, 0, 0, 0);
                rowSelected(evt);
        }

        /**
         * Responds to the user clicking on the <b>Ok</b> button, by
         * firing a FlightRequested event to all registered listeners.
         *
         * @param actionEvent An ActionEvent object describing the event that occurred.
         * @see #fireFlightRequested
         */
        public void actionPerformed(ActionEvent actionEvent) {
//              if (actionEvent.getActionCommand()=="Ticket It!")
                        fireFlightRequested();
        }

        /**
         * Notifies all registered listeners that a flight has been requested.
         * In the demo scenario, this event fires in response to receiving
         * an actionPerformed event from the <b>Ok</b> button, and the
         * itinerary bean is the only registered listener.
         *
         * @see #actionPerformed
         * @see com.ibm.beans.demos.travel.flight.FlightInfoDap#FlightRequested
         */
        public void fireFlightRequested() {
                if (selectedFlight != null) {
                        Vector flightInfo = new Vector();
                        flightInfo.addElement(selectedFlight);
                        FlightRequestedEvent fRE = new FlightRequestedEvent(this, flightInfo);
                        for (int index = 0; index<flightRequestedListeners.size(); index++)
                                ((FlightRequestedListener) flightRequestedListeners.
                                        elementAt(index)).FlightRequested(fRE);
                } /* endif */
        }

        /**
         * Registers a listener for the FlightRequestedEvent.
         *
         * @param fRL The registering FlightRequestedListener.
         * @see #removeFlightRequestedListener
         */
        public synchronized void addFlightRequestedListener(FlightRequestedListener fRL) {
                flightRequestedListeners.addElement(fRL);
        }

        /**
         * Deregisters a listener for the FlightRequestedEvent.
         *
         * @param fRL The deregistering FlightRequestedListener.
         * @see #addFlightRequestedListener
         */
        public synchronized void removeFlightRequestedListener(FlightRequestedListener fRL) {
                flightRequestedListeners.removeElement(fRL);
        }

        /**
         * Returns the preferredSize property of this bean.  This is used to
         * determine the initial size of the viewer in the Layout view).
         *
         * @return A Dimension object containing the preferred width and height.
         */
        public Dimension getPreferredSize() { return new Dimension(450, 200); }

        /**
         * Sizes and positions the MultiColumnListbox within the viewer based on
         * the current size of the viewer.
         */
        public void doLayout() {
                Rectangle rect = getBounds();
                listBox.setBounds(0, 0, rect.width-1, rect.height-1);
        }

        /**
         * Retrieves a reference to the embedded MultiColumnListbox control.
         *
         * @return A reference to the embedded listBox.
         */
        public MultiColumnListbox getListBox() { return listBox; }

        /**
         * Given a key, get a message from the FlightResourceBundle
         * @param key The key for the message
         * @return The message retrieved from the resource bundle
         */
        private String getMsg(String key)
        {
           if (msgs == null)
             msgs = (FlightResourceBundle)ResourceBundle.getBundle(
                    "com.ibm.beans.demos.travel.flight.FlightResourceBundle");

           return (String)msgs.handleGetObject(key);
        }

}

