//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbTravelDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.14 com/java/src/com/ibm/beans/demos/travel/flight/rules/CarrierRestrictionDip.java, bbTravelDemo, bb97c, bb97c9750e 11/18/97 15:35:32 [12/11/97 17:02:58]

package com.ibm.beans.demos.travel.flight.rules;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeSupport;
import java.lang.String;
import java.io.Serializable;
import java.util.ResourceBundle;
import com.ibm.beans.dip.Dip;
import com.ibm.beans.dip.SimpleDip;
import com.ibm.beans.dip.DipPropertyChangeListener;
import com.ibm.beans.dip.DipRejectAdditionException;
import com.ibm.beans.demos.travel.flight.FlightInfoDapProperties;

/**
 * This class implements a business rule which restricts the flight
 * carriers (airlines) accessible via the FlightInfoDapProperties.  It does this by
 * hooking to the sql property and adding an SQL query clause which
 * eliminates the particular carrier.  The carrier to restrict is a
 * property of this dip.
 * <p>
 * See the Carrier Restriction Dip Usage Guide for information regarding how to
 * setup and use this dip.
 */
public class CarrierRestrictionDip extends SimpleDip implements DipPropertyChangeListener, Serializable {

  public CarrierRestrictionDip() {
    support = new PropertyChangeSupport(this);
  }


  // docs say this should be private!?!?
  private void readObject(java.io.ObjectInputStream in) throws
        java.io.IOException, java.lang.ClassNotFoundException {

          // fix up transients
          support = new PropertyChangeSupport(this);

          // now read in the persistent data
          in.defaultReadObject();
  }

  /**
   * Sets the name of the airline to restrict usage of.
   */
  public void setCarrier(String carrierName) {
    String old = carrier;
    carrier = carrierName;

    // fire off bound property notification
    support.firePropertyChange("Carrier", old, carrierName);
  }

  /**
   * Gets the name of the airline to restrict usage of.
   */
  public String getCarrier() {
    return carrier;
  }

  // overrides from SimpleDip

  /**
   * Defines this dip to be related to restricting use of an
   * airline carrier.
   *
   * @return The type name of the dip.
   */
  final public String typeName() { return kDipTypeName; }

  /**
   * Determines if a method should be included in a morphed class.
   * When the dip is applied to a class as a pre-dip during morphing,
   * and the user elects to only override those methods in the dippable
   * class that the pre-dips agrees to, this method is used to determine
   * if a dip agrees to including a specific method.
   * <p>
   * The only method of the dippable bean that the CarrierRestrictionDip
   * needs to be wrapped is the setSQLString method.
   *
   * @param methodName Name of the method to be considered
   * @return boolean indicating whether to wrap given method
   */
  public boolean includeInMorphedClass(String methodName) {
     boolean includeThisMethod = false;

     if (methodName.endsWith("setSQLString(java.lang.String) throws java.beans.PropertyVetoException")) {
        includeThisMethod = true;
     } /* endif */

     return includeThisMethod;
  }

  /**
   * Checks the object to which the dip is being applied to
   * ensure that the object is a FlightInfoDapProperties.  If it is
   * not, then an exception is thrown and the application of the dip
   * is rejected.
   * <p>
   * @param obj The bean being dipped.
   * @exception com.ibm.beans.dip.DipRejectAdditionException This method will throw
   *            this exception if
   *            <i>obj</i> is not a FlightInfoDapProperties.
   */
  public Object createImplementation(Object obj) throws DipRejectAdditionException {

    // this dip can only be applied to FlightInfoDapProperties
    if (obj instanceof FlightInfoDapProperties) {
      // save for later query
      DAP = (FlightInfoDapProperties)obj;

      return null;
    }
    else {
      // error reporting done by dipping fw
      getMessages();
      throw new DipRejectAdditionException((String)messages.handleGetObject("not_DAP"));
    }
  }

  /**
   * Creates the object to receive notification when the SQL property changes.
   *
   * @param imp The implementation returned by createImplementation.
   * @param methods The array of methods in the dippable
   *            bean.
   * @return The array of DipPropertyChangeListeners with the dip itself set to
   *            listen at the appropriate method index to receive
   *            property change notifications.
   * @exception com.ibm.beans.dip.DipRejectAdditionException This method will throw this
   *            exception if some needed methods of the dippable bean were not wrapped.
   */
  public DipPropertyChangeListener[] createPropertyChangeListener(Object imp, String[] methods) throws DipRejectAdditionException {

    // call SimpleDip first to ensure we have properly created the
    // listener array
    super.createPropertyChangeListener(imp, methods);

    int count = 0;
    // set myself as the listener to the SQLString property
    for (int i = 0; i < propertyChangeListeners.length; i++) {
      if ( methods[i].endsWith("setSQLString(java.lang.String) throws java.beans.PropertyVetoException") ) {
        propertyChangeListeners[i] = this;
        count++;
      }
    }

    if (count!=1) {
      // error reporting done by dipping fw
      getMessages();
      throw new DipRejectAdditionException((String)messages.handleGetObject("not_wrapped"));
    }

    return propertyChangeListeners;
  }

  // DipPropertyChangeListener Interface

  /**
   * Handles a property change event.  The Carrier Restriction Dip
   * will enforce appropriate SQL clauses to remove the designated
   * carrier from the list of carriers possible.
   */
  public void propertyChange(PropertyChangeEvent e) {

    // we only care about the SQLString property
    if (e.getPropertyName() == "SQLString") {
      // clause we want
      //String sqlClause = kSQLClausePrefix + "'" + getCarrier() + "'";
      String sqlClause = kSQLClausePrefix + getCarrier();

      // if the clause is NOT already there
      if (DAP.getSQLString().indexOf(sqlClause) == -1) {
        // append the clause
        try {
          DAP.setSQLString(DAP.getSQLString() + " AND " + sqlClause);
        }
        catch (java.beans.PropertyVetoException ex) {
          // we've done our best.  If someone vetoes the change, there's
          // nothing we can do about it.
        }
      }
    }
  }

  // private resource handler
  private void getMessages() {
    if (messages == null) {
      messages = (Messages)ResourceBundle.getBundle("com.ibm.beans.demos.travel.flight.rules.Messages");
    }
  }

  private static final long serialVersionUID = 2000L;
  private static String kDipTypeName = "Airline Carrier Restriction";
  private static String kSQLClausePrefix = "AIRLINE <> ";

  private transient Messages messages;
  private transient FlightInfoDapProperties DAP = null; // rehooked at deserialization
  private transient PropertyChangeSupport support;

  private String carrier = "";

  private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
}

