//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbTravelDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.12 com/java/src/com/ibm/beans/demos/travel/flight/rules/ConnectingFlightDip.java, bbTravelDemo, bb97c, bb97c9750e 11/18/97 15:33:55 [12/11/97 17:02:54]

package com.ibm.beans.demos.travel.flight.rules;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeSupport;
import java.lang.String;
import java.io.Serializable;
import java.util.ResourceBundle;
import java.util.Vector;
import com.ibm.beans.dip.Dip;
import com.ibm.beans.dip.SimpleDip;
import com.ibm.beans.dip.DipPropertyChangeListener;
import com.ibm.beans.dip.DipRejectAdditionException;
import com.ibm.beans.demos.travel.flight.FlightInfoDapProperties;
import com.ibm.beans.demos.travel.flight.FlightInformation;

/**
 * This class implements a business rule which restricts the use of
 * direct flights returned by the FlightInfoDapProperties when the direct flight
 * cost above some threshold over a connecting flight.  It does this by
 * hooking to the flight info property and filtering the results provided
 * by the FlightInfoDapProperties.  The threshold amount is a settable property.
 */
public class ConnectingFlightDip extends SimpleDip implements DipPropertyChangeListener, Serializable {

  public ConnectingFlightDip() {
    support = new PropertyChangeSupport(this);
  }


  // docs say this should be private!?!?
  private void readObject(java.io.ObjectInputStream in) throws
        java.io.IOException, java.lang.ClassNotFoundException {

          // fix up transients
          support = new PropertyChangeSupport(this);

          // now read in the persistent data
          in.defaultReadObject();
  }

  /**
   * Sets the dollar amount threshold before connecting flights
   * are chosen over direct flights.  For instance, if a dollar
   * amount of $100 is set, any direct flights which costs more
   * than $100 than a connecting flight will not be allowed.
   */
  public void setPriceDifferential(double dollarAmount) {
    double old = priceDifferential;
    priceDifferential = dollarAmount;

    // fire off bound property notification
    support.firePropertyChange("priceDifferntial", new Double(old), new Double(priceDifferential));
  }

  /**
   * Gets the name price differential for allowing
   * direct flights.
   */
  public double getPriceDifferential() {
    return priceDifferential;
  }

  // overrides from SimpleDip

  /**
   * Defines this dip to be related to restricting use of
   * direct flights.
   *
   * @return The type name of the dip.
   */
  final public String typeName() { return kDipTypeName; }

  /**
   * Determines if a method should be included in a morphed class.
   * When the dip is applied to a class as a pre-dip during morphing,
   * and the user elects to only override those methods in the dippable
   * class that the pre-dips agrees to, this method is used to determine
   * if a dip agrees to including a specific method.
   * <p>
   * The only method of the dippable bean that the ConnectingFlihgtDip
   * needs to be wrapped is the setFlightInfo method.
   *
   * @param methodName Name of the method to be considered
   * @return boolean indicating whether to wrap given method
   */
  public boolean includeInMorphedClass(String methodName) {
     boolean includeThisMethod = false;

     if (methodName.endsWith("setFlightInfo(java.util.Vector) throws java.beans.PropertyVetoException")) {
        includeThisMethod = true;
     } /* endif */

     return includeThisMethod;
  }

  /**
   * Checks the object to which the dip is being applied to
   * ensure that the object is a FlightInfoDapProperties.  If it is
   * not, then an exception is thrown and the application of the dip
   * is rejected.
   * <p>
   * @param obj The bean being dipped.
   * @exception com.ibm.beans.dip.DipRejectAdditionException This method will throw
   *            this exception if
   *            <i>obj</i> is not a FlightInfoDapProperties.
   */
  public Object createImplementation(Object obj) throws DipRejectAdditionException {

    // this dip can only be applied to FlightInfoDapProperties
    if (obj instanceof FlightInfoDapProperties) {
      // save for later query
      DAP = (FlightInfoDapProperties)obj;

      return null;
    }
    else {
      // error reporting done by dipping fw
      getMessages();
      throw new DipRejectAdditionException((String)messages.handleGetObject("not_DAP"));
    }
  }

  /**
   * Creates the object to receive notification when the flight info property changes.
   *
   * @param imp The implementation returned by createImplementation.
   * @param methods The array of methods in the dippable
   *            bean.
   * @return The array of DipPropertyChangeListeners with the dip itself set to
   *            listen at the appropriate method index to receive
   *            property change notifications.
   * @exception com.ibm.beans.dip.DipRejectAdditionException This method will throw this
   *            exception if some needed methods of the dippable bean were not wrapped.
   */
  public DipPropertyChangeListener[] createPropertyChangeListener(Object imp, String[] methods) throws DipRejectAdditionException {

    // call SimpleDip first to ensure we have properly created the
    // listener array
    super.createPropertyChangeListener(imp, methods);

    int count = 0;
    // set myself as the listener to the flightInfo property
    for (int i = 0; i < propertyChangeListeners.length; i++) {
      if ( methods[i].endsWith("setFlightInfo(java.util.Vector) throws java.beans.PropertyVetoException") ) {
        propertyChangeListeners[i] = this;
        count++;
      }
    }

    if (count!=1) {
      // error reporting done by dipping fw
      getMessages();
      throw new DipRejectAdditionException((String)messages.handleGetObject("not_wrapped"));
    }

    return propertyChangeListeners;
  }

  // DipPropertyChangeListener Interface

  /**
   * Handles a property change event.  The connecting flight dip
   * will filter the results provided by the flight info property
   * and remove any flights that cost more than the threshold over
   * the cheapest connecting flight.
   */
  public void propertyChange(PropertyChangeEvent e) {

    // we only care about the flightInfo property
    if (e.getPropertyName() == "flightInfo") {
      // get the flight information retrieved by the dap
      Vector flightData = DAP.getFlightInfo();
      if (flightData != null && flightData.size() > 0) {

        // initialize the cheapest fare
        int cheapIndex = 0;
        FlightInformation cheapestInfo = (FlightInformation)flightData.elementAt(cheapIndex);
        double cheapest = totalFare(cheapestInfo);

        // walk vector of flight data and look for cheapest flight
        for (int i = 0; i < flightData.size(); i++) {
          FlightInformation info = (FlightInformation)flightData.elementAt(i);
          if (totalFare(info) < cheapest) {
            cheapIndex = i;
            cheapestInfo = info;
            cheapest = totalFare(info);
          }
        }

        // if cheapest flight is a connecting flight, walk the vector
        // again and remove any direct flights that cost more than the
        // cheapest connecting flight + threshold.
        if (cheapestInfo.getbHasConnection()) {

          // clone the flight data so we can manipulate it
          Vector newFlightData = (Vector)flightData.clone();
          boolean modifiedFlightData = false;

          // walk vector looking for direct flights
          for (int i = 0; i < newFlightData.size(); i++) {

            FlightInformation info = (FlightInformation)newFlightData.elementAt(i);

            // if this is a direct flight and it costs more than cheapest + threshold
            if (!info.getbHasConnection() && totalFare(info) > cheapest+priceDifferential) {

              // remove it from the vector
              newFlightData.removeElementAt(i);
              modifiedFlightData = true;

              // back up the index so we increment back to this position
              i--;
            }
          }

          // if changes have been made to the vector, then reset the property
          // in the dap.
          if (modifiedFlightData) {
            try {
              DAP.setFlightInfo(newFlightData);
            }
            catch (java.beans.PropertyVetoException ex) {
              System.err.println("unexpected property veto");
            }
          }

        } // if cheapest is a connecting flight
      } // if there's flight data to examine
    } // if we're dealing with the right property
  }

  // return total fare for flight
  private double totalFare(FlightInformation i) {
    double fare = i.getPrice();
    if (i.getbHasConnection()) {
      fare += i.getPrice2();
    }
    return fare;
  }

  // private resource handler
  private void getMessages() {
    if (messages == null) {
      messages = (Messages)ResourceBundle.getBundle("com.ibm.beans.demos.travel.flight.rules.Messages");
    }
  }

  private static final long serialVersionUID = 2000L;
  private static String kDipTypeName = "Direct Flight Restriction";

  private transient Messages messages;
  private transient FlightInfoDapProperties DAP = null; // rehooked at deserialization
  private transient PropertyChangeSupport support;

  private double priceDifferential = 0;

  private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
}


