//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbBOI
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.12 com/java/src/com/ibm/beans/samples/daps/sql/Common.java, bbDaps, bb97c, bb97c9750e 10/28/97 14:26:57 [12/11/97 17:01:26]
package com.ibm.beans.samples.daps.sql;

import java.io.Serializable;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * The Common class encapsulates common behavior and functionality
 * for the Connection and Statement classes.  This behavior includes
 * support for background thread execution, property change events
 * (both firing and listeners), serialization and the successful
 * and exception properties.
 *
 * <p>The Common class is an abstract class.
 *
 * @see com.ibm.beans.samples.daps.sql.Connection
 * @see com.ibm.beans.samples.daps.sql.Statement
 * @see com.ibm.beans.samples.daps.sql.CommonBeanInfo
 *
 * @version 1.0  March 27, 1997
 * @author J. Russell Boykin
 */
public abstract class Common implements Serializable
{
   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";

   // Instance variables

   /**
    * serial version id to support compatibility with minor versions
    */
   private static final long serialVersionUID = 2000L;

   /**
    * A flag that signifies whether database operations are to be performed
    * asynchronously in a background thread or synchronously in the current
    * thread.
    * <p>This property is used by the myNotify() method.
    */
   protected boolean executeInBackground              = false;

   /**
    * An ouput exception for the latest database operation.
    */
   protected transient Exception exception            = null;

   /**
    * An output flag that signifies whether the method executed was successful.
    */
   protected transient boolean successful             = false;

   /**
    * Initial value of current background thread task.
    */
   static protected transient final int INITIAL    = 0;

   /**
    * Field that signifies which task is being executed in the background.
    */
   protected transient int task = INITIAL;

   /**
    * Supporting class to provide easy firing of property change events.
    */
   protected PropertyChangeSupport changes = new PropertyChangeSupport(this);

   /**
    * Background thread if executeInBackground is true.
    */
   protected transient Thread backgroundThread = null;

   /**
    * Default constructor.
    */
   public Common()
   {
       // Invoke the constructor of the superclass
       super();

       // Perform common initialization for this object.
       initialize();
   }

   /**
    * Finalize this object.
    * @exception java.lang.Throwable Possibly thrown by the superclass.
    */
   protected void finalize() throws Throwable
   {
       // Invoke finalize method of superclass
       super.finalize();
   }

   /**
    * Get the executeInBackground flag for the database operations.
    * @return The executeInBackground flag for the database operations.
    */
   public boolean getExecuteInBackground()
   {
       return executeInBackground;
   }

   /**
    * Set the executeInBackground flag for the database operations.
    * <p>This value is used by the myNotify() method.
    * <p>Fire the property change event for the executeInBackground boolean.
    * @param executeInBackground The executeInBackground flag for the database
    * operations.
    */
   public synchronized void setExecuteInBackground(boolean executeInBackground)
   {
       boolean old = this.executeInBackground;
       this.executeInBackground = executeInBackground;

       // Coordinate the background thread
       coordinateBackgroundThread();

       firePropertyChange("executeInBackground",
                          new Boolean(old),
                          new Boolean(executeInBackground));
   }

   /**
    * Get the exception flag for the latest database operation.
    * @return The exception flag for the last database operation.
    */
   public Exception getException()
   {
       return exception;
   }

   /**
    * Set the exception field.
    * <p>This method can only be called internally.
    * <p>Fire the property change event for the exception Exception.
    * @param exception The exception for the latest database operation.
    */
   protected void setException(Exception exception)
   {
       this.exception = exception;

       // I'm intentionally passing null as the old value so same value will
       // still be fired to listeners.
       firePropertyChange("exception",
                          null,
                          exception);
   }

   /**
    * Get the successful flag for the latest database operation.
    * @return The successful flag for the last database operation.
    */
   public boolean getSuccessful()
   {
       return successful;
   }

   /**
    * Set the successful operation flag.
    * <p>This method can only be called internally.
    * <p>Fire the property change event for the successful boolean.
    * @param successful The successful database operation flag.
    */
   protected void setSuccessful(boolean successful)
   {
       this.successful = successful;

       // I'm intentionally passing null as the old value so same value will
       // still be fired to listeners.
       firePropertyChange("successful",
                          null,
                          new Boolean(successful));
   }

   /**
    * Fire a property change event.
    * <p>This method is necessary so the dip code can see events.
    * @param propertyName The String name of the property.
    * @param oldValue The old value of the property.
    * @param newValue The new value of the property.
    */
   public void firePropertyChange(String propertyName,
                                  Object oldValue,
                                  Object newValue)
   {
       // Fire the property change
       changes.firePropertyChange(propertyName,
                                  oldValue,
                                  newValue);
   }

   /**
    * Method to ensure background exists if the user requires it and it
    * doesn't exist if the user doesn't require it.
    * <p>In JDK 1.1 it is impossible to destroy() a thread
    * once it has been created.
    * <p>This method uses the executeInBackground property.
    * <p>This method sets the backgroundThread property.
    */
   protected synchronized void coordinateBackgroundThread()
   {
       // If executing operations in background
       if (getExecuteInBackground())
       {
           // If we don't have a background thread
           if (backgroundThread == null)
           {
               // Setup background thread for later executions
               backgroundThread = new BackgroundThread(this);
           }
       }
       else
       {
           // If we have a background thread
           if (backgroundThread != null)
           {
/*             Unfortunately, the destroy() method is not implemented in
               JDK 1.1 and will always throw an exception.  The following
               two lines of code will have to be commented out until this
               functionality is implemented.

               This means that once the user chooses to use a background
               thread, the decision to change cannot destroy the thread.
               It will remain.  However, determination of this thread's
               existence will be made every time an instance is created,
               either with a new or serialization.  Therefore the unnecessary
               hit should typically only be taken at design time.

               // Destroy the background thread
               backgroundThread.destroy();

               // Ensure the backgroundThread property is null
               backgroundThread = null;
*/
           }
       }
   }

   /**
    * Common initialization for both initial and serialized instantiation.
    */
   protected void initialize()
   {
       // coordinate the background thread
       coordinateBackgroundThread();
   }

   /**
    * Output Serialization support.
    * @param s ObjectOutputStream used for writing out the object.
    * @exception java.io.IOException Thrown by
    * java.io.ObjectOutputStream.defaultWriteObject.
    */
   private void writeObject(java.io.ObjectOutputStream s)
                            throws java.io.IOException
   {
       // Perform default output serialization
       s.defaultWriteObject();
   }

   /**
    * Input Serialization support.
    * @param s ObjectInputStream used for reading in the object.
    * @exception java.lang.ClassNotFoundException Thrown by
    * java.io.ObjectInputStream.defaultWriteObject.
    * @exception java.io.IOException Thrown by
    * java.io.ObjectInputStream.defaultWriteObject.
    */
   private void readObject(java.io.ObjectInputStream s)
                           throws java.lang.ClassNotFoundException,
                           java.io.IOException
   {
       // Perform default input serialization
       s.defaultReadObject();

       // Call common initialize method
       initialize();
   }

   /**
    * Add a property change event listener.
    * @param l A listener.
    */
   public synchronized void addPropertyChangeListener(PropertyChangeListener l)
   {
       // Add a property change event listener
       changes.addPropertyChangeListener(l);
   }

   /**
    * Remove a property change event listener.
    * @param l A listener.
    */
   public synchronized void removePropertyChangeListener
                            (PropertyChangeListener l)
   {
       // Remove a property change event listener
       changes.removePropertyChangeListener(l);
   }

   /**
    * Determine whether to perform operation in background thread
    * or in current thread.
    * <p>If the executeInBackground property is true, standard
    * java.lang.Object notify() is executed.  Otherwise, the
    * background task is performed via direct call.
    * <p>The executeInBackground property is used in this method.
    */
   protected synchronized void myNotify()
   {
       // If performing operations using a background thread
       if (getExecuteInBackground())
       {
           // Notify the background thread to perform the execution.
           notify();
       }
       else
       {
           // Execute the operation in the current thread
           performWorkAndSetStatus();
       }
   }

   /**
    * Actual background thread operation running in this Common object.
    * <p>This is called from the worker thread.
    */
   protected void runBackgroundThread()
   {
       // Forever
       for (;;)
       {
           // Wait for and process background work
           waitForBackgroundWork();
       }
   }

   /**
    * Method to wait for and drive actual database work is executing in a
    * background thread.
    * <p>waitForBackgroundWork has to be synchronized for wait() to work.
    */
   protected synchronized void waitForBackgroundWork()
   {
       try
       {
           // Initialize task to nothing
           task = INITIAL;

           // Wait for a real task
           do
           {
               // Wait for notification
               wait();

               // Perform actual background thread work and set status
               performWorkAndSetStatus();
           }
           while (task == INITIAL);
       }

       // catch Interrupted Exception
       catch (InterruptedException x)
       {
           // Display exception and stack trace
           System.err.println("Common.waitForBackgroundWork:  " + x);
           x.printStackTrace();
       }
   }

   /**
    * Call method based on the background task to be performed and set status.
    * <p>The exception and successful properties are set by this method.
    */
   protected void performWorkAndSetStatus()
   {
       // If task is not initial
       if (task != INITIAL)
       {
           try
           {
              // Perform the actual work
              performWork();

              // Clear output exception
              setException(null);

              // set successful flag to true
              setSuccessful(true);
           }

           // Catch any exception
           catch (Exception ex)
           {
              // Set exception output field
              setException(ex);

              // set successful flag to false
              setSuccessful(false);
           }
       }
   }

   /**
    * Call method based on the background task to be performed.
    * The caller has guaranteed this will NOT be called for the
    * INITIAL task.
    * @return Whether operation was successful or not.
    * @exception java.lang.Exception Thrown if any exception.
    */
   protected abstract void performWork() throws Exception;
}

/**
 * The BackgroundThread implements a background thread for use in
 * database operations.
 * <p>We need a separate class for the thread as opposed to making
 * Common Runnable so the public run method won't be publicly available.
 */
class BackgroundThread extends Thread
{
   /**
    * Actual Common class.
    */
   private Common common;

   /**
    * Constructor.
    * @param common The actual Common object.
    */
   BackgroundThread(Common common)
   {
       // Save away the Common
       this.common = common;

       // Start the background thread
       this.start();
   }

   /**
    * Method to run the background thread.
    */
   public void run()
   {
       // Run the Common's background thread
       common.runBackgroundThread();
   }
}
