//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbBOI
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.10 com/java/src/com/ibm/beans/samples/daps/sql/Connection.java, bbDaps, bb97c, bb97c9750e 9/29/97 16:27:34 [12/11/97 17:01:20]
package com.ibm.beans.samples.daps.sql;

import java.sql.*;

/**
 * The Connection class supports a single JDBC database connection.
 *
 * <p>The JDBCConnection property is then shared by multiple instances of
 * subclasses of the Statement class.
 *
 * <p>To use the Connection bean, do the following:
 * <ol>
 * <li>Create an instance of the bean.
 * <li>Fill in the bean's properties (described below or in superclass):
 * <ul>
 * <li>autoCommit
 * <li>executeInBackground
 * <li>JDBCDriverName
 * <li>JDBCURL
 * <li>password
 * <li>readOnly
 * <li>transactionIsolation
 * <li>userid
 * </ul>
 * <li>Use the connect() method to connect to the database.
 * The connect() method uses all the above properties to control connection
 * to a database.  The connect() method sets values and fires property change
 * events for the following properties on completion:
 * <ul>
 * <li>exception
 * <li>JDBCConnection
 * <li>successful
 * </ul>
 * <li>Utilize various DDL, InsertUpdateDelete and Select
 * statements to perform SQL operations against the connected database.
 * <li>Use the commit() or rollback() methods to terminate the
 * database transaction.  These methods set values and fire property change
 * events for the following properties:
 * <ul>
 * <li>exception
 * <li>successful
 * </ul>
 * <li>Use the disconnect() method to disconnect from the database.
 * The disconnect() method sets values and fires property change
 * events for the following properties on completion:
 * <ul>
 * <li>excception
 * <li>JDBCConnection
 * <li>successful
 * </ul>
 * </ol>
 *
 * @see com.ibm.beans.samples.daps.sql.Statement
 * @see com.ibm.beans.samples.daps.sql.ConnectionBeanInfo
 *
 * @version 1.0  March 24, 1997
 * @author J. Russell Boykin
 */
public class Connection extends Common
{
   // Instance variables
   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";

   /**
    * The JDBC java.sql.connection.
    * <p>This instance variable is only modified internally.
    * The connectBackground() method sets this value and the
    * disconnectBackground() method clears it.  The disconnectBackground(),
    * commitBackground() and rollbackBackground() methods use this property.
    * <p>The DDL, InsertUpdateDelete and the Select statement beans use
    * this property.
    * @see java.sql.Connection
    */
   protected transient java.sql.Connection JDBCConnection = null;

   /**
    * The JDBC Driver Name is used to instantiate the JDBC Connection.
    * <p>This string identifies the
    * database driver for the connection and is used by the connectBackground()
    * method. It must be set prior to execution of the connect() method.
    */
   protected String JDBCDriverName                    = "sun.jdbc.odbc.JdbcOdbcDriver";

   /**
    * The JDBC URL used to instantiate the JDBC Connection.
    * <p>This string identifies the
    * database target for the connection and is used by the connectBackground()
    * method. It must be set prior to execution of the connect() method.
    */
   protected String JDBCURL                           = "jdbc:odbc:DB2";

   /**
    * The userid of the user using the database.
    * <p>This string is used by the connectBackground() method.
    * It must be set prior to execution of the connect() method.
    */
   protected String userid                            = "";

   /**
    * The password of the user using the database.
    * <p>This string is used by the connectBackground() method.
    * It must be set prior to execution of the connect() method.
    */
   protected String password                          = "";

   /**
    * A flag that specifies whether auto-commit is performed after every
    * statement execution.
    * <p>This flag is used by the connectBackground() method.
    * It must be set prior to execution of the connect() method.
    */
   protected boolean autoCommit                       = true;

   /**
    * A flag that specifies if the database connection is read-only.
    * <p>This flag is used by the connectBackground() method.
    * It must be set prior to execution of the connect() method.
    */
   protected boolean readOnly                         = false;

   /**
    * Transaction isolation values.
    * <p>This value is used by the connectBackground() method.
    * Possible values are:
    * <ul>
    * <li>java.sql.Connection.TRANSACTION_NONE,
    * <li>java.sql.Connection.TRANSACTION_READ_COMMITTED,
    * <li>java.sql.Connection.TRANSACTION_READ_UNCOMMITTED,
    * <li>java.sql.Connection.TRANSACTION_REPEATABLE_READ,
    * <li>java.sql.Connection.TRANSACTION_SERIALIZABLE
    * </ul>
    * It must be set prior to execution of the connect() method.
    * @see com.ibm.beans.samples.daps.sql.IsolationEditor
    */
   protected int transactionIsolation
             = java.sql.Connection.TRANSACTION_REPEATABLE_READ;

   /**
    * Background thread processing connect to database.
    */
   static protected transient final int CONNECT    = INITIAL + 1;

   /**
    * Background thread processing disconnect from database.
    */
   static protected transient final int DISCONNECT = INITIAL + 2;

   /**
    * Background thread processing database commit.
    */
   static protected transient final int COMMIT     = INITIAL + 3;

   /**
    * Background thread processing database rollback.
    */
   static protected transient final int ROLLBACK   = INITIAL + 4;

   /**
    * Default constructor.
    */
   public Connection()
   {
       // Invoke the constructor of the superclass
       super();
   }

   /**
    * Finalize this object.
    * <p>Disconnect from database if not yet disconnected.
    * @exception java.lang.Throwable Possibly thrown by the superclass.
    */
   protected void finalize() throws Throwable
   {
       try
       {
           // Disconnect from this database if connected
           dbDisconnect(false);
       }

       // catch SQL Exception
       catch (SQLException sx)
       {
           // Display exception and stack trace
           System.err.println("Connection.finalize():  " + sx);
           sx.printStackTrace();
       }

       // catch Null Pointer Exception
       catch (NullPointerException nx)
       {
           // Display exception and stack trace
           System.err.println("Connection.finalize():  " + nx);
           nx.printStackTrace();
       }

       // Invoke finalize method of superclass
       super.finalize();
   }

   /**
   * Returns the JDBC database connection object.
   * @return The java.sql.Connection object; null if not connected.
   */
   public java.sql.Connection getJDBCConnection()
   {
       return JDBCConnection;
   }

   /**
    * Set the JDBC database connection object.
    * <p>Fire the property change event for the JDBCConnection object.
    * <p>This method can only be called internally via the connect()
    * and disconnect() methods.
    * @param JDBCConnection The JDBC database connection.
    */
   protected void setJDBCConnection(java.sql.Connection JDBCConnection)
   {
       this.JDBCConnection = JDBCConnection;
       firePropertyChange("JDBCConnection",
                          null,
                          JDBCConnection);
   }

   /**
    * Get the JDBC Driver Name.
    * @return The JDBC Driver Name specifying how to find the JDBC database driver.
    */
   public String getJDBCDriverName()
   {
       return JDBCDriverName;
   }

   /**
    * Set the JDBC Driver Name.
    * <p>This value is used by the connect() method.
    * <p>Fire the property change event for the JDBCDriverName string.
    * @param JDBCDriverName The JDBC Driver name specifying how to find the
    * JDBC database driver.
    */
   public synchronized void setJDBCDriverName(String JDBCDriverName)
   {
       String old = this.JDBCDriverName;
       this.JDBCDriverName = JDBCDriverName;
       firePropertyChange("JDBCDriverName",
                          old,
                          JDBCDriverName);
   }

   /**
    * Get the JDBC URL.
    * @return The JDBC URL specifying how to find the JDBC database.
    */
   public String getJDBCURL()
   {
       return JDBCURL;
   }

   /**
    * Set the JDBC URL.
    * <p>This value is used by the connect() method.
    * <p>Fire the property change event for the JDBCURL string.
    * @param JDBCURL The JDBC URL specifying how to find the JDBC database.
    */
   public synchronized void setJDBCURL(String JDBCURL)
   {
       String old = this.JDBCURL;
       this.JDBCURL = JDBCURL;
       firePropertyChange("JDBCURL",
                          old,
                          JDBCURL);
   }

   /**
    * Get the userid of the database connection.
    * @return The userid for the database connection.
    */
   public String getUserid()
   {
       return userid;
   }

   /**
    * Set the userid of the database connection.
    * <p>This value is used by the connect() method.
    * <p>Fire the property change event for the userid string.
    * @param userid The userid for the database connection.
    */
   public synchronized void setUserid(String userid)
   {
       String old = this.userid;
       this.userid = userid;
       firePropertyChange("userid",
                          old,
                          userid);
   }

   /**
    * Get the password of the user connecting to the database.
    * @return The password for the database password.
    */
   public String getPassword()
   {
       return password;
   }

   /**
    * Set the password of the user connecting to the database.
    * <p>This value is used by the connect() method.
    * <p>Fire the property change event for the password string.
    * @param password The password for the database password.
    */
   public synchronized void setPassword(String password)
   {
       String old = this.password;
       this.password = password;
       firePropertyChange("password",
                          old,
                          password);
   }

   /**
    * Get the auto-commit flag for the database connection.
    * @return The boolean auto-commit flag for the database connection.
    */
   public boolean getAutoCommit()
   {
       return autoCommit;
   }

   /**
    * Set the auto-commit flag for the database connection.
    * <p>This value is used by the connect() method.
    * <p>Fire the property change event for the auto-commit boolean.
    * @param autoCommit The boolean auto-commit flag for the database
    * connection.
    */
   public synchronized void setAutoCommit(boolean autoCommit)
   {
       boolean old = this.autoCommit;
       this.autoCommit = autoCommit;
       firePropertyChange("autoCommit",
                          new Boolean(old),
                          new Boolean(autoCommit));
   }

   /**
    * Get the read-only flag for the database connection.
    * @return The boolean read-only flag for the database connection.
    */
   public boolean getReadOnly()
   {
       return readOnly;
   }

   /**
    * Set the read-only flag for the database connection.
    * <p>This value is used by the connect() method.
    * <p>Fire the property change event for the read-only boolean.
    * @param readOnly The boolean read-only flag for the database connection.
    */
   public synchronized void setReadOnly(boolean readOnly)
   {
       boolean old = this.readOnly;
       this.readOnly = readOnly;
       firePropertyChange("readOnly",
                          new Boolean(old),
                          new Boolean(readOnly));
   }

   /**
    * Get the transaction isolation level for the database connection.
    * @return The integer transaction isolation level for the database
    * connection.
    */
   public int getTransactionIsolation()
   {
       return transactionIsolation;
   }

   /**
    * Set the transaction isolation level for the database connection.
    * <p>This value is used by the connect() method.
    * <p>Fire the property change event for the transactionIsolation value.
    * @param transactionIsolation The integer transaction isolation level for
    *                             the database connection.
    *                             @see com.ibm.beans.samples.daps.sql.IsolationEditor
    */
   public synchronized void setTransactionIsolation(int transactionIsolation)
   {
       int old = this.transactionIsolation;
       this.transactionIsolation = transactionIsolation;
       firePropertyChange("transactionIsolation",
                          new Integer(old),
                          new Integer(transactionIsolation));
   }

   /**
    * Connect to a specified database.
    * <p>The JDBCDriverName, JDBCURL, userid, password, autoCommit, readOnly and
    * transactionIsolation properties are used by this method's background
    * thread operation.
    * <p>The exception, successful and JDBCConnection properties are set
    * indirectly by this method.
    * @see #connectBackground()
    */
   public synchronized void connect()
   {
       // Signify current background task
       task = CONNECT;

       // Notify the background thread
       myNotify();
   }

   /**
    * The actual background thread code that connects to the database.
    * <p>The JDBCDriverName, JDBCURL, userid, password, autoCommit, readOnly and
    * transactionIsolation properties are used by this method.
    * <p>The JDBCConnection property is set by this method.
    * @exception java.sql.SQLException SQL Exception.
    * @exception java.lang.ClassNotFoundException Class not found.
    * @exception java.lang.NullPointerException Pointer not set.
    */
   protected void connectBackground() throws SQLException,
                                             ClassNotFoundException,
                                             NullPointerException
   {
       try
       {
           // Disconnect if we're connected
           // This will send a property change for the JDBCConnection that
           // will then be consumable by the listener Statement beans.
           dbDisconnect(false);

           // Make sure the requested JDBC driver is loaded.
           // Protocol for a driver loading is to register itself so
           // later calls to getConnection() can find the driver.
           // This will cause a nice exception if this driver class isn't found.
           Class.forName(getJDBCDriverName());

           // Attempt to connect to the database
           JDBCConnection = DriverManager.getConnection(getJDBCURL(),
                                                        getUserid(),
                                                        getPassword());

           // Save away the connection.  This also causes the property change
           // event to fire.
           setJDBCConnection(JDBCConnection);

           // Set autoCommit
           JDBCConnection.setAutoCommit(getAutoCommit());

           // Set readOnly
           JDBCConnection.setReadOnly(getReadOnly());

           // Set isolation level
           JDBCConnection.setTransactionIsolation(getTransactionIsolation());
       }

       // catch SQL Exception
       catch (SQLException sx)
       {
           // Display exception and stack trace
           System.err.println("Connection.connectBackground():  " + sx);
           sx.printStackTrace();

           // Rethrow exception
           throw sx;
       }

       // catch ClassNotFound exception
       catch (ClassNotFoundException ex)
       {
           // Display exception and stack trace
           System.err.println("Connection.connectBackground():  " + ex);
           ex.printStackTrace();

           // Rethrow exception
           throw ex;
       }

       // catch Null Pointer Exception
       catch (NullPointerException nx)
       {
           // Display exception and stack trace
           System.err.println("Connection.connectBackground():  " + nx);
           nx.printStackTrace();

           // Rethrow exception
           throw nx;
       }
   }


   /**
    * Method to disconnect from a database and rethrow any exceptions.
    * <p>The JDBCConnection property is used by this method.
    * <p>The JDBCConnection property is set by this method.
    * @param nullException boolean that signifies whether or not to throw
    * NullPointerException if the JDBCConnection property is null.
    * @exception java.sql.SQLException Thrown if unable to perform the close
    * operation.
    * @exception java.lang.NullPointerException Thrown if JDBCConnection is
    * null and the nullException parameter is true;
    */
   private void dbDisconnect(boolean nullException) throws SQLException,
                                                           NullPointerException
   {
       // Get local connection
       java.sql.Connection JDBCConnection = getJDBCConnection();

       // If we are connected
       if (JDBCConnection != null)
       {
           try
           {
               // Close the connection - this may fail.  If it fails, we don't
               // want to clear the JDBCConnection field.  An example of an
               // exception is trying to disconnect when in the midst of a
               // transaction.
               JDBCConnection.close();
               setJDBCConnection(null);
           }

           // catch SQL Exception
           catch (SQLException sx)
           {
               // Display exception and rethrow the exception
               System.err.println("Connection.dbDisconnect():  " + sx);
               throw sx;
           }
       }
       else
       {
           // If we want to throw a null-pointer exception
           if (nullException)
           {
               // throw the null pointer exception for the JDBCConnection
               throw new NullPointerException("Connection.JDBCConnection");
           }
       }
   }

   /**
    * Disconnect from the current database.
    * <p>The JDBCConnection property is used by this method's background
    * thread operation.
    * <p>The exception, successful and JDBCConnection properties are indirectly
    * set by this method.
    * @see #disconnectBackground()
    */
   public synchronized void disconnect()
   {
       // Signify current background task
       task = DISCONNECT;

       // Notify the background thread
       myNotify();
   }

   /**
    * The actual background thread code that disconnects from a database.
    * @exception java.sql.SQLException SQL Exception.
    * @exception java.lang.NullPointerException Pointer not set.
    */
   protected void disconnectBackground() throws SQLException,
                                                NullPointerException
   {
       try
       {
           // Attempt to close the connection
           dbDisconnect(true);
       }

       // catch SQL Exception
       catch (SQLException sx)
       {
           // Display exception and stack trace
           System.err.println("Connection.disconnectBackground():  " + sx);
           sx.printStackTrace();

           // Rethrow exception
           throw sx;
       }

       // catch Null Pointer Exception
       catch (NullPointerException nx)
       {
           // Display exception and stack trace
           System.err.println("Connection.disconnectBackground():  " + nx);
           nx.printStackTrace();

           // Rethrow exception
           throw nx;
       }
   }

   /**
    * Explicitly commit changes to the current database.
    * <p>The JDBCConnection property is used by this method's background
    * thread operation.
    * <p>The exception and successful properties are indirectly set by this
    * method.
    * @see #commitBackground()
    */
   public synchronized void commit()
   {
       // Signify current background task
       task = COMMIT;

       // Notify the background thread
       myNotify();
   }

   /**
    * The actual background thread code that explicitly commits changes to
    * a database.
    * <p>The JDBCConnection property is used by this method.
    * @exception java.sql.SQLException SQL Exception.
    * @exception java.lang.NullPointerException Pointer not set.
    */
   protected void commitBackground() throws SQLException,
                                            NullPointerException
   {
       try
       {
           // Get local JDBCConnection
           java.sql.Connection JDBCConnection = getJDBCConnection();

           // If we are connected to a database
           if (JDBCConnection != null)
           {
              // Commit
              JDBCConnection.commit();
           }
           else
           {
               // Display and throw new NullPointerException.
               NullPointerException nx = new NullPointerException
                                             ("Connection.JDBCConnection");

               // Display exception and stack trace
               System.err.println("Connection.commitBackground():  " + nx);
               nx.printStackTrace();
               throw nx;
           }
       }

       // catch SQL Exception
       catch (SQLException sx)
       {
           // Display exception and stack trace
           System.err.println("Connection.commitBackground():  " + sx);
           sx.printStackTrace();

           // Rethrow exception
           throw sx;
       }
   }

   /**
    * Explicitly rollback changes to the current database.
    * <p>The JDBCConnection property is used by this method's background
    * thread operation.
    * <p>The exception and successful properties are indirectly set by this
    * method.
    * @see #rollbackBackground()
    */
   public synchronized void rollback()
   {
       // Signify current background task
       task = ROLLBACK;

       // Notify the background thread
       myNotify();
   }


   /**
    * The actual background thread code that explicitly rollbacks changes to
    * a database.
    * <p>The JDBCConnection property is used by this method.
    * @exception java.sql.SQLException SQL Exception.
    * @exception java.lang.NullPointerException Pointer not set.
    */
   protected void rollbackBackground() throws SQLException,
                                              NullPointerException
   {
       try
       {
           // Get local JDBCConnection
           java.sql.Connection JDBCConnection = getJDBCConnection();

           // If we are connected to a database
           if (JDBCConnection != null)
           {
               // Rollback
               JDBCConnection.rollback();
           }
           else
           {
               // Display and throw new NullPointerException.
               NullPointerException nx = new NullPointerException
                                             ("Connection.JDBCConnection");

               // Display exception and stack trace
               System.err.println("Connection.rollbackBackground():  " + nx);
               nx.printStackTrace();
               throw nx;
           }
       }

       // catch SQL Exception
       catch (SQLException sx)
       {
           // Display exception and stack trace
           System.err.println("Connection.rollbackBackground():  " +
                              sx);
           sx.printStackTrace();

           // Rethrow exception
           throw sx;
       }
   }

   /**
    * Call method based on the background task to be performed.
    * The caller has guaranteed this will NOT be called for the
    * INITIAL task.
    * @exception java.sql.SQLException SQL Exception.
    * @exception java.lang.ClassNotFoundException Class not found.
    * @exception java.lang.NullPointerException Pointer not set.
    */
   protected void performWork() throws SQLException,
                                       ClassNotFoundException,
                                       NullPointerException
   {
       // Switch on the task to be done
       switch (task)
       {
           // Connect case - perform database connect
           case CONNECT:
               connectBackground();
               break;

           // Disconnect case - perform database disconnect
           case DISCONNECT:
               disconnectBackground();
               break;

           // Commit case - perform database commit
           case COMMIT:
               commitBackground();
               break;

           // Rollback case - perform database rollback
           case ROLLBACK:
               rollbackBackground();
               break;
       }
   }
}
