//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbBOI
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.9 com/java/src/com/ibm/beans/samples/daps/sql/DML.java, bbDaps, bb97c, bb97c9750e 9/29/97 16:27:29 [12/11/97 17:01:20]
package com.ibm.beans.samples.daps.sql;

import java.util.Vector;
import java.sql.SQLException;

/**
 * The DML class is instantiated via subclasses to represent a single
 * SQL DML statement.  This class adds the functionality of input
 * host variable values for DML parameter markers.
 *
 * <p>This class is an abstract class used by the Select and
 * InsertUpdateDelete classes.
 *
 * @see com.ibm.beans.samples.daps.sql.Select
 * @see com.ibm.beans.samples.daps.sql.InsertUpdateDelete
 * @see com.ibm.beans.samples.daps.sql.DMLBeanInfo
 *
 * @version 1.0  March 24, 1997
 * @author J. Russell Boykin
 */
public abstract class DML extends Statement
{
   // Instance variables

   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";

   /**
    * Vector of input objects.
    * <p>These are used by the handleInputValues() method to set
    * values for SQL parameter markers.  This Vector must be set prior
    * to execution of the execute() method
    */
   protected transient Vector inputs = null;

   /**
    * Default constructor.
    */
   public DML()
   {
       // Invoke the constructor of the superclass
       super();
   }

   /**
    * Finalize this object.
    * @exception java.lang.Throwable Possibly thrown by the superclass
    */
   protected void finalize() throws Throwable
   {
       // Ensure the input vector variable gets cleared.  We don't want to
       // remove all elements from the vector as the vector may be being
       // used elsewhere.
       inputs = null;

       // Invoke finalize method of superclass
       super.finalize();
   }

   /**
   * Returns the input Vector for the statement.
   * @return Input Vector for the SQL DML statement.
   */
   public Vector getInputs()
   {
       return inputs;
   }

   /**
    * Set the input vector for the statement.
    * <p>This value is used by the handleInputValues() method.
    * <p>Fire the property change event for the input object.
    * @param inputs The input Vector of objects for use by the SQL DML
    * statement.
    */
   public synchronized void setInputs(Vector inputs)
   {
       Vector old = this.inputs;
       this.inputs = inputs;
       firePropertyChange("inputs",
                          old,
                          inputs);
   }

   /**
    * Method to handle input values for an SQL statement.
    * <p>In a DML statement, these are values for the statement's parameter
    * markers.
    * <p>The preparedStatement and inputs properties are used by this method.
    * @exception java.sql.SQLException Thrown if a setObject fails.
    */
   protected void handleInputValues() throws SQLException
   {
       try
       {
           // Allow the superclass to handle input values
           super.handleInputValues();

           // Clear the statements input parameters
           preparedStatement.clearParameters();

           // If there are input values
           Vector inputs = getInputs();
           if (inputs != null)
           {
               // for all the user's input parameters
               for (int index = 1;
                    index <= inputs.size();
                    index++)
               {
                   // Set the input parameter
                   preparedStatement.setObject(index,
                                               inputs.elementAt(index - 1));
               }
           }
       }

       // catch SQL Exceptions
       catch (SQLException sx)
       {
           // Display exception and rethrow
           System.err.println("DML.handleInputValues():  " + sx);
           throw sx;
       }
   }


   /**
    * The actual background thread code that executes the close operation.
    * <p>The inputs property is set by this method.
    * @exception java.sql.SQLException SQL Exception
    * @exception java.lang.NullPointerException Pointer not set
    */
   protected void closeBackground() throws SQLException,
                                           NullPointerException
   {
       // if the parent's method was successful
       super.closeBackground();

       // Clear the input vector
       setInputs(null);
   }
}
