//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbBOI
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.9 com/java/src/com/ibm/beans/samples/daps/sql/InsertUpdateDelete.java, bbDaps, bb97c, bb97c9750e 9/29/97 16:27:58 [12/11/97 17:01:23]
package com.ibm.beans.samples.daps.sql;

import java.beans.IntrospectionException;
import java.sql.SQLException;

/**
 * The InsertUpdateDelete class is instantiated to represent a single SQL
 * Insert, Update or Delete statement.
 *
 * <p>This class supports number of rows modified by the statement's execution.
 *
 * <p>To use the InsertUpdateDelete bean, do the following:
 * <ol>
 * <li>Create an instance of the bean.
 * <li>Fill in the bean's properties (described below or in superclass):
 * <ul>
 * <li>executeInBackground
 * <li>inputs
 * <li>SQLText
 * </ul>
 * <li>Bind an existing Connection bean's JDBCConnection property
 * to this bean's JDBCConnection property.  This will cause execution of the
 * connect() method on the Connection bean to also set the JDBCConnection
 * property on this bean.
 * <li>If parameter markers are used in the SQL statement, set up the inputs
 * parameter.  Each item in the Vector will correspond to the
 * similarly-ordered parameter marker.
 * <li>Use the execute() method to execute the statement.
 * The execute() method uses all the above properties to perform execution of
 * the DDL statement.  The execute() method sets values and fires property
 * change events for the following properties on completion:
 * <ul>
 * <li>exception
 * <li>numberRowsUpdated
 * <li>successful
 * </ul>
 * <li>Use the close() method to relinquish the statement's resources.
 * The close() method sets values and fires property
 * change events for the following properties on completion:
 * <ul>
 * <li>exception
 * <li>numberRowsUpdated
 * <li>successful
 * </ul>
 * </ol>
 *
 * @see com.ibm.beans.samples.daps.sql.InsertUpdateDeleteBeanInfo
 *
 * @version 1.0  March 24, 1997
 * @author J. Russell Boykin
 */
public class InsertUpdateDelete extends DML
{
   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";

   // Instance variables

   /**
    * An output count of the number of rows updated.
    * <p>This field is set by the executeBackground() and closeBackground()
    * methods.
    */
   protected transient int numberRowsUpdated = 0;

   /**
    * Default constructor.
    */
   public InsertUpdateDelete()
   {
       // Invoke the constructor of the superclass
       super();
   }

   /**
    * Finalize this object.
    * @exception java.lang.Throwable Possibly thrown by the superclass.
    */
   protected void finalize() throws Throwable
   {
       // Invoke finalize method of superclass
       super.finalize();
   }

    // Property Accessors

   /**
    * Get the number of rows modified by the latest statement execution.
    * @return The number of rows modified by the latest statement execution.
    */
   public int getNumberRowsUpdated()
   {
       return numberRowsUpdated;
   }

   /**
    * Set the number of rows modified by the latest statement execution.
    * <p>Methods setting this flag include execute() and close().
    * <p>Fire the property change event for the numberRowsUpdated integer.
    * @param numberRowsUpdated The number of rows modified by the latest
    * statement execution.
    */
   protected void setNumberRowsUpdated(int numberRowsUpdated)
   {
       int old = this.numberRowsUpdated;
       this.numberRowsUpdated = numberRowsUpdated;

       // I'm intentionally passing null as the old value so same value will
       // still be fired to listeners.
       firePropertyChange("numberRowsUpdated",
                          null,
                          new Integer(numberRowsUpdated));
   }

   /**
    * Method to handle result meta data for an SQL Insert/Update/Delete
    * statement.
    * <p>The numberRowsUpdated property is set by this method.
    * @exception java.sql.SQLException Possibly thrown by the superclass
    */
   protected void handleResultMetaData() throws SQLException
   {
       // If we have a prepared statement
       if (preparedStatement != null)
       {
           try
           {
               // Allow the superclass to handle result meta data
               super.handleResultMetaData();

               // Set output count field
               setNumberRowsUpdated(preparedStatement.getUpdateCount());
           }

           // catch SQL Exceptions
           catch (SQLException sx)
           {
               System.err.println
                          ("InsertUpdateDelete.handleResultMetaData():  " +
                           sx);
               throw sx;
           }
       }
   }

   /**
    * The actual background thread code that executes the close operation.
    * <p>The numberRowsUpdated property is set by this method.
    * @exception java.sql.SQLException SQL Exception
    * @exception java.lang.NullPointerException Pointer not set
    */
   protected void closeBackground() throws SQLException,
                                           NullPointerException
   {
       // if the parent's method was successful
       super.closeBackground();

       // Clear row update count
       setNumberRowsUpdated(0);
   }
}
