//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbBOI
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.7 com/java/src/com/ibm/beans/samples/daps/sql/IsolationEditor.java, bbDaps, bb97c, bb97c9750e 9/29/97 16:28:05 [12/11/97 17:01:24]
package com.ibm.beans.samples.daps.sql;

import java.sql.Connection;

/**
 * The IsolationEditor implements a special case property editor for
 * database isolation levels.
 *
 * <p>It is used by the ConnectionBeanInfo class.
 * @see com.ibm.beans.samples.daps.sql.ConnectionBeanInfo
 *
 * @version 1.0  March 24, 1997
 * @author J. Russell Boykin
 */
public class IsolationEditor
               extends java.beans.PropertyEditorSupport
{
   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";

   /**
    * Array of transaction isolation strings.
    * <p>These are used in the setAsText and getAsText methods.
    */
   static protected transient final String isolationStrings[] =
   {
       "None",
       "Read committed",
       "Read uncommitted",
       "Repeatable read",
       "Serializable"
   };

   /**
    * Array of transaction isolation values.
    * <p>These are used in the setAsText and getAsText methods.
    */
   static protected transient final int isolationValues[] =
   {
       java.sql.Connection.TRANSACTION_NONE,
       java.sql.Connection.TRANSACTION_READ_COMMITTED,
       java.sql.Connection.TRANSACTION_READ_UNCOMMITTED,
       java.sql.Connection.TRANSACTION_REPEATABLE_READ,
       java.sql.Connection.TRANSACTION_SERIALIZABLE
   };

   /**
    * Default constructor.
    */
   public IsolationEditor()
   {
       // Invoke the constructor of the superclass
       super();
   }

   /**
    * Finalize this object.
    * @exception java.lang.Throwable Possibly thrown by the superclass.
    */
   protected void finalize() throws Throwable
   {
       // Invoke finalize method of superclass
       super.finalize();
   }

   /**
    * Method to return the array of possible strings.
    * @return The array of possible strings.
    */
   public String[] getTags()
   {
       return isolationStrings;
   }

   /**
    * Method to set the isolation value and convert from text to integer.
    * @param text The string representation of an isolation level.
    * @exception IllegalArgumentException Thrown if invalid isolation value
    */
   public void setAsText(String text) throws IllegalArgumentException
   {
       // For all the isolation strings
       for (int index = 0;
            index < isolationStrings.length;
            index++)
       {
           // If the parameter is the same as one of our predefined strings
           if (isolationStrings[index].compareTo(text) == 0)
           {
               // Set the corresponding integer value and return
               setValue(new Integer(isolationValues[index]));
               return;
           }
       }

       // Throw exception - illegal value
       throw new IllegalArgumentException();
   }

   /**
    * Method to get the string representation of the current integer isolation
    * value.
    * @return The string representation of an isolation level.
    */
   public String getAsText()
   {
       // Get the integer value for the isolation
       int value = ((Integer) getValue()).intValue();

       // For all the possible values
       for (int index = 0;
            index < isolationValues.length;
            index++)
       {
           // If we've found the value
           if (isolationValues[index] == value)
           {
               // Return the corresponding string
               return isolationStrings[index];
           }
       }

       // Value not found - return a null string
       return null;
   }
}
