//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbBOI
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.10 com/java/src/com/ibm/beans/samples/daps/sql/Select.java, bbDaps, bb97c, bb97c9750e 9/29/97 16:28:08 [12/11/97 17:01:25]
package com.ibm.beans.samples.daps.sql;

import java.util.Vector;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

/**
 * The Select class is instantiated to represent a single SQL Select
 * statement.
 *
 * <p>This class supports cursoring through the Select statement's result set.
 *
 * <p>To use the Select bean, do the following:
 * <ol>
 * <li>Create an instance of the bean.
 * <li>Fill in the bean's properties (described below or in superclass):
 * <ul>
 * <li>executeInBackground
 * <li>inputs
 * <li>maxRowsReturned
 * <li>maxFieldSizeReturned
 * <li>SQLText
 * </ul>
 * <li>Bind an existing Connection bean's JDBCConnection property
 * to this bean's JDBCConnection property.  This will cause execution of the
 * connect() method on the Connection bean to also set the JDBCConnection
 * property on this bean.
 * <li>If parameter markers are used in the SQL statement, set up the inputs
 * parameter.  Each item in the Vector will correspond to the
 * similarly-ordered parameter marker.
 * <li>Bind the resultRow property to an existing Viewer bean's
 * additionalRow property.  This will cause the output values to be
 * displayed as they are retrieved from the database.
 * <li>Use the execute() method to execute the statement.
 * The execute() method uses all the above properties to perform execution of
 * the Select statement.  The execute() method sets values and fires
 * property change events for the following properties on completion:
 * <ul>
 * <li>columnNames
 * <li>columnLengths
 * <li>columnNullables
 * <li>columnTypes
 * <li>exception
 * <li>successful
 * </ul>
 * <li>Repeatedly use the next() method to get the resulting rows of the Select.
 * The next() method sets values and fires property
 * change events for the following properties on completion:
 * <ul>
 * <li>exception
 * <li>resultRow
 * <li>rowFound
 * <li>successful
 * </ul>
 * <li>Use the close() method to relinquish the statement's resources.
 * The close() method sets values and fires property
 * change events for the following properties on completion:
 * <ul>
 * <li>columnNames
 * <li>columnLengths
 * <li>columnNullables
 * <li>columnTypes
 * <li>exception
 * <li>resultRow
 * <li>rowFound
 * <li>successful
 * </ul>
 * </ol>
 *
 * @see com.ibm.beans.samples.daps.sql.Viewer
 * @see com.ibm.beans.samples.daps.sql.SelectBeanInfo
 *
 * @version 1.0  March 24, 1997
 * @author J. Russell Boykin
 */
public class Select extends DML
{
   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";

   // Instance variables

   /**
    * Maximum number of rows that can be returned by a Select statement.
    * <p>This field is used by the prepareStatement() method.
    * <p>This field must be set prior to execution of the execute() method.
    */
   protected int maxRowsReturned            = 200;

   /**
    * Maximum size of a single field in a returned row.
    * <p>This field is used by the prepareStatement() method.
    * <p>This field must be set prior to execution of the execute() method.
    */
   protected int maxFieldSizeReturned       = 4096;

   /**
    * A Vector of Strings representing the columns last of this Select.
    * <p>This field is set by the handleResultMetaData() method.
    */
   protected transient Vector columnNames   = null;

   /**
    * A Vector of Strings representing the columns types of this Select.
    * <p>This field is set by the handleResultMetaData() method.
    */
   protected transient Vector columnTypes   = null;

   /**
    * A Vector of Integers representing the columns lengths of this Select.
    * <p>This field is set by the handleResultMetaData() method.
    */
   protected transient Vector columnLengths = null;

   /**
    * A Vector of Integers representing the columns nullable characteristics
    * of this Select.  Possible individual values are
    * <ul>
    * <li>java.sql.ResultSetMetaData.columnNoNulls
    * <li>java.sql.ResultSetMetaData.columnNullable
    * <li>java.sql.ResultSetMetaData.columnNullableUnknown
    * </ul>
    * <p>This field is set by the handleResultMetaData() method.
    */
   protected transient Vector columnNullables = null;

   /**
    * Whether a row has been found or not.
    * <p>This field is set by the nextBackground() method.
    */
   protected transient boolean rowFound     = false;

   /**
    * A Vector of objects representing the last result row found.
    * <p>This field is set by the nextBackground() method.
    */
   protected transient Vector resultRow     = null;

   /**
    * The query's result set.
    * <p>This is only used internally.
    */
   protected transient ResultSet rs         = null;

   /**
    * The query's result set's meta data.
    * <p>This is only used internally.
    */
   protected transient ResultSetMetaData md = null;


   /**
    * Background thread fetching next row for SQL Select statement.
    */
   static protected transient final int NEXT = LASTTASK + 1;

   /**
    * Default constructor.
    */
   public Select()
   {
       // Invoke the constructor of the superclass
       super();
   }

   /**
    * Finalize this object.
    * @exception java.lang.Throwable Possibly thrown by the superclass.
    */
   protected void finalize() throws Throwable
   {
       // Ensure the column names, types, lengths and nullables get cleared
       columnNames     = null;
       columnTypes     = null;
       columnLengths   = null;
       columnNullables = null;

       // Ensure the resultRow vector variable gets cleared
       resultRow = null;

       // Clear result set instance variable
       rs = null;

       // Clear metadata instance variable
       md = null;

       // Invoke finalize method of superclass
       super.finalize();
   }

   // Property Accessors

   /**
    * Get the maximum number of rows that can be returned.
    * @return The maximum number of rows that the SQL Select statement can
    * return.
    */
   public int getMaxRowsReturned()
   {
       return maxRowsReturned;
   }

   /**
    * Set the maximum number of rows that can be returned.
    * <p>This value is used by the prepareStatement() method.
    * <p>Fire the property change event for the maxRowsReturned integer.
    * @param maxRowsReturned The maximum number of rows that the SQL Select
    * statement can return.
    */
   public synchronized void setMaxRowsReturned(int maxRowsReturned)
   {
       int old = this.maxRowsReturned;
       this.maxRowsReturned = maxRowsReturned;

       // This won't fire if the values are the same
       firePropertyChange("maxRowsReturned",
                          new Integer(old),
                          new Integer(maxRowsReturned));
   }

   /**
    * Get the maximum size of a single field that can be returned in a row.
    * @return The maximum size of a single field that can be returned in a row.
    */
   public int getMaxFieldSizeReturned()
   {
       return maxFieldSizeReturned;
   }

   /**
    * Set the maximum size of a single field that can be returned in a row.
    * <p>This value is used by the prepareStatement() method.
    * <p>Fire the property change event for the maxFieldSizeReturned integer.
    * @param maxFieldSizeReturned The maximum size of a single field that can
    * be returned in a row.
    */
   public synchronized void setMaxFieldSizeReturned(int maxFieldSizeReturned)
   {
       int old = this.maxFieldSizeReturned;
       this.maxFieldSizeReturned = maxFieldSizeReturned;

       // This won't fire if the values are the same
       firePropertyChange("maxFieldSizeReturned",
                          new Integer(old),
                          new Integer(maxFieldSizeReturned));
   }

   /**
    * Get the last Vector containing the columns' names.
    * @return The last Vector containing the columns' names.
    */
   public Vector getColumnNames()
   {
       return columnNames;
   }

   /**
    * Set the last Vector containing the columns' names.
    * <p>This method can only be called internally via the
    * handleResultMetaData() method.
    * <p>Fire the property change event for the columnNames Vector.
    * @param columnNames The last Vector containing the columns' names.
    */
   protected void setColumnNames(Vector columnNames)
   {
       Vector old = this.columnNames;
       this.columnNames = columnNames;
       firePropertyChange("columnNames",
                          old,
                          columnNames);
   }

   /**
    * Get the last Vector containing the columns' types.
    * @return The last Vector containing the columns' types.
    */
   public Vector getColumnTypes()
   {
       return columnTypes;
   }

   /**
    * Set the last Vector containing the columns' types.
    * <p>This method can only be called internally via the
    * handleResultMetaData() method.
    * <p>Fire the property change event for the columnTypes Vector.
    * @param columnTypes The last Vector containing the columns' types.
    */
   protected void setColumnTypes(Vector columnTypes)
   {
       Vector old = this.columnTypes;
       this.columnTypes = columnTypes;
       firePropertyChange("columnTypes",
                          old,
                          columnTypes);
   }

   /**
    * Get the last Vector containing the columns' lengths.
    * @return The last Vector containing the columns' lengths.
    */
   public Vector getColumnLengths()
   {
       return columnLengths;
   }

   /**
    * Set the last Vector containing the columns' lengths.
    * <p>This method can only be called internally via the
    * handleResultMetaData() method.
    * <p>Fire the property change event for the columnLengths Vector.
    * @param columnLengths The last Vector containing the columns' lengths.
    */
   protected void setColumnLengths(Vector columnLengths)
   {
       Vector old = this.columnLengths;
       this.columnLengths = columnLengths;
       firePropertyChange("columnLengths",
                          old,
                          columnLengths);
   }

   /**
    * Get the last Vector containing the columns' nulllables.
    * @return The last Vector containing the columns' nullables.
    */
   public Vector getColumnNullables()
   {
       return columnNullables;
   }

   /**
    * Set the last Vector containing the columns' nullables.
    * <p>This method can only be called internally via the
    * handleResultMetaData() method.
    * <p>Fire the property change event for the columnNullables Vector.
    * @param columnNullables The last Vector containing the columns' nullables.
    */
   protected void setColumnNullables(Vector columnNullables)
   {
       Vector old = this.columnNullables;
       this.columnNullables = columnNullables;
       firePropertyChange("columnNullables",
                          old,
                          columnNullables);
   }

   /**
    * Get the last result row Vector.
    * @return The last result row Vector.
    */
   public Vector getResultRow()
   {
       return resultRow;
   }

   /**
    * Set the last result row vector.
    * <p>This method can only be called internally via the
    * nextBackground() method.
    * <p>Fire the property change event for the resultRow Vector.
    * @param resultRow A Vector of objects representing the last result row.
    */
   protected void setResultRow(Vector resultRow)
   {
       Vector old = this.resultRow;
       this.resultRow = resultRow;
       firePropertyChange("resultRow",
                          old,
                          resultRow);
   }

   /**
    * Get a boolean that signifies whether an additional row was found or not.
    * @return Whether an additional row was found or not.
    */
   public boolean getRowFound()
   {
       return rowFound;
   }

   /**
    * Set a boolean that signifies whether an additional row was found or not.
    * <p>This method can only be called internally via the
    * nextBackground() method.
    * <p>Fire the property change event for the rowFound boolean.
    * @param rowFound Whether an additional row was found or not.
    */
   protected void setRowFound(boolean rowFound)
   {
       this.rowFound = rowFound;

       // I'm intentionally passing null as the old value so same value will
       // still be fired to listeners.
       firePropertyChange("rowFound",
                          null,
                          new Boolean(rowFound));
   }

   /**
    * Method to prepare SQL statement and rethrow any exceptions.
    * <p>The maxRowsReturned and maxFieldSizeReturned properties are used
    * by this method.
    * @exception java.sql.SQLException Thrown setting maximum rows or
    * field size.
    */
   protected void prepareStatement() throws SQLException
   {
       try
       {
           // Execute the super class's prepareStatement method
           super.prepareStatement();

           // Set the maximum number of output rows for the execute
           preparedStatement.setMaxRows(getMaxRowsReturned());

           // Set statements max field size
           preparedStatement.setMaxFieldSize(getMaxFieldSizeReturned());
       }

       // catch SQL Exceptions
       catch (SQLException sx)
       {
           // Display the exceptions and rethrow
           System.err.println("Select.prepareStatement():  " + sx);
           throw sx;
       }

   }

   /**
    * Method to handle result meta data for an SQL statement.
    * <p>The columnNames, columnTypes, columnLengths and columnNullables
    * Vectors are set by this method.
    * @exception java.sql.SQLException Thrown by the superclass or by
    * retrieving the column information from the resultSet's metadata.
    */
   protected void handleResultMetaData() throws SQLException
   {
       // If we have a prepared statement
       if (preparedStatement != null)
       {
           try
           {
               // Execute the super class's handleResultMetaData method
               super.handleResultMetaData();

               // If there is a result set
               rs = preparedStatement.getResultSet();
               if (rs != null)
               {
                   // Get the result set's metadata
                   md = rs.getMetaData();

                   // Get the count of columns
                   int columns = md.getColumnCount();

                   // Allocate new columns names, types and lengths vectors
                   Vector newColumnNames     = new Vector(columns);
                   Vector newColumnTypes     = new Vector(columns);
                   Vector newColumnLengths   = new Vector(columns);
                   Vector newColumnNullables = new Vector(columns);

                   // for each column in the result set
                   for (int index = 1;
                        index <= columns;
                        index++)
                   {
                       // Add the column's name to the name vector
                       newColumnNames.addElement(md.getColumnLabel(index));

                       // Add the column's type to the type vector
                       newColumnTypes.addElement(md.getColumnTypeName(index));

                       // Add the column's length to the length vector
                       newColumnLengths.addElement
                                        (new Integer
                                             (md.getColumnDisplaySize(index)));

                       // Add the column's nullable to the nullable vector
                       newColumnNullables.addElement
                                        (new Integer(md.isNullable(index)));
                   }

                   // Set the bound properties for column name, type, length
                   // and nullable.
                   setColumnNames(newColumnNames);
                   setColumnTypes(newColumnTypes);
                   setColumnLengths(newColumnLengths);
                   setColumnNullables(newColumnNullables);
               }
           }

           // catch SQL Exceptions
           catch (SQLException sx)
           {
               // Display and rethrow the exception
               System.err.println("Select.handleResultMetaData():  " + sx);
               throw sx;
           }
       }
   }

   /**
    * Method to retrieve the next row of an SQL result set.
    * <p>The exception, successful, rowFound and resultRow properties are
    * indirectly set by this method.
    * @see #nextBackground()
   */
   public synchronized void next()
   {
       // Signify current background task
       task = NEXT;

       // Notify the background thread
       myNotify();
   }

   /**
    * The actual background thread code that executes the close operation.
    * <p>The rowFound and resultRow properties are set by this
    * method.
    * @exception java.sql.SQLException SQL Exception
    * @exception java.lang.NullPointerException Pointer not set
    */
   protected void nextBackground() throws SQLException,
                                          NullPointerException
   {
       try
       {
           // If we have a JDBC connection
           if (JDBCConnection != null)
           {
               // If we have a preparedStatement
               if (preparedStatement != null)
               {
                   // If we have a resultSet.  We should by definition have one
                   // or prepared statement should have been null
                   if (rs != null)
                   {
                       // If there is another row to get
                       if (rs.next())
                       {
                           // Get the count of columns
                           int columns = md.getColumnCount();

                           // Instantiate a new instance of the result row
                           Vector newRow = new Vector(columns);

                           // Fill in the values of the new output vector
                           // The size of the vector is correct

                           // for each column in the result set
                           for (int index = 1;
                                index <= columns;
                                index++)
                           {
                               // Get the object in an appropriate type as
                               // per JDBC.
                               Object foo = rs.getObject(index);

                               // If the object value was null or unable to
                               // create
                               if ((rs.wasNull()) ||
                                   (foo == null))
                               {
                                   // Add a null object to the output vector
                                   newRow.addElement(null);
                               }
                               else
                               {
                                   // Add the object to the end of the
                                   // results Vector
                                   newRow.addElement(foo);
                               }
                           }
                           // endfor

                           // Set the new result row
                           setResultRow(newRow);

                           // Set that an additional row was found
                           setRowFound(true);
                       }
                       else
                       {
                           // Set that an additional row was not found
                           setRowFound(false);
                       }
                   }
               }
               else
               {
                   // Create new NullPointerException.
                   NullPointerException nx = new NullPointerException
                                                 ("Select.preparedStatement");

                   // Display exception and stack trace
                   System.err.println("Select.nextBackground():  " + nx);
                   nx.printStackTrace();

                   // Throw exception
                   throw nx;
               }
           }
           else
           {
               // Create new NullPointerException
               NullPointerException nx = new NullPointerException
                                             ("Select.JDBCConnection");

               // Display exception and stack trace
               System.err.println("Select.nextBackground():  " + nx);
               nx.printStackTrace();

               // Throw exception
               throw nx;
           }
       }

       // catch SQL Exceptions
       catch (SQLException sx)
       {
           // Display exception and a stacktrace
           System.err.println("Select.nextBackground():  " +
                              sx);
           sx.printStackTrace();

           // Rethrow exception
           throw sx;
       }
   }

   /**
    * The actual background thread code that executes the close operation.
    * <p>The columnNames, columnTypes, columnLengths, columnNullables,
    * resultRow and rowFound properties are set by this method.
    * @exception java.sql.SQLException SQL Exception
    * @exception java.lang.NullPointerException Pointer not set
    */
   protected void closeBackground() throws SQLException,
                                           NullPointerException
   {
       // if the parent's method was successful
       super.closeBackground();

       // Clear result set instance variable
       rs = null;

       // Clear metadata instance variable
       md = null;

       // Clear column metadata output:  names, types, lengths and nullables
       setColumnNames(null);
       setColumnTypes(null);
       setColumnLengths(null);
       setColumnNullables(null);

       // Clear result row
       setResultRow(null);

       // Clear row found flag
       setRowFound(false);
   }

   /**
    * Call method based on the background task to be performed.
    * The caller has guaranteed this will NOT be called for the
    * INITIAL task.
    * @exception java.sql.SQLException SQL Exception
    * @exception java.lang.NullPointerException Pointer not set
    */
   protected void performWork() throws SQLException,
                                       NullPointerException
   {
       // Switch on the task to be done
       switch (task)
       {
           // Next case - perform statement next
           case NEXT:

               nextBackground();

               break;

           // default case
           default:

               // Call the super class to perform the work
               super.performWork();

               break;
       }
   }
}
