//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbBOI
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.11 com/java/src/com/ibm/beans/samples/daps/sql/Statement.java, bbDaps, bb97c, bb97c9750e 9/29/97 16:27:55 [12/11/97 17:01:23]
package com.ibm.beans.samples.daps.sql;

import java.sql.PreparedStatement;
import java.sql.SQLException;

/**
 * The Statement class is instantiated via subclasses to represent a single
 * SQL statement.  Added functionality includes support for the
 * JDBCConnection and SQLText properties, and the general flow of the
 * execute() and close() methods.
 *
 * <p>Multiple instances of subclasses of Statement are all related to a
 * single instance of Connection.
 *
 * <p>This class is an abstract class used by the DDL and DML classes.
 *
 * @see com.ibm.beans.samples.daps.sql.Connection
 * @see com.ibm.beans.samples.daps.sql.DDL
 * @see com.ibm.beans.samples.daps.sql.DML
 * @see com.ibm.beans.samples.daps.sql.StatementBeanInfo
 *
 * @version 1.0  March 24, 1997
 * @author J. Russell Boykin
 */
public abstract class Statement extends Common
{
   // Instance variables

   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";

   /**
    * The JDBC java.sql.connection provided by a Connection object.
    * <p>This value is used by the executeBackground() and prepareStatement()
    * methods.
    * It must be set prior to the execute() method.
    * @see java.sql.Connection
    * @see com.ibm.beans.samples.daps.sql.Connection
    */
   protected transient java.sql.Connection JDBCConnection   = null;

   /**
    * The SQL statement text for this statement.
    * <p>This value is used by the prepareStatement() method.
    * This property must be set prior to the execute() method.
    */
   protected String SQLText                                 = "";

   /**
    * The java.sql.PreparedStatement encapsulated by this Statement object.
    * @see java.sql.PreparedStatement
    */
   protected transient PreparedStatement preparedStatement  = null;

   /**
    * Background thread executing SQL statement.
    */
   static protected transient final int EXECUTE    = INITIAL + 1;

   /**
    * Background thread closing SQL statement.
    */
   static protected transient final int CLOSE      = INITIAL + 2;

   /**
    * Last possible task value.
    * <p>This is used by the Select class.
    */
   static protected transient final int LASTTASK   = CLOSE;

   /**
    * Default constructor.
    */
   public Statement()
   {
       // Invoke the constructor of the superclass
       super();
   }

   /**
    * Finalize this object.
    * @exception java.lang.Throwable Possibly thrown by the superclass.
    */
   protected void finalize() throws Throwable
   {
       // Invoke finalize method of superclass
       super.finalize();
   }

   // Property Accessors

   /**
   * Returns the JDBC database JDBCConnection object from a Connection
   * object.
   * @return The java.sql.Connection object; null if not connected.
   */
   public java.sql.Connection getJDBCConnection()
   {
       return JDBCConnection;
   }

   /**
    * Set the JDBC database connection object.
    * <p>This value is used by the execute() method.
    * <p>This method will typically called via wiring or scripting
    * to set the Connection
    * object from a Connection.
    * <p>Fire the property change event for the JDBCConnection object.
    * @param JDBCConnection The JDBC database connection.
    */
   public synchronized void setJDBCConnection
                            (java.sql.Connection JDBCConnection)
   {
       java.sql.Connection old = this.JDBCConnection;
       this.JDBCConnection = JDBCConnection;
       firePropertyChange("JDBCConnection",
                          null,
                          JDBCConnection);
   }

   /**
    * Get the SQL text.
    * @return The SQL text specifying the SQL statement for this object.
    */
   public String getSQLText()
   {
       return SQLText;
   }

   /**
    * Set the SQL text.
    * <p>This value is used by the execute() method.
    * <p>Fire the property change event for the SQLText string.
    * @param SQLText The SQL text specifying the SQL statement for this object.
    */
   public synchronized void setSQLText(String SQLText)
   {
       String old = this.SQLText;
       this.SQLText = SQLText;
       firePropertyChange("SQLText",
                          old,
                          SQLText);
   }


   /**
    * Execute a statement.
    * <p>The JDBCConnection and SQLText properties are used by this method's
    * background thread operation.
    * <p>The exception and successful properties are indirectly set by this
    * method.
    * <p>This methods background thread operation invokes the
    * prepareStatement(), handleInputValues(), executeStatement() and
    * handleResultMetaData() methods
    * that are overridden by the various Statement subclasses.
    * @see #executeBackground()
    */
   public synchronized void execute()
   {
       // Signify current background task
       task = EXECUTE;

       // Notify the background thread
       myNotify();
   }

   /**
    * The actual background thread code that executes the statement.
    * <p>The JDBCConnection property is used by this method.
    * @exception java.sql.SQLException SQL Exception
    * @exception java.lang.NullPointerException Pointer not set
    */
   protected void executeBackground() throws SQLException,
                                             NullPointerException
   {
       try
       {
           // Get local JDBCConnection
           java.sql.Connection JDBCConnection = getJDBCConnection();

           // If we're connected
           if (JDBCConnection != null)
           {
               // Prepare the statement
               prepareStatement();

               // If the prepare fails, an exception will be thrown
               // and the following method calls won't be reached

               // Handle input values
               handleInputValues();

               // Execute the statement
               executeStatement();

               // Handle output metadata
               handleResultMetaData();
           }
           else
           {
               // Create new NullPointerException.
               NullPointerException nx = new NullPointerException
                                             ("Statement.JDBCConnection");

               // Display exception and stack trace
               System.err.println("Statement.executeBackground():  " + nx);
               nx.printStackTrace();

               // Throw exception
               throw nx;
           }
       }

       // catch SQL Exception
       catch (SQLException sx)
       {
           // Display exception and stack trace
           System.err.println("Statement.executeBackground():  " + sx);
           sx.printStackTrace();

           // Ensure that the prepared statement is null
           preparedStatement = null;

           // Rethrow exception
           throw sx;
       }
   }


   /**
    * Method to prepare SQL statement and rethrow any exceptions.
    * <p>The JDBCConnection property is used by this method.
    * @exception java.sql.SQLException Thrown by the JDBCConnection's
    * prepareStatement method
    */
   protected void prepareStatement() throws SQLException
   {
       try
       {
           // Create a prepared statement
           preparedStatement = JDBCConnection.prepareStatement(getSQLText());
       }

       // catch SQL exceptions
       catch (SQLException sx)
       {
           // Display exception and rethrow the error
           System.err.println("Statement.prepareStatement():  " + sx);
           throw sx;
       }
   }

   /**
    * Method to handle input values for an SQL statement.
    * @exception java.sql.SQLException Thrown by subclasses
    */
   protected void handleInputValues() throws SQLException
   {
       // Default behavior is to do nothing
   }

   /**
    * Method to execute an SQL statement.
    * @exception java.sql.SQLException Thrown by the preparedStatement's
    * execute() method.
    */
   protected void executeStatement() throws SQLException
   {
       // If we have a prepared statement
       if (preparedStatement != null)
       {
           try
           {
              // Execute the prepared statement
              preparedStatement.execute();
           }

           // catch SQL Exceptions
           catch (SQLException sx)
           {
               // Display exception and rethrow the error
               System.err.println("Statement.executeStatement():  " + sx);
               throw sx;
           }
       }
   }

   /**
    * Method to handle result meta data for an SQL statement.
    * @exception java.sql.SQLException Thrown by subclasses
    */
   protected void handleResultMetaData() throws SQLException
   {
       // Default behavior is to do nothing
   }

   /**
    * Close a statement and free the statement resources.
    * <p>The exception and successful properties are indirectly set by this
    * method.
    * @see #closeBackground()
    */
   public synchronized void close()
   {
       // Signify current background task
       task = CLOSE;

       // Notify the background thread
       myNotify();
   }

   /**
    * The actual background thread code that executes the close operation.
    * @exception java.sql.SQLException SQL Exception
    * @exception java.lang.NullPointerException Pointer not set
    */
   protected void closeBackground() throws SQLException,
                                           NullPointerException
   {
       try
       {
           // If we have a JDBC connection
           if (JDBCConnection != null)
           {
               // If we have a prepared statement
               if (preparedStatement != null)
               {
                   // Relinquish the statement's resources
                   preparedStatement.close();

                   // Clear the preparedStatement variable
                   preparedStatement = null;
               }
               else
               {
                   // Create new NullPointerException
                   NullPointerException nx = new NullPointerException
                                             ("Statement.preparedStatement");

                   // Display exception and stack trace
                   System.err.println("Statement.closeBackground():  " + nx);
                   nx.printStackTrace();

                   // Throw exception
                   throw nx;
               }
           }
           else
           {
               // Create new NullPointerException.
               NullPointerException nx = new NullPointerException
                                             ("Statement.JDBCConnection");

               // Display exception and stack trace
               System.err.println("Statement.closeBackground():  " + nx);
               nx.printStackTrace();

               // Throw exception
               throw nx;
           }
       }

       // catch SQL Exceptions
       catch (SQLException sx)
       {
           // Display exception and stack trace
           System.err.println("Statement.closeBackground():  " +
                              sx);
           sx.printStackTrace();

           // Rethrow exception
           throw sx;
       }
   }

   /**
    * Call method based on the background task to be performed.
    * The caller has guaranteed this will NOT be called for the
    * INITIAL task.
    * @exception java.sql.SQLException SQL Exception
    * @exception java.lang.NullPointerException Pointer not set
    */
   protected void performWork() throws SQLException,
                                       NullPointerException
   {
       // Switch on the task to be done
       switch (task)
       {
           // Execute case - perform statement execute
           case EXECUTE:
               executeBackground();
               break;

           // Close case - perform statement close
           case CLOSE:
               closeBackground();
               break;
       }
   }
}
