//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbBOI
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.8 com/java/src/com/ibm/beans/samples/daps/sql/Viewer.java, bbDaps, bb97c, bb97c9750e 9/29/97 16:27:23 [12/11/97 17:01:19]
package com.ibm.beans.samples.daps.sql;

import java.util.Vector;
import java.awt.TextArea;

/**
 * The Viewer class supports viewing of the result row a Select
 * statement.
 *
 * <p>To use the Viewer bean, do the following:
 * <ol>
 * <li>Create an instance of the bean.
 * <li>Fill in the bean's properties.  These are the standard properties
 * of a java.awt.TextArea object.
 * <li>Bind an existing Select bean's resultRow property
 * to this bean's additionalRow property.  This will cause execution of the
 * next() method on the Select bean to also set the additionalRow
 * property on this bean.
 * <li>Repeated uses of the Select bean's next() method will cause
 * additional rows to be displayed within the Viewer.
 * <li>Use the clear() method to clear all data from within the
 * Viewer.
 * </ol>
 *
 * @see com.ibm.beans.samples.daps.sql.Select
 * @see com.ibm.beans.samples.daps.sql.ViewerBeanInfo
 *
 * @version 1.0  March 24, 1997
 * @author J. Russell Boykin
 */
public class Viewer extends TextArea
{

   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";

   /**
    * Vector of input objects.
    * <p>These are used by the createDisplayRow method to create
    * a string representation of the SQL result row.
    */
   protected transient Vector additionalRow = null;

   /**
    * Default constructor.
    */
   public Viewer()
   {
       // Invoke the constructor of the superclass
       super();

       // Set TextArea size
       setSize(100,
               100);

       // Make the TextArea non-editable
       setEditable(false);

       // Ensure the text area is clear
       clear();
   }

   /**
    * Finalize this object.
    * @exception java.lang.Throwable Possibly thrown by the superclass.
    */
   protected void finalize() throws Throwable
   {
       // Invoke finalize method of superclass
       super.finalize();
   }

   /**
   * Returns the current input row.
   * @return Current input Vector row.
   */
   public Vector getAdditionalRow()
   {
       return additionalRow;
   }

   /**
    * Set the input row for viewing.
    * @param additionalRow The input Vector row of objects from the SQL
    * Select statement.
    */
   public synchronized void setAdditionalRow(Vector additionalRow)
   {
       // Save away the additional row
       this.additionalRow = additionalRow;

       // Convert the Vector of objects to its string representations
       String newRow = null;

       // If the input row is non-null
       if (additionalRow != null)
       {
           // Convert the Vector of objects to its string representation
           newRow = createDisplayRow();
       }
       else
       {
           // Be ready to add an empty row
           newRow = "";
       }

       // Append the row to the TextArea
       append(newRow);
   }

   /**
    * Method to convert a Vector of objects to a concatenation of their
    * string representations.
    * @return A string representation of the current row.
    */
   protected String createDisplayRow()
   {
       // Get the number of elements in the current row Vector
       Vector row = getAdditionalRow();
       int size = row.size();

       String output = "";

       // For all the items in the resultRow Vector
       for (int index = 0;
            index < size;
            index++)
       {
           // Get the object at the current location
           Object foo = row.elementAt(index);

           // If an SQL null value was added to the vector
           if (foo == null)
           {
               // add a dash to the output.  This is what the DB2
               // command line processor does.
               output += "-";
           }
           else
           {
               // Add the object's string representation to the string
               // version of the row
               output += foo.toString();
           }

           // If this isn't the last item in the row
           if (index < size - 1)
           {
               // Add a delimiter
               output += " | ";
           }
       }

       // Add in a terminating line feed
       output += "\n";

       return output;
   }

   /**
    * Method to clear out the TextArea's contents.
    */
   public synchronized void clear()
   {
       // Clear the TextArea's contents
       setText("");
   }
}
