//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbContent
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.18 com/java/src/com/ibm/beans/samples/dips/context/AnnotationContextDip.java, bbContextDip, bb97c, bb97c9750e 10/7/97 12:52:46 [12/11/97 17:01:32]
package com.ibm.beans.samples.dips.context;

import com.ibm.beans.dip.Dip;
import com.ibm.beans.dip.Dippable;
import com.ibm.beans.dip.DipRejectAdditionException;
import com.ibm.beans.dip.DipPropertyChangeListener;
import com.ibm.beans.dip.DipVetoableChangeListener;
import com.ibm.beans.dip.DipMethodCallListener;
import com.ibm.beans.dip.DipVetoableMethodListener;
import com.ibm.beans.dip.DipEventFireListener;
import com.ibm.beans.dip.DipVetoableEventListener;
import com.ibm.beans.dip.SimpleDip;
import java.io.Serializable;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.Vector;
import java.lang.reflect.Method;
import java.awt.Frame;
import java.beans.PropertyChangeEvent;


/**
 * The AnnotationContext class is a dip that allows the user to add notes to 
 * an instance of a bean. These notes are restricted to text. 
 * <p>
 * Each annotation allows multiple text notes to a single bean instance. 
 */
public final class AnnotationContextDip extends SimpleDip implements 
    ContextElement, Serializable, DipPropertyChangeListener {
                                                                      
    /**
     * Constructs an AnnotationContext instance.
     */
    public AnnotationContextDip () {
	
	myUpdatePolicy=new ContextUpdatePolicy();
	myAnnotations=new Vector();
	
    }
	
    /**
     * Gets the annotation at a specified index.
     * @param i An integer for the index of the annotation desired.
     * @return An array of Strings, the annotation at the specified index. 
     */
    public String[]     getAnnotationAt(int i) {
	return (String[])myAnnotations.elementAt(i);
    }

    /**
     * Sets the annotation data at a specific index.
     * @param s   The annotation data.
     * @param i   The element of the array at which the update is to occur.
     */
	
    public void         setAnnotationAt(String s, int i) {
	myAnnotations.insertElementAt(s,i);
    }
	
    /**
     * Adds an annotation to the bean.
     * @param s The string that contains the annotation.
     */
	  
    public void         addAnnotation(String s) {
	myAnnotations.addElement(s);
    }

    /**
     * Gets all the annotations for an instance. 
     * @return An AnnotationData object. 
     */
    public Object getContextElement() {
	return myAnnotations;
    }
    
    /**
     * Sets the AnnotationData Vector.
     * @param obj The Vector of annotation data.
     */
    public void         setContextElement(Object obj) {
	if (obj instanceof Vector ) {
	    myAnnotations = (Vector)obj;
	} else {
	    //Get this from the resource bundle
	    System.err.println(getMessage("AnnotationSetContextError"));
	}	    
    }


    /**
     * Gets the name of the Context element.
     * @return A String that identifies the kind of context.
     */
    
    public String       getContextElementName() {
	// This should be in a message file.
	String x=new String(getMessage("AnnotationDip")); 
	return x;
    }

    /**
     * Gets the update policy for the Context element.
     * @return A context policy update object.
     */
    public ContextUpdatePolicy getUpdatePolicy() {
	return myUpdatePolicy;
    }

    /**
     * Sets the update policy for the Context element.
     * @param up The context policy update object for this element.
     */
    public void         setUpdatePolicy(ContextUpdatePolicy up) {
	myUpdatePolicy=up;
    }

    /**
     * Gets the dip type name.
     * @return The type name of the dip.
     */
    public final String typeName()  {
	String x=new String(getMessage("AnnotationDip")); 
	return x;
    }



    /**
     * Creates an implementation of the dip. 
     * For the annotation context, createImplementation returns a pointer
     * to itself. 
     * @param obj The bean being dipped. 
     * @exception DipRejectAdditionException This exception is thrown if
     *		  the dip rejects being added to the bean. Annotation context
     *            does not throw this exception.
     * @return This object.
     */   
    public Object createImplementation(Object obj) throws DipRejectAdditionException {
	myDip = (Dippable)obj;
	myUpdatePolicy.initPolicyOptions(obj);
	return this;
    }


    /**
     * Returns an array of PropertyChangeListeners instances describing the method to 
     * call after the dipped bean property change method is called.
     * The annotation context dip listens for property change events itself.
     * @param imp This is the implementation returned from createImplementation().
     * @param methods[] This is the array of event methods in the dippable bean.
     * @return The array of PropertyChangeListeners. Each one is this instance 
     *         of the annotation context dip.
     * @exception DipRejectAdditionException This exception is thrown if
     *		  the dip rejects being added to the bean. Annotation context
     *            does not throw this exception.
     */
    public DipPropertyChangeListener[] createPropertyChangeListener(Object imp, String[] methods)
	 throws DipRejectAdditionException {

	DipPropertyChangeListener[] returnArray =
	    new DipPropertyChangeListener[methods.length];
	for (int i=0; i<methods.length; i++) {
	    returnArray[i] = this;
	}

	return returnArray;
    }


    /**
     * Satisfies the DipPropertyChangeListener interface.
     * @return A reference to this instance.
     */
    public Dip retrieveDip() {
	return this;
    }

    /**
     * Checks to see if the property changed is in the 
     * context update policy. If it is, the method invokes an annotation Dialog.
     * The annotation dialog allows the user to attach notes to instances of
     * a bean.
     * @param evt <dl><dd>the PropertyChangeEvent that the dip heard. This is 
     *            checked against the properties registered in the context
     *            update policy.</dd></dl>
     */
    public synchronized void propertyChange(PropertyChangeEvent evt) {
	//Compare the event with UpdatePolicy
	String[] policyNames=myUpdatePolicy.getUpdatePolicy();

	boolean match=false;
	for (int i=0; i<policyNames.length; i++) {
	    if (policyNames[i].equals(evt.getPropertyName())) match=true;
	}
	
	//if it is customized to ask then pop up the Annotation Dialog.
	if (match) {
	    Frame f = new Frame();
	    AnnotationDialog a = new AnnotationDialog(this, f,
 			getMessage("AnnotationDialogTitle"));
	    a.show();
	}
	//The dialog is modal and it attaches the annotation results
	// to the vector myAnnotations, by means of the setAnnotation method.
    } 

    private String getMessage(String key) {

	if (myContextResourceBundle==null) {
	    try {
		myContextResourceBundle=(ContextResourceBundle)
		    ResourceBundle.getBundle("com.ibm.beans.samples.dips.context.ContextResourceBundle");
	    } catch (MissingResourceException x) {
		System.err.println("*** Fatal Error for Annotation Context Dip - No Resource Bundle ***");
		System.exit(fileNotFoundError);
	    }
	}

	return myContextResourceBundle.getString(key);
    }

    private Vector myAnnotations;
    private String myAuthor;
    private Dippable myDip;
    private ContextUpdatePolicy myUpdatePolicy;
    private transient ContextResourceBundle myContextResourceBundle;
    private static final int fileNotFoundError=28;

    private static final long serialVersionUID = 2000L;
    private static final String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}
