//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbContent
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.16 com/java/src/com/ibm/beans/samples/dips/context/AuthorContextDip.java, bbContextDip, bb97c, bb97c9750e 11/5/97 16:13:02 [12/11/97 17:01:30]
package com.ibm.beans.samples.dips.context;

import com.ibm.beans.dip.Dip;
import com.ibm.beans.dip.Dippable;
import com.ibm.beans.dip.DipRejectAdditionException;
import com.ibm.beans.dip.DipPropertyChangeListener;
import com.ibm.beans.dip.DipVetoableChangeListener;
import com.ibm.beans.dip.DipMethodCallListener;
import com.ibm.beans.dip.DipVetoableMethodListener;
import com.ibm.beans.dip.DipEventFireListener;
import com.ibm.beans.dip.DipVetoableEventListener;
import com.ibm.beans.dip.EventFireListener;
import com.ibm.beans.dip.MethodCallListener;
import com.ibm.beans.dip.SimpleDip;
import com.ibm.beans.dip.VetoableEventListener;
import com.ibm.beans.dip.VetoableMethodListener;
import java.io.Serializable;
import java.util.Vector;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.ListResourceBundle;
import java.lang.reflect.Method;
import java.awt.Frame;
import java.beans.PropertyChangeEvent;


/**
 * The AuthorContext allows a user or application assembly expert to put an author 
 * name on an instance of a bean. The update policy provides the assembly specialist
 * with a way to identify when the authorship of an instance changes.
 *
 */
public final class AuthorContextDip extends SimpleDip 
          implements Dip, ContextElement, Serializable,  DipPropertyChangeListener {
    /**
     * Constructs an AuthorContextDip.
     */
    public AuthorContextDip () {
	
	myContextUpdatePolicy=new ContextUpdatePolicy();
	myAuthors=new Vector();

    }

    /**
     * Get the names of the author, a convenience method, equivalent to
     * getContextElement.
     * @return A vector of strings representing the names of the authors.
     */
    public Vector getAuthors() { 
	return myAuthors;
    }

    /**
     * Adds an author's name to the list of authors.
     * @param name A String representing the name of the author to be added.
     */
    public void addAuthor(String name) {
	myAuthors.addElement(name);
    }

    /**
     * Sets the names of the authors, a convenience method, equivalent to
     * setContextElement.
     * @param names A Vector of Strings representing the names of the authors.
     */
    public void setAuthors(Vector names) {
	myAuthors=names;
    }
    
	      
    /**
     * Gets the names of the authors as the context element.
     * @return An object representing the names of the author.
     */
    public Object getContextElement() {
	return myAuthors;
    }
    /**
     * Sets the names of the authors as the context element.
     * @param o An object which is a String representing the names of the authors.
     */
    public void         setContextElement(Object o) {
	if (o instanceof Vector) {
	    myAuthors = (Vector)o;
	} else {
	    System.err.println(getMessage("AuthorSetContextError"));
	}
	    
    }
    /**
     * Gets the name of the Context element.
     * @return A String that identifies the kind of context.
     */
    public String       getContextElementName() {
	String x=new String(getMessage("AuthorDip")); 
	return x;
    }
    /**
     * Gets the update policy for the context element.
     * @return A context policy update object
     */
    public ContextUpdatePolicy getUpdatePolicy() {
	return myContextUpdatePolicy;
    }
    
    /**
     * Sets the update policy for the context element.
     * @param up The context policy update object for this element.
     */
    
    public void         setUpdatePolicy(ContextUpdatePolicy up) {
	myContextUpdatePolicy=up;
    }

    /**
     * Returns the dip type name as a String.
     * @return The type name of the dip.
     */
    public final String typeName()  {
	String x=new String(getMessage("AuthorDip")); 
	return x;
    }


    /**
     * Creates the dip implementation, and provides the dip access to the
     * dippable bean.
     * @param obj The bean being dipped. 
     * @exception DipRejectAdditionException This exception is thrown if
     *		  the dip rejects being added to the bean. Author context
     *            does not throw this exception.
     */    
    public Object createImplementation(Object obj) throws DipRejectAdditionException {
	myDip = (Dippable)obj;
	myContextUpdatePolicy.initPolicyOptions(obj);
	return this;
    }

    /**
     * Returns an array of PropertyChangeListeners describing the method to 
     * call after the dipped bean's property change method is called.
     * <p>
     * For context dips this returns the list of PropertyChangeListeners that 
     * correspond to the update policy.
     * @param imp This is the implementation returned from createImplementation
     * @param methods[] This is the array of event methods in the dippable bean.
     * @exception DipRejectAdditionException This exception is thrown if
     *		  the dip rejects being added to the bean. Author context
     *            does not throw this exception.
     * @return The array of PropertyChangeListeners is a one-to-one mapping
     *		with the methods[] input parameter.  This means
     *		PropertyChangeListener[i] will be used with methods[i], and
     *		the length of the two arrays are equal.
     */
    public DipPropertyChangeListener[] createPropertyChangeListener(Object imp, String[] methods)
	 throws DipRejectAdditionException {
	DipPropertyChangeListener[] returnArray =
	    new DipPropertyChangeListener[methods.length];
	for (int i=0; i<methods.length; i++) {
	    returnArray[i] = this;
	}

	return returnArray;
    }
    

    /**
     * Provides the framework with access to the dip.
     * @return A reference to this instance.
     */
    public Dip retrieveDip() {
	return this;
    }

  
    /**
     * Checks to see if the property changed is in the 
     * context update policy. If it is, it invokes an author dialog.
     * The author dialog allows the user to attach a new author to 
     * an instance of a bean.
     * @param evt The PropertyChangeEvent that the dip heard. This is 
     *            checked against the properties registered in the context
     *            update policy.
     */
    public synchronized void propertyChange(PropertyChangeEvent evt) {
	//Compare the event with UpdatePolicy
	String[] policyNames=myContextUpdatePolicy.getUpdatePolicy();

	boolean match=false;
	for (int i=0; i<policyNames.length; i++) {
	    if (policyNames[i].equals(evt.getPropertyName())) match=true;
	}
	
	//if it is customized to ask then pop up the Author Dialog.
	if (match) {
	    Frame f = new Frame();
	    AuthorDialog a = new AuthorDialog(this, f,
					      getMessage("AuthorDialogTitle"));
	    a.show();
	}
    } 


    private String getMessage(String key) {

	if (myContextResourceBundle==null) {
	    try {
		myContextResourceBundle=(ContextResourceBundle)
		    ResourceBundle.getBundle("com.ibm.beans.samples.dips.context.ContextResourceBundle");
	    } catch (MissingResourceException x) {
		System.err.println("*** Fatal Error for Author Context Dip - No Resource Bundle ***");
		System.exit(fileNotFoundError);
	    }
	}

	return myContextResourceBundle.getString(key);
    }

    private Vector myAuthors;
    private Dippable myDip;
    private ContextUpdatePolicy myContextUpdatePolicy;
    private transient ContextResourceBundle myContextResourceBundle;
    private static final int fileNotFoundError=28;

    private static final long serialVersionUID = 2000L;  
    private static final String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}
