//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbContent
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.16 com/java/src/com/ibm/beans/samples/dips/context/ContextUpdatePolicy.java, bbContextDip, bb97c, bb97c9750e 11/6/97 12:25:54 [12/11/97 17:01:31]
package com.ibm.beans.samples.dips.context;

import com.ibm.beans.dip.Dip;
import com.ibm.beans.dip.Dippable;
import com.ibm.beans.dip.DipRejectAdditionException;
import com.ibm.beans.dip.DipPropertyChangeListener;
import com.ibm.beans.dip.DipVetoableChangeListener;
import com.ibm.beans.dip.DipMethodCallListener;
import com.ibm.beans.dip.DipVetoableMethodListener;
import com.ibm.beans.dip.DipEventFireListener;
import com.ibm.beans.dip.DipVetoableEventListener;
import com.ibm.beans.dip.EventFireListener;
import com.ibm.beans.dip.EventFireEvent;
import com.ibm.beans.dip.MethodCallListener;
import com.ibm.beans.dip.MethodCallEvent;
import com.ibm.beans.dip.VetoableEventListener;
import com.ibm.beans.dip.VetoableMethodListener;
import java.beans.PropertyChangeEvent;
import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.beans.EventSetDescriptor;
import java.io.Serializable;
import java.util.Vector;
import java.util.Hashtable;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.lang.reflect.Method;

/**
 * ContextUpdatePolicy class keeps a record of the methods that will  cause
 * a context element to change. Each context element has an update policy as
 * a private member and provides getters and setters so ContextDip can manage
 * the update policy. 
 */

public class ContextUpdatePolicy implements Serializable {

    /**
     * Constructs a ContextUpdatePolicy.
     */
    public ContextUpdatePolicy () {
	try {
	    myContextResourceBundle=(ContextResourceBundle)
		ResourceBundle.getBundle("com.ibm.beans.samples.dips.context.ContextResourceBundle");
	} catch (MissingResourceException x) {
	    System.err.println("*** Fatal Error for ContextUpdatePolicy - No Resource Bundle ***");
	    System.exit(fileNotFoundError);
	}
    }


    /**
     * Provides access to the available properties that can be
     * used to trigger an annotation.
     * @return String[]. An array of the properties as strings.
     */
    public String[] getUpdatePolicyOptions () {

	//Check a transient to see if it needs to be restored.
	if (myPropertyOptions==null) restoreTransients();

	String[] aListOfPropertiesAsStrings=new String[myPropertyOptions.length];
	
	if (myPropertyOptions.length > 0) {
	    for (int i = 0; i < myPropertyOptions.length; i++) {
		aListOfPropertiesAsStrings[i] = myPropertyOptions[i].getName();
		myPropertiesMap.put(aListOfPropertiesAsStrings[i],
				  myPropertyOptions[i]);
	    }
	}
	    
	return aListOfPropertiesAsStrings;
    }

    /**
     * Provides access to the properties that trigger an annotation.
     * @return String[]. An array of the properties as strings.
     */
    public String[] getUpdatePolicy () {

	//Check a transient to see if it needs to be restored.
	if (myUpdateProperties==null) restoreTransients();

	String[] returnArray= new String[myUpdateProperties.length];
	for (int i=0; i < myUpdateProperties.length; i++) {
	    returnArray[i]=myUpdateProperties[i].getName();
	}
	return returnArray;
    }
    
    /**
     * Sets the properties in the context policy using a String array.
     * @param updatePolicyAsStrings The array of strings representing
     *        the properties in the context update policy.
     */
    public void setUpdatePolicy (String[] updatePropertiesAsStrings) {
	
	myUpdatePropertiesAsStrings=updatePropertiesAsStrings;

	myUpdateProperties = new
	    PropertyDescriptor[updatePropertiesAsStrings.length];

	//Check the transient myPropertiesMap to see if it needs to be restored.
	if (myPropertiesMap==null) restoreTransients();

	//Create vector myUpdateProperties
	for (int i = 0; i < updatePropertiesAsStrings.length; i++) {
	    myUpdateProperties[i]=(PropertyDescriptor)
				   myPropertiesMap.get(updatePropertiesAsStrings[i]);
	}
    } 



    /**
     * Initializes the introspector to provide information
     * on what properties are available for a context update
     * policy.
     * @param bean The bean that the context dip listens to for property changes.
     */
    public void initPolicyOptions (Object bean) {

	
	myBeanForPolicy = bean;

	try {
	    BeanInfo bi = Introspector.getBeanInfo(myBeanForPolicy.getClass());
	    myPropertyOptions = bi.getPropertyDescriptors();
	    
	}
	catch (IntrospectionException ex) {
	    System.err.println(
		myContextResourceBundle.getString("IntrospectionFailedInfo"));
	}

    }

    /**
     * Provides access to the properties which trigger a context change
     * @param s A string which represents the property descriptor. 
     * @return PropertyDescriptor, the property descriptor for the string
     *         provided.
     */
    public PropertyDescriptor getPDforString (String s) {
	return (PropertyDescriptor)myPropertiesMap.get(s);
    }

    private void restoreTransients() {
	//Message resource bundle
	if (myContextResourceBundle==null) {
	    try {
		myContextResourceBundle=(ContextResourceBundle)
		    ResourceBundle.getBundle("com.ibm.beans.samples.dips.context.ContextResourceBundle");
	    } catch (MissingResourceException x) {
		System.err.println("*** Fatal Error for ContextUpdatePolicy  - No Resource Bundle ***");
		System.exit(fileNotFoundError);
	    }
	}

	//If dipped bean didn't stream in, no sense restoring context policy 
	if (myBeanForPolicy!=null) {

	    //reflect on dipped bean to get property descriptors
	    //and make myPropertyOptions
	    try {
		BeanInfo bi = Introspector.getBeanInfo(myBeanForPolicy.getClass());
		myPropertyOptions = bi.getPropertyDescriptors();
	    
	    }
	    catch (IntrospectionException ex) {
		System.err.println(
		   myContextResourceBundle.getString("IntrospectionFailedInfo"));
	    }

	    //make myPropertiesMap
	    myPropertiesMap = new Hashtable();
	    if (myPropertyOptions.length > 0) {
		for (int i = 0; i < myPropertyOptions.length; i++) {
		    myPropertiesMap.put(myPropertyOptions[i].getName(),
					myPropertyOptions[i]);
		}
	    }

	    //setUpdatePolicy
	    myUpdateProperties = new
		PropertyDescriptor[myUpdatePropertiesAsStrings.length];
	    for (int i = 0; i < myUpdatePropertiesAsStrings.length; i++) {
		myUpdateProperties[i]=(PropertyDescriptor)
		    myPropertiesMap.get(myUpdatePropertiesAsStrings[i]);
	    }
	}
    }


    private transient PropertyDescriptor[] myUpdateProperties=new PropertyDescriptor[0];
    private transient PropertyDescriptor[] myPropertyOptions=new PropertyDescriptor[0];
    private String[] myUpdatePropertiesAsStrings;
    private Object myBeanForPolicy;
    private transient Hashtable myPropertiesMap= new Hashtable();
    private transient ContextResourceBundle myContextResourceBundle;
    private static final int fileNotFoundError=28;

    private static final long serialVersionUID = 2000L;
    private static final String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}

