//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbContent
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.11 com/java/src/com/ibm/beans/samples/dips/context/VersionContextCustomizer.java, bbContextDip, bb97c, bb97c9750e 11/4/97 09:45:53 [12/11/97 17:01:33]
package com.ibm.beans.samples.dips.context;

import java.lang.reflect.Method;
import java.awt.Panel;
import java.awt.Choice;
import java.awt.Label;
import java.awt.List;
import java.awt.Button;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.util.Vector;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.beans.Customizer;
import java.beans.PropertyChangeListener;
import java.beans.IntrospectionException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * The VersionContextCustomizer customizes a version context object.
 * This customization provides a facility for changing the update policy 
 * which the dip uses to determine when the version changes. 
 * 
 */
public class VersionContextCustomizer extends Panel
   implements Customizer, ItemListener, ActionListener  {
       
    // Panel overrides

       /**
	* Construct a VersionContextCustomizer.
	*/
    public VersionContextCustomizer () {
	
	try {
	    myContextResourceBundle=(ContextResourceBundle)
		ResourceBundle.getBundle("com.ibm.beans.samples.dips.context.ContextResourceBundle");
	} catch (MissingResourceException x) {
	    System.err.println("*** Fatal Error for Version Context Dip - No Resource Bundle ***");
	    System.exit(fileNotFoundError);
	}

	//Version on only one thing at a time
	//This is a limitation of introspection
	myPolicyOptionsList= new List(linesVisible, false);

	myApplyButton= new Button((String)
		myContextResourceBundle.getString("ApplyButtonLabel"));
	myApplyButton.setActionCommand("ApplyButton");
	myApplyButton.addActionListener(this);


	versionLabel=new Label(
		myContextResourceBundle.getString("VersionDialogCaption"));
	                                           
	myVersionChoice = new Choice();
	myVersionChoice.addItemListener(this);
	
    }
       

   // Customizer interface
       
   /**
    * Sets the object instance with which the version customizer interacts.
    * Once the object is set, the proper GUI is determined and created.
    *
    * @param bean	The bean instance to customize
    */
    public void setObject(Object bean) {


	myDip = (VersionContextDip) bean;      
        setLayout( new BorderLayout() );
      
	Label l=new Label((String)
		myContextResourceBundle.getString("VersionPolicyOptionLabel"));	
	add("North", l);

	Panel c=new Panel();
	c.setLayout(new BorderLayout());		

	//Build the List entries from the Dip's Policy object. 
	String[] opts=myDip.getUpdatePolicy().getUpdatePolicyOptions();
	String[] selectedOpts=myDip.getUpdatePolicy().getUpdatePolicy();
	int i=0; //i is used as an index into the selectedOpts array.


	if (opts.length>0) {
	    myReadyFlag=true;
	    for (int j=0;j<opts.length;j++) {
		myPolicyOptionsList.addItem(opts[j]);
		if (selectedOpts.length>i) {
		    if (opts[j].equals(selectedOpts[i])) {
			myPolicyOptionsList.select(j);
			i++;
		    }
		}
	    }
	} else {
	    myPolicyOptionsList.addItem(
			myContextResourceBundle.getString("NoProperties"));
	}
	
	c.add("Center",myPolicyOptionsList);


	Panel soPanel=new Panel();
	soPanel.setLayout( new FlowLayout(java.awt.FlowLayout.CENTER) );
	soPanel.add(myApplyButton);
	c.add("South",soPanel);

	add("Center",c);


	Panel s=new Panel();
	s.setLayout(new FlowLayout());
	s.add(versionLabel);
	Vector v=myDip.getVersions();
	for (int j=0;j<v.size(); j++) {
	    Long id=new Long(((VersionData)v.elementAt(j)).getVersionID());
	    myVersionChoice.addItem(id.toString());
	}
	s.add(myVersionChoice);
	add("South",s);

    }
       
   /**
    * Adds a listener to get notified whenever the customizer changes the
    * bean in a significant manner. A significant change is a policy update.
    *
    * @param listener	Object to be notified of changes.
    */
    public void addPropertyChangeListener(PropertyChangeListener listener) {
	propertyChangeSupport.addPropertyChangeListener( listener );
    }
       
   /**
    * Removes a listener from the notification list.
    *
    * @param listener	Object to be removed from the notification list.
    */
    public void removePropertyChangeListener(PropertyChangeListener listener) {
	propertyChangeSupport.removePropertyChangeListener( listener );
    }

    /**
     * Changes the version as the item selected in the list box changes.
     * 
     * @param itmEvt The item state changed event information.
     *
     */
       
    public void itemStateChanged(ItemEvent itmEvt) {
	if (myDip!=null) {
	    Long l=Long.valueOf(myVersionChoice.getSelectedItem());
	    myDip.changeVersion(l.longValue());
	}
    }

    /**
     * Responds to the buttons which the
     * user may press in the dialog. 
     * @param event The event that provides information on which,
     *              if any, button was pressed.
     */
    public void actionPerformed( ActionEvent event ) {

        String actionCommand = event.getActionCommand();

	if ( myApplyButton.getActionCommand().equals(actionCommand)) {
	    if (myReadyFlag) {
		//get the selections from the list box
		String[] name= myPolicyOptionsList.getSelectedItems();
		
		//Set the updatePolicy for the dip
		myDip.getUpdatePolicy().setUpdatePolicy(name);
	    }
        }
    }


    public Dimension getPreferredSize() {
        Dimension d=new Dimension(preferedWidth, preferedHeight);
	return d;
    }

    private boolean myReadyFlag=false;
    private List   myPolicyOptionsList;
    private Button myApplyButton;
    private Label   versionLabel;
    private Choice myVersionChoice;
    private VersionContextDip myDip;
    private ContextResourceBundle myContextResourceBundle;
    private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);

    private static final int preferedWidth=300;
    private static final int preferedHeight=200;

    private static final int linesVisible=4;
    private static final int fileNotFoundError=28;
    private static final String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}
