//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbContent
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.21 com/java/src/com/ibm/beans/samples/dips/context/VersionContextDip.java, bbContextDip, bb97c, bb97c9750e 11/4/97 16:22:28 [12/11/97 17:01:33]
package com.ibm.beans.samples.dips.context;

import com.ibm.beans.dip.Dip;
import com.ibm.beans.dip.Dippable;
import com.ibm.beans.dip.DipRejectAdditionException;
import com.ibm.beans.dip.DipPropertyChangeListener;
import com.ibm.beans.dip.DipVetoableChangeListener;
import com.ibm.beans.dip.DipMethodCallListener;
import com.ibm.beans.dip.DipVetoableMethodListener;
import com.ibm.beans.dip.DipEventFireListener;
import com.ibm.beans.dip.DipVetoableEventListener;
import com.ibm.beans.dip.EventFireListener;
import com.ibm.beans.dip.MethodCallListener;
import com.ibm.beans.dip.SimpleDip;
import com.ibm.beans.dip.VetoableEventListener;
import com.ibm.beans.dip.VetoableMethodListener;
import java.io.Serializable;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.Vector;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.lang.IllegalAccessException;
import java.awt.Frame;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyDescriptor;


/**
 * Version Context allows  end user or assembly expert to keep track of the 
 * version of an instance of a bean. The update policy determines when the version
 * changes. The update policy is set at dip time.
 */
public final class VersionContextDip extends SimpleDip implements  
    ContextElement, Serializable, DipPropertyChangeListener {


    /**
     * Constructs a VersionContext.
     */
    public VersionContextDip () {

	myContextUpdatePolicy=new ContextUpdatePolicy();
	myVersions=new Vector();
	currentVersionID=initialVersionValue;
	
    }

    /**
     * Gets the version context element.
     * @return A Vector containing the version information.
     */
    public Vector   getVersions() { 
	//Verison is not a string (it is probably a vector
	//of property descriptors), but for this purpose a string 
	// will suffice.

	return myVersions;
    }

	/**
	 * Gets the current version identifier.
	 * @return A long, the current version identifier.
	 */
    public long getCurrentVersionID() {
	return currentVersionID;
    }


    /**
     * Gets version data for an instance. 
     * @return A Vector of version data, as an Object.
     */
    public Object       getContextElement() {
	return myVersions;
    }

    /**
     * Sets the version data for an instance.
     * @param o A VersionData object
     */
    public void         setContextElement(Object o) {
	if (o instanceof Vector) {
	    //Probably need to pull this apart ot get the info piece by piece.
	    myVersions = (Vector)o;
	} else {
	    System.err.println(getMessage("VersionSetContextError")); 
	}
    }

    /**
     * Gets the context element name, <tt>version</tt> in this case.
     * @return  The string <tt>version</tt>, representing the context element name.
     */
    public String       getContextElementName() {
	String x=new String(getMessage("VersionDip")); 
	return x;
    }

    /**
     * Gets the update policy for this context element.
     * @return A context policy update object
     */
    public ContextUpdatePolicy getUpdatePolicy() {
	return myContextUpdatePolicy;
    }

    /**
     * Sets the update policy for this context element.
     * @param up The context policy update object for this element.
     */
    public void         setUpdatePolicy(ContextUpdatePolicy up) {
	myContextUpdatePolicy=up;
    }


    /**
     * Returns the dip type name as a String.
     * @return The type name of the dip.
     */
    public final String typeName()  {
	String x=new String(getMessage("VersionDip")); 
	return x;
    }


    /**
     * Returns a pointer to this dip. 
     *
     * @param obj The bean being dipped.
     * @exception DipRejectAdditionException This exception is thrown if
     *		  the dip rejects being added to the bean.
     */   
    public Object createImplementation(Object obj) throws DipRejectAdditionException {
	myDip = (Dippable)obj;
	myContextUpdatePolicy.initPolicyOptions(obj);
	return this;
    }



   /**
     * Returns an array of PropertyChangeListeners describing the method to 
     * call after the dipped bean's property change method is called.
     * 
     * The version context dip listens for property change events itself.
     * @param imp This is the implementation returned from createImplementation
     * @param methods[] This is the array of event methods in the dippable bean.
     * @exception DipRejectAdditionException This exception is thrown if
     *		  the dip rejects being added to the bean.
     * @return The array of PropertyChangeListeners, each one is this instance 
     *         of the version context dip.
     */
    public DipPropertyChangeListener[] createPropertyChangeListener(Object imp, String[] methods) 
	 throws DipRejectAdditionException {

	DipPropertyChangeListener[] returnArray =
	    new DipPropertyChangeListener[methods.length];
	for (int i=0; i<methods.length; i++) {
	    returnArray[i] = this;
	}

	return returnArray;
    }
    

    /**
     * Satisfies the DipPropertyChangeListener interface.
     * @return a reference to this instance.
     */
    public Dip retrieveDip() {
	return this;
    }

    /**
     * Checks to see if the property changed is in the 
     * context update policy. If it is, it updates the version data.
     * @param evt the PropertyChangeEvent that the dip heard. This is 
     *            checked against the properties registered in the context
     *            update policy.
     */
    public synchronized void propertyChange(PropertyChangeEvent evt) {

	//When the user changes version don't create a new version
	if (changingVersion) return;

	//Compare the event with UpdatePolicy

	//A zero length array saves us from a null pointer exception.
	String[] policyNames=new String[0];
	if (myContextUpdatePolicy != null) {
	    policyNames=myContextUpdatePolicy.getUpdatePolicy();
	}


	boolean match=false;
	for (int i=0; i<policyNames.length; i++) {
	    if (policyNames[i].equals(evt.getPropertyName())) match=true;
	}


	if (match) {
	    String propertyName = evt.getPropertyName();
	    Object value=evt.getNewValue();
	    VersionData v=new VersionData(currentVersionID, propertyName, value);
	    myVersions.addElement(v);
	    currentVersionID++;	    
	}
    }

	/**
	 * Change the version based on the version identifier.
	 * @param versionID a long identifying the version the user wants to change to.
	 */
    public synchronized void changeVersion(long versionID) {
	changingVersion=true;
	VersionData v=getVersion(versionID);
	PropertyDescriptor p =
	    myContextUpdatePolicy.getPDforString(v.getPropertyName());
	Method m = p.getWriteMethod();
	//This next, unusual line makes a one element array of the value
	//To match the parameters to Method.invoke() 
	Object[] values={v.getValue()};
	
	try {
	    m.invoke(myDip, values);
	} catch (InvocationTargetException itEx) {
	    System.err.println(getMessage("InvokeFailedInfo"));
	} catch (IllegalAccessException ilEx) {
	    System.err.println(getMessage("IllegalAccessInfo"));
	}
	changingVersion=false;
	    
    }

	/*
	  Gets the current version identifier as a long.
	  versionID the versionID for the version data sought.
	  This method returns the version data for the supplied
	  identifier.
	 */
	
    private VersionData getVersion (long versionID) {

	for (int i=0; i<myVersions.size(); i++) {
	    VersionData v=(VersionData)myVersions.elementAt(i);
	    if (v.getVersionID()==versionID) {
		return v;
	    }
	}
	return null;
    }


    private String getMessage(String key) {

	if (myContextResourceBundle==null) {
	    try {
		myContextResourceBundle=(ContextResourceBundle)
		    ResourceBundle.getBundle("com.ibm.beans.samples.dips.context.ContextResourceBundle");
	    } catch (MissingResourceException x) {
		System.err.println("*** Fatal Error for Annotation Context Dip - No Resource Bundle ***");
		System.exit(fileNotFoundError);
	    }
	}

	return myContextResourceBundle.getString(key);
    }

    private Vector myVersions;
    private String myAuthor;
    private Dippable myDip;
    private ContextUpdatePolicy myContextUpdatePolicy;
    private transient static ContextResourceBundle myContextResourceBundle;
    private long currentVersionID;

    private boolean changingVersion=false;

    private static final int fileNotFoundError=28;
    private static final long serialVersionUID = 2000L;
    private static final long initialVersionValue = 1L;
    
    private static final String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}
