//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbContent
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.17 com/java/src/com/ibm/beans/samples/dips/license/HostAndDurationLicensePolicy.java, bbLicenseDip, bb97c, bb97c9750e 11/18/97 17:52:00 [12/11/97 17:01:37]
package com.ibm.beans.samples.dips.license;

import java.io.Serializable;
import java.lang.Error;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.util.Date;
import java.util.ResourceBundle;
import com.ibm.beans.samples.dips.license.Messages;

/**
 * This class implements methods that restict running to a particular
 * host name up to a date when the license expires.
 */
public class HostAndDurationLicensePolicy implements Serializable {

  /**
   * Constructs a HostAndDurationLicensePolicy object.
   */
  public HostAndDurationLicensePolicy() { }

  /**
   * Copy constructor.  Constructs a copy of a license policy object.
   *
   * @param p	policy instance to copy
   */
  HostAndDurationLicensePolicy(HostAndDurationLicensePolicy p) {

    license = null;		// will be initialized when used
    privateKey = p.privateKey;
    publicKey = p.publicKey;
    encodedSignature = p.encodedSignature;
    hostname = p.hostname;
    expires = p.expires;
  }

  /**
   * Gets the <i>PrivateKey</i> property of the license policy.
   *
   * @return The private key property object.
   */
  public PrivateKey getPrivateKey() { return privateKey; }

  /**
   * Sets the <i>PrivateKey</i> property of the license policy.  Only one of the
   * <i>PrivateKey</i> and <i>PublicKey</i> properties can be set at any time.  A key
   * property cannot be set more than once. The
   * keys must be generated using a KeyPairGenerator
   * which uses the DSA algorithm.  See <tt>java.security.KeyPairGenerator</tt> for
   * details.
   *
   * @param key	The private key to which the property is set.
   * @exception InvalidKeyException	This method throws this exception if
   *		the public key is already set, the private key is
   *		already set, or an invalid key object is given.
   */
  public synchronized void setPrivateKey(PrivateKey key) throws InvalidKeyException {
    // cannot set a key if any key has already been set
    if (publicKey != null || privateKey != null) {
      throw new InvalidKeyException();
    }

    privateKey = key;
  }

  /**
   * Gets the <i>PublicKey</i> property of the license policy.
   *
   * @return The public key property object.
   */
  public PublicKey getPublicKey() { return publicKey; }

  /**
   * Sets the <i>PublicKey</i> property of the license policy.  Only one of the
   * <i>PrivateKey</i> and <i>PublicKey</i> properties can be set at any time.  A key
   * property cannot be set more than once.  The
   * keys must be generated using a KeyPairGenerator
   * which uses the DSA algorithm.  See <tt>java.security.KeyPairGenerator</tt> for
   * details.
   *
   * @param key	The public key to which the property is set.
   * @exception InvalidKeyException	This method throws this exception if
   *		the public key is already set, the private key is
   *		already set, or an invalid key object is given.
   */
  public synchronized void setPublicKey(PublicKey key) throws InvalidKeyException {
    // cannot set a key if any key has already been set
    if (publicKey != null || privateKey != null) {
      throw new InvalidKeyException();
    }

    publicKey = key;

    // reset the hostname property
    hostname = null;
  }

  /**
   * Gets the X.509 encoded signature.  The encoded
   * signature is obtained from the <a href="#sign_method">sign</a> method.
   *
   * @return A byte array containing the X.509 encoded signature.
   */
  public byte[] getEncodedSignature() { return encodedSignature; }

  /**
   * Sets the encoded signature property of the license policy.
   * This information is required to verify a license and is obtained
   * from the <a href="#sign_method">sign</a> method.
   *
   * @param ec[]	The X.509 encoded signature.
   */
  public void setEncodedSignature(byte[] ec) {
    encodedSignature = ec;
  }

  /**
   * Gets the host name property of the license policy.  The host name
   * property restricts execution to the specified host.  <p>The property
   * always returns the name of the host the bean is running on if the public
   * key property is set.
   *
   * @return A string containing the host name property.
   */
  public String getHost() {
    // if hostname is not set and public key is set, then just get the
    // current hostname.
    if (hostname == null && publicKey != null) {
      try {
	hostname = InetAddress.getLocalHost().getHostName();
      }
      catch (UnknownHostException e) {
	hostname = "localhost";
      }
    }
    
    return hostname;
  }

  /**
   * Sets the host name property of the license policy.  The host name
   * property restricts execution to the specified host.  The setting
   * of this property is ignored if the policy object has a public key
   * property set.  Setting the public key resets the host name. 
   *
   * @param host	The host name that allows the bean to run.
   */
  public void setHost(String host) {
    // property is only settable without a public key
    if (publicKey == null) {
      hostname = host;
    }
  }

  /**
   * Gets the license expiration date property of the license policy.
   * The verify method will not return true past the specified date.
   *
   * @return A Date containing the expiration date of the license property.
   */
  public Date getExpirationDate() { return expires; }

  /**
   * Sets the license expiration date property of the license policy.
   * The verify method does not return true past the specified date.
   *
   * @param d	The expiration date of the license.
   */
  public void setExpirationDate(Date d) {
    expires = d;
  }

  // delegated Signature methods

  /**
   * Encapsulates the 
   * update methods of the Signature class to ensure the proper data
   * is being validated.  It should be overriden by subclasses to
   * define what data is validated for the license.  It is called for
   * both signing and verifying a license.
   *
   * @exception SignatureException	The method throws
   *		a signature exception if the license policy is not properly
   *		initialized.
   */
  protected void update() throws SignatureException {
    // if we don't have a license object by now, something is wrong.
    if (license == null) {
      getMessages();
      throw new Error((String)messages.handleGetObject("no_sig"));
    }

    // if we aren't initialized
    if (expires == null) {
      getMessages();
      throw new SignatureException((String)messages.handleGetObject("no_init"));
    }

    // convert expiration date to byte array and update the signature
    String canonicalDate = expires.toString();
    byte[] ba = canonicalDate.getBytes();
    license.update(ba);
    
    // convert hostname to byte array and update the signature
    // we use getHost() here to handle programmatic data
    license.update(getHost().getBytes());
  }

  /**
   * Creates a license based on current property
   * settings.  <a name="sign_method">This is done by a license issuer.
   *
   * @return A byte array containing the X.509 encoded signature for the
   *		newly created license.
   * @exception SignatureException	This method will throw a
   *		signature exception
   *		if the license policy is not properly initialized.
   */
  public byte[] sign() throws SignatureException {
    // ensure we have a license to sign
    if (license == null) {
      try {
	license = Signature.getInstance("DSA");
      }
      catch (NoSuchAlgorithmException e) {
	getMessages();
	throw new Error((String)messages.handleGetObject("no_DSA"));
      }
    }

    // install the private key since we only sign with it
    if (privateKey == null) {
      getMessages();
      throw new SignatureException((String)messages.handleGetObject("no_privkey"));
    }
    try {
      license.initSign(privateKey);
    }
    catch (InvalidKeyException e) {
      getMessages();
      throw new Error((String)messages.handleGetObject("inv_privkey"));
    }

    // update will the proper data
    update();

    // return result of signing
    return license.sign();
  }

  /**
   * Validates a license based on the current property
   * settings.
   *
   * @return <tt>true</tt> if license is valid; <tt>false</tt> otherwise.
   * @exception java.security.SignatureException	This method
   *		throws a signature exception
   *		if the license policy is not properly initialized.
   */
  public boolean verify() throws SignatureException { 
    // ensure we have a license to verify
    if (license == null) {
      try {
	license = Signature.getInstance("DSA");
      }
      catch (NoSuchAlgorithmException e) {
	getMessages();
	throw new Error((String)messages.handleGetObject("no_DSA"));
      }
    }

    // install the public key since we only verify with it
    if (publicKey == null) {
      getMessages();
      throw new SignatureException((String)messages.handleGetObject("no_pubkey"));
    }
    try {
      license.initVerify(publicKey);
    }
    catch (InvalidKeyException e) {
      getMessages();
      throw new Error((String)messages.handleGetObject("inv_pubkey"));
    }

    // update with the proper data
    update();

    // if no license, we give up
    if (!license.verify(encodedSignature))
      return false;

    // no need to check hostname, it was checked as part of
    // the license
    
    // license verifies, but has it expired?
    if (new Date().after(getExpirationDate()))
      return false;

    return true;
  }

  private void getMessages() {
    if (messages == null) {
      messages = (Messages)ResourceBundle.getBundle("com.ibm.beans.samples.dips.license.Messages");
    }
  }

  private static final long serialVersionUID = 2000L;
  
  // transient data
  private transient Signature license;
  private transient Messages messages;

  // keys to retain
  private PrivateKey	privateKey;
  private PublicKey	publicKey;

  // encoded signature for license
  private byte[]	encodedSignature;

  // hostname for the license
  private String	hostname;

  // expiration date
  private Date		expires;

  private final static String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}
