//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbContent
//
//   CLASSES: none
//
//   ORIGINS: 27,180
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.18 com/java/src/com/ibm/beans/samples/dips/license/HostAndDurationLicensePolicyCustomizer.java, bbLicenseDip, bb97c, bb97c9750e 11/18/97 17:52:01 [12/11/97 17:01:39]
//
// file save dialog source comes from JavaSoft beanbox source.
//
package com.ibm.beans.samples.dips.license;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Customizer;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.SignatureException;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.ResourceBundle;
import com.ibm.beans.samples.dips.license.Messages;

/**
 * The HostAndDurationLicensePolicyCustomizer serves three purposes depending on the
 * state of the object it is customizing.  The three states of the object
 * are:
 * <ul>
 * <li> Initialization
 * <li> Issue license
 * <li> Customer
 * </ul>
 * The customizer presents a different GUI and customizes the object
 * differently depending on the state.  What happens in each state is:
 * <dl>
 * <dt>Initialization
 * <dd>During this state, the customizer presents an initialization GUI
 *	that initializes the license policy with proper keys and saves
 *	a clone of the policy for later use by license issuers.
 * <dt>Issue license
 * <dd>GUI for retrieving customer information such as the name of the host the bean
 *	runs on and the expiration date of the license.  Then, an encoded
 *	signature is generated and the information given to the customer.
 * <dt>Customer
 * <dd>GUI for entering the encoded license information from the license
 *	issuer.
 */
public class HostAndDurationLicensePolicyCustomizer extends Panel implements Customizer {

  // ---------- Customizer interface ----------

  /**
   * Sets the object instance with which this customizer is to interact.
   * Once the object is set, the proper GUI is determined and created.
   *
   * @param bean	The bean instance to customize.
   */
  public void setObject(Object bean) {
    // retain reference to the policy object
    policy = (HostAndDurationLicensePolicy)bean;
    
    // go through mode determination checks
    // and create the appropriate GUI
    if (policy.getPrivateKey() != null) {
      guiMode = kIssuer;
      createIssuerGUI();
    }
    else if (policy.getPublicKey() != null) {
      guiMode = kCustomer;
      createCustomerGUI();
    }
    else {
      guiMode = kInitialize;
      createInitializeGUI();
    }
  }

  // ---------- Bound Properties ----------

  /**
   * Adds a listener to get notified when the customizer changes the
   * bean in a significant manner.  Significant changes include changes
   * to the property values of the license policy.
   *
   * @param listener	The object to be notified of changes.
   */
  public void addPropertyChangeListener(PropertyChangeListener listener) {
    support.addPropertyChangeListener(listener);
  }

  /**
   * Removes a listener from the notification list.
   *
   * @param listener	The object to be removed from the notification list.
   */
  public void removePropertyChangeListener(PropertyChangeListener listener) {
    support.removePropertyChangeListener(listener);
  }

  // ---------- Initialization GUI methods ----------

  // this method is not designed for subclassing
  private void createInitializeGUI() {
    // need more control over this UI
    GridBagLayout layout = new GridBagLayout();
    setLayout(layout);

    GridBagConstraints gbc = new GridBagConstraints();
    // gbc.fill = GridBagConstraints.BOTH;
    gbc.weightx = 1.0;

    // create title
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    getMessages();
    Label title = new Label((String)messages.handleGetObject("b_lic_init"));
    layout.setConstraints(title, gbc);
    add(title);

    // vertical white space
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    Label whitespace = new Label("");
    layout.setConstraints(whitespace, gbc);
    add(whitespace);
  
    // create text clustered in center
    gbc.weightx = 0.0;
    Label text = new Label((String)messages.handleGetObject("txt_1"));
    layout.setConstraints(text, gbc);
    add(text);
    text = new Label((String)messages.handleGetObject("txt_2"));
    layout.setConstraints(text, gbc);
    add(text);
    text = new Label((String)messages.handleGetObject("txt_3"));
    layout.setConstraints(text, gbc);
    add(text);
    
    // vertical white space
    gbc.weightx = 1.0;
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    gbc.fill = GridBagConstraints.NONE;
    whitespace = new Label("");
    layout.setConstraints(whitespace, gbc);
    add(whitespace);
  
    // create button
    Button saveIssuerButton = new Button((String)messages.handleGetObject("save_issuer"));
    saveIssuerButton.addActionListener(new SaveIssuerListener());
    layout.setConstraints(saveIssuerButton, gbc);
    add(saveIssuerButton);
  }

  /**
   * Gets the preferred width and height of the customizer.
   *
   * @return The preferred dimensions of the customizer.
   */
  public Dimension getPreferredSize() {
    Dimension x = new Dimension(400,300);
    return x;
  }

  // called to set the appropriate keys in the appropriate policy objects
  void setPolicyKeys() {
    // first create the keys necessary to put into the policies
    KeyPairGenerator keyGen = null;
    try {
      keyGen = KeyPairGenerator.getInstance("DSA");
    }
    catch (NoSuchAlgorithmException e) {
      getMessages();
      throw new Error((String)messages.handleGetObject("no_DSA"));
    }
    keyGen.initialize(512);	// minimal strength, this is a sample
    KeyPair keyPair = keyGen.generateKeyPair();

    // "clone" the policy object
    clonePolicy = new HostAndDurationLicensePolicy(policy);

    try {
      clonePolicy.setPrivateKey(keyPair.getPrivate());
      policy.setPublicKey(keyPair.getPublic());
    }
    catch (InvalidKeyException e) {
      // assert
      getMessages();
      System.err.println((String)messages.handleGetObject("inv_keypair"));
    }
  }

  // Sets keys in both policy objects, saves, issuer's policy object.
  class SaveIssuerListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      // setup the policies so they are ready to save
      setPolicyKeys();
      
      // The following is necessary to create a file
      // dialog, select a file to write to, ensure that it can be
      // created or opened, open it for writing, make sure it is
      // written to as an object stream, streaming the license
      // issuer's policy clone object and then closing the stream.
      // It is basically copied from JavaSoft's beanbox Save() method.
      String fname = null;
      java.awt.Component c = (java.awt.Component)e.getSource();
      while (c != null && !(c instanceof java.awt.Frame)) c = c.getParent();
      getMessages();
      FileDialog fd = new FileDialog((java.awt.Frame)c , (String)messages.handleGetObject("save_issuer_obj"), FileDialog.SAVE);
      // the setDirectory() is not needed, except for a bug under Solaris...
      fd.setDirectory(System.getProperty("user.dir"));
      // Solaris currently ignores the setFile()
      fd.setFile("HDLIssuer.ser");
      fd.show();
      fname = fd.getFile();
      // if cancelled
      if (fname == null) {
	return;
      }
      String dname = fd.getDirectory();

      try {
	// create the single ObjectOutputStream
	File serFile = new File(dname, fname);
	File dir = new File(serFile.getParent());
	if (dir != null) {
	  dir.mkdirs();
	}
	FileOutputStream f = new FileOutputStream(serFile);
	ObjectOutputStream oos = new ObjectOutputStream(f);
	oos.writeObject(clonePolicy);
	oos.close();
      }
      catch (Exception ex) {
	getMessages();
	System.err.println((String)messages.handleGetObject("save_issuer_failed"));
      }
    }
  }

  // ---------- Issuer GUI methods ----------

  /**
   * Creates a customizable GUI for the issuance of licenses.  This
   * GUI includes capabilities to load existing issuer license policy
   * objects that were serialized during initialization.  It must
   * allow the entering of the same data that the license will
   * validate, allow for the generation of a new license (calling
   * sign() on the policy object), and the distribution of the encoded
   * signature to the customer using some means.
   */
  protected void createIssuerGUI() {
    // need more control over this UI
    GridBagLayout layout = new GridBagLayout();
    setLayout(layout);

    GridBagConstraints gbc = new GridBagConstraints();
    // gbc.fill = GridBagConstraints.BOTH;
    gbc.weightx = 1.0;

    // title
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    getMessages();
    Label title = new Label((String)messages.handleGetObject("HDL_issuer"));
    layout.setConstraints(title, gbc);
    add(title);

    // vertical white space
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    Label whitespace = new Label("");
    layout.setConstraints(whitespace, gbc);
    add(whitespace);
  
    // text item for hostname
    gbc.gridwidth = 1;
    gbc.weightx = 0.0;
    gbc.fill = GridBagConstraints.HORIZONTAL;
    Label hn = new Label((String)messages.handleGetObject("hn_prompt"));
    layout.setConstraints(hn, gbc);
    add(hn);
    gbc.weightx = 1.0;
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    hostname = new TextField(policy.getHost(), 30);
    layout.setConstraints(hostname, gbc);
    add(hostname);

    // text item for expiration date
    gbc.gridwidth = 1;
    Label exp = new Label((String)messages.handleGetObject("exp_prompt"));
    layout.setConstraints(exp, gbc);
    add(exp);
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    expirationDate = new TextField(30);
    layout.setConstraints(expirationDate, gbc);
    add(expirationDate);

    // give it a default value to use for formatting example
    expirationDate.setText(DateFormat.getDateInstance(DateFormat.LONG).format(new Date()));

    // vertical white space
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    gbc.fill = GridBagConstraints.NONE;
    whitespace = new Label("");
    layout.setConstraints(whitespace, gbc);
    add(whitespace);
  
    // generate license button
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    Button b1 = new Button((String)messages.handleGetObject("b_genlic"));
    layout.setConstraints(b1, gbc);
    add(b1);
    b1.addActionListener(new GenerateLicenseListener());
  }

  // Generates encoded signature and saves it.
  class GenerateLicenseListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      // set the hostname property
      String h = hostname.getText();
      policy.setHost(h);
      // is it necessary to fire two property changes for one button evt?
      support.firePropertyChange("", null, null);

      try {
	// set the expiration date property
	String dateString = expirationDate.getText();
	DateFormat df = DateFormat.getDateInstance(DateFormat.LONG);
	Date expires = df.parse(dateString);
	// bug? in java.text.DateFormat returns null on bad parse
	// rather than throw the exception.  we must do it manually.
	if (expires == null) {
	  throw new ParseException("unknown parse problem", 0);
	}
	policy.setExpirationDate(expires);
	support.firePropertyChange("", null, null);
	
	// sign the license
	byte[] encsig = policy.sign();
	
	// The following is necessary to create a file
	// dialog, select a file to write to, ensure that it can be
	// created or opened, open it for writing, streaming the signature
	// and then closing the stream.
	// It is basically copied from JavaSoft's beanbox Save() method.
	String fname = null;
	java.awt.Component c = (java.awt.Component)e.getSource();
	while (c != null && !(c instanceof java.awt.Frame)) c = c.getParent();
	getMessages();
	FileDialog fd = new FileDialog((java.awt.Frame)c , (String)messages.handleGetObject("save_sig"), FileDialog.SAVE);
	// the setDirectory() is not needed, except for a bug under Solaris...
	fd.setDirectory(System.getProperty("user.dir"));
	// Solaris currently ignores the setFile()
	fd.setFile("signature.dat");
	fd.show();
	fname = fd.getFile();
	// if cancelled
	if (fname == null) {
	  return;
	}
	String dname = fd.getDirectory();
	
	try {
	  // create the single FileOutputStream
	  File sigFile = new File(dname, fname);
	  File dir = new File(sigFile.getParent());
	  if (dir != null) {
	    dir.mkdirs();
	  }
	  FileOutputStream f = new FileOutputStream(sigFile);
	  DataOutputStream dos = new DataOutputStream(f);
	  dos.writeInt(encsig.length);
	  dos.write(encsig, 0, encsig.length);
	  dos.close();
	}
	catch (Exception ex) {
	  getMessages();
	  System.err.println((String)messages.handleGetObject("save_sig_failed"));
	}
      }
      catch (ParseException ex) {
	getMessages();
	System.err.println((String)messages.handleGetObject("no_parse"));
      }
      catch (SignatureException ex) {
	System.err.println("PrintLicenseDip: "+ex.getMessage());
      }
    }
  }

  // Converts ASN.1/DER byte encoding to hex string.
  String ASN1DERToString(byte[] encoding) {
    String hex = "";

    for (int i = 0; i < encoding.length; i++) {
      // make it easier to read
      if ((i % 4) == 0 && i != 0) {
	hex += " ";
      }

      // convert signed byte to "unsigned" integer
      int byteVal = encoding[i];
      if (byteVal < 0) {
	byteVal = java.lang.Math.abs(byteVal) + 127;
      }

      // convert unsigned int to hex
      String hexByte = Integer.toHexString(byteVal);
      if (hexByte.length() == 1) {
	hex += "0"+hexByte;
      }
      else {
	hex += hexByte;
      }
    }
    return hex;
  }
  
  // ---------- Customer GUI methods ----------

  /**
   * Creates the GUI that customers use to update their
   * license policy with appropriate information from the license
   * issuer.  This information includes the presentation of programmatically
   * determined data, such as current host name, and input of non-programmatic data.
   * Non-programmatic data include dates and encoded license signatures.
   */
  protected void createCustomerGUI() {
    // need more control over this UI
    GridBagLayout layout = new GridBagLayout();
    setLayout(layout);

    GridBagConstraints gbc = new GridBagConstraints();
    // gbc.fill = GridBagConstraints.BOTH;
    gbc.weightx = 1.0;

    // title
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    getMessages();
    Label title = new Label((String)messages.handleGetObject("HDL"));
    layout.setConstraints(title, gbc);
    add(title);

    // vertical white space
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    Label whitespace = new Label("");
    layout.setConstraints(whitespace, gbc);
    add(whitespace);
  
    // text item for expiration date
    gbc.gridwidth = 1;
    Label exp = new Label((String)messages.handleGetObject("exp_prompt"));
    layout.setConstraints(exp, gbc);
    add(exp);
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    expirationDate = new TextField(30);
    layout.setConstraints(expirationDate, gbc);
    add(expirationDate);

    // give it a default value to use for formatting example
    Date d = policy.getExpirationDate();
    if (d == null) {
      d = new Date();
    }
    expirationDate.setText(DateFormat.getDateInstance(DateFormat.LONG).format(d));

    // text item for encoded signature
    gbc.gridwidth = 1;
    gbc.fill = GridBagConstraints.NONE;
    Label sig = new Label((String)messages.handleGetObject("enc_sig_prompt"));
    layout.setConstraints(sig, gbc);
    add(sig);
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    gbc.fill = GridBagConstraints.HORIZONTAL;
    encodedSignature = new TextField(40);
    layout.setConstraints(encodedSignature, gbc);
    add(encodedSignature);

    // show current setting
    byte[] encsig = policy.getEncodedSignature();
    if (encsig != null) {
      encodedSignature.setText(ASN1DERToString(encsig));
    }

    // vertical white space
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    gbc.fill = GridBagConstraints.NONE;
    whitespace = new Label("");
    layout.setConstraints(whitespace, gbc);
    add(whitespace);
  
    // import signature button
    gbc.gridwidth = 1;
    Button b2 = new Button((String)messages.handleGetObject("b_import"));
    layout.setConstraints(b2, gbc);
    add(b2);
    b2.addActionListener(new ImportSignatureListener());
    
    // save license button
    gbc.gridwidth = GridBagConstraints.REMAINDER;// end of row
    Button b1 = new Button((String)messages.handleGetObject("b_setlic"));
    layout.setConstraints(b1, gbc);
    add(b1);
    b1.addActionListener(new SetLicenseListener());
  }
    
  // Sets expiration date property on policy.  Hostname isn't needed
  // since it is programmatic data.  The encoded signature is handled
  // by the import button.
  class SetLicenseListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      try {
	// set expiration date property
	String dateString = expirationDate.getText();
	DateFormat df = DateFormat.getDateInstance(DateFormat.LONG);
	Date expires = df.parse(dateString);
	policy.setExpirationDate(expires);
      }
      catch (ParseException ex) {
	getMessages();
	System.err.println((String)messages.handleGetObject("no_parse"));
      }
    }
  }

  // Sets the encoded signature property on the policy.
  class ImportSignatureListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      // The following is necessary to create a file
      // dialog, select a file to read from, ensure that it can be
      // opened, open it for reading, streaming the signature
      // and then closing the stream.
      // It is basically copied from JavaSoft's beanbox Save() method.
      String fname = null;
      java.awt.Component c = (java.awt.Component)e.getSource();
      while (c != null && !(c instanceof java.awt.Frame)) c = c.getParent();
      getMessages();
      FileDialog fd = new FileDialog((java.awt.Frame)c , (String)messages.handleGetObject("load_sig"), FileDialog.LOAD);
      // the setDirectory() is not needed, except for a bug under Solaris...
      fd.setDirectory(System.getProperty("user.dir"));
      // Solaris currently ignores the setFile()
      fd.setFile("signature.dat");
      fd.show();
      fname = fd.getFile();
      // if cancelled
      if (fname == null) {
	return;
      }
      String dname = fd.getDirectory();
	
      try {
	// create the single FileInputStream
	File sigFile = new File(dname, fname);
	FileInputStream f = new FileInputStream(sigFile);
	DataInputStream dis = new DataInputStream(f);
	int eclen = dis.readInt();
	byte[] encsig = new byte[eclen];
	dis.readFully(encsig);
	dis.close();

	encodedSignature.setText(ASN1DERToString(encsig));
	policy.setEncodedSignature(encsig);
	support.firePropertyChange("", null, null);
      }
      catch (Exception ex) {
	getMessages();
	System.err.println((String)messages.handleGetObject("import_failed"));
      }
    }
  }

  // must be package access for action listeners
  void getMessages() {
    if (messages == null) {
      messages = (Messages)ResourceBundle.getBundle("com.ibm.beans.samples.dips.license.Messages");
    }
  }

  // GUI modes
  private static final short kInitialize = 1;	// needs initialization of keys
  private static final short kIssuer = 2;	// interface for license issuing
  private static final short kCustomer = 3;	// interface for customer

  // the following data members are package access so that the
  // inner class action listeners can access them.

  // property change listener data
  transient PropertyChangeSupport support = new PropertyChangeSupport(this);

  transient Messages messages;

  // license policy objects
  HostAndDurationLicensePolicy policy;
  HostAndDurationLicensePolicy clonePolicy;

  // what mode is the customizer running in
  short guiMode;

  // GUI controls
  TextField hostname;
  TextField expirationDate;
  TextField encodedSignature;
  private final static String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}
