//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbLicenseDip
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.20 com/java/src/com/ibm/beans/samples/dips/license/PrintLicenseDip.java, bbLicenseDip, bb97c, bb97c9750e 11/18/97 18:06:37 [12/11/97 17:01:35]
package com.ibm.beans.samples.dips.license;

import java.awt.Component;
import java.lang.String;
import java.io.Serializable;
import java.util.ResourceBundle;

import com.ibm.beans.dip.SimpleDip;
import com.ibm.beans.dip.DipVetoableMethodListener;
import com.ibm.beans.dip.DipRejectAdditionException;
import com.ibm.beans.dip.MethodVetoException;
import com.ibm.beans.dip.MethodCallEvent;

/**
 * This class implements a simple form of licensing dip by restricting
 * access to print and printAll methods of a java.awt.Component
 * instance.  This class can only be applied to objects that are instances of
 * java.awt.Component.
 * <p>
 * The licensing policy chosen for this dip ensures that the
 * printing is enabled only on specific hosts and for a specific
 * duration.
 * <p>
 * See the <i>License Dip Usage Guide</i> for information regarding how to
 * setup and use a license dip.
 */
public class PrintLicenseDip extends SimpleDip implements DipVetoableMethodListener, Serializable {

  /**
   * Constructs a new print restricting license dip.  The constructor
   * handles instantiating a license policy.  The owner of the
   * dip is responsible for ensuring proper customization
   * by invoking the customizer for this class.
   */
  public PrintLicenseDip() {
    policy = new HostAndDurationLicensePolicy();
  }

  /**
   * Get the policy object used by the dip.  Only used by the customizer.
   *
   * @return the policy object
   */
  HostAndDurationLicensePolicy getPolicy() {
    return policy;
  }

  // overrides from SimpleDip

  /**
   * Defines this dip to be related to licensing of print
   * function where the license restricts use to a particular
   * host for a particular duration.
   *
   * @return The type name of the dip.
   */
  final public String typeName() { return kDipTypeName; }

  /**
   * Allows removal of licensing.
   * When an object is licensed, it must never be allowed to remove
   * licensing.  This method always returns false.
   *
   * @return <tt>true</tt> if removal allowed; <tt>false</tt> otherwise.
   */
  public boolean isRemovable() { return false; }

  /**
   * Determines if a method should be included in a morphed class.
   * When the dip is applied to a class as a pre-dip during morphing,
   * and the user elects to only override those methods in the dippable
   * class that the pre-dips agrees to, this method is used to determine
   * if a dip agrees to including a specific method.
   * <p>
   * The only methods of the dippable bean that the PrintLicenseDip needs to be
   * wrapped are the print and printAll methods.
   *
   * @param methodName Name of the method to be considered
   * @return boolean indicating whether to wrap given method
   */
  public boolean includeInMorphedClass(String methodName) {
     boolean includeThisMethod = false;

     if ((methodName.indexOf("print") != -1) || (methodName.indexOf("printAll") != -1)) {
        includeThisMethod = true;
     } /* endif */

     return includeThisMethod;
  }

  /**
   * Checks the object to which the dip is being applied to
   * ensure that the object is an awt.compenent.  If it is
   * not, then an exception is thrown and the application of the dip
   * is rejected.
   * <p>
   * @param obj	The bean being dipped.
   * @return <code>null</code> (no other implementation necessary).
   * @exception com.ibm.beans.dip.DipRejectAdditionException This method will throw
   *		this exception if
   *		<i>obj</i> is not a java.awt.Component.
   */
  public Object createImplementation(Object obj) throws DipRejectAdditionException {

    // this dip can only be applied if the license has been initialized
    if (policy.getPublicKey() == null) {
      // error reporting done by dipping fw
      getMessages();
      throw new DipRejectAdditionException((String)messages.handleGetObject("no_apply"));
    }

    // this dip can only be applied to java.awt.Components
    if (obj instanceof Component) {
      return null;
    }
    else {
      // error reporting done by dipping fw
      getMessages();
      throw new DipRejectAdditionException((String)messages.handleGetObject("not_Component"));
    }
  }

  /**
   * Creates the dip to receive notification when the print and printAll methods
   * are called.  If either of these methods are called, the dip may veto
   * the methods if the license is not
   * valid.
   *
   * @param imp	The implementation returned by createImplementation.
   * @param methods[] The array of methods in the dippable
   *		bean.
   * @return The array of DipVetoableMethodListeners with the dip itself set to
   *		listen at the appropriate method index to receive
   *		print and printAll calls.
   * @exception com.ibm.beans.dip.DipRejectAdditionException This method will throw
   *		this exception if the dippable bean's print or printAll methods were not
   *     wrapped when it was morphed.
   */
  public DipVetoableMethodListener[] createVetoableMethodListener(Object imp, String[] methods) throws DipRejectAdditionException {

    // call SimpleDip first to ensure we have properly created the
    // listener array
    super.createVetoableMethodListener(imp, methods);

    int count=0;
    // set myself as the listener to the print and printAll methods
    for (int i = 0; i < vetoableMethodListeners.length; i++) {
      if ( methods[i].endsWith("print(java.awt.Graphics)") ) {
	vetoableMethodListeners[i] = this;
	count++;
      }
      else if ( methods[i].endsWith("printAll(java.awt.Graphics)") ) {
	vetoableMethodListeners[i] = this;
	count++;
      }
    }

    if (count!=2) {
      // error reporting done by dipping fw
      getMessages();
      throw new DipRejectAdditionException((String)messages.handleGetObject("methods_not_wrapped"));
    }

    return vetoableMethodListeners;
  }

  // DipVetoableMethodListener Interface

  /**
   * Indicates whether method calls need arguments.
   * This method states that the dip does not care about arguments for any
   * method calls.
   *
   * @return <tt>true</tt> if arguments needed; <tt>false</tt> otherwise.
   */
  public boolean needsArguments() { return false; }

  /**
   * Checks for a valid license and vetos if the license is
   * invalid.
   *
   * @param e	Describes the method that was called.
   * @exception MethodVetoException This method throws this exception if
   *		the license is
   *		not valid.
   */
  public void handleVetoableMethod(MethodCallEvent e) throws MethodVetoException {
    try {
      if (!policy.verify()) {
	getMessages();
	String s = (String)messages.handleGetObject("terms_cond");
	System.err.println(s);
	throw new MethodVetoException(s, e);
      }
    }
    catch (java.security.SignatureException ex) {
      System.err.println("PrintLicenseDip: "+ex.getMessage());
      throw new MethodVetoException("PrintLicenseDip: "+ex.getMessage(), e);
    }
  }

  /**
   * Recovers from a print veto.
   *
   * @param e	Description of the particular method that was vetoed.
   */
  public void recoverFromMethodVeto(MethodCallEvent e) {
    // this method does nothing because there's nothing to do
    // if a print method is vetoed.
  }

  private void getMessages() {
    if (messages == null) {
      messages = (Messages)ResourceBundle.getBundle("com.ibm.beans.samples.dips.license.Messages");
    }
  }

  private static final long serialVersionUID = 2000L;
  private static String kDipTypeName = "Print License Dip";

  private HostAndDurationLicensePolicy policy;

  private transient Messages messages;

  private final static String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}

