//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbPrintDip
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved - Patent(s) Pending
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.13 com/java/src/com/ibm/beans/samples/dips/print/PrintButtonFrame.java, bbPrintDip, bb97c, bb97c9750e 10/24/97 17:52:53 [12/11/97 17:01:46]
package com.ibm.beans.samples.dips.print;

import java.awt.Button;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Frame;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

/**
 * <p>
 * This class is a support class used by the Print Dip.
 * This class implements a small dialog that contains a <b>Print</b> button.
 * When the button is clicked, a callback to the dip is made which causes
 * the dippable bean to be printed.
 * <p>
 * This class is basically the view side of the dip.
 * The model is primarily in the PrintDip class.
 *
 * @author John W. Alcorn (jalcorn@us.ibm.com)
 * @see com.ibm.beans.samples.dips.print.PrintDip
 */
public class PrintButtonFrame extends Frame implements ActionListener, ComponentListener, WindowListener {

  /**
   * Constructs a new <b>Print</b> button dialog.  The dialog's listeners are attached
   * and the <b>Print</b> button is created and wired to deliver events to the dialog.
   * It is the caller's responsibility to size, position, and show the dialog.
   *
   * @param dip The dip to which this frame belongs.
   * @param title The text to appear in the frame's title bar.
   * @param buttonText The label for the <b>Print</b> button.
   *
   * @see com.ibm.beans.samples.dips.print.PrintDip#createPrintButtonFrame
   */
   public PrintButtonFrame(PrintDip dip, String title, String buttonText)
   {
      // Let our superclass initialize itself
      super(title); // set the frame's title bar text

      // Initialize internal variables
      printDip = dip;

      // Attach appropriate listeners
      addWindowListener(this); // allows us to receive the windowClosing event (we ignore the others)
      addComponentListener(this); // we need to let the dip know about move and resize events

      setLayout(null); // we do a custom layout in our doLayout() method

      // Create and wire the <b>Print</b> button
      printButton = new Button(buttonText);
      add(printButton);
      printButton.addActionListener(this);

      // For some reason, the following line not only changes PrintButtonFrame's icon image,
      // but also the icon image of the frame hosting the dippable bean (such as the Visual
      // Layout sub-assembly view, and the main Assembly Surface window).

   // setIconImage(getToolkit().getImage(getClass().getResource("PrintDipIconColor16.gif")));
   }


  /**
   * Sizes and positions the <b>Print</b> button within the frame.
   * If the frame is resized, the button dynamically resizes
   * itself to be 5 pels from each edge.
   */
   public void doLayout()
   {
      Dimension mySize = getSize();
      Insets insets = getInsets();

      printButton.setBounds(insets.left+BUTTON_INSET,
                            insets.top+BUTTON_INSET,
                            mySize.width-(insets.left+insets.right+2*BUTTON_INSET),
                            mySize.height-(insets.top+insets.bottom+2*BUTTON_INSET));
   }


  /**
   * Provides access to the <b>Print</b> button within this frame.
   * The dip uses this method to access the <b>Print</b> button
   * in order to modify button-related properties.
   *
   * @return The <b>Print</b> button.
   */
   public Button getPrintButton()
   {
      return printButton;
   }

  // Methods from ActionListener interface

  /**
   * Responds to the <b>Print</b> button being pressed.  This method
   * calls the printDippableBean method of the dip.
   *
   * @param evt Describes the action that occurred.
   *
   * @see com.ibm.beans.samples.dips.print.PrintDip#printDippableBean
   */
   public void actionPerformed(ActionEvent evt)
   {
      printDip.printDippableBean();
   }

  // Methods from ComponentListener interface

  /**
   * Notification that the <b>Print</b> button frame has been moved.
   * The Print Dip is notified by setting its <i>printButtonFrameXPos</i>
   * and/or <i>printButtonFrameYPos</i> properties.  Each property is
   * only set if it actually changed.
   *
   * @param evt Describes the event that occurred.
   *
   * @see com.ibm.beans.samples.dips.print.PrintDip#setPrintButtonFrameXPos
   * @see com.ibm.beans.samples.dips.print.PrintDip#setPrintButtonFrameYPos
   */
   public void componentMoved(ComponentEvent evt)
   {
      Point myLocation = getLocation();
      short xPos = (short) myLocation.x;
      short yPos = (short) myLocation.y;

      if ((xPos!=oldXPos) || (yPos!=oldYPos)) {
         printDip.lockPrintButtonFrame(true); //keeps annoying flickers from happening
         if (xPos!=oldXPos) {
            oldXPos = xPos;
            printDip.setPrintButtonFrameXPos(xPos);
         } /* endif */

         if (yPos!=oldYPos) {
            oldYPos = yPos;
            printDip.setPrintButtonFrameYPos(yPos);
         } /* endif */
         printDip.lockPrintButtonFrame(false);
      } /* endif */
   }


  /**
   * Notification that the <b>Print</b> button frame has been resized.
   * The Print Dip is notified by setting its <i>printButtonFrameWidth</i>
   * and/or <i>printButtonFrameHeight</i> properties.  Each property is
   * only set if it actually changed.
   *
   * @param evt Describes the event that occurred.
   *
   * @see com.ibm.beans.samples.dips.print.PrintDip#setPrintButtonFrameWidth
   * @see com.ibm.beans.samples.dips.print.PrintDip#setPrintButtonFrameHeight
   */
   public void componentResized(ComponentEvent evt)
   {
      Dimension mySize = getSize();
      short width = (short) mySize.width;
      short height = (short) mySize.height;

      if ((width!=oldWidth) || (height!=oldHeight)) {
         printDip.lockPrintButtonFrame(true); //keeps annoying flickers from happening
         if (width!=oldWidth) {
            oldWidth = width;
            printDip.setPrintButtonFrameWidth(width);
         } /* endif */

         if (height!=oldHeight) {
            oldHeight = height;
            printDip.setPrintButtonFrameHeight(height);
         } /* endif */
         printDip.lockPrintButtonFrame(false);
      } /* endif */
   }


  /**
   * Notification that the <b>Print</b> button frame has been shown.
   * No action is taken; the <tt>ComponentListener</tt> interface requires this method.
   *
   * @param evt Describes the event that occurred.
   */
   public void componentShown(ComponentEvent evt) {}


  /**
   * Notification that the <b>Print</b> button frame has been hidden.
   * No action is taken; the <tt>ComponentListener</tt> interface requires this method.
   *
   * @param evt Describes the event that occurred.
   */
   public void componentHidden(ComponentEvent evt) {}

  // Methods from WindowListener interface

  /**
   * Notification that the <b>Print</b> button frame is being closed.
   * This event is fired in response to the title bar <b>Close</b> button
   * being pressed (or </b>Close</b> being chosen from the frame's system menu).
   * Rather than actually closing the window, it is simply hidden (so that it
   * can be shown again if requested).  Also, rather than directly hiding it,
   * the <i>showPrintButtonFrame</i> property of the PrintDip is set to
   * <tt>false</tt> (deferring the implementation of what to do when this
   * property is modified to the dip).
   *
   * @param evt Describes the action that occurred.
   *
   * @see com.ibm.beans.samples.dips.print.PrintDip#setShowPrintButtonFrame
   */
   public void windowClosing(WindowEvent evt)
   {
      printDip.setShowPrintButtonFrame(false);
   }


  /**
   * Notification that the <b>Print</b> button frame has been created.
   * No action is taken; the <tt>WindowListener</tt> interface requires this method.
   *
   * @param evt Describes the event that occurred.
   */
   public void windowOpened(WindowEvent evt) {}


  /**
   * Notification that the <b>Print</b> button frame has been destroyed.
   * No action is taken; the <tt>WindowListener</tt> interface requires this method.
   *
   * @param evt Describes the event that occurred.
   */
   public void windowClosed(WindowEvent evt) {}


  /**
   * Notification that the <b>Print</b> button frame has been activated.
   * No action is taken; the <tt>WindowListener</tt> interface requires this method.
   *
   * @param evt Describes the event that occurred.
   */
   public void windowActivated(WindowEvent evt) {}


  /**
   * Notification that the <b>Print</b> button frame has been de-activated.
   * No action is taken; the <tt>WindowListener</tt> interface requires this method.
   *
   * @param evt Describes the event that occurred.
   */
   public void windowDeactivated(WindowEvent evt) {}


  /**
   * Notification that the <b>Print</b> button frame has been minimized.
   * No action is taken; the <tt>WindowListener</tt> interface requires this method.
   *
   * @param evt Describes the event that occurred.
   */
   public void windowIconified(WindowEvent evt) {}


  /**
   * Notification that the <b>Print</b> button frame has been restored from minimized state.
   * No action is taken; the <tt>WindowListener</tt> interface requires this method.
   *
   * @param evt Describes the event that occurred.
   */
   public void windowDeiconified(WindowEvent evt) {}


   //----------------------------------------------------------------------
   private PrintDip printDip;    //reference back to the dip, so we can call its methods
   private Button   printButton; //the button that fires off the printing

   private short oldXPos   = 0;  //previous horizontal location
   private short oldYPos   = 0;  //previous vertical location
   private short oldWidth  = 0;  //previous width
   private short oldHeight = 0;  //previous height

   private static short BUTTON_INSET = 5;

   private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
}

