//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbPrintDip
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved - Patent(s) Pending
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.22 com/java/src/com/ibm/beans/samples/dips/print/PrintDip.java, bbPrintDip, bb97c, bb97c9750e 11/14/97 14:15:20 [12/11/97 17:01:43]

package com.ibm.beans.samples.dips.print;

import com.ibm.beans.dip.DipMethodCallListener;
import com.ibm.beans.dip.DipRejectAdditionException;
import com.ibm.beans.dip.MethodCallEvent;
import com.ibm.beans.dip.SimpleDip;
import com.ibm.beans.widgets.PrintingSupport;

import java.awt.Button;
import java.awt.Component;
import java.awt.Frame;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.lang.ClassNotFoundException;
import java.util.Properties;
import java.util.ResourceBundle;


/**
 * <p>
 * This class provides basic printing support to any dippable AWT bean.
 * The dip provides a small floating frame containing a <b>Print</b> button.
 * When this button is pressed, a standard print dialog is presented.
 * This dialog allows the user to select options, such as the target
 * print queue, number of copies, and printer-driver-specific settings,
 * such as paper-size, orientation, resolution, and so forth.  The user
 * may then submit the print job or cancel.
 * <p>
 * By using the customizer, the assembler or user can set properties
 * such as whether to show the floating frame, where it should appear,
 * its size, what text to display in the <b>Print</b> button, the
 * title bar text, the print job title, and so forth.
 * <p>
 * This class essentially implements the dip model, while the
 * PrintButtonFrame class implements its view.
 * <p>
 * See the <i>Print Dip Usage Guide</i> for information regarding
 * how to set up and use the Print Dip.
 *
 * @author John W. Alcorn (jalcorn@us.ibm.com)
 * @see com.ibm.beans.samples.dips.print.PrintButtonFrame
 * @see com.ibm.beans.samples.dips.print.PrintDipBeanInfo
 * @see com.ibm.beans.samples.dips.print.PrintDipCustomizer
 * @see com.ibm.beans.widgets.PrintingSupport
 */
public class PrintDip extends SimpleDip implements DipMethodCallListener, Serializable {

  /**
   * Constructs a new Print Dip, using default values for each property.
   * The owner of the dip is responsible for ensuring proper customization
   * by invoking the customizer for this class.
   */
  public PrintDip()
  {
     printButtonFrame = null;

     showPrintButtonFrame = true;

     printButtonFrameXPos   = 0;
     printButtonFrameYPos   = 0;
     printButtonFrameWidth  = 175; //just wide enough for default title bar string
     printButtonFrameHeight = 100;

     resourceBundle = (PrintDipResourceBundle) ResourceBundle.getBundle(
        "com.ibm.beans.samples.dips.print.PrintDipResourceBundle");

     printButtonText = resourceBundle.getString("DefaultPrintButtonText");
     printButtonFrameTitleText = resourceBundle.getString("DefaultPrintButtonFrameTitleText");
     printJobTitleText = resourceBundle.getString("DefaultPrintJobTitleText");

//   printJobProperties = new Properties(Properties.defaults);
     printJobProperties = new Properties();

     propertyChangeSupport = new PropertyChangeSupport(this);
  }


  /**
   * This method is called whenever the Print Dip is instantiated from
   * serialized data (such as a .ser file).  It is overridden so that
   * transient data (that is, data which is not serialized) can be
   * initialized (then it just calls defaultReadObject on the
   * ObjectInputStream to let it take care of initializing all of the
   * non-transient data).
   */
  private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
     propertyChangeSupport = new PropertyChangeSupport(this);
     resourceBundle = (PrintDipResourceBundle) ResourceBundle.getBundle(
        "com.ibm.beans.samples.dips.print.PrintDipResourceBundle");

     in.defaultReadObject();
  }


  // overrides from SimpleDip

  /**
   * Defines this dip to be related to printing.
   *
   * @return The type name of the dip.
   */
  public String typeName() {
     return resourceBundle.getString("TypeName");
  }


  /**
   * Determines if a method should be included in a morphed class.
   * When the dip is applied to a class as a pre-dip during morphing,
   * and the user elects to only override those methods in the dippable
   * class that the pre-dips agrees to, this method is used to determine
   * if a dip agrees to including a specific method.
   * <p>
   * The only methods of the dippable bean that the PrintDip needs to be
   * wrapped are the addNotify and removeNotify methods.
   *
   * @param methodName Name of the method to be considered
   * @return boolean indicating whether to wrap given method
   */
  public boolean includeInMorphedClass(String methodName) {
     boolean includeThisMethod = false;

     if ((methodName.indexOf("addNotify") != -1) || (methodName.indexOf("removeNotify") != -1)) {
        includeThisMethod = true;
     } /* endif */

     return includeThisMethod;
  }


  /**
   * Checks the object to which the dip is being applied to ensure that
   * the object is a <tt>java.awt.Component</tt>.  If it is not, then an
   * exception is thrown, and the application of the dip is rejected.
   * <p>
   * @param obj The bean being dipped.
   * @exception DipRejectAdditionException
   *            Thrown if <i>obj</i> is not a descendant of
   *            <tt>java.awt.Component</tt>.
   */
  public Object createImplementation(Object obj) throws DipRejectAdditionException {
    // introspect DippableBean to see if it has java.awt.Component as an ancestor
    if (obj instanceof Component) {
      dippableBean = (Component) obj;
      if (dippableBean.isShowing()) {
         createPrintButtonFrame(); //jwa: Don't do this unless dippable bean is visible
      } /* endif */
      return null;
    } else {
      throw new DipRejectAdditionException(resourceBundle.getString("RejectAdditionException"));
    } /* endif */
  }

  /**
   * Notifies the PrintDip that it is being removed from its dippable bean.
   * The PrintDip uses this notification to take down the floating <b>Print</b>
   * button dialog, by calling the destroyPrintButtonFrame method.
   *
   * @see #destroyPrintButtonFrame
   * @see com.ibm.beans.dip.Dip#releaseImplementation()
   */
  public void releaseImplementation() {
     destroyPrintButtonFrame();
  }


  /**
   * Wires the dip to be notified when the addNotify and removeNotify methods
   * of the dippable bean are called.  The dip hooks these methods in order
   * to know when to show or hide the <b>Print</b> button dialog.
   *
   * @param impl This value should be <tt>null</tt>.  There is no separate implementation.
   * @param methods An array of method signatures from the dippable bean.
   * @exception DipRejectAdditionException  Thrown if dippable bean's
   *            addNotify and/or removeNotify methods not wrapped.
   * @return Array of DipMethodCallListeners with PrintDip set to
   *         listen at the appropriate method indices.
   * @see #handleMethodCall
   * @see com.ibm.beans.dip.Dip#createMethodCallListener()
   */
  public DipMethodCallListener[] createMethodCallListener(Object impl, String[] methods) throws DipRejectAdditionException {

    // call SimpleDip first to ensure we have properly created
    // the listener array
    super.createMethodCallListener(impl, methods);

    // keep track of number of methods hooked - it should be two, or we reject addition of the dip
    int count = 0;

    // set myself as the listener to the addNotify() and removeNotify() methods
    for (int i = 0; i < methodCallListeners.length; i++) {
       if ((methods[i].indexOf("addNotify") != -1) || (methods[i].indexOf("removeNotify") != -1)) {
          methodCallListeners[i] = this;
          count++;
       } /* endif */
    } /* endfor */

    if (count!=2) throw new DipRejectAdditionException(resourceBundle.getString("RejectAdditionException2"));

    return methodCallListeners;
  }


  // DipMethodCallListener Interface

  /**
   * Communicates to the dipping framework that this dip does not require
   * the arguments for intercepted methods of the dippable bean.  Returning
   * <tt>false</tt> avoids the performance impact of parameter marshaling.
   *
   * @return Always <tt>false</tt>, indicating that this dip does not require
   *         the arguments from intercepted methods.
   */
  public boolean needsArguments() { return false; }


  /**
   * Responds to method calls on the dippable bean.  Calls the
   * createPrintButtonFrame method in response to the addNotify
   * method of the dippable bean being called.  Calls the
   * destroyPrintButtonFrame method in response to the removeNotify
   * method of the dippable bean being called.
   *
   * @param methodDescription	Describes the method of the dippable bean that was
   *        called.  This dip only intercepts the addNotify and removeNotify methods.
   * @see #createMethodCallListener
   * @see #createPrintButtonFrame
   * @see #destroyPrintButtonFrame
   */
  public void handleMethodCall(MethodCallEvent methodDescription) {
     if (methodDescription.getMethodId().indexOf("addNotify")!=-1) { //addNotify
        createPrintButtonFrame();
     } /* endif */

     if (methodDescription.getMethodId().indexOf("removeNotify")!=-1) { //removeNotify
        destroyPrintButtonFrame();
     } /* endif */
  }


  /**
   * Creates the <b>Print</b> button dialog and initializes it.
   * Whether the dialog is shown depends on the value of
   * the <i>showPrintButtonFrame</i> property.
   *
   * @see #getPrintButtonFrame
   * @see #destroyPrintButtonFrame
   * @see com.ibm.beans.samples.dips.print.PrintButtonFrame
   */
  public void createPrintButtonFrame() {
     if (printButtonFrame==null) {
        printButtonFrame = new PrintButtonFrame(this, printButtonFrameTitleText, printButtonText);
        printButtonFrame.setBounds(printButtonFrameXPos, printButtonFrameYPos,
           printButtonFrameWidth, printButtonFrameHeight);
        printButtonFrameLocked = false;
        printButtonFrame.setVisible(showPrintButtonFrame);
     } /* endif */
  }


  /**
   * Destroys the <b>Print</b> button dialog.  Should be called when the dippable
   * bean is being destroyed.
   *
   * @see #getPrintButtonFrame
   * @see #createPrintButtonFrame
   * @see com.ibm.beans.samples.dips.print.PrintButtonFrame
   */
  public void destroyPrintButtonFrame() {
     if (printButtonFrame!=null) printButtonFrame.dispose();
     printButtonFrame = null;
  }


  /**
   * Locks the <b>Print</b> button dialog.  This keeps a series of updates to the
   * properties related to the floating dialog from causing unnecessary flicker.
   *
   * @param lock Specifies whether the floating dialog
   * @see com.ibm.beans.samples.dips.print.PrintButtonFrame
   */
  public void lockPrintButtonFrame(boolean lock) {
     printButtonFrameLocked = lock;
  }


  /**
   * Prints the dippable bean.  This method is generally called
   * from the action listener attached to the <b>Print</b> button
   * in the PrintButtonFrame class.
   * <p>
   * Another scenario is that an assembly tool could wire this method
   * to an event fired by another bean.  In this scenario, the
   * showPrintButtonFrame property is set to <tt>false</tt>,
   * and printing of the dippable bean is initiated by an external
   * event such as a button click or menu item selection from another
   * bean in the aggregation comprising the application.
   * <p>
   * The implementation of the actual printing is deferred to the
   * PrintingSupport utility class from the <tt>com.ibm.beans.widgets</tt>
   * package, which is called from this method.
   *
   * @see com.ibm.beans.widgets.PrintingSupport
   * @see com.ibm.beans.samples.dips.print.PrintButtonFrame#actionPerformed
   */
  public void printDippableBean()
  {
     PrintingSupport.printComponent(dippableBean, printButtonFrame,
                                    printJobTitleText, printJobProperties);
  }


  // Begin support for bound properties


  /**
   * Adds a listener to be notified whenever a change occurs to one of the
   * properties of the dip.  Another bean would call this method if it
   * wished to be notified of changes occuring within this dip.  All of the
   * properties of the Print Dip are bound, but none are constrained.
   *
   * @param listener	The object to be notified of changes.
   */
  public void addPropertyChangeListener(PropertyChangeListener listener) {
     propertyChangeSupport.addPropertyChangeListener( listener );
  }


  /**
   * Removes a listener from the notification list.
   *
   * @param listener	The object to be removed from the notification list.
   */
  public void removePropertyChangeListener(PropertyChangeListener listener) {
     propertyChangeSupport.removePropertyChangeListener( listener );
  }


  // Begin accessors for the Print Dip properties


  /**
   * Gets the <em>dippableBean</em> property.
   *
   * @return The dippable bean to which this dip is applied.
   */
  public Object getDippableBean() { return dippableBean; }


  /**
   * Gets the <em>printButtonFrame</em> property.
   *
   * @return The <tt>java.awt.Frame</tt> containing the <b>Print</b> button.
   *         The returned frame can be cast up to an instance of the
   *         PrintButtonFrame class if needed.  Note that, in general, the
   *         returned frame should not be modified directly, but rather by
   *         modifying the other properties exposed by this dip.
   * @see #createPrintButtonFrame
   * @see #destroyPrintButtonFrame
   * @see com.ibm.beans.samples.dips.print.PrintButtonFrame
   */
  public Frame getPrintButtonFrame() { return printButtonFrame; }


  /**
   * Gets the <em>printJobProperties</em> property.
   *
   * @return The properties used for the next print job.
   * @see #setPrintJobProperties
   * @see #printDippableBean
   */
  public Properties getPrintJobProperties() { return printJobProperties; }


  /**
   * Sets the <em>printJobProperties</em> property.
   *
   * @param newPrintJobProperties The properties used for the next print job.
   * @see #getPrintJobProperties
   * @see #printDippableBean
   */
  public void setPrintJobProperties(Properties newPrintJobProperties)
  {
     Properties oldPrintJobProperties = printJobProperties;
     printJobProperties = newPrintJobProperties;

     propertyChangeSupport.firePropertyChange("printJobProperies",
        oldPrintJobProperties, newPrintJobProperties);
  }


  /**
   * Gets the <em>printButtonFrameTitleText</em> property.
   *
   * @return The title bar text of the <b>Print</b> button dialog.
   * @see #setPrintButtonFrameTitleText
   */
  public String getPrintButtonFrameTitleText() { return printButtonFrameTitleText; }


  /**
   * Sets the <em>printButtonFrameTitleText</em> property.
   *
   * @param newPrintButtonFrameTitleText The title bar text of the <b>Print</b> button dialog.
   * @see #getPrintButtonFrameTitleText
   */
  public void setPrintButtonFrameTitleText(String newPrintButtonFrameTitleText)
  {
     String oldPrintButtonFrameTitleText = printButtonFrameTitleText;
     printButtonFrameTitleText = newPrintButtonFrameTitleText;

     propertyChangeSupport.firePropertyChange("printButtonFrameTitleText",
        oldPrintButtonFrameTitleText, newPrintButtonFrameTitleText);

     if (printButtonFrame!=null) printButtonFrame.setTitle(printButtonFrameTitleText);
  }


  /**
   * Gets the <em>printButtonText</em> property.
   *
   * @return The label of the button inside the floating dialog.
   * @see #setPrintButtonText
   */
  public String getPrintButtonText() { return printButtonText; }


  /**
   * Sets the <em>printButtonText</em> property.
   *
   * @param newPrintButtonText The label of the button inside the floating dialog.
   * @see #getPrintButtonText
   */
  public void setPrintButtonText(String newPrintButtonText)
  {
     String oldPrintButtonText = printButtonText;
     printButtonText = newPrintButtonText;

     propertyChangeSupport.firePropertyChange("printButtonText",
        oldPrintButtonText, newPrintButtonText);

     if (printButtonFrame!=null) {
        Button printButton = printButtonFrame.getPrintButton();
        if (printButton!=null) printButton.setLabel(printButtonText);
     } /* endif */
  }


  /**
   * Gets the <em>printJobTitleText</em> property.
   *
   * @return The title of the next print job.
   * @see #setPrintJobTitleText
   * @see #printDippableBean
   */
  public String getPrintJobTitleText() { return printJobTitleText; }


  /**
   * Sets the <em>printJobTitleText</em> property.
   *
   * @param newPrintJobTitleText The title of the next print job.
   * @see #getPrintJobTitleText
   * @see #printDippableBean
   */
  public void setPrintJobTitleText(String newPrintJobTitleText)
  {
     String oldPrintJobTitleText = printJobTitleText;
     printJobTitleText = newPrintJobTitleText;

     propertyChangeSupport.firePropertyChange("printJobTitleText",
        oldPrintJobTitleText, newPrintJobTitleText);
  }


  /**
   * Gets the <em>showPrintButtonFrame</em> property.
   *
   * @return <tt>true</tt>, if the <b>Print</b> button dialog is visible;
   *         <tt>false</tt> otherwise.
   * @see #setShowPrintButtonFrame
   */
  public boolean getShowPrintButtonFrame() { return showPrintButtonFrame; }


  /**
   * Sets the <em>showPrintButtonFrame</em> property.
   *
   * @param newShowPrintButtonFrame Sets whether the <b>Print</b> button dialog is visible.
   * @see #getShowPrintButtonFrame
   */
  public void setShowPrintButtonFrame(boolean newShowPrintButtonFrame)
  {
     Boolean oldShowPrintButtonFrame = new Boolean(showPrintButtonFrame);
     showPrintButtonFrame = newShowPrintButtonFrame;

     propertyChangeSupport.firePropertyChange("showPrintButtonFrame",
        oldShowPrintButtonFrame, new Boolean(showPrintButtonFrame));

     if (printButtonFrame!=null) printButtonFrame.setVisible(showPrintButtonFrame);
  }


  /**
   * Gets the <em>printButtonFrameXPos</em> property.
   *
   * @return The horizontal position of the <b>Print</b> button dialog.
   * @see #setPrintButtonFrameXPos
   */
  public short getPrintButtonFrameXPos() { return printButtonFrameXPos; }


  /**
   * Sets the <em>printButtonFrameXPos</em> property.
   *
   * @param newPrintButtonFrameXPos The horizontal position of the <b>Print</b> button dialog.
   * @see #getPrintButtonFrameXPos
   */
  public void setPrintButtonFrameXPos(short newPrintButtonFrameXPos)
  {
     Short oldPrintButtonFrameXPos = new Short(printButtonFrameXPos);
     printButtonFrameXPos = newPrintButtonFrameXPos;

     propertyChangeSupport.firePropertyChange("printButtonFrameXPos",
        oldPrintButtonFrameXPos, new Short(printButtonFrameXPos));

     if ((printButtonFrame!=null) && !printButtonFrameLocked) {
        printButtonFrame.setLocation(printButtonFrameXPos, printButtonFrameYPos);

  /*
     jwa: Since the frame may not accept the specified location (such as if
          the operating system won't let it be offscreen), query it for what
          value we were actually granted.
  */
        printButtonFrame.validate();
        printButtonFrameXPos = (short) printButtonFrame.getLocation().x;
     } /* endif */
  }


  /**
   * Gets the <em>printButtonFrameYPos</em> property.
   *
   * @return The vertical position of the <b>Print</b> button dialog.
   * @see #setPrintButtonFrameYPos
   */
  public short getPrintButtonFrameYPos() { return printButtonFrameYPos; }


  /**
   * Sets the <em>printButtonFrameYPos</em> property.
   *
   * @param newPrintButtonFrameYPos The vertical position of the <b>Print</b> button dialog.
   * @see #getPrintButtonFrameYPos
   */
  public void setPrintButtonFrameYPos(short newPrintButtonFrameYPos)
  {
     Short oldPrintButtonFrameYPos = new Short(printButtonFrameYPos);
     printButtonFrameYPos = newPrintButtonFrameYPos;

     propertyChangeSupport.firePropertyChange("printButtonFrameYPos",
        oldPrintButtonFrameYPos, new Short(printButtonFrameYPos));

     if ((printButtonFrame!=null) && !printButtonFrameLocked) {
        printButtonFrame.setLocation(printButtonFrameXPos, printButtonFrameYPos);

  /*
     jwa: Since the frame may not accept the specified location (such as if
          the operating system won't let it be offscreen), query it for what
          value we were actually granted.
  */
        printButtonFrame.validate();
        printButtonFrameYPos = (short) printButtonFrame.getLocation().y;
     } /* endif */
  }


  /**
   * Gets the <em>printButtonFrameWidth</em> property.
   *
   * @return The width of the <b>Print</b> button dialog.
   * @see #setPrintButtonFrameWidth
   */
  public short getPrintButtonFrameWidth() { return printButtonFrameWidth; }


  /**
   * Sets the <em>printButtonFrameWidth</em> property.
   *
   * @param newPrintButtonFrameWidth The width of the <b>Print</b> button dialog.
   * @see #getPrintButtonFrameWidth
   */
  public void setPrintButtonFrameWidth(short newPrintButtonFrameWidth)
  {
     Short oldPrintButtonFrameWidth = new Short(printButtonFrameWidth);
     printButtonFrameWidth = newPrintButtonFrameWidth;

     propertyChangeSupport.firePropertyChange("printButtonFrameWidth",
        oldPrintButtonFrameWidth, new Short(printButtonFrameWidth));

     if (printButtonFrame!=null) {
        printButtonFrame.setSize(printButtonFrameWidth, printButtonFrameHeight);

  /*
     jwa: Since the frame may not accept the specified width (such as if
          the operating system won't let it be wider than the screen),
          query it for what value we were actually granted.
  */
        printButtonFrame.validate();
        printButtonFrameWidth = (short) printButtonFrame.getSize().width;
     } /* endif */
  }


  /**
   * Gets the <em>printButtonFrameHeight</em> property.
   *
   * @return The height of the <b>Print</b> button dialog.
   * @see #setPrintButtonFrameHeight
   */
  public short getPrintButtonFrameHeight() { return printButtonFrameHeight; }


  /**
   * Sets the <em>printButtonFrameHeight</em> property.
   *
   * @param newPrintButtonFrameHeight The height of the <b>Print</b> button dialog.
   * @see #getPrintButtonFrameHeight
   */
  public void setPrintButtonFrameHeight(short newPrintButtonFrameHeight)
  {
     Short oldPrintButtonFrameHeight = new Short(printButtonFrameHeight);
     printButtonFrameHeight = newPrintButtonFrameHeight;

     propertyChangeSupport.firePropertyChange("printButtonFrameHeight",
        oldPrintButtonFrameHeight, new Short(printButtonFrameHeight));

     if (printButtonFrame!=null) {
        printButtonFrame.setSize(printButtonFrameWidth, printButtonFrameHeight);

  /*
     jwa: Since the frame may not accept the specified height (such as if
          the operating system won't let it be taller than the screen),
          query it for what value we were actually granted.
  */
        printButtonFrame.validate();
        printButtonFrameHeight = (short) printButtonFrame.getSize().height;
     } /* endif */
  }


  //----------------------------------------------------------------------
  // Standard Copyright statement
  //----------------------------------------------------------------------
  private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";

  //----------------------------------------------------------------------
  // The PrintDip Serial Version Unique ID
  //----------------------------------------------------------------------
  private static final long serialVersionUID = 2000L;

  //----------------------------------------------------------------------
  // The PrintDip Resource Bundle (holds our translatable strings)
  //----------------------------------------------------------------------
  private transient PrintDipResourceBundle resourceBundle = null;

  //----------------------------------------------------------------------
  // The PrintDip Property Change Support (fires property change events)
  //----------------------------------------------------------------------
  private transient PropertyChangeSupport propertyChangeSupport = null;

  //----------------------------------------------------------------------
  // The PrintDip Properties
  //----------------------------------------------------------------------
  private Component dippableBean;                      // we provide "getter"
  private transient PrintButtonFrame printButtonFrame; // we provide "getter"
  private Properties printJobProperties;               // we provide "getter" and "setter"
  private String printButtonFrameTitleText;            // we provide "getter" and "setter"
  private String printButtonText;                      // we provide "getter" and "setter"
  private String printJobTitleText;                    // we provide "getter" and "setter"
  private boolean showPrintButtonFrame;                // we provide "getter" and "setter"
  private short printButtonFrameXPos;                  // we provide "getter" and "setter"
  private short printButtonFrameYPos;                  // we provide "getter" and "setter"
  private short printButtonFrameWidth;                 // we provide "getter" and "setter"
  private short printButtonFrameHeight;                // we provide "getter" and "setter"
  private transient boolean printButtonFrameLocked;    // we provide "getter" and "setter"
}

