//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbContent
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.20 com/java/src/com/ibm/beans/samples/dips/sessauth/SAImpl.java, bbSecurityDip, bb97c, bb97c9750e 11/19/97 10:24:52 [12/11/97 17:01:55]
package com.ibm.beans.samples.dips.sessauth;

import com.ibm.beans.dip.Dippable;

import java.io.Serializable;
import java.io.ObjectInputStream;
import java.text.MessageFormat;
import java.util.Vector;
import java.util.ResourceBundle;
import java.util.MissingResourceException;

import java.lang.reflect.Modifier;
import java.lang.reflect.Method;

/**
    This class is the implementation of the Session Authentication Dip.  This object contains references
    to the principal, principal query, and authenticator that were configured
    by the assembler using the Session Authentication Dip customizer.
    <p>
    This object contains the methods that initiate authentication on a principal, and track
    if the principal has been successfully authenticated yet.
*/

public class SAImpl implements Serializable
{
    private static final long serialVersionUID = 2000L;

    private SAImpl () { /* Don't allow default constructor */ }

    /**
        Constructs the Session Authentication Implementation.

        @param dip A reference to the Session Authentication Dip.
    */
    public SAImpl( SessionAuthenticationDip dip ) {
        this.dip = dip;
        setDefaultCustomization();
        constructTransientFields();
    }

    /**
        This method initiates the entire authentication process.  This process includes:

        - Obtaining the principal information
        - Invoking the appropriate object to check the validity of that <em>principal</em>
        - Returning whether the authentciation attempt was successful.

        This method WILL BLOCK the calling thread until the authentication process completes.

        @return Returns <tt>true</tt> if authentication was successful,
            or <tt>false</tt> if unsuccessful.

        @see com.ibm.beans.samples.dips.sessauth.SAPrincipalQuery
    */
    boolean authenticate() {
        authenticationSuccessfull = logonQuery.queryForPrincipal( principal );
        return authenticationSuccessfull;
    }

    /**
        Invokes the authenticator to verify the principal.
        It is called from the object that queried for the principal.  That object is a subclass of
        SAPrincipalQuery.  This method invokes the appropriate authenticator
        to validate the principal entered by the user.

        @return <tt>true</tt> if the authentication attempt was successful; <tt>false</tt> otherwise.
    */
    public boolean doAuthentication() {
        boolean authenticationOK = false;
        SADip.traceln( 1000, "doAuthentication" );

        // The 'authenticators' Vector will eventually contain a list > 1 of
        // possible authenticators.  Thus, some (not all) of the necessary
        // list processing logic is here.

        SADip.println( "ONLY using the first authenticator" );

        SAAuthenticator currentAuthenticator = (SAAuthenticator) authenticators.firstElement();

        authenticationOK = currentAuthenticator.authenticatePrincipal( principal );

        return authenticationOK;
    }

    /**
        Sets the value of the principal.  This is called from the object that
        queried for the principal to store the principal related information.  That object is a subclass of
        SAPrincipalQuery.

        @param principal The new value for the principal.
        @see com.ibm.beans.samples.dips.sessauth.SAPrincipalQuery
    */
    public void setPrincipal ( SAPrincipal principal ) {
        this.principal = principal;
    }

    /**
        Gets the current value of the principal.
        @return The current value of the principal.
    */
    public SAPrincipal getPrincipal( ) {
        return principal;
    }

    //----------------------------------------
    // Protected methods

    String retrieveMessage( String messageKey ) {
        return (String) resourceBundle.handleGetObject( messageKey );
    }

    SessionAuthenticationDip getDip() {
        return dip;
    }

    String getPrincipalQueryClassName( ) {
        return principalQueryClassName;
    }

    void setPrincipalQueryClassName( String principalQueryClassName ) {
        this.principalQueryClassName = principalQueryClassName;
    }

    Vector getAuthenticators( ) {
        return authenticators;
    }

    void setAuthenticators( Vector authenticators ) {
        this.authenticators = (Vector) authenticators.clone();
    }

    boolean associateDippableBean( Object dippableBean ) {

        if ( beanCanBeDipped( dippableBean ) ) {
            dipApplied = true;
            return true;
        }
        else { // Bean has final methods - can't wrap
            dipApplied = false;
            return false;
        }
    }

    final boolean getAuthenticationSuccessfull() {
        return authenticationSuccessfull;
    }

    final boolean isDipApplied() {
        return dipApplied;
    }

    //----------------------------------------
    // Private methods

    private final void setAuthenticationSuccessfull( boolean authenticationSuccessfull ) {
        this.authenticationSuccessfull = authenticationSuccessfull;
    }

    private boolean beanCanBeDipped( Object dippableBean ) {

        boolean isBeanDippable = true;
        /*
            Beans with FINAL or STATIC methods can't be dipped since the dipping
            framework can't intercept those methods, thus this dip can't disable those
            methods in a bean.

            Get the oringal bean, and see if it had any declared static or final methods
        */

        String dippableBeanName = dippableBean.getClass().getName();
        int startOfDippable = dippableBeanName.lastIndexOf( "Dippable" );
        String originalBeanName = dippableBeanName.substring( 0, startOfDippable );

        Class originalBeanClass = null;
        Method[] declaredMethods = null;

        try {
            originalBeanClass = Class.forName( originalBeanName );
            declaredMethods = originalBeanClass.getDeclaredMethods();
        }
        catch ( ClassNotFoundException e ) {
            isBeanDippable = false;
        }
        catch ( SecurityException e ) {
            isBeanDippable = false;
        }

        if ( declaredMethods != null ) {
            for ( int i = 0 ; i < declaredMethods.length; i++ ) {
                int modifiers = declaredMethods[i].getModifiers();
                if ( Modifier.isFinal( modifiers ) || Modifier.isStatic( modifiers ) ) {
                    isBeanDippable = false;
                    break;
                }
            }
        }

        return isBeanDippable;
    }

    private void readObject( ObjectInputStream stream )
        throws java.lang.ClassNotFoundException, java.io.IOException,  java.io.NotActiveException {
        SADip.traceln( 100, "SAImpl.readObject() called" );
        stream.defaultReadObject();
        constructTransientFields();
    }

    /*
        This method will reconstruct the transient fields.  It is used by
        both the constructor and readObject
    */
    private void constructTransientFields() {

        SADip.traceln( 1000, "construct transient " + principalQueryClassName );

        try {
            resourceBundle = (SAResourceBundle) ResourceBundle.getBundle( SADip.RESOURCE_BUNDLE );
        }
        catch ( MissingResourceException e ) {
            SADip.errorMessage( "SAImpl: Unable to open resource bundle" );
        }

        try {
            logonQuery = (SAPrincipalQuery) Class.forName( principalQueryClassName ).newInstance();
            logonQuery.setImplementation( this );
        }
        catch ( ClassNotFoundException exception ) {
            MessageFormat formatString = new MessageFormat( retrieveMessage( "SA_PRINCIPAL_CLASS" ) );
            StringBuffer messageString = new StringBuffer();
            Object[] arguments = new Object[1];
            arguments[0] = principalQueryClassName;
            formatString.format( arguments, messageString, null );
            SADip.errorMessage( new String( messageString ) );
            exception.printStackTrace();
        }
        catch ( InstantiationException exception ) {
            MessageFormat formatString = new MessageFormat( retrieveMessage( "SA_PRINCIPAL_CLASS" ) );
            StringBuffer messageString = new StringBuffer();
            formatString.format( principalQueryClassName, messageString, null );
            SADip.errorMessage( new String( messageString ) );
            exception.printStackTrace();
        }
        catch ( IllegalAccessException exception ) {
            MessageFormat formatString = new MessageFormat( retrieveMessage( "SA_PRINCIPAL_CLASS" ) );
            StringBuffer messageString = new StringBuffer();
            formatString.format( principalQueryClassName, messageString, null );
            SADip.errorMessage( new String( messageString ) );
            exception.printStackTrace();
        }
    }

    //----------------------------------------
    // These values can also be set during customization
    // This method simply provides default values

    private void setDefaultCustomization() {

        setPrincipalQueryClassName( "com.ibm.beans.samples.dips.sessauth.demo.SamplePrincipalQuery" );

        setPrincipal( new com.ibm.beans.samples.dips.sessauth.demo.SamplePrincipal() );

        SAAuthenticator simpleAuthenticator = new com.ibm.beans.samples.dips.sessauth.demo.SampleSimpleAuthenticator();
        Vector authenticatorVector = new Vector();
        authenticatorVector.addElement( simpleAuthenticator );
        setAuthenticators( authenticatorVector );

    }

    //----------------------------------------
    // Properties

    private transient boolean authenticationSuccessfull = false;
    private transient SAPrincipalQuery logonQuery = null;
    private transient SAResourceBundle resourceBundle;

    private boolean dipApplied = false;

    private SAPrincipal principal;

    // A collection of valid authenticators for the dippable bean
    // as selected via the customizer when this dip was applied.
    private Vector authenticators;          // Contains SAAuthenticator objects
    private String principalQueryClassName = null;
    private SessionAuthenticationDip dip;

    private final static String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}
