//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbContent
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.13 com/java/src/com/ibm/beans/samples/dips/sessauth/SAVetoableListener.java, bbSecurityDip, bb97c, bb97c9750e 11/19/97 10:24:56 [12/11/97 17:01:52]
package com.ibm.beans.samples.dips.sessauth;

import com.ibm.beans.dip.Dip;

import com.ibm.beans.dip.DipVetoableMethodListener;
import com.ibm.beans.dip.MethodVetoException;
import com.ibm.beans.dip.MethodCallEvent;

import com.ibm.beans.dip.DipVetoableEventListener;
import com.ibm.beans.dip.EventVetoException;
import com.ibm.beans.dip.EventFireEvent;

import com.ibm.beans.dip.DipVetoableChangeListener;
import java.beans.PropertyVetoException;
import java.beans.PropertyChangeEvent;

/**
    This object implements all the vetoable listeners for the Session
    Authentication dip.  These listeners check to see if the principal
    has been successfully authenticated.  If the principal has been successfully
    authenticated, the listener returns; otherwise, it vetos the constrained property,
    method, or event.

    @see com.ibm.beans.samples.dips.sessauth.SAPrincipal
    @see com.ibm.beans.samples.dips.sessauth.SAPrincipalQuery
    @see com.ibm.beans.samples.dips.sessauth.SAAuthenticator
*/
public class SAVetoableListener implements
    com.ibm.beans.dip.DipVetoableMethodListener,
    com.ibm.beans.dip.DipVetoableEventListener,
    com.ibm.beans.dip.DipVetoableChangeListener
{
    /*
        We need an implementation, so don't allow construction without one.
    */
    private SAVetoableListener() { };

    /**
        Constructs an SAVetoableListener.

        @param implementation A reference to the Session Authentication Dip's implementation
    */
    public SAVetoableListener( SAImpl implementation ) {
        this.implementation = implementation;
    }
    //-------------------------------------------------------------
    // Constrained Methods

    /**
        Handles vetoing of methods.  This method is called
        before the dippable bean's method is invoked.

        @param evt A MethodCallEvent object describing the event source
            and the method that was called.
        @exception MethodVetoException This exception may be thrown if
	the VetoableMethodListener
            wants the method operation not to be performed.
    */

    public void handleVetoableMethod( MethodCallEvent evt ) throws MethodVetoException {
        SADip.traceln( 1000, "VetoableListener.handleVetoableMethod" );
        if ( authenticationSuccessfull() ) {
            SADip.traceln( 1000, "handleVetoableMethod OK" );
            return;
        }
        else {
            SADip.traceln( 1000, "handleVetoableMethod VETOED: " + evt.getMethodId() );
            throw new MethodVetoException( implementation.retrieveMessage( "SA_VETO_NOT_AUTHENTICATED" ),
                                           evt );
        }
    }

    /**
        Recovers from a vetoed method.
        This method is called if the dippable bean's method was vetoed.
        <p>
        This method is called if the dippable bean's method was vetoed
        by another dip and when this dip already had its handleVetoableMethod
        method called and did not veto the event.

        @param evt A MethodCallEvent object describing the event source
            and the method that was called.
    */
    public void recoverFromMethodVeto( MethodCallEvent evt ) {
        SADip.traceln( 10000, "VetoableListener.recoverFromMethodVeto" );
        // No recovery necessary
    }

    /**
        Indicates whether the listener needs arguments.  By not needing arguments,
        performance is improved because the dipping framework does not need to do
        any marshaling.

        @return <tt>true</tt> if arguments are needed, otherwise <tt>false</tt>.
    */
    public boolean needsArguments( ) {
        return false;
    }

    //-------------------------------------------------------------
    // Constrained Events

    /**
        Handles the vetoing of events.
        This method is called before the dippable bean fires an event.

        @param evt An EventFireEvent object describing the event source
            and the event that was fired.
        @exception EventVetoException This exception is thrown if the VetoableEventListener
            wants the event firing to be canceled.
    */
    public void handleVetoableEvent( EventFireEvent evt ) throws EventVetoException {
        SADip.traceln( 1000, "VetoableListener.handleVetoableEvent" );
        if ( authenticationSuccessfull() ) {
            SADip.traceln( 1000, "handleVetoableEvent OK" );
           return;
        }
        else {
            SADip.traceln( 1000, "handleVetoableEvent VETOED" );
            throw new EventVetoException( implementation.retrieveMessage( "SA_VETO_NOT_AUTHENTICATED" ),
                                          evt );
        }
    }

    /**
        Recovers from a vetoed event.  This method is called if a dippable bean's event was vetoed.
        <p>
        This method gets called if the dippable bean's event was vetoed
        by another dip and when this dip already had its handleVetoableEvent
        method called and did not veto the event.

        @param evt A EventFireEvent object describing the event source
            and the event that was vetoed.
    */
    public void recoverFromEventVeto( EventFireEvent evt ) {
        SADip.traceln( 10000, "VetoableListener.recoverFromEventVeto" );
        // No recovery necessary
    }

    //-------------------------------------------------------------
    // Constrained Properties

    /**
        Handles vetoing of property changes and recovery from vetoed changes.  It
        is called before a dippable beans property changes, and
        if that change is vetoed by another dip.
        <p>
        This method is called under two conditions:
        <ol>
        <li> Before the dippable bean's property change method is called.
        <li> If the dippable bean's property change was vetoed
            by another dip and when this dip already had it's vetoableChange
            method called and did not veto the event.
        </ol>
        <p>
        Because the dipping framework is using the JavaBeans vetoable change
        support, it differs slightly from the vetoable methods and vetoable events.

        @param evt A PropertyChangeEvent object describing the event source
            and the property change.

        @exception PropertyVetoException This exception may be thrown if
	the VetoableMethodListener wants the property operation not to be performed.

        @see java.beans.VetoableChangeListener
    */
    public void vetoableChange( PropertyChangeEvent evt ) throws PropertyVetoException {

        // As noted above, this method may get called twice for any given event - once
        // to give it a chance to veto, and a second time if the change is vetoed.  Note
        // that if called because the event is vetoed:
        //   - We are called even if we are the ones that issued the veto
        //   - We are called during this "cleanup" even if we hadn't been called yet
        //     to have the opportunity to veto the change
        //   - The Dip Framework (and the relate Beans support) ignores any veto
        //     exceptions during the cleanup

        SADip.traceln( 1000, "VetoableListener.vetoableChange: " + evt.getPropertyName() );
        if ( authenticationSuccessfull() ) {
            SADip.traceln( 1000, "vetoableChange OK" );
            return;
        }
        else {
            SADip.traceln( 1000, "vetoableChange VETOED" );
            throw new PropertyVetoException( implementation.retrieveMessage( "SA_VETO_NOT_AUTHENTICATED" ),
                                             evt );
        }
    }

    //-------------------------------------------------------------
    // Dip Auxiliary methods

    /**
        Retrieves a pointer to the dip that created the listener.

        @return A reference to the dip that created this listener.
    */
    public Dip retrieveDip() {
        return implementation.getDip();
    }

    //-------------------------------------------------------------
    // Private methods and data

    private boolean authenticationSuccessfull() {
        return implementation.getAuthenticationSuccessfull();
    }

    private SAImpl implementation;

    private final static String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}
