//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbSecurityDip
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.21 com/java/src/com/ibm/beans/samples/dips/sessauth/SessionAuthenticationDip.java, bbSecurityDip, bb97c, bb97c9750e 11/19/97 10:24:57 [12/11/97 17:01:59]
package com.ibm.beans.samples.dips.sessauth;

import com.ibm.beans.dip.DipVetoableMethodListener;
import com.ibm.beans.dip.DipVetoableChangeListener;
import com.ibm.beans.dip.DipVetoableEventListener;
import com.ibm.beans.dip.DipPropertyChangeListener;
import com.ibm.beans.dip.DipEventFireListener;
import com.ibm.beans.dip.DipMethodCallListener;
import com.ibm.beans.dip.DipRejectAdditionException;

import java.io.Serializable;
import java.io.ObjectInputStream;

/**
    The Session Authentication Dip provides access control for the bean
    that is dipped.
    <p>
    Once a bean is dipped in Session Authentication, the end users are
    required to validate themselves using a valid principal, a User ID and Password for example,
    before the bean is
    functional. Until the end user is validated or if validation fails, the bean is disabled by the Session
    Authentication dip.
    Disabling a bean that has not been successfully validated is accomplished by vetoing
    all properties, methods, and events.
    <p>
    This class is serializable since the dip interface it implements is serializable
    <p>
    Access control is implemented using a principal, a principal query
    and an authticator.
    <p>
    The principal is an entity that can be authenticated.  A principal might consist of a
    a User ID and a Password.
    <p>
    The principal query is an object that can obtain the information required for a principal.
    A principal query might consist of an AWT dialog that prompts the end user for a User ID and
    Password.
    <p>
    The authenticator is the entity that can validate a principal.  An authenticator might validate a
    User ID and Password entered by the user against a password file for example.

    @see com.ibm.beans.samples.dips.sessauth.SAPrincipal
    @see com.ibm.beans.samples.dips.sessauth.SAPrincipalQuery
    @see com.ibm.beans.samples.dips.sessauth.SAAuthenticator
*/

public final class SessionAuthenticationDip implements com.ibm.beans.dip.Dip
{
    private static final long serialVersionUID = 2000L;

    /**
        Constructs a SessionAuthenticationDip.
        This also creates the dip implementation.

        @see com.ibm.beans.samples.dips.sessauth.SAImpl
    */
    public SessionAuthenticationDip() {
        // Create the implementation now so that we can
        // be customized prior to being added to a dippable bean.
        // Necessary since the SADip customizer references the
        // implementation.
        implementation = new SAImpl( this );
    }
    //----------------------------------------------------------------------
    // Methods from interface Dip

    /**
        Sets the priority of the Session Authentication Dip.
        <p>
        The Session Authentication dip requests the
        hightest priority allowable.  The dip with the highest priority is
        the first
        Dip given that can veto a method, property, or event.

	@return an integer which sets the priority to the highest possible value.
        @see com.ibm.beans.dip.DipInfo#priority()
    */
    public final int priority() {
        return SADip.PRIORITY;
    }

    /**
        Uniquely identifies this Dip.  The string
        identifies the Session Authentication Dip
        object, not an instance.
        All Session Authentication Dip instances return the
        same value.

        @see com.ibm.beans.dip.DipInfo#typeName()

        @return A String that uniquely identifes the Session Authentication Dip.
    */
    public String typeName() {
        return SADip.TYPE_NAME;
    }

    /**
        Indicates if the Session Authentication can work with the new dip.  If the Session
        Authentication Dip cannot work with this dip, the Dipping Framework does not
        add the new dip to the dippable bean.

        @param newDip A reference to the new dip with which the Session Authentication dip should
            determine if it can work.
        @return <tt>true</tt> if the Session Authentication Dip can work with the new dip; <tt>false</tt>
            otherwise.

        @see com.ibm.beans.dip.DipInfo#worksWith()

    */
    public boolean worksWith( String newDip ) {
        return true;
    }

    /**
        Indicates if this dip is removable.  The Session Authentication Dip is
        <em>not</em> removable.

        @return <tt>true</tt> if this dip is removable; <tt>false</tt>otherwise.
        @see com.ibm.beans.dip.DipInfo#isRemovable()

    */
    public boolean isRemovable() {
        return false;
    }

    /**
        Creates the Session Authentication Dip implementation.  This object drives the
        actual work of querying for a principal and authenticating that principal.
        <p>
        The Session Authentication Dip cannot be added to a dippable bean under the
        following circumstances:
        <ul>
        <li> If the dippable bean contains public final or static methods.  A final or static method cannot
            be intercepted by the dipping framework.
        <li> The dippable bean must be using DippableBeanImp for its dippable
            implementation because this dip relies on a priority
            in which dips are invoked.
        </ul>

        @param obj A reference to the dippable bean.
        @return A reference to the object that implements Session Authentication Dip.
        @exception com.ibm.beans.dip.DipRejectAdditionException Indicates that the
            Session Authentication Dip cannot be added to the dippable bean.

        @see com.ibm.beans.dip.Dip#createImplementation()

    */
    public Object createImplementation( Object obj ) throws DipRejectAdditionException {
        SADip.traceln( 100, "Entering createImplementation" );
        if ( implementation.associateDippableBean( obj ) == false ) {
            String errorString = implementation.retrieveMessage( "SA_FINAL_METHODS" );
            implementation = null;      // Don't need the implementation any longer
            SADip.errorMessage( errorString );
            throw new DipRejectAdditionException( errorString );
        }

        /*
            Note that createImplementation is called when the SessionAuthenticationDip
            Dip is applied.  However, we don't want this dip to become "active" (i.e.
            start vetoing things pending authentication) when it's applied.  We don't want
            become active until after the dippable bean has be serialized, deployed (presumably),
            and deserialized (via readObject).

            So, a minor "trick" ... Even when the dip is first applied, the create*Listener() methods
            are called.  We set the event listeners to null, which is an indication to the dipping
            framework not to call anything.  In readObject, where the dipped bean is deserialized, we
            will create the REAL listeners.
        */
        vetoableListener = null;
        instantiationListener = null;
        SADip.traceln( 100, "Returning from createImplementation");
        return implementation;
    }

    /**
     * Indicates SessionAuthenticationDip has nothing that needs to be released
     * when the Dip is removed from the bean.
     * @see com.ibm.beans.dip.Dip#releaseImplementation()
     *
     */
    public void releaseImplementation() {}

    /**
     * Determines if a method should be included in a morphed class.
     * When the dip is applied to a class as a pre-dip during morphing,
     * and the user elects to only override those methods in the dippable
     * class that the pre-dips agrees to, this method to used to determine
     * if a dip agrees to including a specific method.
     */
    public boolean includeInMorphedClass(String methodName) {
      // all methods should be morphed for security purposes
      return true;
    }

    //----------------------------------------------------------------------
    //  Methods pertaining to the Dippable Bean's Properties, Methods, and
    //  events.

    /**
        Registers methods to call after the dippable bean
        property changed methods is called.  The Session Authentication Dip does
        not register any methods for the dippable bean property changed methods.

        @param imp The Session Authentication Dip implementation returned from
            the createImplementation method.
        @param methods[] An array of property changed methods in the dippable bean.
        @return <tt>null</tt> indicating that no methods are to be registered.

        @see com.ibm.beans.dip.Dip#createPropertyChangeListener()
    */
    public DipPropertyChangeListener[] createPropertyChangeListener(Object imp, String[] methods) {
        SADip.traceln( 200, "createPropertyChangeListener - none registered");
        return null;
    }

    /**
        Registers methods to call before the dippable bean
        property changed method is called.  These methods can prevent the dippable
        bean method from being called by throwing the appropriate exception.
        The Session Authentication Dip registers a method for all
        of the dippable bean property changed methods.  Until authentication is successful,
        this dip vetos all property changed methods, preventing any state change in
        the dippable bean.
        <p>
        The same Session Authentication dip method is registered for all
        the dippable bean
        vetoable methods.  This method vetos the property <tt>set</tt> and <tt>get</tt>
        methods
        if authentication has not been successful, or returns otherwise.

        @param imp The Session Authentication Dip implementation returned from
            createImplementation.
        @param methods[] An array of property changed methods in the dippable bean.
        @return An array of methods to call before the dippable bean corresponding method.

        @see com.ibm.beans.dip.Dip#createVetoableChangeListener()
    */
    public DipVetoableChangeListener[] createVetoableChangeListener(Object imp, String[] methods) {
        int numberOfMethods = methods.length;
        DipVetoableChangeListener[] vetoableChangeListener = new DipVetoableChangeListener[ numberOfMethods ];

        for ( int i = 0 ; i < numberOfMethods ; i++ ) {
            SADip.traceln( 10000, "registering vetoable listener for " + methods[ i ] );
            vetoableChangeListener[ i ] = vetoableListener;
        }

        return vetoableChangeListener;
    }

    /**
        Registers methods to call after the dippable bean
        method is called.  The Session Authentication Dip only
        registers a method for the dippable bean's readObject method.  This is
        the point at which the Session Authentication dip  attempts to authenticate
        the dippable bean.
        <p>
        If authentication is unsuccessful, then all subsequent properties, methods, and events
        are vetoed by this dip.

        @param imp The Session Authentication Dip implementation returned from
            createImplementation.
        @param methods[] An array of property changed methods in the dippable bean.
        @return An array of nulls except for the element corresponding to the
            dippable bean's readObject method.

        @see com.ibm.beans.dip.Dip#createMethodCallListener()
    */
    public DipMethodCallListener[] createMethodCallListener(Object imp, String[] methods) {
        int numberOfMethods = methods.length;
        DipMethodCallListener[] methodCallListener = new DipMethodCallListener[ numberOfMethods ];

        // Only intercept the readObject() method call
        for ( int i = 0 ; i < numberOfMethods ; i++ ) {

            if ( methodNameMatches( methods[i], "readObject" ) ) {
                SADip.traceln( 10000, "Intercept readObject" );
                methodCallListener[ i ] = instantiationListener;
            }
            else {
                methodCallListener[ i ] = null;
            }
        }

        return methodCallListener;
    }

    /**
        Registers methods to call before the dippable bean
        method is called.  These dip methods can prevent the dippable
        bean method from being called by throwing the appropriate exception.
        The Session Authentication Dip registers methods for all
        of the dippable bean methods with the exception of:
        <dl>
        <dt>writeObject
        <dd>
            Vetoing a writeObject call would prevent a bean that contained the dippable bean from
            being serialized correctly. If the dippable bean's writeObject was not called,
            a subsequent readObject on the containing bean could not de-serialize the
            dippable bean.
        <dt>finalize
        <dd>
            Because finalize is called when the dippable bean is garbage collected, it should not be
            vetoed. It may contain important resource releasing logic or other cleanup logic.
        <dt>readObject and the dippable bean constructor
        <dd>
            Because the dippable bean does not exist until these methods complete, they are
            not vetoable.
        </dl>
        <p>
        The same Session Authentication method is registered for all the dippable bean
        vetoable methods.  This method vetos the dippable bean method by throwing
        the appropriate exception if
        authentication has not been successful, or returns otherwise.

        @param imp The Session Authentication Dip implementation returned from
            the createImplementation method.
        @param methods[] An array of methods in the dippable bean.
        @return An array of methods to call before the dippable bean's corresponding method.

        @see com.ibm.beans.dip.Dip#createVetoableMethodListener()
    */
    public DipVetoableMethodListener[] createVetoableMethodListener(Object imp, String[] methods) {
        int numberOfMethods = methods.length;
        DipVetoableMethodListener[] vetoableMethodListener = new DipVetoableMethodListener[ numberOfMethods ];

        // Don't intercept the following method calls (see javadoc comment above):
        // - finalize
        // - writeObject
        // - constructor
        // - readObject

        for ( int i = 0 ; i < numberOfMethods ; i++ ) {

            if ( methodNameMatches( methods[i], "finalize" )
                 || methodNameMatches( methods[i], "writeObject" )
                 || methodNameMatches( methods[i], "readObject" )

               ) {
                SADip.traceln( 10000, "Not intercepting post " + methods[i] );
                vetoableMethodListener[ i ] = null;
            }
            else {
                vetoableMethodListener[ i ] = vetoableListener;
            }
        }

        return vetoableMethodListener;
    }

    /**
        Registers methods to call after the dippable bean
        event method is called.  The Session Authentication Dip does
        not register any methods for the dippable bean event methods.

        @param imp The Session Authentication Dip implementation returned from
            createImplementation.
        @param methods[] An array of event methods in the dippable bean.
        @return <tt>null</tt> indicating that no event methods are intercepted.

        @see com.ibm.beans.dip.Dip#createEventFireListener()
    */
    public DipEventFireListener[] createEventFireListener(Object imp, String[] methods) {
        return null;
    }

    /**
        Registers methods to call before the dippable bean
        event method is called.  These dip methods can prevent the dippable
        bean event method from being called by throwing the appropriate exception.
        The Session Authentication Dip registers a method for all
        of the dippable bean event methods.  Until authentication is successful,
        this dip vetos all event methods, preventing events from being
        fired by the dippable bean.
        <p>
        The same Session Authentication dip method is registered for all
        the dippable bean's
        vetoable methods.  This method vetos the event
        if authentication has not been successful, or returns otherwise.

        @param imp The Session Authentication Dip implementation returned from
            the createImplementation method.
        @param methods[] An array of event methods in the dippable bean.
        @return An array of methods to call before the dippable bean corresponding method.

        @see com.ibm.beans.dip.Dip#createVetoableEventListener()
    */
    public DipVetoableEventListener[] createVetoableEventListener(Object imp, String[] methods) {
        int numberOfMethods = methods.length;
        DipVetoableEventListener[] vetoableEventListener = new DipVetoableEventListener[ numberOfMethods ];

        for ( int i = 0 ; i < numberOfMethods ; i++ ) {
            SADip.traceln( 10000, "Intercepting event: " + methods[ i ] );
            vetoableEventListener[ i ] = vetoableListener;
        }

        return vetoableEventListener;
    }

    // Protected methods
    SAImpl getImplementation() {
        return implementation;
    }

    // Private Methods
    private void readObject( ObjectInputStream stream )
        throws java.lang.ClassNotFoundException, java.io.IOException,  java.io.NotActiveException {
        SADip.traceln( 100, "*** SessionAuthenticationDip.readObject() called ***" );
        stream.defaultReadObject();

        if ( implementation.isDipApplied() ) {
            constructListeners();
        }
    }

    private void constructListeners() {
        // Create the objects we will need for the create*Listener() methods which are
        // called by the dip framework.

        vetoableListener = new SAVetoableListener( implementation );
        instantiationListener = new SAInstantiationListener( implementation );
    }

    private boolean methodNameMatches( String dippableMethod, String nameToMatch ) {
        // String.indexOf returns a -1 if it doesn't contain the parameter string
        return ( dippableMethod.indexOf( nameToMatch ) != ( -1 ) );
    }

    // Data
    private SAImpl implementation;
    transient private SAVetoableListener vetoableListener;
    transient private SAInstantiationListener instantiationListener;

    private final static String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}
