//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbContent
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.15 com/java/src/com/ibm/beans/samples/dips/sessauth/SessionAuthenticationDipCustomizer.java, bbSecurityDip, bb97c, bb97c9750e 11/19/97 10:24:59 [12/11/97 17:01:57]
package com.ibm.beans.samples.dips.sessauth;

import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;

import java.beans.Beans;
import java.beans.BeanInfo;
import java.beans.BeanDescriptor;
import java.beans.Introspector;
import java.beans.IntrospectionException;
import java.beans.Customizer;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;

import java.awt.Panel;

/**
    This object allows customization of the Session Authentication Dip using an AWT-based
    interaction with a user.  The three items that are customizable are the class names
    to be used for the:
    <ul>
    <li>Principal
    <li>Principal query
    <li>Authenticator
    </ul>
    @see com.ibm.beans.samples.dips.sessauth.SAPrincipal
    @see com.ibm.beans.samples.dips.sessauth.SAPrincipalQuery
    @see com.ibm.beans.samples.dips.sessauth.SAAuthenticator

*/
public class SessionAuthenticationDipCustomizer extends Panel implements java.beans.Customizer, ActionListener
{
    /**
        Constructs the Session Authentication Customizer.
    */
    public SessionAuthenticationDipCustomizer () {

        try {
            resourceBundle = (SAResourceBundle) ResourceBundle.getBundle( SADip.RESOURCE_BUNDLE );
        }
        catch ( MissingResourceException e ) {
            SADip.errorMessage( "SessionAuthenticationDipCustomizer: Unable to open resource bundle" );
        }

        /*
            Put the NLS text in the labels
        */

        applyButton = new Button( retrieveMessage( "SA_APPLY" ) );
        applyButton.addActionListener( this );

        cancelButton = new Button( retrieveMessage( "SA_CANCEL" ) );
        cancelButton.addActionListener( this );

        principalLabel = new Label( retrieveMessage( "SA_PRINCIPAL_LABEL" ) );
        principalQueryLabel = new Label( retrieveMessage( "SA_PRINCIPAL_QUERY_LABEL" ) );
        authenticatorLabel = new Label( retrieveMessage( "SA_AUTHENTICATOR_LABEL" ) );

        principalEntry = new TextField();
        principalQueryEntry = new TextField();
        authenticatorEntry = new TextField();

    }

    /**
        Sets the Session Authentication Dip object that this customizer is to customize.

        @param objectToCustomize The SessionAuthenticationDip instance to customize.
    */
    public void setObject( Object objectToCustomize ) {

        myDip = (SessionAuthenticationDip) objectToCustomize;
        myDipImpl = myDip.getImplementation();

        /*
            Build up our customizer.  It consists of three entry fields, one each
            for the Principal, Principal Query, and the Authenticator.

            We will not act on any changes until the APPLY button is pressed.
            The CANCEL button will revert the display to the current values.
        */

        setLayout( new BorderLayout() );

        queryPanel.setLayout( new GridLayout(  3 /* rows: Principal, Principal Query, Authenticator */,
                                               2 /* cols: Label and entry field */ ) );

        // Fill in the current Policy class name value, if any
        queryPanel.add( principalLabel );
        principalEntrySetText();
        queryPanel.add( principalEntry );

        // Fill in the current Policy Query class name value, if any
        queryPanel.add( principalQueryLabel );
        principalQueryEntrySetText();
        queryPanel.add( principalQueryEntry );

        // Fill in the current authenticator class name value, if any
        // There is some scalfolding for making this a collection of
        // authenticators.  However, not all logic is in place yet, so we only actually
        // use the first authenticator in the list...
        queryPanel.add( authenticatorLabel );
        authenticatorEntrySetText();
        queryPanel.add( authenticatorEntry );

        add( "Center", queryPanel );

        buttonPanel.setLayout( new FlowLayout( FlowLayout.CENTER ) );

        buttonPanel.add( applyButton );
        buttonPanel.add( cancelButton );

        add( "South", buttonPanel );
    }

    /**
        Returns this panel's preferred width and height.

        @return A Dimension structure containing this panel's preferred width and height.
    */
    public Dimension getPreferredSize() {
       return(new Dimension(355, 90));
    }

    /**
        Registers a listener for the PropertyChange event.
        This customizer fires a PropertyChange
        event when it changes the Session Authentication dip such that
        the displayed properties are to be refreshed.

        @param listener The object to be notified when this customizer changes a property.
    */
    public void addPropertyChangeListener( PropertyChangeListener listener ) {
	    propertyChangeSupport.addPropertyChangeListener( listener );
    }

    /**
        Removes the listener from the list of objects to be notified when this
        customizer changes a property in the Session Authentication dip.

        @param listener The object to be removed from the list of listeners.
    */
    public void removePropertyChangeListener( PropertyChangeListener listener ) {
	    propertyChangeSupport.removePropertyChangeListener( listener );
    }

    /**
        Handle the <b>Apply</b> and <b>Cancel</b> buttons.
	@param event The event that describes which action was performed.
    */
    public void actionPerformed( ActionEvent event ) {

        String actionCommand = event.getActionCommand();

        if ( actionCommand.equals( retrieveMessage( "SA_APPLY" ) ) ) {
            SADip.traceln( 1000, "Customizer: APPLY button" );
            handleApplyButton();
        }
        else if ( actionCommand.equals( retrieveMessage( "SA_CANCEL" ) ) ) {
            SADip.traceln( 1000, "Customizer: CANCEL button" );
            handleCancelButton();
        }
        else {
            SADip.errorMessage( "*** unknown action button ***" );
        }
    }

    //----------------------------------------
    // Private methods and data

    private void handleApplyButton() {
        // Check that all the changes are OK, and if so, apply them

        // the following will be set based on parsing the user's input.
        // reinitialize them in case we find invalid input from the user.
        enteredPrincipal = null;
        enteredPrincipalQueryClassName = null;
        enteredAuthenticator.removeAllElements();

        boolean allEntriesValid = true; // hope for the best :-)

        if ( !principalEntryValid() ) {
            SADip.errorMessage( retrieveMessage( "SA_INVALID_PRINCIPAL_ENTRY" ) );
            allEntriesValid = false;
        }
        else if ( !principalQueryEntryValid() ) {
            SADip.errorMessage( retrieveMessage( "SA_INVALID_PRINCIPAL_QUERY_ENTRY" ) );
            allEntriesValid = false;
        }
        else if ( !authenticatorEntryValid() ) {
            SADip.errorMessage( retrieveMessage( "SA_INVALID_AUTHENTICATOR_ENTRY" ) );
            allEntriesValid = false;
        }

        if ( allEntriesValid ) {
            // Make the changes to SAImpl, re-display the values from there
            // then fire property changed event (?)
            myDipImpl.setPrincipal( enteredPrincipal );
            myDipImpl.setPrincipalQueryClassName( enteredPrincipalQueryClassName );
            myDipImpl.setAuthenticators( enteredAuthenticator );

    	    propertyChangeSupport.firePropertyChange( "", null, null );

            principalEntrySetText();
            principalQueryEntrySetText();
            authenticatorEntrySetText();
        }
    }

    private void handleCancelButton() {
        // just reset the values in the entry boxes to whatever is there now
        principalEntrySetText();
        principalQueryEntrySetText();
        authenticatorEntrySetText();
    }

    private String retrieveMessage( String messageKey ) {
        return (String) resourceBundle.handleGetObject( messageKey );
    }

    //----------------------------------------
    // Process the APPLY button action

    private boolean principalEntryValid() {
        SADip.traceln( 1000, "process Principal" );

        String enteredClassName = principalEntry.getText();
        Object enteredObject = null;
        Object currentObject = myDipImpl.getPrincipal();
        boolean entryValid = false; // assume the worst :-(

        if ( ( ( enteredObject = retrieveObject( enteredClassName, currentObject ) ) != null )
             && enteredObject instanceof SAPrincipal ) {
            enteredPrincipal = (SAPrincipal) enteredObject;
            // If the entered principal has a customizer, invoke it
            customizeEntry( enteredObject, enteredClassName );
            entryValid = true;
        }
        else {
            enteredPrincipal = null;
            entryValid = false;
        }

        return entryValid;
    }

    private void customizeEntry( Object enteredObject, String enteredClassName ) {
        SADip.traceln( 1000, "customizeEntry" );
        Class customizerClass = null;

        try {
            BeanInfo beanInfo = Introspector.getBeanInfo( enteredObject.getClass() );
            BeanDescriptor beanDescriptor = beanInfo.getBeanDescriptor();
    	    customizerClass = beanDescriptor.getCustomizerClass();
        }
        catch ( IntrospectionException ex ) {
            SADip.errorMessage( retrieveMessage( "SA_INTROSPECTION_EXCEPTION" ) );
        }

        SADip.println( "Entered Customizer class: " + customizerClass );

        if ( customizerClass != null ) {

            Customizer customizer = null;
            try {
                customizer = (Customizer) customizerClass.newInstance();
            }
            catch ( InstantiationException ex ) {
                SADip.errorMessage( retrieveMessage( "SA_INSTANTIATION_EXCEPTION" ) );
                ex.printStackTrace();
                return;
            }
            catch ( IllegalAccessException ex ) {
                SADip.errorMessage( retrieveMessage( "SA_INSTANTIATION_EXCEPTION" ) );
                ex.printStackTrace();
                return;
            }

            Frame frame = new Frame();
            SACustomizerDialog customizerDialog = new SACustomizerDialog( customizer,
                                                  retrieveMessage( "SA_CUSTOMIZER_DIALOG" ),
                                                  frame,
                                                  enteredObject,
                                                  retrieveMessage( "SA_DONE_LABEL" ));
            // Block on show until the window is no longer visible
            SADip.println( "Showing entered customizer" );
            customizerDialog.show();
            SADip.println( "returned from show" );
        }
    }

    private boolean principalQueryEntryValid() {
        SADip.traceln( 1000, "process Principal Query" );

        String enteredPrincipalQuery = principalQueryEntry.getText();
        Object enteredObject = null;
        boolean entryValid = false;

        if ( ( ( enteredObject = retrieveObject( enteredPrincipalQuery, null ) ) != null )
             && enteredObject instanceof SAPrincipalQuery ) {

            /*
                Unlike the Principal and Authenticators, we only save a class name
                so there is nothing to customize here.  Thus no call to customizeEntry
            */
            enteredPrincipalQueryClassName = enteredPrincipalQuery;
            entryValid = true;
        }
        else {
            enteredPrincipalQueryClassName = null;
            entryValid = false;
        }

        return entryValid;
    }

    private boolean authenticatorEntryValid() {
        SADip.traceln( 1000, "process Authenticator" );

        String enteredClassName = authenticatorEntry.getText();
        // Currently, the authenticators vector only supports one entry,
        // so we need only get the first element.
        Object currentObject = myDipImpl.getAuthenticators().firstElement();
        Object enteredObject = null;
        boolean entryValid = false; // assume the worst :-(

        /*
            The authenticator will eventually be a collection of
            authenticators.  The scaffolding is only partly in place, thus the use
            of Vector, and why we removeAllElements - we assume elsewhere throughout
            the dip that there is only a single authenticator.
        */

        if ( ( ( enteredObject = retrieveObject( enteredClassName, currentObject ) ) != null )
             && enteredObject instanceof SAAuthenticator ) {

            customizeEntry( enteredObject, enteredClassName );

            enteredAuthenticator.removeAllElements(); // We only use one element right now.
            enteredAuthenticator.addElement( enteredObject );
            entryValid = true;
        }
        else {
            SADip.traceln( 1000, "authenticator entered invald: " + enteredClassName );
            enteredAuthenticator.removeAllElements(); // We only use one element right now.
            entryValid = false;
        }

        return entryValid;
    }

    private void principalEntrySetText() {
        if ( myDipImpl.getPrincipal() != null ) {
            Class currentClass = myDipImpl.getPrincipal().getClass();
            SADip.traceln( 1000, "class = " + currentClass + " principal entry = " + principalEntry );
            principalEntry.setText( currentClass.getName() );
        }
        else {
            principalEntry.setText( null );
        }
    }

    private Object retrieveObject( String className, Object currentObject ) {

        // If the class name is the same as the current object's class name
        // then return the current object.  Otherwise, allocate a new object

        if ( currentObject != null
             &&  className.equals( currentObject.getClass().getName() ) ) {
            return currentObject;
        }

        Class enteredClass = null;
        Object enteredObject = null;

        try {
            enteredClass = Class.forName( className );
        }
        catch ( ClassNotFoundException exception ) {
            SADip.traceln( 1, "Unable to load/create class: " + className );
            enteredClass = null;
            enteredObject = null;
        }

        if ( enteredClass != null ) {
            try {
                enteredObject = enteredClass.newInstance();
            }
            catch ( InstantiationException ex ) {
                enteredObject = null;
            }
            catch ( IllegalAccessException ex ) {
                enteredObject = null;
            }
        }

        return enteredObject;
    }

    private void principalQueryEntrySetText() {
        principalQueryEntry.setText( myDipImpl.getPrincipalQueryClassName() );
    }

    private void authenticatorEntrySetText() {
        if ( myDipImpl.getAuthenticators() != null ) {
            Class currentClass = ((SAAuthenticator) myDipImpl.getAuthenticators().firstElement()).getClass(); // Only work on the first now
            authenticatorEntry.setText( currentClass.getName() );
        }
        else {
            authenticatorEntry.setText( null );
        }
    }

    private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);

    private SessionAuthenticationDip myDip = null;
    private SAImpl myDipImpl = null;

    private SAResourceBundle resourceBundle = null;

    private SAPrincipal enteredPrincipal = null;
    private String enteredPrincipalQueryClassName = null;
    private Vector enteredAuthenticator = new Vector(); // Collection of ONE SAAuthenticator

    //----------------------------------------
    // Customizer panel elements

    // We use our own reference to the resource bundle rather than the
    // one in SAImpl so we can initialize the labels more effieiently

    private Panel queryPanel = new Panel();
    private Panel buttonPanel = new Panel();

    private Button applyButton = null;
    private Button cancelButton = null;

    private Label principalLabel = null;
    private Label principalQueryLabel = null;
    private Label authenticatorLabel = null;

    private TextField principalEntry = null;
    private TextField principalQueryEntry = null;
    private TextField authenticatorEntry = null;

    private final static String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}
