//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbContent
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.11 com/java/src/com/ibm/beans/samples/dips/sessauth/demo/SamplePrincipalQuery.java, bbSecurityDip, bb97c, bb97c9750e 11/19/97 10:24:45 [12/11/97 17:01:57]
package com.ibm.beans.samples.dips.sessauth.demo;

import com.ibm.beans.samples.dips.sessauth.SAPrincipalQuery;
import com.ibm.beans.samples.dips.sessauth.SAPrincipal;
import com.ibm.beans.samples.dips.sessauth.SAImpl;

import java.awt.*;
import java.util.ResourceBundle;
import java.util.MissingResourceException;

/**
    This sample for the Session Authentication dip queries the
    user using an AWT modal dialog for the information required by the
    SamplePrincipal object.  It attempts to use the SampleSimpleAuthenticator
    object to validate that principal information.
*/
public class SamplePrincipalQuery extends SAPrincipalQuery
{
    /**
        Construct a SamplePrincipalQuery.
    */
    public SamplePrincipalQuery() {

        try {
            resourceBundle = (SampleResourceBundle) ResourceBundle.getBundle(
                "com.ibm.beans.samples.dips.sessauth.demo.SampleResourceBundle" );

        }
        catch ( MissingResourceException e ) {
            System.out.println( "SamplePrincipalQuery: Unable to open resource bundle" );
        }


        queryFrame = new Frame( );
        queryDialog = new SamplePrincipalQueryAWT( this, queryFrame, retrieveMessage( "SM_DIALOG_TITLE" ) );
    }

    /**
        Query for the principal information and attempt to authenticate it.
        This method queries the
        end user to enter a principal consisting of a User ID and Password using a modal dialog.
        It then invokes the authenticator to validate that principal.  The Session
        Authentication Dip
        state indicating whether authentication was successful is updated
        based on the return value of this method.
        <p>
        This method must suspend the calling thread until
        authentication is complete (successful or not).

        @param principal The principal object queried.
        @return <tt>true</tt> to indicate authentication successful; <tt>false</tt> otherwise.

        @see com.ibm.beans.samples.dips.sessauth.SAImpl#setPrincipal()
        @see com.ibm.beans.samples.dips.sessauth.SAImpl#doAuthentication()
    */
    public boolean queryForPrincipal( SAPrincipal principal ) {
        boolean authenticationOK = false;
        SAImpl dipImplementation = getImplementation();

        SamplePrincipal samplePrincipal = (SamplePrincipal) principal;

        int maxRetries = 3;         // Make this configurable later
        int attempt;

        for ( attempt = 0; attempt < maxRetries && !authenticationOK; attempt++ ) {

            queryDialog.getPrincipalInfo( samplePrincipal );

            dipImplementation.setPrincipal( samplePrincipal );

            authenticationOK = dipImplementation.doAuthentication();
        }

        if ( ( attempt == maxRetries ) && ( authenticationOK == false ) ) {
            System.out.println( resourceBundle.handleGetObject( "SM_MAX_RETRY" ) );
        }

        // If authentication failed or not, the dialogs won't be redisplayed
        // from this instantiation.  The user will have to close and reactivate

        queryDialog.dispose();
        queryFrame.dispose();

        return authenticationOK;
    }

    //----------------------------------------
    // Private methods & data

    String retrieveMessage( String messageKey ) {
        return (String) resourceBundle.handleGetObject( messageKey ) ;
    }

    private Frame   queryFrame;
    private SamplePrincipalQueryAWT  queryDialog;
    private SampleResourceBundle resourceBundle;

    private final static String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
}
