//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbTraceDip
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.15 com/java/src/com/ibm/beans/samples/dips/trace/MsgDispClient.java, bbTraceDip, bb97c, bb97c9750e 10/29/97 10:45:35 [12/11/97 17:02:03]
package com.ibm.beans.samples.dips.trace;

import java.io.DataOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Serializable;
import java.net.InetAddress;
import java.net.Socket;
import java.net.SocketException;

/**
* MsgDisp is a logging facility which will run remotely. It provides
* function to log or print messages to
* a scrolling window as opposed to printing to standard out, etc.  It also
* allows you to save the contents of the window.  The MsgDispServer, the server, can run
* on one machine while the MsgDispClient(the client which is called to log messages)
* runs on another. This utility uses java Sockets for communications between the client and server.
* MsgDispServer supports calls from MsgDispClients in multiple processes and threads at
* the same time.  Property configuration information is used to point the client to the
* correct server location.
* The user must deploy MsgDispServer manually, if not the log output goes to the
* bit bucket(not captured in MsgDispServer window), however, the log output will also
* be written to standard out.
*
* MsgDisp Implementation
*
*    Server Functionality
*
*        MsgDispServer at deployment creates a GUI frame and TextArea.
*        It then creates a ServerSocket and blocks for a client to connect.
*        It then instantiates an instance of  the SocketHd class as a new
*        thread for every connecting client.  When a client attaches to the
*        socket the SocketHd thread reads the log message from the socket
*        and send a message to MsgDispServer to display it in its TextArea
*        within the GUI frame.
*   Client Functionality
*        The MsgDispClient creates a socket connection to the desired host
*         where MsgDispServer is running and writes to the socket.
*/

//=======================================================================================
/**
* This class represents the MsgDispClient instance
* that connects to the MsgDispServer instance and
* sends data through a socket connection.
* (The socket connection is implemented using a Java Socket instance.)
* A MsgDispServer instance is a server GUI that enables
* the MsgDispClient instance to log messages.
*/
//=======================================================================================
public class MsgDispClient implements Serializable
{
    /**
     * Constructs a MsgDispClient.
     */
  public MsgDispClient()
    {
    super();
    }

   //=====================================================================
   /**
   * Logs output to the MsgDispServer GUI through a socket.
   * The method opens a socket on the server and writes to it.
   * The MsgDispServer GUI reads the data and displays it.
   * <p>
   * @param myBuffer The message to log.
   *
   */
   //======================================================================
   public synchronized void qprintf(String myBuffer)
   {
    MessageToDisplay = myBuffer;
    try
      { // if we have not created a socket or the MsgDispServer is not running
        if(((mySocket == null) && (isServerUp == true)) || (msgDispHostName != saveMsgDispHostName))
         {
                 isServerUp = true;
          try
            {
             saveMsgDispHostName = msgDispHostName; // CJC
             mySocket = new Socket(msgDispHostName, msgDispPortNumber);
            }
            catch (Exception e) {
              if(e instanceof FileNotFoundException)
                {
                 msgDispHostName = "127.0.0.1";  // set msgDispHostName to the local machine
                 saveMsgDispHostName = msgDispHostName; // CJC
                 mySocket = new Socket(msgDispHostName, msgDispPortNumber);
                }
              else {
                // System.out.println("Error: " + e.toString() );
                // MsgDispServer is not running
                isServerUp = false;
              }
            } // end catch
         } // end mySocket == null
         // System.out.println("after create socket!\n");
	 if(isServerUp) // CJC
           {
           is = new DataOutputStream(mySocket.getOutputStream());
           boolean more = true;
           // System.out.println("Before WriteBytes!\n");
           InetAddress myAddress = InetAddress.getLocalHost();  // get host name of machine
           // prefix host name to message and write it to socket.
           is.writeBytes(myAddress.getHostName() + ": " + MessageToDisplay + "\n");
           System.out.println(MessageToDisplay + "\n");
           //         is.close();
           //         t.close();
           }
         else System.out.println(MessageToDisplay + "\n"); // MsgDispServer not running,output to stdout
       } // end first try
       catch(IOException e)
         {
          if(e instanceof SocketException)
            {
             // Will get this exception if MsgDispServer is not running!
             // System.out.println("SocketException in MsgDispServer!\n");
            }
          else
             System.err.println(e);
          }
   }


   /**
   * Sets the host name where the MsgDispServer GUI is running.
   * <p>
   * @param myHost The location of the MsgDispServer GUI.
   */
   public void setMsgDispHostName(String myHost)
      {
      msgDispHostName = myHost;
      }


   /**
   * Gets the host name where the MsgDispServer GUI window is running.
   * <p>
   * @return The location of the MsgDispServer GUI.
   */
   public String getMsgDispHostName()
      {
      return msgDispHostName;
      }

   /**
   * Sets the port number where the MsgDispServer listens for communication.
   * <p>
   * @param port The number of the MsgDispServer port.
   */
   public void setMsgDispPortNumber(int port)
      {
      msgDispPortNumber = port;
      }


   /**
   * Gets the port number where the MsgDispServer listens for communication.
   * <p>
   * @return The number of the MsgDispServer port.
   */
   public int getMsgDispPortNumber()
      {
      return msgDispPortNumber;
      }


   private transient boolean isServerUp = true;
   private transient DataOutputStream is = null;
   private transient Socket mySocket = null;
   private String msgDispHostName = "127.0.0.1";  // set msgDispHostName equal to the local machine.
   private String saveMsgDispHostName = "InitValue";  // CJC
   private int msgDispPortNumber = 8189;
   private transient String MessageToDisplay;

   // Serializable Stream Unique ID
   private static final long serialVersionUID = 2000L;

   private static final String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
}

