//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbTraceDip
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.11 com/java/src/com/ibm/beans/samples/dips/trace/MsgDispServer.java, bbTraceDip, bb97c, bb97c9750e 11/14/97 14:14:55 [12/11/97 17:02:03]
package com.ibm.beans.samples.dips.trace;

import com.ibm.beans.widgets.CloseWindowAdapter;
import java.awt.FileDialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Menu;
import java.awt.MenuBar;
import java.awt.MenuItem;
import java.awt.MenuShortcut;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ResourceBundle;

/**
* MsgDisp is a logging facility which will run remotely. It provides
* function to log or print messages to
* a scrolling window as opposed to printing to standard out, etc.  It also
* allows you to save the contents of the window.  The MsgDispServer, the server, can run
* on one machine while the MsgDispClient(the client which is called to log messages)
* runs on another. This utility uses java Sockets for communications between the client and server.
* MsgDispServer supports calls from MsgDispClients in multiple processes and threads at
* the same time.  Property configuration information is used to point the client to the
* correct server location.
* The user must deploy MsgDispServer manually, if not the log output goes to the
* bit bucket(not captured in MsgDispServer window), however, the log output will also
* be written to standard out.
*
* MsgDisp Implementation
*
*    Server Functionality
*
*        MsgDispServer at deployment creates a GUI frame and TextArea.
*        It then creates a ServerSocket and blocks for a client to connect.
*        It then instantiates an instance of  the SocketHd class as a new
*        thread for every connecting client.  When a client attaches to the
*        socket the SocketHd thread reads the log message from the socket
*        and send a message to MsgDispServer to display it in its TextArea
*        within the GUI frame.
*   Client Functionality
*        The MsgDispClient creates a socket connection to the desired host
*         where MsgDispServer is running and writes to the socket.
*/


//=======================================================================================
/**
* The MsgDispServer class is a server GUI
* that enables MsgDispClient instances to log messages to it.
* This utility serves as a centralized place where
* Java beans can log messages.
* To start the MsgDispServer GUI, enter <br>
* "java com.ibm.beans.samples.dips.trace.MsgDispServer".
*/
//========================================================================================
public class MsgDispServer extends Frame implements ActionListener
{
   /**
    * Constructs a MsgDispServer instance.
    */
  public MsgDispServer()
    { try {
      getMessages();
      menuFile = (String)messages.handleGetObject("menuFile");
      menuNew = (String)messages.handleGetObject("menuNew");
      menuSave = (String)messages.handleGetObject("menuSave");
      menuSaveAs = (String)messages.handleGetObject("menuSaveAs");
      menuQuit = (String)messages.handleGetObject("menuQuit");
      dialogSaveFile = (String)messages.handleGetObject("dialogSaveFile");
      traceOutput = (String)messages.handleGetObject("TraceOutput");

      filename = "msgdisp.log";
      InetAddress myAddress = InetAddress.getLocalHost();
      setTitle(traceOutput + " (" + myAddress.getHostName() + ")");
      Panel p = new Panel();
      p.setLayout(new FlowLayout());

      add("South", p);

      MenuBar mbar = new MenuBar();
      Menu m = new Menu(menuFile,true);
      MenuItem mi = new MenuItem(menuNew, new MenuShortcut(27)); //lets 'Esc' key clear window
      mi.addActionListener(this);
      m.add(mi);
      m.addSeparator();
      mi = new MenuItem(menuSave);
      mi.addActionListener(this);
      m.add(mi);
      mi = new MenuItem(menuSaveAs);
      mi.addActionListener(this);
      m.add(mi);
      m.addSeparator();
      mi = new MenuItem(menuQuit);
      mi.addActionListener(this);
      m.add(mi);

      mbar.add(m);

      setMenuBar(mbar);

      myTextArea = new TextArea(10,60);
      myTextArea.setEditable(false);

      add("Center",myTextArea);

      addWindowListener(new CloseWindowAdapter(this)); //jwa: Makes window close button work
    }
    catch(Exception e)
      {
       System.out.println(e);
      }
   }

  private void getMessages() {
    if (messages == null) {
      messages = (Messages)ResourceBundle.getBundle("com.ibm.beans.samples.dips.trace.Messages");
    }
  }

   /**
    * Implements the functions behind
    * the menu options. These options are New, Save, SaveAs, and Quit.
    * @param evt The event.
    */
   public void actionPerformed(ActionEvent evt)
   {
       Object arg = evt.getSource();

         if(arg.equals(menuQuit))
                System.exit(0);
         if(arg.equals(menuSave))
            {
           try {
            if(filename != "msgdisp.log")
             {
                FileOutputStream myfile = new FileOutputStream(filename);
                PrintWriter pout = new PrintWriter(myfile);
                pout.print(myTextArea.getText());
                pout.flush();
                pout.close();

             }
            else {
              FileDialog d = new FileDialog(this,dialogSaveFile,FileDialog.SAVE);
               d.setDirectory(".");
               d.setFile(filename);
               d.show();
               filename = d.getFile();
               if(filename != null)
                {
                FileOutputStream myfile = new FileOutputStream(filename);
                PrintWriter pout = new PrintWriter(myfile);
                pout.print(myTextArea.getText());
                pout.flush();
                pout.close();
                }
            }
           }
          catch (Exception e)
          {  System.out.println(e);
          }
            }
          if(arg.equals(menuNew))
            {
            myTextArea.setText("");
            }

          if(arg.equals(menuSaveAs))
            {
             try {
               FileDialog d = new FileDialog(this,dialogSaveFile,FileDialog.SAVE);
               d.setDirectory(".");
               d.setFile(filename);
               d.show();
               filename = d.getFile();
               if(filename != null)
                {
                FileOutputStream myfile = new FileOutputStream(filename);
                PrintWriter pout = new PrintWriter(myfile);
                pout.print(myTextArea.getText());
                pout.flush();
                pout.close();
                }

             }
          catch (Exception e)
          {  System.out.println(e);
          }
        }
   }


   //===========================================
   /**
   *  Adds the message to the GUI TextArea.
   *  @param myString The message to display in the GUI TextArea.
   */
   //===========================================
   public static synchronized void DisplayMessage(String myString)
   {
   myTextArea.append(myString + '\n');
   }

    //=================================================
    /** Creates a ServerSocket instance and one socket for
    *  each process that needs to display a message.
    * (Generally a client wants to display a message
    * because they create a socket attached to the same
    * port.) The accept() method remains blocked until
    * a new client attaches to the socket.
    * @param args[] The arguments passed to the MsgDispServer GUI
    *        (the first argument, if present, specifies the port number).
    */
    //==================================================
   public static void main(String args[])
   {  Frame f = new MsgDispServer();
      f.setBounds(0, 0, 500, 300);
      f.setVisible(true);

      int portNumber = 8189;
      if (args.length>0) portNumber = Integer.parseInt(args[0]);

      int i = 1;
      try
      {  ServerSocket s = new ServerSocket(portNumber);

         for (;;)
         {  Socket incoming = s.accept( );
            // System.out.println("Spawning " + i);
            new SocketHd(incoming, i).start();
            i++;
         }
      }
      catch (Exception e)
      {  System.out.println(e);
      }
   }

private static Frame parent;
private static String filename;
private static String saveFileName;
private static TextArea myTextArea;
private static  String menuFile;
private static  String menuNew;
private static  String menuSave;
private static  String menuSaveAs;
private static  String menuQuit;
private static  String traceOutput;
private static  String dialogSaveFile;
private transient Messages messages;
private static final String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
}




//====================================================
/**
* SocketHd class is responsible for reading data from socket and
* calling MsgDispServer to displaying it in the Message window.
* There is an instance of this class created for every
* Process logging messages to MsgDispServer.
*/
//=====================================================
class SocketHd extends Thread
{  private static final String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
   Socket incoming;
   int counter;

   SocketHd(Socket i, int c) { incoming = i; counter = c; }

   public void run()
   {  try
//    {  DataInputStream in = new DataInputStream(incoming.getInputStream());
      {  BufferedReader  in = new BufferedReader (new InputStreamReader(incoming.getInputStream()));
         PrintWriter out = new PrintWriter(incoming.getOutputStream());
         // byte [] myInput = new byte[4096];
         boolean done = false;
         while (!done)
         {  String myString = in.readLine();
            //in.readFully(myInput);
            // String myString = new String(myInput);
            if (myString == null) done = true;
            else
            {
            MsgDispServer.DisplayMessage(myString);
            }
         }
         incoming.close();
         // System.out.println("closing Spawned Thread!\n");
      }
      catch (Exception e)
      { // System.out.println(e + " :end of Stream");
        // This exception will be thrown when the client terminates.
      }

   }
}

