//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbTraceDip
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.21 com/java/src/com/ibm/beans/samples/dips/trace/TraceDip.java, bbTraceDip, bb97c, bb97c9750e 10/29/97 08:35:10 [12/11/97 17:02:02]
package com.ibm.beans.samples.dips.trace;

import com.ibm.beans.dip.EventFireEvent;
import com.ibm.beans.dip.EventVetoException;
import com.ibm.beans.dip.Dip;
import com.ibm.beans.dip.DipEventFireListener;
import com.ibm.beans.dip.DipMethodCallListener;
import com.ibm.beans.dip.DipPropertyChangeListener;
import com.ibm.beans.dip.DipRejectAdditionException;
import com.ibm.beans.dip.DipVetoableEventListener;
import com.ibm.beans.dip.DipVetoableMethodListener;
import com.ibm.beans.dip.DipVetoableChangeListener;
import com.ibm.beans.dip.MethodCallEvent;
import com.ibm.beans.dip.MethodVetoException;

import java.beans.PropertyChangeEvent;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;


/**
 *   The TraceDip class defines a dip that can be applied to a
 *   dippable bean. This dip is used for debugging. It optionally prints debug information
 *   when entering and leaving methods, when properties change, and
 *   when events are generated by the dippable bean.
 */

public class TraceDip implements Serializable, Dip, DipEventFireListener,
         DipMethodCallListener, DipVetoableEventListener, DipVetoableMethodListener,
         DipPropertyChangeListener, DipVetoableChangeListener {

    /** Constructs the TraceDip instance.
     */
    public TraceDip()
      {
      super();
      traceEventsFlag = true;
      traceMethodsFlag = true;
      tracePropertiesFlag = true;
      if(myMsgDispClient == null)
           myMsgDispClient = new MsgDispClient();
      }

    /**
     * Gets the priority of this dip in relation to all other
     * dips.  This priority is used to determine which dips
     * are acted on first.
     * <p>
     * @return The priority is 700. This priority is of a higher priority
               than the default priority of 500.
     */
    public final int priority()
      {
        return 700;
      }


    /**
     * Gets the dip type name as a String.
     * <P>
     * @return The type name of the dip.
     */
    public String typeName()
      {
      return kTraceDip;
      }


    /**
     * Indicates whether another dip can work with this dip.
     * <P>
     * @param dipTypeName The new dip requesting to be added to the bean.
     * @return <tt>true</tt> if new dip is to be added; <tt> false</tt> otherwise.
     */
    public boolean worksWith(String dipTypeName)
      {
      return true;
      }


    /**
     * Indicates whether the dip can be removed from the dipped bean.
     * <p>
     * @return <tt>true</tt> if dip can be removed; <tt> false</tt> otherwise.
     */
    public boolean isRemovable()
      {
       return true;
      }


    /**
     * Indicates whether this dip can have a separate instantiation.
     * <p>
     * @param obj The bean being dipped. This is the only
     *            handle to the bean provided to the dip.
     * @exception DipRejectAdditionException This exception is thrown if the
     *		      dip rejects being added to the bean.  This dip never rejects any beans.
     * @return This implementation disallows separate instantiation, always returns <tt>null</tt>.
     */
    public Object createImplementation(Object obj) throws DipRejectAdditionException
      {
        myDippableBean = obj; // save handle to dippable bean.
        dippableBeanClassName = myDippableBean.getClass().getName();
        dippableBeanClassName += ".";

        if(myMsgDispClient == null)
           myMsgDispClient = new MsgDispClient();
        return null;
      }


    /**
     * Indicates that the TraceDip instance
     * has nothing that needs to be released when
     * the dip is removed from the bean.
     * @see com.ibm.beans.dip.Dip#releaseImplementation()
     *
     */
    public void releaseImplementation() {}


    /**
     * Determines if a method should be included in a morphed class.
     * When the dip is applied to a class as a pre-dip during morphing,
     * and the user elects to only override those methods in the dippable
     * class that the pre-dips agrees to, this method is used to determine
     * if a dip agrees to including a specific method.
     * <p>
     * @param methodName Name of the method to be considered
     * @return boolean indicating whether to wrap given method
     */
    public boolean includeInMorphedClass(String methodName) {
        return true;   // include all methods
    }


    //----------------------------------------------------------------------
    // Dip Methods affecting all the beans properties:

    /**
     * Creates an array of DipPropertyChangeListener
     * instances describing the methods to
     * call after the dipped bean propertyChange() method is called.
     * <p>
     * @param imp The implementation returned from the createImplementation() method.
     * @param methods[] The array of event methods in the dippable bean.
     * @exception DipRejectAdditionException This exception is thrown if
     *		  the dip rejects being added to the bean.
     * @return The array of DipPropertyChangeListener instances is a one-to-one mapping
     *		with the <i>methods[]</i> input parameter.  For example,
     *		DipPropertyChangeListener[i] is used with <i>methods[i]</i>, and
     *		the length of the two arrays are equal.
     */
    public DipPropertyChangeListener[] createPropertyChangeListener(Object imp, String[] methods)
			throws DipRejectAdditionException
      {
        myPropertyChangeListener = new DipPropertyChangeListener[methods.length];
	// set myself the listener for all Properties in the dippable ban by returning
        // an array with all elements containing this as the listener.
	for(int x = 0; x < myPropertyChangeListener.length; x++) {
	   myPropertyChangeListener[x] = this;
	}
        return myPropertyChangeListener;
      } 	


    /**
     * Creates an array of DipVetoableChangeListener instances describing the methods to
     * call before the dipped bean vetoableChange() method is called.
     * <p>
     * @param imp The implementation returned from createImplementation() method.
     * @param methods[] The array of event methods in the dippable bean.
     * @exception DipRejectAdditionException This exception is thrown if
     *		  the dip rejects being added to the bean.
     * @return The array of DipVetoableChangeListener instances is a one-to-one mapping
     *		with the <i>methods[]</i> input parameter.  For example,
     *		DipVetoableChangeListener[i] is used with <i>methods[i]</i>, and
     *		the length of the two arrays are equal.
     */
    public DipVetoableChangeListener[] createVetoableChangeListener(Object imp, String[] methods)
			throws DipRejectAdditionException
      {
        myVetoableChangeListener = new DipVetoableChangeListener[methods.length];
	// set myself the listener for all contrained Properties in the dippable bean
        //  by returning an array with all elements containing this as the listener.
	for(int x = 0; x < myVetoableChangeListener.length; x++) {
	   myVetoableChangeListener[x] = this;
	}
        return myVetoableChangeListener;
      }

    /**
     * Creates an array of DipMethodCallListener instances describing the methods to
     * call after the dipped bean handleMethodCall() method is called.
     * <p>
     * @param imp The implementation returned from createImplementation() method.
     * @param methods[] The array of event methods in the dippable bean.
     * @exception DipRejectAdditionException This exception is thrown if
     *		  the dip rejects being added to the bean.
     * @return The array of DipMethodCallListener instances is a one-to-one mapping
     *		with the <i>methods[]</i> input parameter.  For example,
     *		DipMethodCallListener[i] is used with <i>methods[i]</i>, and
     *		the length of the two arrays are equal.
     */
    public DipMethodCallListener[] createMethodCallListener(Object imp, String[] methods)
			throws DipRejectAdditionException
      {
        myMethodCallListener = new DipMethodCallListener[methods.length];
	// set myself the listener for all bound methods in the dippable bean by returning
        // an array with all elements containing this as the listener.
	for(int x = 0; x < myMethodCallListener.length; x++) {
	   myMethodCallListener[x] = this;
	}
        return myMethodCallListener;
      }


    /**
     * Creates an array of DipVetoableMethodListener instances describing the method to
     * call before the dipped bean vetoableMethodCall() method is called.
     * <p>
     * @param imp The implementation returned from createImplementation() method.
     * @param methods[] The array of event methods in the dippable bean.
     * @exception DipRejectAdditionException This exception is thrown if
     *		  the dip rejects being added to the bean.
     * @return The array of DipVetoableMethodListener instances is a one-to-one mapping
     *		with the <i>methods[]</i> input parameter.  This means
     *		DipVetoableMethodListener[i] is used with <i>methods[i]</i>, and
     *		the length of the two arrays are equal.
     */
    public DipVetoableMethodListener[] createVetoableMethodListener(Object imp, String[] methods)
			throws DipRejectAdditionException
      {
        myVetoableMethodListener = new DipVetoableMethodListener[methods.length];
	// set myself the listener for all constained methods in the dippable bean by returning
        // an array with all elements containing this as the listener.
	for(int x = 0; x < myVetoableMethodListener.length; x++) {
	   myVetoableMethodListener[x] = this;
	}
        return myVetoableMethodListener;
      }


    /**
     * Creates an array of DipEventFireListener instances describing the methods to
     * call after the dipped bean handleEventFire() method is called.
     * <p>
     * @param imp The implementation returned from createImplementation() method.
     * @param methods[] The array of event methods in the dippable bean.
     * @exception DipRejectAdditionException This exception is thrown if
     *		  the dip rejects being added to the bean.
     * @return The array of  DipEventFireListener instances is a one-to-one mapping
     *		with the <i>methods[]</i> input parameter.  For example,
     *		 DipEventFireListener[i] is used with <i>methods[i]</i>, and
     *		the length of the two arrays are equal.
     */
    public DipEventFireListener[] createEventFireListener(Object imp, String[] methods)
			throws DipRejectAdditionException
      {
        myEventFireListener = new DipEventFireListener[methods.length];
	// set myself the listener for events generated by the dippable bean by returning
        // an array with all elements containing "this" as the listener.
	for(int x = 0; x < myEventFireListener.length; x++) {
	   myEventFireListener[x] = this;
	}
        return myEventFireListener;
      }


    /**
     * Creates an array of DipVetoableEventListener instances describing the methods to
     * call before the dipped bean handleVetoableEvent() method is called.
     * <p>
     * @param imp The implementation returned from createImplementation() method.
     * @param methods The array of event methods in the dippable bean.
     * @exception DipRejectAdditionException This exception is thrown if
     *		  the dip rejects being added to the bean.
     * @return The array of DipVetoableEventListener instances is a one-to-one mapping
     *		with the <i>methods[]</i> input parameter.  For example,
     *		DipVetoableEventListener[i] will be used with <i>methods[i]</i>, and
     *		the length of the two arrays are equal.
     */
  public DipVetoableEventListener[] createVetoableEventListener(Object imp, String[] methods)
			throws DipRejectAdditionException
      {
        myVetoableEventListener = new DipVetoableEventListener[methods.length];
	// set myself the listener for events generated by the dippable bean by returning
        // an array with all elements containing "this" as the listener.
	for(int x = 0; x < myVetoableEventListener.length; x++) {
	   myVetoableEventListener[x] = this;
	}
        return myVetoableEventListener;
      }

     /** Sets the <i> traceEventsFlag </i>.
      * <P>
      *  @param traceEvents  Indicates whether the events of a dippable
      *  bean are traced.
      *   <tt>true</tt> causes
      *  events to be traced; <tt>false</tt>
      *  disables event tracing.
      *
      */
  public void setTraceEventsFlag(boolean traceEvents)
       {
     	traceEventsFlag = traceEvents;
       }


     /** Sets the <i> traceMethodsFlag </i>.
      * <P>
      *  @param traceMethods Indicates whether the methods of a dippable
      *  bean are traced.
      *  <tt>true</tt> causes
      *   methods to be traced; <tt>false</tt> disables method tracing.
      *
      */
  public void setTraceMethodsFlag(boolean traceMethods)
       {
     	traceMethodsFlag = traceMethods;
       }

     /** Sets the <i> tracePropertiesFlag </i>.
      * <P>
      *  @param traceProperties Indicates whether the properties of
      *  the dippable bean are traced.
      <tt>true</tt> causes properties
      *  to be traced; <tt>false</tt> disables property tracing.
      *
      */
  public void setTracePropertiesFlag(boolean traceProperties)
       {
     	tracePropertiesFlag = traceProperties;
       }

     /** Gets the <i> traceEventsFlag </i> value, which controls whether
      *  the dippalbe bean's events are traced.
      * <P>
      * @return <tt>true</tt> if events are traced; <tt> false</tt> otherwise.
      */
  public boolean getTraceEventsFlag()
      {
       return traceEventsFlag;
      }

     /** Gets the <i> traceMethodsFlag </i> value, which controls whether the
      *  dippable bean's methods are traced.
      * <P>
      * @return <tt>true</tt> if methods are traced; <tt> false</tt> otherwise.
      */
  public boolean getTraceMethodsFlag()
      {
       return traceMethodsFlag;
      }

     /** Gets the <i>tracePropertiesFlag</i> value,
      * which controls whether the dippable bean's properties are traced.
      * <P>
      * @return <tt>true</tt> if properties are traced; <tt> false</tt> otherwise.
      */
  public boolean getTracePropertiesFlag()
      {
       return tracePropertiesFlag;
      }

     /** Uses a MsgDispClient instance to log messages to a
      * MsgDispServer GUI.
      * @param MessageToLog The message to log.
      */
  public void logTraceOutput(String MessageToLog)
      {
       myMsgDispClient.qprintf(MessageToLog);
      }

    /** Sets the <i> msgDispHostName </i> that is the machine name where
     *  the MsgDispServer GUI is running.
     * <p>
     * @param ServerHost The location of the MsgDispServer GUI.
     */
  public void setMsgDispHostName(String ServerHost)
      {
       msgDispHostName = ServerHost;
       // set host in MsgDispClient code.
       myMsgDispClient.setMsgDispHostName(ServerHost);
      }

    /**
     * Gets the <i> msgDispHostName </i> value that is the machine name where
     * the MsgDispServer GUI is running.
     * <P>
     * @return The machine name.
     *
     */
  public String getMsgDispHostName()
      {
       return myMsgDispClient.getMsgDispHostName();
      }



 // EventFireListener interface,  method implementations
    /**
     *
     *  Logs a message describing the event that was fired.
     *  This method is called when a bound event is fired.
     * <P>
     * @param evt An EventFireEvent object describing the event source
     *   	and the event that was fired.
     */
  public void handleEventFire(EventFireEvent evt)
    {
     if(traceEventsFlag)
	 logTraceOutput(kTraceDipLogName + kHandleEventFire + "|dippableBean=" + dippableBeanClassName + "|event=" + evt.getEventSetName());
    }



// DipListenerAuxiliary interface, method implementations
    /**
    * Retrieves a pointer to the dip that created the listener.
    * In this implementation, the dip returns itself.
    * <p>
    * @return This dip.
    */
   public Dip retrieveDip()
      {
      return this;
      }

    /**
     * Indicates whether the implementation of the MethodCallListener
     * instance needs arguments marshaled in the MethodCallEvent instance.
     * <p>
     * @return <tt>true</tt> if MethodCallListener needs arguments marshaled; <tt> false</tt> otherwise.
     */
   public boolean needsArguments()
      {
      return true;
      }


// MethodCallListener interface implementations
    /**
     *  Logs a message describing the method that was called.
     * This method is called when a bound method is called.
     * <P>
     * @param evt A MethodCallEvent object describing the event source
     *   	and the property that changed.
     */
    public void handleMethodCall(MethodCallEvent evt)
      {
     if(traceMethodsFlag)
        {
         Object[] myArgs = evt.getArguments(); // get args to method that was wrapped
         if(myArgs != null)
	   {
           String myArgsString = new String();
           myArgsString += "(";
           // loop through myArgs converting them to strings
           for(int i=0;i<myArgs.length;i++)
	     {
             myArgsString += myArgs[i].toString();
             myArgsString += ",";
             }
           myArgsString += ")";
           // args present
           logTraceOutput(kTraceDipLogName + kHandleMethodCall + "|dippableBean=" + dippableBeanClassName + "|MethodWrapped=" + evt.getMethodId() + "|Args=" + myArgsString);
	   }
        else
         logTraceOutput(kTraceDipLogName + kHandleMethodCall + "|dippableBean=" + dippableBeanClassName + "|MethodWrapped=" + evt.getMethodId() ); 	 // no args
        }
      }



// VetoableMethodListener interface implementations
    /**
     *  Logs a message describing the method that was called.
     * This method gets called when a constrained method is called.
     * <P>
     * @param evt An MethodCallEvent object describing the event source
     *   	and the method that was called.
     * @exception MethodVetoException This exception is thrown if the VetoableMethodListener
     *          wants the method operation not to be performed.
     */
    public void handleVetoableMethod(MethodCallEvent evt)
				throws MethodVetoException
      {
     if(traceMethodsFlag)
        {
         Object[] myArgs = evt.getArguments(); // get args to method that was wrapped
         if(myArgs != null)
	   {
           String myArgsString = new String();
           myArgsString += "(";
           // loop through myArgs converting them to strings
           for(int i=0;i<myArgs.length;i++)
	     {
             myArgsString += myArgs[i].toString();
             myArgsString += ",";
             }
           myArgsString += ")";
           // args present
           logTraceOutput(kTraceDipLogName + kHandleVetoableMethod + "|dippableBean=" + dippableBeanClassName + "|MethodWrapped= "+ evt.getMethodId() + "|args=" + myArgsString);
	   }
         else
           logTraceOutput(kTraceDipLogName + kHandleVetoableMethod + "|dippableBean=" + dippableBeanClassName + "|MethodWrapped= "+ evt.getMethodId()); 	   // no args

        }
      }

    /**
     * Allows listeners to perform recovery operations when a constrained method is vetoed.
     *  The logged message describes the method that was vetoed.
     * <P>
     * @param evt A MethodCallEvent object describing the event source
     *   	and the method that was called.
     */
    public void recoverFromMethodVeto(MethodCallEvent evt)
      {
     if(traceMethodsFlag)
        {
         Object[] myArgs = evt.getArguments(); // get args to method that was wrapped
         if(myArgs != null)
	   {
           String myArgsString = new String();
           myArgsString += "(";
           // loop through myArgs converting them to strings
           for(int i=0;i<myArgs.length;i++)
	     {
             myArgsString += myArgs[i].toString();
             myArgsString += ",";
             }
           myArgsString += ")";
           // args present
           logTraceOutput(kTraceDipLogName + kMethodVetoed + "|dippableBean=" + dippableBeanClassName + "|MethodWrapped= "+ evt.getMethodId() + "|args=" + myArgsString);
	   }
         else
	   logTraceOutput(kTraceDipLogName + kMethodVetoed + "|dippableBean=" + dippableBeanClassName + "|MethodWrapped= "+ evt.getMethodId()); // no args
        }
     }

// VetoableEventListener interface implementations
    /**
     *  Logs a message describing the event that was fired.
     * This method gets called when a constrained event is fired.
     * <P>
     * @param evt An EventFireEvent object describing the event source
     *   	and the event that was fired.
     * @exception EventVetoException This exception is thrown if the VetoableEventListener
     * 		instance wants the event firing canceled.
     */
    public void handleVetoableEvent(EventFireEvent evt)
				throws EventVetoException
      {
      if(traceEventsFlag)
	 logTraceOutput(kTraceDipLogName +  kHandleVetoableEvent + "|dippableBean=" + dippableBeanClassName + "|event=" + evt.getEventSetName());
      }

    /**
     * Allow listeners to perform recovery operations when a constrained event was vetoed.
     * The logged message describes the event that was vetoed.
     * <P>
     * @param evt An EventFireEvent object describing the event source
     *   	and the event that was vetoed.
     */
    public void recoverFromEventVeto(EventFireEvent evt)
      {
      if(traceEventsFlag)
	 logTraceOutput(kTraceDipLogName + kEventVetoed + "|dippableBean=" + dippableBeanClassName + "|event=" + evt.getEventSetName());
      }

// PropertyChangeListener interface implementation
    /**
     * Logs a message describing the property when a bound property is changed.
     * <P>
     * @param evt A PropertyChangeEvent object describing the event
     * source and the property that changed.
     */
   public void propertyChange(PropertyChangeEvent evt)
     {
     if(tracePropertiesFlag)
	 logTraceOutput(kTraceDipLogName + kPropertyChange  + "|dippableBean=" + dippableBeanClassName + "|Property=" + evt.getPropertyName() + "|NewValue=" + evt.getNewValue() + "|OldValue=" + evt.getOldValue());
     }

// VetoableChangeListener interface implementation
   /**
    * Logs a message describing the property when a bound property is changed.
    * <P>
    *  @param evt A PropertyChangeEvent object describing the
    *  event source and the property that changed.
    */
   public void vetoableChange(PropertyChangeEvent evt)
     {
     if(tracePropertiesFlag)
	 logTraceOutput(kTraceDipLogName + kVetoableChange  + "|dippableBean=" + dippableBeanClassName + "|Property=" + evt.getPropertyName() + "|NewValue=" + evt.getNewValue() + "|OldValue=" + evt.getOldValue());
     }


    // Private Methods
    private void readObject( ObjectInputStream stream ) throws ClassNotFoundException, IOException{
        stream.defaultReadObject();
        if(myMsgDispClient == null)
           myMsgDispClient = new MsgDispClient();
    }

/**
   * A String representing the location where the MsgDispServer is running.
   *
   */
  private String msgDispHostName;

  /**
   * A boolean flag indicating whether or not to trace Methods
   * in the dippable bean.
   */
  private boolean traceMethodsFlag;

  /**
   * A boolean flag indicating whether or not to trace Events
   * in the dippable bean.
   */
  private boolean traceEventsFlag;

  /**
   * A boolean flag indicating whether or not to trace Properties
   * in the dippable bean.
   */
  private boolean tracePropertiesFlag;


  /**
   * A constant String representing the name for this dip.
   *
   */
  private static final String kTraceDip = "Trace Dip";


  /**
   * Array of property change listeners.  Initialized by the
   * createPropertyChangeListener() method.
   */
  private DipPropertyChangeListener[] myPropertyChangeListener;

  /**
   * Array of property change listeners.  Initialized by the
   * createVetoableChangeListener() method.
   */
  private DipVetoableChangeListener[] myVetoableChangeListener;

  /**
   * Array of property change listeners.  Initialized by the
   * createMethodCallListener() method.
   */
  private DipMethodCallListener[] myMethodCallListener;

  /**
   * Array of property change listeners.  Initialized by the
   * createVetoableMethodListener() method.
   */
  private DipVetoableMethodListener[] myVetoableMethodListener;

  /**
   * Array of property change listeners.  Initialized by the
   * createEventFireListener() method.
   */
  private DipEventFireListener[] myEventFireListener;

  /**
   * Arrary of property change listeners.  Initialized by the
   * createVetoableEventListener() method.
   */
  private DipVetoableEventListener[] myVetoableEventListener;

  // Serializable Stream Unique ID
  private static final long serialVersionUID = 2000L;

  private Object myDippableBean;
  private String dippableBeanClassName;

  private MsgDispClient myMsgDispClient = null;

  //------------------------------
  // Messages presented to the user via logging facility.	
  //------------------------------	
  private static final String kTraceDipLogName = "TraceDip.";
  private static final String kEventVetoed = "recoverFromEventVeto";
  private static final String kMethodVetoed = "recoverFromMethodVeto";
  private static final String kHandleEventFire = "handleEventFire";
  private static final String kHandleMethodCall = "handleMethodCall";
  private static final String kHandleVetoableMethod = "handleVetoableMethod";
  private static final String kHandleVetoableEvent = "handleVetoableEvent";
  private static final String kPropertyChange = "propertychange";
  private static final String kVetoableChange = "vetoableChange";

  private static final String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
}

