//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbMediaDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.6 com/java/src/com/ibm/beans/samples/media/AudioPlayer.java, bbMediaDemo, bb97c, bb97c9750e 10/31/97 17:08:10 [12/11/97 17:02:47]
package com.ibm.beans.samples.media;

import java.applet.Applet;
import java.applet.AudioClip;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.PopupMenu;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.net.URL;
import com.ibm.beans.tools.BeanLauncher;
import com.ibm.beans.widgets.CloseWindowAdapter;
import com.ibm.beans.widgets.FileExtension;

/**
 * <p>
 * The AudioPlayer bean is a simple player for au files.
 * An audio file can be selected by double-clicking on the bean.
 * The AudioPlayer also offers a pop-up menu (accessed via the
 * secondary mouse button), with options for loading a new audio clip
 * (via a standard file dialog), or playing the current audio clip.
 * The audio clip can also be set by wiring something in another bean
 * to the setFileName method of this bean.  This bean is also an
 * applet - see AudioPlayer.html for a sample invocation.
 *
 * @author John W. Alcorn (jalcorn@us.ibm.com)
 */
public class AudioPlayer extends Applet implements Serializable, MouseListener, ActionListener {
	public AudioPlayer() {
		fileName = ""; //jwa: If it's null, then it can't be set via the PropertySheet
		img = loadIcon(true);
	}

	public AudioPlayer(String audioFileName) {
		fileName = audioFileName; //jwa: we don't call setFileName, since getAudioClip won't work until constructor returns
		img = loadIcon(true);
	}

	public void init() {
		String appletTagParam = getParameter("AudioFile");
		if (appletTagParam!=null) {
			File appletTagFile = new File(appletTagParam);
			if (appletTagFile.exists()) setFileName(appletTagFile.getAbsolutePath());
		} /* endif */
		setAllowMouseInteraction("true".equalsIgnoreCase(getParameter("MouseInteraction")));
	}

	public void destroy() {
		if (img != null) img.flush();
	}

	public void start() {
		if (fileName=="") {
			URL url = getClass().getResource("default.au");
			clip = getAudioClip(url);
		} else {
			setFileName(fileName); //since we can't load an audio clip in our constructor
		} /* endif */
	
		appletStarted = true;
		if (allowMouseInteraction) enableMouseInteraction(true);
	}

	public void stop() {
		appletStarted = false;
		if (allowMouseInteraction) enableMouseInteraction(false);
	}

	public Image loadIcon(boolean large) {
		String iconFileName = large ? "AudioPlayerIconColor32.gif" : "AudioPlayerIconColor16.gif";
		URL url = getClass().getResource(iconFileName);
		return getToolkit().getImage(url);
	}

	public String chooseAudioFile() {
		Frame parentFrame = new Frame("");
		FileDialog fd = new FileDialog(parentFrame, "Select an au file", FileDialog.LOAD);
		fd.setFilenameFilter(new FileExtension("*.au"));
		fd.show();
		return(fd.getFile()!=null ? fd.getDirectory()+fd.getFile() : null);
	}

	public void loadAudioClip() { setFileName(chooseAudioFile()); }

	public void playAudioClip() { if (clip!=null) clip.play(); }

	public void stopAudioClip() { if (clip!=null) clip.stop(); }

	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
		in.defaultReadObject();
		img = loadIcon(true);
		if (!fileName.equals("")) setFileName(fileName); /* Cause transient audio clip to be reloaded */
	}

	public void actionPerformed(ActionEvent evt) {
		if (evt.getActionCommand().equals("Load...")) loadAudioClip();
		if (evt.getActionCommand().equals("Play")) playAudioClip();
		if (evt.getActionCommand().equals("Stop")) stopAudioClip();

		if (popupAdded) {
			remove(popup); //jwa: we remove it due to a deserialization bug with popups (4040848)
			popupAdded = false;
		} /* endif */
	}

	public void mouseClicked(MouseEvent evt) {
		if (allowMouseInteraction && (evt.getClickCount()==2)) setFileName(chooseAudioFile());
	}

	public void mouseReleased(MouseEvent evt) {
		if (allowMouseInteraction && evt.isPopupTrigger()) {
			if (!popupAdded) {
				add(popup);
				popupAdded = true;
			} /* endif */

			popup.show(this, evt.getX(), evt.getY());
			// jwa: We remove the popup after processing the ActionEvent it generates, in order
			// to workaround a bug in JDK 1.1.x that keeps you from being able to deserialize
			// a component that has a popup attached to it.  Note that, if the popup is
			// dismissed via a means other than choosing one of its items (such as by hitting
			// the 'Esc' key), then we don't get a chance to remove it (the call to show the
			// popup isn't modal, so it returns immediately, meaning we can't call remove from
			// here).  Hopefully JavaSoft will address this bug in a future release of the
			// JDK (it still happens in JDK 1.1.3 for WIN32, and in the GA JDK 1.1.1 for OS/2).
			// Note: this is JavaSoft bug #4040848.
		} /* endif */
	}

	public void mousePressed(MouseEvent evt) {}
	public void mouseEntered(MouseEvent evt) {}
	public void mouseExited(MouseEvent evt) {}

	public void paint(Graphics g) {
		if (img != null) g.drawImage(img, 0, 0, this);
	}

	public Dimension getPreferredSize() {
		Dimension size = new Dimension(imgWidth, imgHeight);
		if (size.width<=0) size.width = 32;
		if (size.height<=0) size.height = 32;
		return size;
	}

	public Dimension getMinimumSize() {	return(getPreferredSize()); }

	public String getFileName() { return fileName; }

	public synchronized void setFileName(String newFileName) {
		if (newFileName != null) {
			fileName = newFileName;
		} else {
			return;
		} /* endif */
	
		File audioFile = new File(fileName);
		if (audioFile.exists()) {
			try {
				URL url = new URL("file:/"+fileName);
				clip = getAudioClip(url);

				if (clip==null) { //if that fails, try using getResource()
					URL url2 = getClass().getResource(fileName);
					clip = getAudioClip(url2);
				} /* endif */

				if (clip==null) System.err.println("getAudioClip("+url.toString()+")==null");
			} catch(Exception ex) {
				System.err.println("Exception '"+ex.getMessage()+"' loading "+fileName);
			} /* endtry */
		} else {
			System.err.println("File " + fileName + " not found!");
		} /* endif */
	}

	public boolean getAllowMouseInteraction() {	return allowMouseInteraction; }

	public void setAllowMouseInteraction(boolean allow) {
		allowMouseInteraction = allow;
		if (appletStarted) enableMouseInteraction(allowMouseInteraction);
	}

	private void enableMouseInteraction(boolean enable) {
		if (enable) {
			if (popup==null) {
				popup = new PopupMenu();
				popup.add("Load...");
				popup.add("Play");
				popup.add("Stop");
			} /* endif */
	
			popup.addActionListener(this);
			addMouseListener(this);
		} else {
			popup.removeActionListener(this);
			removeMouseListener(this);
		} /* endif */
	}

	public static void main(String argv[]) {
		Frame myFrame = new Frame("Audio Player");
	
		CloseWindowAdapter adapter = new CloseWindowAdapter(myFrame);
		myFrame.addWindowListener(adapter);
	
		AudioPlayer myAudio;
		if (argv.length==0)
			myAudio = new AudioPlayer();
		else
			myAudio = new AudioPlayer(argv[0]);
	
		myFrame.add(myAudio);

		//the following line sets up the applet environment, so that we don't get
		//NullPointerException's when calling getAudioClip().
		BeanLauncher.assignAppletStub(myAudio, myAudio.getClass());
		myAudio.setAllowMouseInteraction(true);
		myAudio.start();
	
		myFrame.pack();
		myFrame.setIconImage(myAudio.loadIcon(false));
		myFrame.show();
	}

	transient Image img = null;
	transient AudioClip clip = null;
	transient PopupMenu popup = null;
	transient boolean popupAdded = false;
	transient boolean appletStarted = false;
	transient int imgWidth = 32;
	transient int imgHeight = 32;

	private boolean allowMouseInteraction = false;
	private String fileName;

	private final static long serialVersionUID = 2000L;
	private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
}

