//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbMediaDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.18 com/java/src/com/ibm/beans/samples/media/ImageViewer.java, bbMediaDemo, bb97c, bb97c9750e 12/10/97 17:25:11 [12/11/97 17:02:44]
package com.ibm.beans.samples.media;

import java.applet.Applet;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.PopupMenu;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.net.URL;
import com.ibm.beans.widgets.CloseWindowAdapter;
import com.ibm.beans.widgets.FileExtension;
import com.ibm.beans.widgets.PrintingSupport;

/**
 * <p>
 * The ImageViewer bean is a simple viewer for gif and jpg images.
 * It also supports animation, via the animated gif file format.
 * An image can be selected by double-clicking on the bean.
 * The ImageViewer also offers a pop-up menu (accessed via the
 * secondary mouse button), with options for loading a new image
 * (via a standard file dialog), or printing the current image.
 * Note that interaction with the bean via the mouse is only
 * enabled if the allowMouseInteraction property is set to true.
 * The image can also be set by wiring something in another bean
 * to the setFileName method of this bean.  This bean is also an
 * applet - see ImageViewer.html for a sample invocation.
 *
 * @author John W. Alcorn (jalcorn@us.ibm.com)
 */
public class ImageViewer extends Applet implements Serializable, MouseListener, ActionListener {
	public ImageViewer() {
		fileName = ""; //jwa: If it's null, then it can't be set via the PropertySheet

		tracker = new MediaTracker(this);
		img = loadIcon(true);
	}

	public ImageViewer(String imageFileName) {
		setFileName(imageFileName);
	}

	public void init() {
		try {
			setFileName(getParameter("ImageFile"));
			setAllowMouseInteraction("true".equalsIgnoreCase(getParameter("MouseInteraction")));
		} catch (NullPointerException ex) {}
	}

	public void destroy() {
		if (img != null) img.flush();
	}

	public void start() {
		appletStarted = true;
		if (allowMouseInteraction) enableMouseInteraction(true);
	}

	public void stop() {
		appletStarted = false;
		if (allowMouseInteraction) enableMouseInteraction(false);
	}

	public Image loadIcon(boolean large) {
	   try {
		String iconFileName = large ? "ImageViewerIconColor32.gif" : "ImageViewerIconColor16.gif";
		URL url = getClass().getResource(iconFileName);
		return getToolkit().getImage(url);
	    }
	    catch (Exception ex) {
		return null;
	    }
	}

	public String chooseImageFile() {
		Frame parentFrame = new Frame("");
		FileDialog fd = new FileDialog(parentFrame, "Select a gif or jpg file", FileDialog.LOAD);
		fd.setFilenameFilter(new FileExtension("*.gif"));
		fd.show();
		parentFrame.dispose();
		return(fd.getFile()!=null ? fd.getDirectory()+fd.getFile() : null);
	}

	public void loadImage() { setFileName(chooseImageFile()); }

	public void printImage() {
		PrintingSupport.printComponent(this, null, fileName, null);
	}

	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException {
		in.defaultReadObject();

		if (fileName.equals("")) {
			img = loadIcon(true);
		} else {
			setFileName(fileName); /* Cause transient image to be reloaded */
		} /* endif */
	}

	public void actionPerformed(ActionEvent evt) {
		if (evt.getActionCommand().equals("Load...")) loadImage();
		if (evt.getActionCommand().equals("Print...")) printImage();

		if (popupAdded) {
			remove(popup); //jwa: we remove it due to a deserialization bug with popups (4040848)
			popupAdded = false;
		} /* endif */
	}

	public void mouseClicked(MouseEvent evt) {
		if (allowMouseInteraction && (evt.getClickCount()==2)) setFileName(chooseImageFile());
	}

	public void mouseReleased(MouseEvent evt) {
		if (allowMouseInteraction && evt.isPopupTrigger()) {
			if (!popupAdded) {
				add(popup);
				popupAdded = true;
			} /* endif */

			popup.show(this, evt.getX(), evt.getY());
			// jwa: We remove the popup after processing the ActionEvent it generates, in order
			// to workaround a bug in JDK 1.1.x that keeps you from being able to deserialize
			// a component that has a popup attached to it.  Note that, if the popup is
			// dismissed via a means other than choosing one of its items (such as by hitting
			// the 'Esc' key), then we don't get a chance to remove it (the call to show the
			// popup isn't modal, so it returns immediately, meaning we can't call remove from
			// here).  Hopefully JavaSoft will address this bug in a future release of the
			// JDK (it still happens in JDK 1.1.3 for WIN32, and in the GA JDK 1.1.1 for OS/2).
			// Note: this is JavaSoft bug #4040848.
		} /* endif */
	}

	public void mousePressed(MouseEvent evt) {}
	public void mouseEntered(MouseEvent evt) {}
	public void mouseExited(MouseEvent evt) {}

	public void paint(Graphics g) {
		if (img != null) {
			if (stretchImage) {
				Dimension size = getSize();
				g.drawImage(img, 0, 0, size.width, size.height, this);
			} else {
				g.drawImage(img, 0, 0, this);
			} /* endif */
		} /* endif */
	}

	public Dimension getPreferredSize() {
		Dimension size = new Dimension(imgWidth, imgHeight);
		if (size.width<=0) size.width = 32;
		if (size.height<=0) size.height = 32;
		return size;
	}

	public Dimension getMinimumSize() {	return(getPreferredSize()); }

	public String getFileName() { return fileName; }

	public synchronized void setFileName(String newFileName) {
		if (newFileName != null) {
			fileName = newFileName;
		} else {
			return;
		} /* endif */

		Image imgOld = img;
		File imgFile = new File(fileName);

		boolean exists=false;
		try {
		    exists=imgFile.exists();
		} catch (SecurityException s) {}

		if (exists) {
			img = getToolkit().getImage(fileName);
		} else {
			//try loading it via getResource
			try {
				URL url = getClass().getResource(fileName);
				img = getToolkit().getImage(url);
			} catch (Exception ex) {
				System.err.println("File " + fileName + " not found!");
				return;
			} /* endtry */
		} /* endif */

		if (img != null) {
			if (tracker == null) tracker = new MediaTracker(this);

			if (imgOld != null) {
				tracker.removeImage(imgOld, 0);
				imgOld.flush();
			} /* endif */

			tracker.addImage(img, 0);
			try { tracker.waitForID(0, 5000); } catch (InterruptedException ex) {};

			imgWidth = img.getWidth(this);
			imgHeight = img.getHeight(this);
			if ((imgWidth>0) && (imgHeight>0)) resize(imgWidth, imgHeight);

			Container parent = getParent();
			if (parent instanceof Frame) {
				Frame parentFrame = (Frame) parent;
				parentFrame.pack();
			} /* endif */

			repaint();
		} else {
			System.err.println("Error loading " + fileName);
		} /* endif */
	}

	public boolean getStretchImage() { return stretchImage; }

	public void setStretchImage(boolean stretch) {
		stretchImage = stretch;
		if (appletStarted) repaint();
	}

	public boolean getAllowMouseInteraction() { return allowMouseInteraction; }

	public void setAllowMouseInteraction(boolean allow) {
		allowMouseInteraction = allow;
		if (appletStarted) enableMouseInteraction(allowMouseInteraction);
	}

	private void enableMouseInteraction(boolean enable) {
		if (enable) {
			if (popup==null) {
				popup = new PopupMenu();
				popup.add("Load...");
				popup.add("Print...");
			} /* endif */
	
			popup.addActionListener(this);
			addMouseListener(this);
		} else {
			popup.removeActionListener(this);
			removeMouseListener(this);
		} /* endif */
	}

	public static void main(String argv[]) {
		Frame myFrame = new Frame("Image Viewer");
	
		CloseWindowAdapter adapter = new CloseWindowAdapter(myFrame);
		myFrame.addWindowListener(adapter);
	
		ImageViewer myImage;
		if (argv.length==0)
			myImage = new ImageViewer();
		else
			myImage = new ImageViewer(argv[0]);
	
		myFrame.add(myImage);
		myImage.setAllowMouseInteraction(true);
		myImage.start();

		myFrame.pack();
		myFrame.setIconImage(myImage.loadIcon(false));
		myFrame.show();
	}

	transient Image img = null;
	transient MediaTracker tracker = null;
	transient PopupMenu popup = null;
	transient boolean popupAdded = false;
	transient boolean appletStarted = false;
	transient int imgWidth = 32;
	transient int imgHeight = 32;

	private boolean allowMouseInteraction = false;
	private boolean stretchImage = true;
	private String fileName;

	private final static long serialVersionUID = 2000L;
	private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
}

