//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbMediaDemo
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.4 com/java/src/com/ibm/beans/samples/media/ImageViewerCustomizer.java, bbMediaDemo, bb97c, bb97c9750e 10/31/97 17:06:15 [12/11/97 17:02:46]
package com.ibm.beans.samples.media;

import java.awt.Button;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;
import java.beans.Customizer;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ResourceBundle;

/**
 * <p>
 * The ImageViewerCustomizer allows the user of the ImageViewer to specify
 * the image to be displayed, by providing a descriptive label, an entry
 * field, and a "Browse" button which will pop up a file dialog.
 *
 * @author John W. Alcorn (jalcorn@us.ibm.com)
 * @see com.ibm.beans.samples.media.ImageViewer
 */
public class ImageViewerCustomizer extends Panel implements Customizer, TextListener, ActionListener {

  // Panel overrides

  /**
   * Constructs a new Image Viewer Customizer.  All of the AWT controls for
   * the customizer panel are created here.  These controls are initialized
   * in <i>setObject</i>.
   *
   * @see #setObject
   */
  public ImageViewerCustomizer() {
     propertyChangeSupport = new PropertyChangeSupport(this);

//   resourceBundle = (ImageViewerResourceBundle) ResourceBundle.getBundle(
//      "COM.ibm.beans.samples.media.ImageViewerResourceBundle");

     FlowLayout layout = new FlowLayout();
     setLayout(layout);

/* Create the descriptive label */
     Label fileNameLabel = new Label(
//      resourceBundle.getString("FileNameLabel"), Label.RIGHT);
        "Specify a gif or jpg file:", Label.RIGHT);
     add(fileNameLabel);

/* Create the text field */
     fileNameTextField = new TextField(16);
     fileNameTextField.addTextListener(this);
     add(fileNameTextField);

/* Create the "Browse" button */
     browseButton = new Button(
//      resourceBundle.getString("BrowseButton"));
        "Browse");
     add(browseButton);
  }


  /**
   * Returns the smallest width and height at which this panel can be displayed
   * with all of its AWT controls visible.
   *
   * @return A Dimension structure containing this panel's minimum width and height
   */
  public Dimension getMinimumSize() {
     return(new Dimension(209, 237));
  }


  // Customizer interface

  /**
   * Sets the object instance with which this customizer is to interact.
   * This is where each of the AWT controls (created in the constructor
   * method) are initialized, based on the values of the properties
   * associated with the instance of the ImageViewer provided as input.
   *
   * @param bean	The instance of the ImageViewer to customize.
   */
  public void setObject(Object bean) {
     if ((bean != null) && (bean instanceof ImageViewer)) {
        imageViewer = (ImageViewer) bean;
     } else {
        return; //not much we can do - caller isn't expecting any exceptions
                //returning at least keeps us from trapping...
     } /* endif */

     fileNameTextField.setText(imageViewer.getFileName());

     //following call needs to be after above setText() to keep
     //ImageViewer.setFileName() from being called just because
     //the customizer was brought up.
     fileNameTextField.addTextListener(this);
     browseButton.addActionListener(this);
  }


  /**
   * Adds a listener to be notified when the customizer changes
   * one of the bean's properties.
   *
   * @param listener	The object to be notified of changes.
   */
  public void addPropertyChangeListener(PropertyChangeListener listener) {
     propertyChangeSupport.addPropertyChangeListener( listener );
  }


  /**
   * Removes a listener from the notification list.
   *
   * @param listener	The object to be removed from the notification list.
   */
  public void removePropertyChangeListener(PropertyChangeListener listener) {
     propertyChangeSupport.removePropertyChangeListener( listener );
  }


  // ActionListener interface

  /**
   * Receives button clicks from the "Browse" button.
   *
   * @param evt	Describes the event that occurred.
   */
  public void actionPerformed(ActionEvent evt) {
/* "Browse" button clicked */
     if (evt.getSource()==browseButton) {
        String newFileName = imageViewer.chooseImageFile();
        if (newFileName!=null) {
           fileNameTextField.setText(newFileName); //this causes textValueChanged to be called
        } /* endif */
     } /* endif */
  }


  // TextListener interface

  /**
   * Receives keystrokes from the text fields.
   *
   * @param evt	Describes the event that occurred.
   */
  public void textValueChanged(TextEvent evt) {
/* Image FileName text changed */
     if (evt.getSource()==fileNameTextField) {
        String oldFileName = imageViewer.getFileName();
        String newFileName = fileNameTextField.getText();
        imageViewer.setFileName(newFileName);
        propertyChangeSupport.firePropertyChange("fileName", oldFileName, newFileName);
     } /* endif */
  }


/* Instance of the ImageViewer being customized */
  private ImageViewer imageViewer = null;

/* AWT controls */
  private TextField fileNameTextField = null;
  private Button    browseButton = null;

/* Helper class for handling property updates via the customizer */
  private PropertyChangeSupport propertyChangeSupport = null;

/* Helper class for retrieving translatable strings */
//private ImageViewerResourceBundle resourceBundle = null;

/* Standard BeanExtender Copyright statement */
  private final static String kBeanExtenderIBMCopyright = "(c) Copyright IBM Corporation 1997";
}

