//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbNetDeploy
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.14 com/java/src/com/ibm/beans/samples/netdeploy/client/Launcher.java, bbNetDeploy, bb97c, bb97c9750e 11/19/97 11:44:36 [12/11/97 16:58:17]

package com.ibm.beans.samples.netdeploy.client;

import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.applet.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import com.ibm.beans.netdeploy.client.NetworkClassLoader;

/**
 * This application downloads and runs the specified application.
 * The custom classloader COM.ibm.beans.netdeploy.client.NetworkClassLoader
 * is used to download the specified class from any file group
 * configured with the On Demand download option.
 *
 * @see com.ibm.beans.netdeploy.client.NetworkClassLoader
 */
public class Launcher
{
	private final static String kBeanExtenderIBMCopyright="(c) Copyright IBM Corporation 1997";
	/**
	 * This method downloads and launches the specified application.  The first
	 * parameter supplied is an array of strings entered from the command line.
	 * The first string must be the class name of an application which resides
	 * in a file group configured on the locally running Network Deployment
	 * client.  The file group must be configured with the "On Demand" download
	 * control option.  Any more strings in the String array passed to this
	 * method are used as arguments for the specified application.
	 * <p>
	 * Once the application's class file is downloaded, it is used to create a
	 * Method object for the "main" method in the class.  Next, an argument list
	 * for the method is constructed.  This argument list consists of the String
	 * array passed into this method, minus the first element.  The Method
	 * object is then used to invoke the "main" method with the argument list 
	 * just constructed.
	 */
	public static void main(String argv[])
		throws Throwable
	{      
                ClassLoader l = null;

		// Make sure an argument was passed in.
		if (argv == null || argv.length == 0 || argv[0] == "")
		{
			System.out.println("Class name required.  Syntax:  java Launcher " +
			                   "<application class name> <args>");
			System.exit(0);
		}

                try {

		   l = new NetworkClassLoader();
                }
                catch(java.rmi.NotBoundException ex)
                {
                  System.out.println("Error: Client daemon is NOT running.");
                  System.out.println("Please start the Client daemon and try again.");
                  //System.out.println(ex.getMessage());
                  ex.printStackTrace();
		  System.exit(0);
                  
                }

                catch(com.ibm.beans.netdeploy.NetDeployUnknownHostException unex)
                {
                  System.out.println("Please verify the client daemon hostname in the BeanSystem properties file."); 
                  System.out.println(unex.getMessage());
                  unex.printStackTrace();
		  System.exit(0);
                  
                }

                catch(java.lang.Exception ex)
                {
                  System.out.println("Error: Unable to contact the Client daemon."); 
                  System.out.println("Please restart the Client daemon and try again.");
                  //System.out.println(ex.getMessage());
                  ex.printStackTrace();
		  System.exit(0);
                  
                }



		try {
			Class c = null;
			Method m = null;

			try
			{
				// Attempt to load class
				c = l.loadClass(argv[0]);

                                if(c == null)
                                {
				 System.out.println("Class " + argv[0] + " not found.");
				 System.exit(0);
                                }
			}
			catch(ClassNotFoundException xcpt)
			{
				System.out.println(xcpt.getMessage());

				System.exit(0);
			}

			try
			{
				// Construct a Method object for the application class's "main"
				// method.  To construct a Method object, the method name is
				// required, plus an array of classes that corresponds to the
				// method's parameter list.  Since the parameter list for 
				// application's "main" method is the same as that for
				// Launcher.main(), simply use the "argv" variable to construct
				// the required class object.
				Class[] parmTypes = new Class[1];
				parmTypes[0] = argv.getClass();
				m = c.getMethod("main", parmTypes);
			}
			catch(NoSuchMethodException xcpt)
			{
				String mName = new String("public static main(String argv[])");
				String cName = c.getName();
				System.out.println("The method \"" + mName + 
				                   "\" does not exist in class " + cName);
				System.exit(0);
			}
			catch(SecurityException xcpt)
			{
				String mName = new String("main(String argv[])");
				String cName = c.getName();
				System.out.println
					("Security violation occurred while attempting to access " +
					 "method \"" + mName + "\" in class \"" + cName + "\".");
				System.exit(0);
			}

			try
			{
				String stringArg[];

				// Construct a String array to pass to the application "main"
				// method.  The array consists of all strings passed to
				// Launcher.main(), except for the first string.  This first string
				// is the class name of the application and thus is not part of
				// the application's argument list.
				int numArgs = argv.length - 1;
				if (numArgs > 0)
				{
					stringArg = new String[numArgs];
					for (int i = 0; i < numArgs; i++)
						stringArg[i] = argv[i + 1];
				}
				else
					stringArg = new String[0];

				// Note:  The argument list must be passed in as an array of
				//        objects.  The String array is essentially one object and
				//        is therefore one argument to the method call.  Since
				//        there are no other arguments, an Object array with only
				//        one element is created, with the one element being the
				//        String array.
				Object[] argArray = new Object[1];
				argArray[0] = stringArg;

				// Invoke the application's "main" method.  Note that a null
				// object reference is passed to the Method.invoke().  No
				// object reference is required for static methods.  We want
				// to avoid instantiating the specified class ourselves anyway,
				// since we don't necessarily know how.
				Object dummy = m.invoke(null, argArray);
			}
			catch(NullPointerException xcpt)
			{
				// Since a null object reference is passed to Method.invoke(), this
				// exception will occur if the "main" method is not defined as
				// static.
				System.out.println("The method \"" + c.getName() + ".main()\" " +
				                   "must be declared a static method.");
				System.exit(0);
			}
			catch(IllegalAccessException xcpt)
			{
				System.out.println("Illegal access of class \"" + argv[0] + "\".");
				System.exit(0);
			}
			catch(IllegalArgumentException xcpt)
			{
				System.out.println("Internal error:  IllegalArgumentException.");
				System.exit(0);
			}
			catch(InvocationTargetException xcpt)
			{
				// Wrapper for exception thrown by method invoked by
				// Method.invoke()
				Throwable t = xcpt.getTargetException();
				throw t;
			}
		} catch (Exception xcpt) {
			String msg = xcpt.getMessage();
			Class c = xcpt.getClass();
			if (msg != null)
				System.out.println("Caught exception " + c.getName() +
				                   ": " + msg);
			else
				System.out.println("Caught exception: [No exception message]");
         xcpt.printStackTrace();
			throw xcpt;
		}
	}
}


