//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbNetDeploy
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//       
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.5 com/java/src/com/ibm/beans/samples/netdeploy/warehouse/SimpleStorageSystemImpl.java, bbNetDeploy, bb97c, bb97c9750e 11/24/97 18:41:46 [12/11/97 16:58:53]
/*-------------------------------------------------------------------+
|                    StorageSystemImpl.java                          |
| ================================================================== |
|                                                                    |
| Description:  Class used to abstract the underlying storage system.| 
|                                                                    |
| Author(s):    Sreedhar Arda and Ajay Apte                          |
|               Copyright IBM Corporation, 1997                      |
|                                                                    |
+-------------------------------------------------------------------*/
package com.ibm.beans.samples.netdeploy.warehouse;

import java.io.IOException;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Properties;

import com.ibm.beans.netdeploy.warehouse.*;
import com.ibm.beans.netdeploy.*;

import com.ibm.beans.samples.netdeploy.warehouse.DefaultAuthenticationImpl;

/**
 * Class used to abstract the underlying storage system. 
 */

public class SimpleStorageSystemImpl extends StorageSystemImpl

{
   /*--------------------------------------------------------------------------+
   |  Copyright notice:                                                        |
   +--------------------------------------------------------------------------*/

   private final static String kBeanExtenderIBMCopyright =
           "(c) Copyright IBM Corporation 1997";

    protected String pluginFileName = null;

    protected String pluginClassName = null;


    /**
     * SimpleStorageSystemImpl constructor.
     */

    public SimpleStorageSystemImpl()
    {
	super();
    }

    /**
     * Adds contents to the storage system.  A content publisher publishes
     * her contents to a warehouse which in turn delegates the stock call
     * to the storage system.
     * @param id The Warhouse Item ID.
     * @param content The content data.
     * @param metadata The content's metadata.
     * @param authData The authentication information needed to perform the task.
     * @return <tt>true</tt> if the contents are stocked; <tt>false</tt>
     * otherwise. 
     * @exception InvalidPrimaryKeyException The Warehouse item ID is not
     * valid or does not exist.
     * @exception IOException Data cannot be stocked in the storage system.
     * @exception UserNotAuthorizedException The publisher does not have
     * permissions to stock the contents.
     */
    public boolean stock(WarehouseItem newItem, 
                         AuthenticationData authData)       
                            throws IOException, NetDeployException
    {

        if (authData == null)
            return false;

        try {
            if ((authTable != null) && (authTable.checkUserAuthentication (authData) == true))
                return super.stock (newItem, authData);
        }
        catch (Exception e) {
            System.out.println(e);
            e.printStackTrace();
        }
        return false;
    }

   
    /**
     * Returns the contents associated with a Warehouse item ID. 
     * @param id The Warehouse Item ID. 
     * @param authData The authentication information needed to perform the task.
     * @return The contents associated with the Warehouse item ID.
     * @exception InvalidPrimaryKeyException The Warehouse item ID is not
     * valid or does not exist. 
     * @exception IOException Content cannot be retrieved from the 
     * storage system.
     * @exception UserNotAuthorizedException The user does not have
     * permission to request the content.  
     */
    public RemoteInputStreamInterface order(WarehouseItemID id, AuthenticationData authData)
                            throws IOException
    {

        if (authData == null)
            return null;

        try {
            if ((authTable != null) && (authTable.checkUserAuthentication (authData) == true))
                return super.order (id, authData);
        }
        catch (Exception e) {
            System.out.println(e);
            e.printStackTrace();
        }
        return null;

    }


    /**
     * Updates the contents for a specified Warehouse item ID. 
     * @param id The Warehouse item ID. 
     * @param content The updated content data. 
     * @param metadata The updated metadata information. 
     * @param authData The authentication information needed to perform the task.
     * @return <tt>true</tt> if the contents are updated; <tt>false</tt>
     * otherwise.
     * @exception InvalidPrimaryKeyException The Warehouse item ID is not
     * valid or does not exist. 
     * @exception IOException Contents can not be updated. 
     * @exception UserNotAuthorizedException The publisher does not have
     * permission to update the contents. 
     */

    public boolean restock(WarehouseItem newItem, 
                           AuthenticationData authData)
                            throws IOException, NetDeployException
    {


        if (authData == null)
            return false;

        try {
            if ((authTable != null) && (authTable.checkUserAuthentication (authData) == true))
                return super.restock (newItem, authData);
        }
        catch (Exception e) {
            System.out.println(e);
            e.printStackTrace();
        }
        return false;

    }



    /**
     * Deletes a Warehouse item ID from the storage system. 
     * @param id The Warehouse item ID. 
     * @return <tt>true</tt> if the contents are deleted; <tt>false</tt> otherwise.
     * @param authData The authentication information needed to perform the task.
     * @exception InvalidPrimaryKeyException The Warehouse item ID is 
     * not valid or does not exist. 
     * @exception IOException The contents cannot be deleted from 
     * the storage system.
     * @exception UserNotAuthorizedException The user does not have
     * permission to delete the data.
     */
    public boolean unstock (WarehouseItemID id, AuthenticationData authData)
				throws IOException
    {

        if (authData == null)
            return false;

        try {
            if ((authTable != null) && (authTable.checkUserAuthentication (authData) == true))
                return super.unstock (id, authData);
        }
        catch (Exception e) {
            System.out.println(e);
            e.printStackTrace();
        }
        return false;

    }


    /**
     * Checks if a client is authorized to stock the data.
     * If the authentication data is <tt>null</tt>, the 
     * storage system asks the client negotiator to add/run
     * an authentication plug-in on the client to collect 
     * the necessary authentication information. 
     * @param authData The authentication information needed to stock the 
     * contents.
     * @param cn A client negotiator that fetches the required authentication information form the client. 
     * @return The AuthenticationData object that can be used in the stock call; 
     * <tt>null</tt> if the client is not allowed to stock the content.
     */

    public AuthenticationData getAuthorizationToStock (
                                                 AuthenticationData authData,
                                                 ClientNegotiator cn)
                                                 throws IOException
    {
        return internalCheckAuthentication (cn, authData);
    }

    /**
     * Checks if a client is authorized to restock the data.
     * If the authentication data is <tt>null</tt>, the 
     * storage system asks the client negotiator to add/run
     * an authentication plug-in on the client to collect 
     * the necessary authentication information. 
     * @param id The Warehouse item ID. 
     * @param authData The authentication information needed to restock 
     * the contents.
     * @param cn A client negotiator that fetches the required authentication information. 
     * @return The AuthenticationData object that can be used in the restock call; 
     * <tt>null</tt> if the client is not allowed to restock the content.
     */

    public AuthenticationData getAuthorizationToRestock (WarehouseItemID id,
                                                 AuthenticationData authData,
                                                 ClientNegotiator cn)
                                                 throws IOException
    {
        return internalCheckAuthentication (cn, authData);
    }


    /**
     * First checks whether the requested Warehouse item ID exists
     * or not. If it exists checks whether the user is authorized 
     * to order the data.
     * If the authentication data is <tt>null</tt>, the 
     * storage system asks the client negotiator to add/run 
     * an authentication plug-in on the client to collect 
     * the necessary authentication information. 
     * @param id The Warehouse item ID. 
     * @param authData The authentication information needed to order 
     * the contents.
     * @param cn A client negotiator that fetches the required authentication information from the client. 
     * @return The AuthenticationData object that can be used in the order call; 
     * <tt>null</tt> if the user is not allowed to order the contents.
     */

    public AuthenticationData getAuthorizationToOrder (WarehouseItemID id,
                                               AuthenticationData authData,
                                               ClientNegotiator cn)
                                               throws IOException
    {
        return internalCheckAuthentication (cn, authData);

    }

    protected void setAuthenticationClassName (String clsName) {

        try {
            Class c = Class.forName (clsName);
            this.authTable = (AuthenticationInterface)c.newInstance ();
        }
        catch (Exception e) {
            System.out.println ("Cannot create auth system :" + e);
            e.printStackTrace();
        }
    }

    protected void setAuthenticationPluginFileName (String fileName) {

        pluginFileName = fileName;
    }

    protected void setAuthenticationPluginClassName (String clsName) {

        pluginClassName = clsName;
    }

  
    private AuthenticationData internalCheckAuthentication (ClientNegotiator cn,
                                               AuthenticationData authData)
                                               throws IOException
    {
         if (authData != null) {

             try {
                 if (authTable.checkUserAuthentication (authData) == true)
                     return authData;
             }
             catch (IOException ioe) {
                 throw ioe;
             }
             catch (Exception e) {
                 System.out.println(e);
                 e.printStackTrace();
             }
         }

         if (cn == null)
             return null;

         UIDPasswordData data = null;
         try {

             data = (UIDPasswordData) 
                               cn.getAuthenticationData(pluginClassName);
         }
         catch (AuthPluginNotFoundException authEx) {

             System.out.println ("Trying to install plugin .. ");
             PluginItem item = new PluginItem ();
             try {

                 item.metadata = new Properties();
                 String fileName = pluginFileName.substring ( 
                           pluginFileName.lastIndexOf (File.separator) + 
                           File.separator.length());
                 item.metadata.put (PluginItem.ITEM_NAME, fileName);
                 item.metadata.put (PluginItem.ITEM_TYPE, "application/java-archive");
                 DataTransferImpl dd = new DataTransferImpl();
                 item.content = dd.createRemoteInputStream (pluginFileName);
                 cn.addAuthenticationPlugin (item);
                 data = (UIDPasswordData) 
                               cn.getAuthenticationData(pluginClassName);
             }
             catch (Exception ee) {
                 System.out.println ("SimpleStorage: Exception from plugin install:: " + ee);
                 ee.printStackTrace();
             } 
         }
         catch (IOException ioe) {
             throw ioe;
         }
         catch (Exception e) {
             System.out.println(e);
             e.printStackTrace();
         }

         if ((data != null) && 
             (authTable.checkUserAuthentication(data) == true))
             return data;
         else
             return null;
    }
}
