//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbScripting
//
//   CLASSES: none
//
//   ORIGINS: 27, 180
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.7 com/java/src/com/ibm/beans/samples/script/form/JarInfo.java, bbScripting, bb97c, bb97c9750e 9/30/97 15:38:27 [12/11/97 16:54:29]

package com.ibm.beans.samples.script.form;

import java.beans.*;

/**
 * Represents a loaded Jar file
 */


public class JarInfo {
   private static final String kBeanExtenderIBMCopyright=
                 "(c) Copyright IBM Corporation 1997";
    /**
     * Private data
     */
    private SimpleClassLoader classLoader;
    private String[] beanNames;
    private BeanInfo[] beanInfos;
    private boolean[] fromPrototype;
    private MessageHeader[] manifestData;
    private String jarName;

    // REMIND! We currently don't care much on whether a bean is from
    // a serialized prototype or not; the only distinction is in the lazy
    // computation of the BeanInfo...

    /** 
     * Create a JarInfo.
     * @param	jarName	The name of the file containing the Jar
     * @param	cl	The ClassLoader instance
     * @param	beanName	The names for all the beans
     * @param	fromPrototype	Whether this bean is from a serialized prototype
     */
    public JarInfo(String jarName,
		   SimpleClassLoader cl,
		   String[] beanNames,
		   boolean[] fromPrototype,
		   MessageHeader[] manifestData) {
	if (beanNames.length != fromPrototype.length) {
	    throw new Error("beanNames and fromPrototype need to have the same length");
	}
	this.jarName = jarName;
	this.classLoader = cl;
	this.beanNames = beanNames;
	this.fromPrototype = fromPrototype;
	this.manifestData = manifestData;
	this.beanInfos = new BeanInfo[beanNames.length];
	for (int i = 0; i < beanNames.length; i++) {
	    if (fromPrototype[i]) {
		// delay instantiating it
		continue;
	    }
	    // get the BeanInfo data
	    Class c = cl.findClass(beanNames[i]);
	    BeanInfo bi;
	    try {
		bi = Introspector.getBeanInfo(c);
	    } catch (Exception ex) {
		System.err.println("JarInfo: couldn't find BeanInfo for "+c+
				   "; caught "+ex);
		continue;
	    }
	    beanInfos[i] = bi;
	    debug("JarInfo:: @ "+i+"; beanName: "+beanNames[i]+
		  "; fromPrototype: "+fromPrototype[i]);
	}
	// Sort?
    }

    // Accessing the information in this JarInfo

    /**
     * Get the name of the file containing this JAR
     */
    public String getJarName() {
	return jarName;
    }

    /**
     * Get the number of beans in this Jar file
     */
    public int getCount() {
	return beanNames.length;
    }

    /**
     * Get the BeanInfo for the ith bean in this jar file
     */
    public BeanInfo getBeanInfo(int i) {
	if (beanInfos[i] != null) {
	    return beanInfos[i];
	} else {
	    Object b = getInstance(beanNames[i]);
	    if (b != null) {
		Class c = b.getClass();
		BeanInfo bi;
		try {
		    bi = Introspector.getBeanInfo(c);
		} catch (Exception ex) {
		    System.err.println("JarInfo: couldn't find BeanInfo for "+c+"; caught "+ex);
		    return null;
		}
		beanInfos[i] = bi;
		return bi;
	    }
	    return null;
	}
    }

    /**
     * The bean name of this  bean
     * REMIND - should this be an attribute of the BeanDescriptor of its BeanInfo?
     */
    public String getName(int i) {
	return beanNames[i];
    }

    /**
     * Was this bean from a serialized prototype? -- unused so far?
     */
    public boolean isFromPrototype(String name) {
	return fromPrototype[indexForName(name)];
    }

    /**
     * Get to Manifest Headers for this bean
     */
    public MessageHeader getManifestData(String name) {
	return manifestData[indexForName(name)];
    }

    /**
     * Get a new Bean instance given its name
     */
    public Object getInstance(String name) {
	try {
	    return Beans.instantiate(classLoader, name);
        } catch (Exception ex) {
	    System.err.println("Could not instantiate "+name);
	    System.err.println("  caught an exception: "+ex);
	    ex.printStackTrace();
	    return null;
	}
    }

    // REMIND - may need to clean up this a bit.
    private int indexForName(String name) {
	for (int i=0; i<beanNames.length ; i++) {
	    if (beanNames[i].equals(name)) {
		return i;
	    }
	}
	return -1;
    }


    // Debugging stuff

    private static boolean debug = false;
    private static void debug(String msg) {
	if (debug) {
	    System.err.println("JarInfo:: "+msg);
	}
    }

}
