//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbUtils
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.10 com/java/src/com/ibm/beans/tools/debugger/BeanDebugger.java, bbDebugger, bb97c, bb97c9750e 11/18/97 17:01:21 [12/11/97 17:00:44]
package com.ibm.beans.tools.debugger;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import sun.tools.debug.*;

import com.ibm.beans.util.MessageFormatter;
import com.ibm.beans.tools.debugger.BeanDbgView;
import com.ibm.beans.tools.debugger.DebuggerResourceBundle;
import com.ibm.beans.widgets.ToolTipCanvas;

public class BeanDebugger implements DebuggerCallback
{

    private final static String kBeanExtenderIBMCopyright =
                               "(c) Copyright IBM Corporation 1997";

private DebuggerResourceBundle res;
private MessageFormatter mf;
private BeanDbgView beanDbgView;
private RemoteDebugger debugger;
private ThreadContext currentThreadContext; // a new one is created only when 
                          // a breakpoint is hit or when a thread is switched.


/********************************************************
 Initialization
********************************************************/

public BeanDebugger (String argv[], BeanDbgView view)
  {
   res = DebuggerResourceBundle.getRes();
   mf = new MessageFormatter("com.ibm.beans.tools.debugger.DebuggerResourceBundle");
   beanDbgView = view;
   debugger = createRemoteDebugger (argv);
  }

public BeanDebugger (String argv[])
  {
   res = DebuggerResourceBundle.getRes();
   mf = new MessageFormatter("com.ibm.beans.tools.debugger.DebuggerResourceBundle");
   debugger = createRemoteDebugger (argv);
  }


public BeanDebugger (String host, String password) {

   res = DebuggerResourceBundle.getRes();
   mf = new MessageFormatter("com.ibm.beans.tools.debugger.DebuggerResourceBundle");
   debugger = createRemoteDebugger (host, password);
}

protected RemoteDebugger createRemoteDebugger (String host, String password)
{

   try {

       Thread.currentThread().setPriority(Thread.NORM_PRIORITY);

       RemoteDebugger dbg = new RemoteDebugger(host, password, this, false);

       return dbg;
   }
   catch (Exception e) {

       e.printStackTrace();
   }

     return null;
}

protected RemoteDebugger createRemoteDebugger (String [] argv)
  {

   try
     {

       Thread.currentThread().setPriority(Thread.NORM_PRIORITY);
       
       String s = new String("");

       if (argv != null) {
           for (int i = 0; i < argv.length; i++)
               s = s + "  " + argv[i];
       }
       RemoteDebugger dbg = new RemoteDebugger(s, this, false);
       return dbg;
     }

   catch(Exception e)
     {
       e.printStackTrace();
     }

     return null;
  }

public synchronized RemoteDebugger getDebugger()
  {
   return debugger;
  }

public ThreadContext getThreadContext () {

    if (currentThreadContext == null) {
        currentThreadContext = new ThreadContext(this);
        return currentThreadContext;
    }
    return currentThreadContext;

}

public synchronized BeanDbgView getBeanDbgView () {

    return beanDbgView;
}

public int Close () {

    if (debugger != null)
        debugger.close();
    return 0;
}

public void setBeanDbgView (BeanDbgView view) {

    beanDbgView = view;
}

/********************************************************
 Callback Listener
********************************************************/

public synchronized void printToConsole(String string)
{
    Vector v = beanDbgView.getDebuggerPanels();
    int s =  v.size();

    for(int i = 0; i < s; i++)
    {
        Object o = v.elementAt(i);
        if (o instanceof ConsolePanel)
        {
             ConsolePanel console = (ConsolePanel) o;
             console.print(string);
             break;
        }
    }
}


public synchronized void breakpointEvent(RemoteThread thread)
  {
   ThreadContext context = new ThreadContext(this);
   context.setRemoteThread(thread);
   try
     {
       RemoteStackFrame stack[] = thread.dumpStack();
       context.setStackFrame(stack[0]);
     }
   catch(Exception e)
     {
       e.printStackTrace();
       return;
     }

   currentThreadContext = context;
   beanDbgView.updateContext(context);
  }


public synchronized void exceptionEvent(RemoteThread thread, String string)
{

  //            System.out.println("exception event");
}

public synchronized void threadDeathEvent(RemoteThread thread)
{
        try {
            RemoteThreadGroup[] glist = debugger.listThreadGroups(null);
            RemoteThread[] threads = glist[0].listThreads(true);

            ThreadContext context = new ThreadContext (this);
            context.setRemoteThread(threads[0]);
            context.setStackFrame(threads[0].getCurrentFrame());

            currentThreadContext = context;
            beanDbgView.updateContext(currentThreadContext);
        }
        catch (Exception e) {
            return;
        }
}

public synchronized void quitEvent()
{
  //            System.out.println("quit event");
}

/********************************************************
 Public APIS for other panles and BeanDbgView 
********************************************************/

public synchronized void startDebugging (String clsString, String argString) {

    RemoteClass dbgClass;
    String dbgClassName;

    try {
        dbgClass = debugger.findClass(clsString);
        dbgClassName = dbgClass.getName();
    }
    catch (Exception e) {

        String s = new String (mf.format(DebuggerResourceBundle.LOAD_CLASS_ERROR,
                        clsString));
        beanDbgView.getTip().setMessage (s);
        return;
    }

    if (dbgClass == null) {

        String s = new String (
            res.getString(DebuggerResourceBundle.LOAD_CLASS_ERROR) + clsString);

        beanDbgView.getTip().setMessage (s);
        return;
    }

    try {
        RemoteField method;
        try {
            method = dbgClass.getMethod("main");
            String err = dbgClass.setBreakpointMethod(method);
        }
        catch (NoSuchMethodException nsme) {

            String s = new String (mf.format(DebuggerResourceBundle.NO_MAIN_ERROR,
                                clsString));
            beanDbgView.getTip().setMessage (s);
            return;
        } 

        int dbgArgc = 2;
        String[] dbgArgv= new String[2];
        dbgArgv[0] = dbgClassName;
        if (argString != null)
            dbgArgv[1] = argString;
        else
            dbgArgv[1] = new String ("");

        RemoteThreadGroup tg = debugger.run(dbgArgc, dbgArgv);

    }
    catch (Exception e) {
   
        String s = new String (mf.format(DebuggerResourceBundle.NOT_RUN_ERROR,
                                 clsString));
        beanDbgView.getTip().setMessage (s);
        return;
    }
}

public synchronized void breakPointsUpdate(RemoteClass remote_class,
                     int line_no, boolean add_bp)
  {

  }

public synchronized boolean setBreakpointMethod (String clsName, String methodName) {

    try {

        RemoteClass cls = debugger.findClass (clsName);
        RemoteField method = cls.getMethod (methodName);
        String err = cls.setBreakpointMethod (method);
        return true;
    
    }
    catch (Exception exp) {
        exp.printStackTrace();
    }
    return false;

}

public synchronized boolean setBreakpointLine (String clsName, int num) {

    try {

        RemoteClass cls = debugger.findClass (clsName);
        String err = cls.setBreakpointLine (num);
        return true;
    }
    catch (Exception exp) {
        String s = "" + num;
        System.out.println(mf.format(DebuggerResourceBundle.BP_SET_LINE_ERROR, s));
    }
    return false;

}

public synchronized void viewStackFrame(RemoteStackFrame stackFrame, RemoteThread rt)
  {
   currentThreadContext = new ThreadContext(this);
   currentThreadContext.setStackFrame(stackFrame);
   currentThreadContext.setRemoteThread(rt);
   beanDbgView.updateContext(currentThreadContext);
  }

public synchronized void performThreadAction (ThreadPanel panel, 
                         RemoteThread thr, int cmd) {

    if (thr == null)
        return;

    if (cmd == ThreadPanel.ACTION_SWITCH) {

        ThreadContext context = new ThreadContext (this);
        context.setRemoteThread(thr);
        try {
            context.setStackFrame(thr.getCurrentFrame());
        }
        catch (Exception e) {
            e.printStackTrace();
            return;
        }
        currentThreadContext = context;
        beanDbgView.updateContext(currentThreadContext);
        return;
    }
    
    try {

        switch (cmd) {

            case (ThreadPanel.ACTION_SUSPEND):
                thr.suspend();
                break;
            case ThreadPanel.ACTION_RESUME:
                thr.resume();
                break;
            case ThreadPanel.ACTION_STOP:
                thr.stop();
                break;
        }
    }
    catch (Exception e) {
        e.printStackTrace();
    }

}

/********************************************************
 Debug Function Handlers : Step, Next, Cont etc..
********************************************************/


public synchronized void Step(RemoteThread thread)
{
    if (thread == null) {
        if (currentThreadContext != null)
            thread = currentThreadContext.getRemoteThread();
    }

    if (thread == null)
     printToConsole(res.getString(DebuggerResourceBundle.NO_THREAD_ERROR) + "\n");
 
    else {
        try {
            thread.step(true);
        }
        catch(Exception e) {
             System.out.println(res.getString(
                  DebuggerResourceBundle.STEP_INTO_ERROR));
        }
    }

}

public synchronized void Next(RemoteThread thread)
{
     if (thread == null) {

         if (currentThreadContext != null)
             thread = currentThreadContext.getRemoteThread();
     }

     if (thread == null)
         printToConsole(
            res.getString(DebuggerResourceBundle.NO_THREAD_ERROR) + "\n");
     else {
         try {
               thread.next();
         }
         catch(Exception e) {
                 System.out.println(res.getString(
                     DebuggerResourceBundle.STEP_OVER_ERROR));
         }
     }
}

public synchronized void Cont(RemoteThread thread)
{

    if (thread == null) {

        if (currentThreadContext != null)
            thread = currentThreadContext.getRemoteThread();
    }

    if (thread == null)
        
        printToConsole(res.getString(DebuggerResourceBundle.NO_SUSPEND_ERROR));
    else {

        try {

            RemoteThreadGroup glist[] = debugger.listThreadGroups(null);
            RemoteThreadGroup rtg;

            for (int i = 0; i < glist.length; i++) {

                if (glist[i].getName().equalsIgnoreCase("system") == true)
                    continue;

                RemoteThread list[] = glist[i].listThreads(true);

                for (int j = 0; j < list.length; j++) {

                    if (list[j] == thread)  {

                        for (int k = 0; k < list.length; k++)
                            list[k].cont();
                        thread.resetCurrentFrameIndex();

                        i = glist.length + 1;

                        break;
                    }
                }
            }
        }
        catch(Exception e)
        {
             System.out.println(res.getString(
               DebuggerResourceBundle.CONTINUE_ERROR));
        }
    }

}



}


