//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbUtils
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.8 com/java/src/com/ibm/beans/tools/debugger/DbgTreeNode.java, bbDebugger, bb97c, bb97c9750e 11/18/97 17:00:39 [12/11/97 17:00:29]
package com.ibm.beans.tools.debugger;

import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Enumeration;

import com.ibm.beans.tools.debugger.DebuggerPanel;


public class DbgTreeNode {

    private final static String kBeanExtenderIBMCopyright =
                               "(c) Copyright IBM Corporation 1997";

    public DbgTreeNode (DebuggerPanel newDbgPanel, String newLabel) {

        dbgPanel = newDbgPanel;
        label = newLabel;
    }


    public synchronized DebuggerPanel getDbgPanel () {
        return dbgPanel;
    }

    public synchronized Vector  getChildren() {
        return children;
    }

    public synchronized boolean hasChildren() {
        if (children == null) return false;
        else return true;
    }

    public synchronized String  getLabel() {
        return label;
    }

    public synchronized void setLabel(String newLabel) {
        label = null;
        label = newLabel;
    }

    public synchronized void toggleExpansionState() {
        if (bExpanded)
           bExpanded = false;
        else
           bExpanded = true;
    }

    public synchronized void setExpansionState(boolean b) {
        bExpanded = b;
    }

    public synchronized boolean getExpansionState() {
        return bExpanded;
    }

    public synchronized void addChild(DbgTreeNode n) {
        if (children == null)
            children = new Vector(0);
        children.addElement(n);
    }

    public synchronized int drawContents (Graphics g, int x, int y) {

        int retVal = y;
        int drawLabelOffset = Toolkit.getDefaultToolkit().getFontMetrics(g.getFont()).getHeight() / 2 - 2;

        // draw the expansion box and the line

        if (children != null) {
            g.drawRect (x - (fBoxWidth/2), y - (fBoxHeight/2), fBoxWidth, fBoxHeight);
            g.drawLine (x + (fBoxWidth/2), y, x + fHorizontalLineWidth, y);

            g.drawLine (x - fExpLength, y, x + fExpLength, y);
            if (bExpanded == false)
                g.drawLine (x, y - fExpLength, x, y + fExpLength);

        }
        else {
            g.drawLine (x, y, x + fHorizontalLineWidth, y);
        }

        // draw the label itself

        if (label != null)
            g.drawString (label, x + fHorizontalLineWidth + 2, y + drawLabelOffset);

        // draw the children and the vertical line

        if ((children != null) && (bExpanded == true)) {

            int vertLineX = x + fXDelta;
            int vertLineY = y;
            int vertLineY2 = 0;

            for (Enumeration e = children.elements(); e.hasMoreElements();) {
                DbgTreeNode n = (DbgTreeNode)e.nextElement();

                vertLineY2 = retVal + fYDelta;
                if (n.hasChildren() == true)
                    vertLineY2 -= fBoxHeight/2;

                retVal = n.drawContents (g, vertLineX, retVal + fYDelta);

                g.drawLine (vertLineX, vertLineY, vertLineX, vertLineY2);
                vertLineY = vertLineY2;
                if (n.hasChildren() == true)
                    vertLineY += fBoxHeight;
            }
        }

        // store the last draw position so that we can use it for checking if the
        // expansion box was clicked. This is just a fast way of doing it.

            lastDrawX = x;
            lastDrawY = y;
        
        return retVal;
    }

    public synchronized boolean handleMouseClickInView (int x, int y) {

        if (children == null)
            return false;

        if ((Math.abs (x - lastDrawX) <= fBoxWidth / 2) &&
            (Math.abs (y - lastDrawY) <= fBoxHeight / 2)) {
            toggleExpansionState();
            return true;
        }

        else if (bExpanded == true) {
            for (Enumeration e = children.elements(); e.hasMoreElements();) {
                DbgTreeNode n = (DbgTreeNode)e.nextElement();
                if (n.handleMouseClickInView (x, y) == true)
                    return true;
            }
        }

        return false;

    }

    public synchronized void dumpTree (int level) {
        System.out.println("Node at level " + level + " " + label);
        if (children == null)
            return;
        for (Enumeration e = children.elements(); e.hasMoreElements();) {
            DbgTreeNode n = (DbgTreeNode)e.nextElement();
            n.dumpTree(level + 1);
        }
    }

    public Vector children;
    public DebuggerPanel dbgPanel;
    public String label;
    public boolean bExpanded;

    protected int lastDrawX;
    protected int lastDrawY;


    protected static final int fBoxHeight = 10;
    protected static final int fBoxWidth = 10;

    private static final int fXDelta = 18;
    private int fYDelta = 30;

    private static final int fHorizontalLineWidth = 42;
    private static final int fExpLength = 3;
}
