//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbUtils
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.10 com/java/src/com/ibm/beans/tools/debugger/DbgTreePanel.java, bbDebugger, bb97c, bb97c9750e 11/18/97 17:10:51 [12/11/97 17:00:35]
package com.ibm.beans.tools.debugger;

import java.lang.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Enumeration;

import sun.tools.debug.*;

import com.ibm.beans.tools.debugger.DebuggerPanel;

public class DbgTreePanel extends DebuggerPanel implements ComponentListener, Serializable {

    private final static String kBeanExtenderIBMCopyright =
                               "(c) Copyright IBM Corporation 1997";
    private static final long serialVersionUID = 2000L;

    public DbgTreePanel (ThreadContext threadContext) {

        super(threadContext);
        initData();
    }

    public DbgTreePanel () {

        super();
        initData();
    }

    public synchronized void initData() {

        setLayout(null);

        topLevels = new Vector(0);
        add(vBar = new Scrollbar(Scrollbar.VERTICAL));
        add(hBar = new Scrollbar(Scrollbar.HORIZONTAL));
        add(dbgTreeView = new DbgTreeView(this));

        addComponentListener(this);
        vBar.addAdjustmentListener(dbgTreeView);
        hBar.addAdjustmentListener(dbgTreeView);

    }

    public Vector getTreeList () {
        return topLevels;
    }

    public DbgTreeView getDbgTreeView () {
        return dbgTreeView;
    }

    public synchronized Scrollbar getVerticalScrollbar () {
        return vBar;
    }

    public synchronized Scrollbar getHorizontalScrollbar () {
        return hBar;
    }



    public void componentResized(ComponentEvent event) {

        Dimension d = getSize();

        vBar.setBounds(d.width-kScrollbarWidth, 0, kScrollbarWidth, d.height-kScrollbarHeight);
        hBar.setBounds(0, d.height-kScrollbarHeight, d.width, kScrollbarHeight);
        dbgTreeView.setBounds(0, 0, d.width-kScrollbarWidth, d.height-kScrollbarHeight);

        adjustScrollbars ();
    }

    public void componentHidden(ComponentEvent event) {

    }

    public void componentMoved(ComponentEvent event) {

    }


    public void componentShown(ComponentEvent event) {

    }

    public synchronized boolean handleMouseClickInView (int x, int y) {

       if (topLevels == null)
           return true;

       for (Enumeration e = topLevels.elements(); e.hasMoreElements();) {
           DbgTreeNode n = (DbgTreeNode)e.nextElement();
           if (n.handleMouseClickInView (x, y) == true) {
               adjustScrollbars ();
               break;
           }
       }
       return true;
    }

    public synchronized void adjustScrollbars () {


            int num = topLevels.size();

            for (Enumeration e = topLevels.elements(); e.hasMoreElements();) {
                DbgTreeNode n = (DbgTreeNode)e.nextElement();
                num += getNumChildren (n);
            }

            int y = num * dbgTreeView.getDrawYDelta();
            int oldhBar = hBar.getValue();
            int oldvBar = vBar.getValue();

            int visible = this.getSize().height-kScrollbarHeight;
            if (y > visible)
                vBar.setValues (oldvBar, visible, 0, y);
            else
                vBar.setValues (0, visible, 0, visible);

            visible =  this.getSize().width-kScrollbarWidth;
            if (visible < kXScrollMaxVal)
               hBar.setValues (oldhBar, visible, 0, kXScrollMaxVal);
            else
               hBar.setValues (oldhBar, visible, 0, visible);

            dbgTreeView.setScrollOffset (hBar.getValue(),vBar.getValue());
            dbgTreeView.repaint();
    }

    private synchronized int getNumChildren (DbgTreeNode n) {

        int num = 0;
        if (n.hasChildren() && (n.getExpansionState() == true)) {
            Vector children = n.getChildren();
            num += children.size();

            for (Enumeration e = children.elements(); e.hasMoreElements();) {
                DbgTreeNode n1 = (DbgTreeNode)e.nextElement();
                num += getNumChildren (n1);
            }
        }
        return num;
    }

    private void readObject (ObjectInputStream in) throws IOException {

        try {

            in.defaultReadObject ();
            initData ();
        }
        catch (Exception exp) {
            exp.printStackTrace();
        }
    }

    private void writeObject (ObjectOutputStream out) throws IOException {

        try {

            removeComponentListener(this);
            out.defaultWriteObject ();
            addComponentListener(this);
        }
        catch (Exception exp) {
            exp.printStackTrace();
        }
    }


    public transient Vector topLevels;
    public transient DbgTreeView dbgTreeView;
    private transient Scrollbar hBar;
    private transient Scrollbar vBar;
    private transient boolean bAdjustScrollBars = false;

    private static final int kScrollbarWidth = 20;
    private static final int kScrollbarHeight = 20;

    private static final int kXScrollMaxVal = 600;

}

