//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbUtils
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.9 com/java/src/com/ibm/beans/tools/debugger/DbgTreeView.java, bbDebugger, bb97c, bb97c9750e 12/3/97 11:08:33 [12/11/97 17:00:42]
package com.ibm.beans.tools.debugger;

import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Enumeration;

import sun.tools.debug.*;

import com.ibm.beans.tools.debugger.DebuggerPanel;


public class DbgTreeView extends Canvas implements AdjustmentListener {

    private final static String kBeanExtenderIBMCopyright =
                               "(c) Copyright IBM Corporation 1997";

    public DbgTreeView (DbgTreePanel newDbgPanel) {

        super();

        dbgPanel = newDbgPanel;
        mouseAdapter = new DbgPanelMouseAdapter(this);
        addMouseListener(mouseAdapter);

        dx = dy = 0;
    }

    public void paint(Graphics g)
    {
        Vector topLevels = dbgPanel.getTreeList();
        if (topLevels != null) {

            // set the scrolling translation
            g.translate (-dx, -dy);
            drawTree (g, topLevels);
        }
    }

    private synchronized int drawTree(Graphics g, Vector topLevels) {

       int x = 30;
       int y = fYDelta;
       int vertLineY = y;
       int vertLineY2 = 0;
       boolean bFlag = false;

       g.setColor(Color.black);
       g.setFont(fFont);

       for (Enumeration e = topLevels.elements(); e.hasMoreElements();) {
           DbgTreeNode n = (DbgTreeNode)e.nextElement();

           vertLineY2 = y;
           if (n.hasChildren())
               vertLineY2 -= fBoxHeight / 2;

           y = n.drawContents (g, x, y) + fYDelta;

           if (bFlag)
               g.drawLine (x, vertLineY, x, vertLineY2);
           else
               bFlag = true;
           vertLineY = vertLineY2;
           if (n.hasChildren() == true)
               vertLineY += fBoxHeight;

       }


       // we need to subtract fYDelta from y since we areadding it to y
       // in the above loop. This way y represents the y value
       // where the last string was drawn.

       return y - fYDelta;
    }


    public synchronized boolean handleMouseClick (MouseEvent evt) {

       dbgPanel.handleMouseClickInView (evt.getX()+dx, evt.getY()+dy);
       return true;
    }

    public synchronized void setScrollOffset (int x, int y) {
        dx = x;
        dy = y;
    }

    public synchronized void changeScrollOffset (int x, int y) {
        dx += x;
        dy += y;
    }

    public synchronized int getScrollOffsetX () {
        return dx;
    }

    public synchronized int getScrollOffsetY () {
        return dy;
    }

    public synchronized int getDrawYDelta () {
        return fYDelta;
    }

    public void adjustmentValueChanged(AdjustmentEvent e) {

        if (e.getAdjustable() == dbgPanel.getVerticalScrollbar()) {
            setScrollOffset (dx, e.getValue());
            repaint();
        }

        if (e.getAdjustable() == dbgPanel.getHorizontalScrollbar()) {
            setScrollOffset (e.getValue(), dy);
            repaint();
        }
    }

    public DbgTreePanel dbgPanel;
    public DbgPanelMouseAdapter mouseAdapter;

    private int dx;
    private int dy;

    private static final int fBoxHeight = 10;
    private int fYDelta = 30;
    private static final Font fFont = new Font ("Helvetica", Font.PLAIN, 12);
}


// Mouse adapter

class DbgPanelMouseAdapter extends MouseAdapter {

   private final static String kBeanExtenderIBMCopyright =
                              "(c) Copyright IBM Corporation 1997";

   public DbgPanelMouseAdapter (DbgTreeView panel) {
       super();
       varView = panel;
   }

   public void mouseClicked(MouseEvent e) {
       varView.handleMouseClick(e);
   }

   public DbgTreeView varView = null;

}
