//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbUtils
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.7 com/java/src/com/ibm/beans/tools/debugger/SRCPanel.java, bbDebugger, bb97c, bb97c9750e 11/18/97 17:01:24 [12/11/97 17:00:45]
package com.ibm.beans.tools.debugger;

import java.awt.*;
import java.awt.event.*;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.net.*;
import java.awt.event.*;
import sun.tools.debug.*;

import com.ibm.beans.util.MessageFormatter;
import com.ibm.beans.tools.debugger.ScrollSourceList;
import com.ibm.beans.tools.debugger.BeanDebugger;
import com.ibm.beans.tools.debugger.BeanDbgView;

public abstract class SRCPanel extends DebuggerPanel 
      implements ComponentListener, ItemListener, Externalizable
{
    private final static String kBeanExtenderIBMCopyright =
                               "(c) Copyright IBM Corporation 1997";

    protected static final int LISTHEIGHT = 20;

    protected transient Choice choices;
    protected transient Vector sourceLists;
    protected transient String [] breakPoints;
    protected ScrollSourceList currentSourceList;
    protected transient DebuggerResourceBundle res;
    protected transient MessageFormatter mf;

    public SRCPanel()
    {
        super();
        res = DebuggerResourceBundle.getRes();
        mf = new MessageFormatter("com.ibm.beans.tools.debugger.DebuggerResourceBundle");
        initData();
    }

    public SRCPanel(ThreadContext thread_context)
    {
        super(thread_context);
        res = DebuggerResourceBundle.getRes();
        mf = new MessageFormatter("com.ibm.beans.tools.debugger.DebuggerResourceBundle");
        initData();

        contextUpdated (thread_context);
    }

    protected void initData () {

        sourceLists = new Vector();
        createPanel();
    }

    protected int getChoiceOffset()
    { 
       return(0);
    }

    protected void createPanel()
    {
        setLayout(null);

        choices = new Choice();
        add(choices);
        choices.setLocation(getChoiceOffset(), 0);        
        //choices.setVisible(true);
        createSourceLists();

        addComponentListener (this);
        choices.addItemListener(this);
    }

    protected abstract void createSourceLists(); 

    protected void showTip (ThreadContext c, String s) {
        c.getBeanDebugger().getBeanDbgView().getTip().setMessage(s);
    }

    public void contextUpdated (ThreadContext context) {

        super.contextUpdated (context);

        RemoteThread thr;
        RemoteStackFrame frame;
        String str;
        RemoteClass cls;
        DebuggerResourceBundle res = DebuggerResourceBundle.getRes();
        String s = res.getString(DebuggerResourceBundle.NULL_THREAD_ERROR);

        try {
            thr = context.getRemoteThread();
            if (thr == null)
	    {
                System.out.println(s);
                return;
            }
            frame = context.getStackFrame();
            if (frame == null)
                frame = thr.getCurrentFrame();
            if (frame == null)
	    {
                s = res.getString(DebuggerResourceBundle.NULL_FRAME_ERROR);
                System.out.println(s);
                return;
            }
            cls = frame.getRemoteClass();
            if (cls == null)
	    {
                s = res.getString(DebuggerResourceBundle.NULL_CLASS_ERROR);
                System.out.println(s);
                return;
            }
            contextUpdate(cls, frame.getLineNumber());
        }
        catch (Exception e) {
            e.printStackTrace();
            return;
        }
    }

    protected abstract void contextUpdate(RemoteClass cls, int line_no);


    public void itemStateChanged (ItemEvent evt) {

        if (evt.getStateChange() != ItemEvent.SELECTED)
            return;
        
        changeChoice(choices.getSelectedIndex(), -1);
    }

    protected void changeChoice(int index, int lineNum) {

        ScrollSourceList list = (ScrollSourceList) 
                                   sourceLists.elementAt(index);

        if (list == currentSourceList) {
            currentSourceList.selectLine (lineNum);
            return;
        }

        // change the source list
        remove (currentSourceList);
        currentSourceList = list;
        currentSourceList.setVisible (false);
        add (currentSourceList);
        currentSourceList.setVisible (true);
        currentSourceList.setBounds (0, LISTHEIGHT, getSize().width,
                           getSize().height - LISTHEIGHT);
        choices.select(index);
        if (lineNum >= 0)
            currentSourceList.selectLine (lineNum);
    }

    protected int getRealSourceLineNum(int num, int index)
    {
       return(num);
    }

    protected abstract RemoteClass getRemoteClass(int index);

    public boolean changeBreakPoint (ScrollSourceList l, int num, boolean bSet) {


        int index = sourceLists.indexOf (l);
        if (index == -1) {
            return false;
        }
        
        
        RemoteClass cls = getRemoteClass(index);
        num = getRealSourceLineNum(num, index);
 
        try {

            String str;
            if (bSet)
                str = cls.setBreakpointLine (num);
            else 
                str = cls.clearBreakpointLine (num);

            if (str.length() > 2)
                str = str.concat ("\n");

            getThreadContext().getBeanDebugger().printToConsole(str);

            if (str.length() > 2)
                return false;
            else
                return true;
        }
        catch (Exception e) {
        }
        return false;
    }

    public void writeExternal (ObjectOutput out) throws IOException {

        try {

            super.writeExternal (out);

            String [] bps = getBeanDebugger().getDebugger().listBreakpoints();
            if ((bps != null) && (bps.length > 0))
                out.writeObject (bps);
            else if (breakPoints != null)
                out.writeObject (breakPoints);
            else
                out.writeObject (bps);
            
        }
        catch (Exception exp) {
            exp.printStackTrace();
        }
    }


    public void readExternal (ObjectInput in) throws IOException {

        try {

            super.readExternal (in);
            res = DebuggerResourceBundle.getRes();
            mf = new MessageFormatter(
                   "com.ibm.beans.tools.debugger.DebuggerResourceBundle");
            breakPoints = (String[]) in.readObject ();
            initData ();
        }
        catch (Exception exp) {
            exp.printStackTrace();
        }
    }

    public void componentHidden(ComponentEvent event) {
    }

    public void componentMoved(ComponentEvent event) {
    }

    public void componentResized(ComponentEvent event) {

        Dimension d = getSize();
        choices.setSize (d.width - getChoiceOffset(), LISTHEIGHT);        
        currentSourceList.setSize (d.width, d.height - LISTHEIGHT);
        repaint();
    }

    public void componentShown(ComponentEvent event) {
    }

}





