//====START_GENERATED_PROLOG======================================
//
//
//   COMPONENT_NAME: bbUtils
//
//   CLASSES: none
//
//   ORIGINS: 27
//
//
//   (C) COPYRIGHT International Business Machines Corp. 1997
//   All Rights Reserved
//   Licensed Materials - Property of IBM
//   US Government Users Restricted Rights - Use, duplication or
//   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
//   	
//   IBM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
//   ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
//   PURPOSE. IN NO EVENT SHALL IBM BE LIABLE FOR ANY SPECIAL, INDIRECT OR
//   CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
//   USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
//   OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
//   OR PERFORMANCE OF THIS SOFTWARE.
//
//====END_GENERATED_PROLOG========================================
//
// @(#) 1.11 com/java/src/com/ibm/beans/tools/debugger/ThreadPanel.java, bbDebugger, bb97c, bb97c9750e 11/18/97 17:10:54 [12/11/97 17:00:42]
package com.ibm.beans.tools.debugger;

import java.lang.*;
import java.awt.*;
import java.io.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.Enumeration;

import sun.tools.debug.*;

import com.ibm.beans.tools.debugger.DebuggerPanel;


public class ThreadPanel extends DbgTreePanel implements ActionListener, Serializable {
    private final static String kBeanExtenderIBMCopyright =
                               "(c) Copyright IBM Corporation 1997";
    private static final long serialVersionUID = 2000L;

    private transient PopupMenu popup;
    private transient ThreadTree lastPopupClient;
    private transient DebuggerResourceBundle res;
    

    public static final int ACTION_SUSPEND = 1;
    public static final int ACTION_RESUME = 2;
    public static final int ACTION_STOP = 3;
    public static final int ACTION_SWITCH = 4;

    public ThreadPanel (ThreadContext threadContext) {

        super(threadContext);
        res = DebuggerResourceBundle.getRes();
        contextUpdated (threadContext);
    }

    public ThreadPanel () {

        super();
        res = DebuggerResourceBundle.getRes();

    }

    public synchronized void contextUpdated(ThreadContext newThreadContext)
    {

        // remember the previous state
        Vector prevState  = getPreviousState();

        super.contextUpdated(newThreadContext);
        RemoteDebugger newDbg = newThreadContext.getBeanDebugger().getDebugger();

        // Add the default ones
        Vector topLevels = getTreeList();
        topLevels.removeAllElements();

        if (newDbg == null) {
        ThreadTree errMsg = new ThreadTree (this, new String (
           res.getString(DebuggerResourceBundle.NO_DEF_THREAD_MSG)));
            topLevels.addElement (errMsg);
        }

        else {

            try {

               RemoteThreadGroup[] tG = newDbg.listThreadGroups(null);
               for (int i = 0; i < tG.length; i++) {
                   ThreadTree thTree = new ThreadTree (this, tG[i]);
                   topLevels.addElement (thTree);
               }

            } catch(Exception e) {
                e.printStackTrace();
            }
       }

       // match the previous state
       matchPreviousState (prevState);
       prevState = null;

       adjustScrollbars ();
    }

    private synchronized Vector getPreviousState() {

        Vector topLevels = getTreeList();
        if (topLevels == null)
            return null;

        Vector prevState = new Vector (topLevels.size());

        for (Enumeration e = topLevels.elements(); e.hasMoreElements();) {
            ThreadTree n = (ThreadTree)e.nextElement();

            try {
                String name = n.getThreadGroup().getName();
                prevState.addElement (new String (name));
                prevState.addElement (new Boolean (n.getExpansionState()));
            } catch (Exception exp) {}

        } // end for

        return prevState;
    }

    private synchronized void matchPreviousState (Vector prevState) {

        if (prevState == null)
            return;

        Vector topLevels = getTreeList();
        if (topLevels == null)
            return;

        for (Enumeration e = topLevels.elements(); e.hasMoreElements();) {
            ThreadTree n = (ThreadTree)e.nextElement();

            try {
                String name = n.getThreadGroup().getName();

                for (Enumeration f = prevState.elements(); f.hasMoreElements();)
                {
                    String str = (String)f.nextElement();
                    Boolean state = (Boolean) f.nextElement();

                    if (name.equals(str) == true) {
                        n.setExpansionState (state.booleanValue());
                        break;
                    }
                } // end for

            } catch (Exception exp) {}

        } // end for

    }

    public PopupMenu getPopup (ThreadTree popupClient) {

        lastPopupClient = popupClient;

        if (popup != null)
            return popup;

        popup = new PopupMenu(res.getString(
                        DebuggerResourceBundle.THREAD_LABEL));
        {
            MenuItem mi = new MenuItem (
                res.getString(DebuggerResourceBundle.SUSPEND_LABEL));
            popup.add(mi);
        }
        {
            MenuItem mi = new MenuItem (
                res.getString(DebuggerResourceBundle.RESUME_LABEL));
            popup.add(mi);
        }
        {
            MenuItem mi = new MenuItem (
                res.getString(DebuggerResourceBundle.STOP_LABEL));

            popup.add(mi);
        }
        popup.addSeparator();
        {
            MenuItem mi = new MenuItem (
                res.getString(DebuggerResourceBundle.SWITCH_TO_LABEL));
            popup.add(mi);
        }
        add(popup);

        popup.addActionListener (this);

        return popup;

    }

    public void actionPerformed (ActionEvent evt) {

        if (lastPopupClient == null)
            return;
        RemoteThread thr = lastPopupClient.getThread();
        if (thr == null)
            return;

        String cmd = evt.getActionCommand();

        if (cmd.equals(
                res.getString(DebuggerResourceBundle.SUSPEND_LABEL)))
            getThreadContext().getBeanDebugger().
                              performThreadAction(this, thr, ACTION_SUSPEND); 
        else if (cmd.equals(
                res.getString(DebuggerResourceBundle.RESUME_LABEL)))
            getThreadContext().getBeanDebugger().
                              performThreadAction(this, thr, ACTION_RESUME); 
        else if (cmd.equals(
                res.getString(DebuggerResourceBundle.STOP_LABEL)))
            getThreadContext().getBeanDebugger().
                              performThreadAction(this, thr, ACTION_STOP); 
        else if (cmd.equals(
                res.getString(DebuggerResourceBundle.SWITCH_TO_LABEL)))
            getThreadContext().getBeanDebugger().
                              performThreadAction(this, thr, ACTION_SWITCH); 

    }

}

